jQuery.sap.declare("sas.stress.config.model.Hierarchy");
jQuery.sap.require("sas.stress.config.model.HierarchyLineItem");
jQuery.sap.require("sas.stress.config.StressAPI");
jQuery.sap.require("sas.stress.wkbench.StressUtil");

sap.ui.base.Object.extend("sas.stress.config.model.Hierarchy", {
    constructor: function (hierarchyKey,
                           categoryKey,
                           categoryName,
                           name,
                           version,
                           description,
                           createdByUser,
                           updatedByUser,
                           createdTimestamp,
                           updatedTimestamp,
                           lineItems,
                           levelNames,
                           hierarchyCode,
                           taxonomyPackageOwners,
                           dimensionKey,
                           dimensionName,
                           defaultHierarchy) {
        this.hierarchyKey = hierarchyKey;
        this.categoryKey = categoryKey;
        this.categoryName = categoryName;
        this.name = name;
        this.version = version;
        this.description = description;
        this.createdByUser = createdByUser;
        this.updatedByUser = updatedByUser;
        this.createdTimestamp = createdTimestamp;
        this.updatedTimestamp = updatedTimestamp;
        this.lineItems = lineItems; //Array of HierarchyLineItem
        this.levelNames = levelNames;
        this.hierarchyCode = hierarchyCode; // Hierarchy type (scenario, entity, custom, etc)
        this.taxonomyPackageOwners = taxonomyPackageOwners;
        this.nameVersion=sas.stress.wkbench.StressUtil.getNameVersion(this.name,this.version);
        this.dimensionKey = dimensionKey;
        this.dimensionName = dimensionName;
        this.defaultHierarchy = defaultHierarchy;
    }
});

sas.stress.config.model.Hierarchy.prototype.getHierarchyKey = function () {
    return this.hierarchyKey;
};

sas.stress.config.model.Hierarchy.prototype.getCategoryKey = function () {
    return this.categoryKey;
};

sas.stress.config.model.Hierarchy.prototype.getCategoryName = function () {
    return this.categoryName;
};

sas.stress.config.model.Hierarchy.prototype.getName = function () {
    return this.name;
};

sas.stress.config.model.Hierarchy.prototype.getVersion = function () {
    return this.version;
};

sas.stress.config.model.Hierarchy.prototype.getNameVersion = function () {
    return this.nameVersion;
};

sas.stress.config.model.Hierarchy.prototype.getDescription = function () {
    return this.description;
};

sas.stress.config.model.Hierarchy.prototype.getLineItems = function () {
    return this.lineItems;
};

sas.stress.config.model.Hierarchy.prototype.getLevelNames = function () {
    return this.levelNames;
};

sas.stress.config.model.Hierarchy.prototype.setLineItems = function (lineItems) {
    if ($.isArray(lineItems)) {
        this.lineItems = lineItems;
    }
};

sas.stress.config.model.Hierarchy.prototype.getCreatedByUser = function () {
    return this.createdByUser;
};

sas.stress.config.model.Hierarchy.prototype.getUpdatedByUser = function () {
    return this.updatedByUser;
};

sas.stress.config.model.Hierarchy.prototype.getCreatedTimestamp = function () {
    return this.createdTimestamp;
};

sas.stress.config.model.Hierarchy.prototype.getUpdatedTimestamp = function () {
    return this.updatedTimestamp;
};

sas.stress.config.model.Hierarchy.prototype.getHierarchyCode = function () {
    return this.hierarchyCode;
};

sas.stress.config.model.Hierarchy.prototype.getTaxonomyPackageOwners = function () {
    return this.taxonomyPackageOwners;
};

sas.stress.config.model.Hierarchy.prototype.getLineItemsByLevel = function (levelno) {
    var item, items = [];
    for (var i = 0; i < this.lineItems.length; i++) {
        item = this.lineItems[i];
        var level = item.getLevelNum();
        if (level === (levelno + 1)) {
            items.push(item);
        }
    }

    return items;
};

sas.stress.config.model.Hierarchy.prototype.getDimensionKey = function () {
    return this.dimensionKey;
};

sas.stress.config.model.Hierarchy.prototype.getDimensionName = function () {
    return this.dimensionName;
};

sas.stress.config.model.Hierarchy.prototype.getDefaultHierarchy = function () {
    return this.defaultHierarchy;
};

sas.stress.config.model.Hierarchy._hierarchies = {};

sas.stress.config.model.Hierarchy.getHierarchy = function (hierarchyKey) {
    return sas.stress.config.model.Hierarchy._hierarchies[hierarchyKey];
};

sas.stress.config.model.Hierarchy.getCachedHierarchies = function () {
    return _.values(sas.stress.config.model.Hierarchy._hierarchies);
};

sas.stress.config.model.Hierarchy.getSpecificHierarchyDetails = function (hierarchyKey) {
    var deferred = $.Deferred();
    //sas.stress.config.model.Hierarchy._hierarchies = {}; //reset cache
    sas.stress.config.StressAPI.getSpecificHierarchyDetails(hierarchyKey).done(function (hierarchyDetailsJSON) {
            var hierarchies = [];
            //_.each(hierarchyDetailsJSON, function (hierarchyDetailJSON) {
            var lineItems = [];
            _.each(hierarchyDetailsJSON.lineItems, function (lineItem) {
                var hierarchyItem = new sas.stress.config.model.HierarchyLineItem(lineItem.liKey, lineItem.hierarchyKey,
                    lineItem.liOrder, lineItem.contra, lineItem.aggregationType, lineItem.levelNames, lineItem.memberName,
                    lineItem.format, lineItem.formulaCode, lineItem.levelNum, lineItem.workgroupNames, lineItem.formulaCode);
                lineItems.push(hierarchyItem);
            });

            var hierarchyJSON = hierarchyDetailsJSON.hierarchy;
            if (hierarchyJSON) {
                var hierarchy = new sas.stress.config.model.Hierarchy(hierarchyJSON.hierarchyKey,
                                                                      hierarchyJSON.categoryKey,
                                                                      hierarchyJSON.categoryName,
                                                                      hierarchyJSON.name,
                                                                      hierarchyJSON.version,
                                                                      hierarchyJSON.description,
                                                                      hierarchyJSON.createdByUser,
                                                                      hierarchyJSON.updatedByUser,
                                                                      hierarchyJSON.createdTimestamp,
                                                                      hierarchyJSON.updatedTimestamp,
                                                                      lineItems,
                                                                      hierarchyJSON.levelNames,
                                                                      hierarchyJSON.code,
                                                                      hierarchyJSON.taxonomyPackageOwners,  //@@DD This was missing
                                                                      hierarchyJSON.dimensionKey,
                                                                      hierarchyJSON.dimensionName,
                                                                      hierarchyJSON.defaultHierarchy);
                hierarchies.push(hierarchy);
                //sas.stress.config.model.Hierarchy._hierarchies[hierarchy.getHierarchyKey()] = hierarchy;
                //sas.stress.config.model.Hierarchy._hierarchies = hierarchy;

            }
            //});
            deferred.resolve(hierarchy);
        })
        .fail(function (jqxhr) {
            deferred.reject(jqxhr);
        });
    return deferred;
};

sas.stress.config.model.Hierarchy.getHierarchyDetails = function () {
    var deferred = $.Deferred();
    sas.stress.config.model.Hierarchy._hierarchies = {}; //reset cache
    sas.stress.config.StressAPI.getHierarchyDetails().done(function (hierarchyDetailsJSON) {
           var hierarchies = [];
           _.each(hierarchyDetailsJSON, function (hierarchyDetailJSON) {
               var lineItems = [];
               _.each(hierarchyDetailJSON.lineItems, function (lineItem) {
                   var hierarchyItem = new sas.stress.config.model.HierarchyLineItem(lineItem.liKey, lineItem.hierarchyKey,
                       lineItem.liOrder, lineItem.contra, lineItem.aggregationType, lineItem.levelNames, lineItem.memberName?lineItem.memberName:lineItem.name,
                       lineItem.format, lineItem.formulaCode, lineItem.levelNum, lineItem.workgroupNames, lineItem.formulaCode);
                   lineItems.push(hierarchyItem);
               });

               var hierarchyJSON = hierarchyDetailJSON.hierarchy;
               if (hierarchyJSON) {
                   var hierarchy = new sas.stress.config.model.Hierarchy(hierarchyJSON.hierarchyKey,
                                                                         hierarchyJSON.categoryKey,
                                                                         hierarchyJSON.categoryName,
                                                                         hierarchyJSON.name,
                                                                         hierarchyJSON.version,
                                                                         hierarchyJSON.description,
                                                                         hierarchyJSON.createdByUser,
                                                                         hierarchyJSON.updatedByUser,
                                                                         hierarchyJSON.createdTimestamp,
                                                                         hierarchyJSON.updatedTimestamp,
                                                                         lineItems,
                                                                         hierarchyJSON.levelNames,
                                                                         hierarchyJSON.code,
                                                                         hierarchyJSON.taxonomyPackageOwners,
                                                                         hierarchyJSON.dimensionKey,
                                                                         hierarchyJSON.dimensionName,
                                                                         hierarchyJSON.defaultHierarchy);
                   hierarchies.push(hierarchy);
                   sas.stress.config.model.Hierarchy._hierarchies[hierarchy.getHierarchyKey()] = hierarchy;
               }
           });
           deferred.resolve(hierarchies);
       })
       .fail(function (jqxhr) {
           deferred.reject(jqxhr);
       });
    return deferred;
};

sas.stress.config.model.Hierarchy.getHierarchies = function () {
    var deferred = $.Deferred();
    sas.stress.config.model.Hierarchy._hierarchies = {}; //reset cache
    sas.stress.config.StressAPI.getHierarchies().done(function (hierarchiesJSON) {
           var hierarchies = [];
           $.each(hierarchiesJSON, function (index, hierarchyJSON) {
               var hierarchy = new sas.stress.config.model.Hierarchy(hierarchyJSON.hierarchyKey,
                                                                     hierarchyJSON.categoryKey,
                                                                     hierarchyJSON.categoryName,
                                                                     hierarchyJSON.name,
                                                                     hierarchyJSON.version,
                                                                     hierarchyJSON.description,
                                                                     hierarchyJSON.createdByUser,
                                                                     hierarchyJSON.updatedByUser,
                                                                     hierarchyJSON.createdTimestamp,
                                                                     hierarchyJSON.updatedTimestamp,
                                                                     hierarchyJSON.lineItems,
                                                                     hierarchyJSON.levelNames,
                                                                     hierarchyJSON.code,
                                                                     hierarchyJSON.taxonomyPackageOwners,
                                                                     hierarchyJSON.dimensionKey,
                                                                     hierarchyJSON.dimensionName,
                                                                     hierarchyJSON.defaultHierarchy);
               hierarchies.push(hierarchy);
               sas.stress.config.model.Hierarchy._hierarchies[hierarchy.getHierarchyKey()] = hierarchy;
           });
           deferred.resolve(hierarchies);
       })
       .fail(function (jqxhr) {
           deferred.reject(jqxhr);
       });
    return deferred;
};

sas.stress.config.model.Hierarchy.addHierarchy = function (newHierarchy) {
    var deferred = $.Deferred();
    sas.stress.config.StressAPI.addHierarchy(newHierarchy).done(function (hierarchyJSON) {
           var hierarchy = new sas.stress.config.model.Hierarchy(hierarchyJSON.hierarchyKey,
                                                                 hierarchyJSON.categoryKey,
                                                                 hierarchyJSON.categoryName,
                                                                 hierarchyJSON.name,
                                                                 hierarchyJSON.version,
                                                                 hierarchyJSON.description,
                                                                 hierarchyJSON.createdByUser,
                                                                 hierarchyJSON.updatedByUser,
                                                                 hierarchyJSON.createdTimestamp,
                                                                 hierarchyJSON.updatedTimestamp,
                                                                 hierarchyJSON.lineItems,
                                                                 hierarchyJSON.levelNames,
                                                                 hierarchyJSON.code,
                                                                 hierarchyJSON.taxonomyPackageOwners,  //@@DD This was missing
                                                                 hierarchyJSON.dimensionKey,
                                                                 hierarchyJSON.dimensionName,
                                                                 hierarchyJSON.defaultHierarchy);
           sas.stress.config.model.Hierarchy._hierarchies[hierarchy.getHierarchyKey()] = hierarchy;
           deferred.resolve(hierarchy);
       })
       .fail(function (jqxhr) {
           deferred.reject(jqxhr);
       });
    return deferred;
};

/* Used to initially set the Name input in the New Hierarchy dialog with a unique name. */
sas.stress.config.model.Hierarchy.getNewHierarchyName = function () {
    var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
    var newHierarchyName = "";
    var count = 1;
    while (newHierarchyName === "") {
        var tempHierarchyName = rb.getText("hierarchy.name.fmt", [count++]);
        //check if Hierarchy with this name exists
        var hierarchy = _.find(sas.stress.config.model.Hierarchy._hierarchies, function (hierarchy) {
            return hierarchy.getName().toLocaleLowerCase() === tempHierarchyName.toLocaleLowerCase();
        });
        //if not, use this name
        if (hierarchy === undefined) {
            newHierarchyName = tempHierarchyName;
        }
    }
    return newHierarchyName;
};

sas.stress.config.model.Hierarchy.deleteHierarchy = function (hierarchyKey) {
    var deferred = $.Deferred();
    sas.stress.config.StressAPI.deleteHierarchy(hierarchyKey).done(function () {
           sas.stress.config.model.Hierarchy._hierarchies[hierarchyKey] = undefined;
           delete sas.stress.config.model.Hierarchy._hierarchies[hierarchyKey];
           deferred.resolve();
       })
       .fail(function (jqxhr) {
           deferred.reject(jqxhr);
       });
    return deferred;
};

sas.stress.config.model.Hierarchy.updateHierarchy = function (hierarchy) {
    var deferred = $.Deferred();
    sas.stress.config.StressAPI.updateHierarchy(hierarchy).done(function (updatedHierarchyJson) {
       var updatedHierarchy = new sas.stress.config.model.Hierarchy(updatedHierarchyJson.hierarchyKey,
                                                                    updatedHierarchyJson.categoryKey,
                                                                    updatedHierarchyJson.categoryName,
                                                                    updatedHierarchyJson.name,
                                                                    updatedHierarchyJson.version,
                                                                    updatedHierarchyJson.description,
                                                                    updatedHierarchyJson.createdByUser,
                                                                    updatedHierarchyJson.updatedByUser,
                                                                    updatedHierarchyJson.createdTimestamp,
                                                                    updatedHierarchyJson.updatedTimestamp,
                                                                    updatedHierarchyJson.lineItems,
                                                                    updatedHierarchyJson.levelNames,
                                                                    updatedHierarchyJson.code,
                                                                    updatedHierarchyJson.taxonomyPackageOwners,  //@@DD This was missing
                                                                    updatedHierarchyJson.dimensionKey,
                                                                    updatedHierarchyJson.dimensionName,
                                                                    updatedHierarchyJson.defaultHierarchy);

        // fixes for S1268377.. if the existing items has "lineItems" or "levelNames", copy them to the new item
        var existinghierarchy=  sas.stress.config.model.Hierarchy._hierarchies[updatedHierarchy.getHierarchyKey()];
        //if ((existinghierarchy.getLineItems()&&existinghierarchy.getLineItems().length>0)||existinghierarchy.getLevelNames().length>0){
            updatedHierarchy.setLineItems(existinghierarchy.getLineItems());
            updatedHierarchy.levelNames=existinghierarchy.levelNames;
        //}
        //delete hierarchy in cache
       sas.stress.config.model.Hierarchy._hierarchies[updatedHierarchy.getHierarchyKey()] = undefined;
       delete sas.stress.config.model.Hierarchy._hierarchies[updatedHierarchy.getHierarchyKey()];
       //set hierarchy in cache
       sas.stress.config.model.Hierarchy._hierarchies[updatedHierarchy.getHierarchyKey()] = updatedHierarchy;
       deferred.resolve(updatedHierarchy);
    })
       .fail(function (jqxhr) {
           deferred.reject(jqxhr);
    });
    return deferred;
};

// deprecated, use Hierarchy.getHierarchy(key) instead
//sas.stress.config.model.Hierarchy.getHierarchyByKey = function (hierarchykey) {
//    return sas.stress.config.model.Hierarchy._hierarchies[hierarchykey];
//};