jQuery.sap.require("sap.ui.model.Filter");
jQuery.sap.require("sap.ui.model.Sorter");
jQuery.sap.require("sas.hc.m.MessageDialog");
jQuery.sap.require("sas.hc.m.MessageToast");
jQuery.sap.require("sas.hc.ui.layout.SplitterLayoutData");
jQuery.sap.require("sas.risk.common.models.RiskJSONModel");
jQuery.sap.require("sas.stress.config.model.Hierarchy");
jQuery.sap.require("sas.stress.config.model.HierarchyMaps");
jQuery.sap.require("sas.stress.config.model.HierarchyMapPair");
jQuery.sap.require("sas.stress.models.Model");
jQuery.sap.require("sas.stress.wkbench.StressFixes");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.stress.wkbench.ViewManager");

sap.ui.controller("sas.stress.config.view.HierarchyMapping", {

    onInit: function (event) {
        var controller = this;
        var view = this.getView();
        view.targetTreeTableMaximized = false;

        controller.targetHierarchyKey = view.getViewData().hierarchyKey;
        controller.fromGraphLayout=view.getViewData().fromGraphLayout||false;
        controller.targetHierarchy = sas.stress.config.model.Hierarchy.getHierarchy(controller.targetHierarchyKey);
        sas.stress.config.model.Hierarchy.getHierarchies().done(function () {

            var hierarchy = controller.targetHierarchy;
            var model = new sas.risk.common.models.RiskJSONModel({
                hierarchyName: hierarchy.getName()
            });
            view.setModel(model);

            var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
            //sap.ui.getCore().byId("targetTreeTableLabel").setText(rb.getText("targetTree.title.fmt", [hierarchy.getNameVersion()]));

            view.breadCrumb.removeAllItems();
            view.breadCrumb.addBreadcrumbItem("Top", rb.getText("hierarchies.header.title"));
            view.breadCrumb.addBreadcrumbItem("New", rb.getText("hierarchyMappingPage.contentInfoLabel.fmt.txt", hierarchy.getName()));

            controller._loadData();
            view.saveButton.setEnabled(false);
        });

        this.navChecker = sas.stress.wkbench.StressUtil.createNavChecker(true, view);
    },

    onExit: function () {
        //S1312016: Need to make sure the router is back working after save or navigate to another page under configure without hash change
        var router = sap.ui.core.routing.Router.getRouter("router");
        if (!router._bIsInitialized) {
            router.initialize();
        }

        this.navChecker.stop();
        // Depending on where the entry point was for edit Hierarchy mapping, the user can edit Hierarchy
        // Mapping from hierarchy Map screen or Hierarchy screen; the app has to switch back to that screen
        if (this.fromGraphLayout) {
            sas.eventBus.publish(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.VIEW_CONFIG_ROOT, {});
        }else {
            sas.eventBus.publish(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.VIEW_HIERARCHIES, {});
        }
    },

    _loadData: function () {
        var deferred = $.Deferred();
        var controller = this;
        // SOPHIE TODO: why even call this anymore?
        controller._loadHierarchyMappingInfo(controller.targetHierarchyKey).done(function () {
            deferred.resolve();
        }).fail(function (jqxhr) {
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    /* Gets info about all the source hierarchies that are mapped to the given target hierarchy.
     * Result also contains line items for the source hierarchies, line items for the target hierarchy
     * and line item mappings between the source hierarchies and target hierarchy.
     * This function then uses this data to set the view. */
    _loadHierarchyMappingInfo: function (targetHierarchyKey) {
        var view = this.getView();
        var controller = this;
        var deferred = $.Deferred();
        sas.stress.config.model.HierarchyMaps.getHierarchyMaps(targetHierarchyKey).done(function (hierarchyMaps) {
            controller.hierarchyMapPairs = hierarchyMaps.getHierarchyMapPairs(); //Mapping between hierarchies.
            //Build source and target hierarchy trees.
            var sourceHierarchiesRootNode = { children: [] };
            controller.hierarchyMapInfos = hierarchyMaps.getHierarchyMapInfos(); //Cache this so it can be used in the Save function.
            _.each(hierarchyMaps.getHierarchyMapInfos(), function (hierarchyMapInfo) { //Contains line items for source and target hierarchies involved.
                if (hierarchyMapInfo.getHierarchyKey() !== targetHierarchyKey) {
                    //assume this is source hierarchy
                    //Add a node with this hierarchy name
                    var sourceHierarchy = sas.stress.config.model.Hierarchy.getHierarchy(hierarchyMapInfo.getHierarchyKey());
                    var sourceHierarchyRootNode = {
                        name: sourceHierarchy.getName(),
                        version: sourceHierarchy.getVersion(),
                        hierarchyKey: hierarchyMapInfo.getHierarchyKey(),
                        children: []
                    };
                    sourceHierarchiesRootNode.children.push(sourceHierarchyRootNode);
                    //Add nodes in the source hierarchy to the source hierarchy root node
                    _.each(hierarchyMapInfo.getLineItems(), function (lineItem) {
                        controller._addToSourceTreeNode(sourceHierarchyRootNode, lineItem);
                    });
                } else {
                    var targetHierarchy = sas.stress.config.model.Hierarchy.getHierarchy(targetHierarchyKey);
                    var targetHierarchyRootNode = {
                        name: targetHierarchy.getNameVersion(),
                        hierarchyKey: hierarchyMapInfo.getHierarchyKey(),
                        children: []
                    };
                    _.each(hierarchyMapInfo.getLineItems(), function (lineItem) {
                        controller._addToTargetTreeNode(targetHierarchyRootNode, lineItem);
                    });

                    var targetTreeTableModel = new sas.risk.common.models.RiskJSONModel();
                    targetTreeTableModel.setData(targetHierarchyRootNode);
                    view.targetTreeTable.setModel(targetTreeTableModel);
                    view.targetTreeTable.bindRows("/children");
                }
            });
            var sourceTreeTableModel = new sas.risk.common.models.RiskJSONModel();
            //Sort by Hierarchy name ascending
            sourceHierarchiesRootNode.children.sort(function (a, b) {
                var aName = a.name.toLowerCase();
                var bName = b.name.toLowerCase();
                return ((aName < bName) ? -1 : ((aName > bName) ? 1 : 0));
            });
            sourceTreeTableModel.setData(sourceHierarchiesRootNode);
            view.sourceTreeTable.setModel(sourceTreeTableModel);
            view.sourceTreeTable.bindRows("/children");

            //Add mapped nodes to target hierarchy tree.
            //Add hierarchy nodes.
            _.each(hierarchyMaps.getHierarchyItemMaps(), function (hierarchyItemMap) { //For each source hierarchy to target hierarchy mapping
                _.each(hierarchyItemMap.getItemMapPairs(), function (itemMapPair) { //For each line item mapping in the source hierarchy to target hierarchy mapping
                    //find node in target tree with liKey = toLiKey.
                    var targetNode = controller._findNode(view.targetTreeTable.getModel().getData(), { "liKey": itemMapPair.getToLiKey() });
                    //find node in source tree for hierarchy matching the fromHierKey in the map pair and with liKey = fromLiKey.
                    //First, find the source tree.
                    var hierarchyMapPair = _.find(controller.hierarchyMapPairs, function (hierarchyMapPair) {
                        return hierarchyMapPair.getHierarchyMapKey() === hierarchyItemMap.getHierarchyMapKey();
                    });
                    if (hierarchyMapPair !== undefined) {
                        var sourceNode = controller._findNode(
                            view.sourceTreeTable.getModel().getData(),
                            {
                                liKey: itemMapPair.getFromLiKey(),
                                hierarchyKey: hierarchyMapPair.getFromHierKey()
                            }
                        );

                        //add a node under this node with liKey = fromLiKey
                        targetNode.children.push(
                            {
                                name: sourceNode.path,
                                liKey: itemMapPair.getFromLiKey(),
                                toLiKey: itemMapPair.getToLiKey(),
                                children: [],
                                hierarchyKey: sourceNode.hierarchyKey
                            }
                        );
                        //Update mappings count of the node in source tree.
                        sourceNode.mappings++;
                    }
                });
            });

            view.targetTreeTable.getModel().refresh();
            deferred.resolve();
        }).fail(function (jqxhr) {
            var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
            sas.hc.m.MessageDialog.error({text: rb.getText('error.message.txt'), title: rb.getText('error.window.title')});
            deferred.reject(jqxhr);
        }).always(function() {
            var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
            view.sourceTreeTable.setNoData(new sas.risk.common.widgets.ZeroStateLayout({
                icon: sas.icons.HC.ZEROSTATEADD,
                text: rb.getText("source.hierarchies.noData.txt"),
                buttons: [
                    new sas.hc.m.Button({
                        text: rb.getText("addHierarchyMappingDialog.window.title"),
                        press: $.proxy(controller.sourceTreeTableAddButtonPress, controller)
                    }).addStyleClass("sasPrimaryButton")
                ]
            }));

        });
        return deferred;
    },

    /* Recursively searches the tree model for a node where node matches keyValueObj. Returns the node, if one is found. */
    _findNode: function (treeModelData, keyValueObj) {
        var controller = this;
        for (var i = 0; i < treeModelData.children.length; i++) {
            var childNode = treeModelData.children[i];
            if (controller._isMatch(childNode, keyValueObj) === true) {
                return childNode;
            }
            //find in children of this node
            var node = controller._findNode(childNode, keyValueObj);
            if (node) {
                return node;
            }
        }
    },

    //Copied from underscore.js as this function is not available in Underscore 1.6.
    // Returns whether an object has a given set of `key:value` pairs.
    _isMatch: function (object, attrs) {
        var keys = _.keys(attrs), length = keys.length;
        if (object === null) {
            return !length;
        }
        var obj = Object(object);
        for (var i = 0; i < length; i++) {
            var key = keys[i];
            if (attrs[key] !== obj[key] || !(key in obj)) {
                return false;
            }
        }
        return true;
    },


    /* Adds a branch of nodes listed in levelNames to the rootNode. */
    _addToSourceTreeNode: function (rootNode, lineItem) {
        var currentNode = rootNode;
        $.each(lineItem.getLevelNames(), function (index, levelName) {
            //check if currentNode.children contains node with levelName[i].
            var node = _.find(currentNode.children, function (childNode) {
                return childNode.name === levelName;
            });
            //If not, add node with levelName[i].
            if (node === undefined) {
                node = { name: levelName, hierarchyKey: lineItem.getHierarchyKey(), children: [] };
                //set liKey and path on the leaf node.
                if (index === lineItem.getLevelNames().length - 1) {
                    node.liKey = lineItem.getLiKey();
                    node.path = rootNode.name + "," + lineItem.getLevelNames();
                    node.mappings = 0;
                }
                currentNode.children.push(node);
            }
            //set currentNode as the node with levelName[i] for levelName[i+1].
            currentNode = node;
        });
    },

    /* Adds a branch of nodes listed in levelNames to the rootNode. */
    _addToTargetTreeNode: function (rootNode, lineItem) {
        var currentNode = rootNode;
        $.each(lineItem.getLevelNames(), function (index, levelName) {
            //check if currentNode.children contains node with levelName[i].
            var node = _.find(currentNode.children, function (childNode) {
                return childNode.name === levelName;
            });
            //If not, add node with levelName[i].
            if (node === undefined) {
                node = { name: levelName, hierarchyKey: lineItem.getHierarchyKey(), children: [] };
                //set HierarchyLineItem properties on the last level node.
                if (index === lineItem.getLevelNames().length - 1) {
                    node.liKey = lineItem.getLiKey();
                    node.liOrder = lineItem.getLiOrder();
                    node.contra = lineItem.getContra();
                    node.format = lineItem.getFormat();
                    node.aggregationType = lineItem.getAggregationType();
                    node.formula = lineItem.getFormula();
                    node.formulaCode = lineItem.getFormulaCode();
                    node.levelNames = lineItem.getLevelNames();
                    node.memberName = lineItem.getMemberName();
                    node.supportsFormula = true; //Init
                }
                currentNode.children.push(node);
            }
            //set currentNode as the node with levelName[i] for levelName[i+1].
            currentNode = node;
        });
    },

    addMappingButtonPress: function (event) {
        var view = this.getView();
        var controller = this;
        var targetTreeTableSelectedIndices = view.targetTreeTable.getSelectedIndices();
        if (targetTreeTableSelectedIndices.length !== 1) {
            return; //return if no or more than 1 target node selected
        }
        //Get target node
        var selectedTargetNodePath = view.targetTreeTable.getContextByIndex(targetTreeTableSelectedIndices[0]).getPath();
        var selectedTargetNode = view.targetTreeTable.getModel().getProperty(selectedTargetNodePath);
        if (selectedTargetNode.formula !== null && selectedTargetNode.formula.length > 0) {
            var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
            sas.hc.m.MessageDialog.warning({
                text: rb.getText("hierarchyMappingPage.removeFormula.confirmation.msg.txt"),
                title: rb.getText("hierarchyMappingPage.addMapping.window.title"),
                buttonActions: [sas.hc.m.MessageDialogAction.Yes, sas.hc.m.MessageDialogAction.No],
                buttonPress: function (event) {
                    var action = event.getParameter("origin").getProperty("messageDialogAction");
                    if (action === sas.hc.m.MessageDialogAction.Yes) {
                        selectedTargetNode.formula = undefined;
                        controller._addModelMapping();
                    }
                }
            });
        } else {
            controller._addModelMapping();
        }
    },

    /* Recursively searches for all the leaf nodes under the given node. */
    _getLeafNodes: function (node) {
        var controller = this;
        _.each(node.children, function (childNode) {
            if (childNode.children.length === 0) {
                controller.leafNodes.push(childNode);
            } else {
                controller._getLeafNodes(childNode);
            }
        });
        return controller.leafNodes;
    },

    removeMappingButtonPress: function (event) {
        var view = this.getView();
        var controller = this;
        var targetTreeTableModel = view.targetTreeTable.getModel();

        //Get the list of selected nodes. We get this list first because getContextByIndex will return a different row once we remove a node.
        var selectedNodes = [];
        $.each(view.targetTreeTable.getSelectedIndices(), function (index, selectedIndex) {
            var selectedNodePath = view.targetTreeTable.getContextByIndex(selectedIndex).getPath();
            var selectedNode = targetTreeTableModel.getProperty(selectedNodePath);
            selectedNodes.push(selectedNode);
        });

        controller._removeModelMappings(selectedNodes);
    },

    _removeAllMappedModelNodesFromTargetTree: function (modelData) {
        var controller = this;
        _.each(modelData.children, function (node) {
            if (controller._containsMappedModelNodes(node, false)) {
                //Remove all child model nodes.
                for (var i = node.children.length - 1; i >= 0; i--) {
                    if (node.children[i].modelKey !== undefined) {
                        node.children.splice(i, 1);
                    }
                }
                node.supportsFormula = true; //Formula can only be given for node that does not have mapped model nodes.
            } else { //remove mapped nodes from children of this node
                controller._removeAllMappedModelNodesFromTargetTree(node);
            }
        });
    },

    saveHierarchyMappingButtonPress: function (event) {
        var view = this.getView();
        var controller = this;
        this._saveHierarchyMappingPage().done(function () {
            controller._setDirty(false);
            var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
            sas.hc.m.MessageToast.show(rb.getText("hierarchyMappingPage.save.successful.msg.txt"), { at: sap.ui.core.Popup.Dock.CenterTop });
            //Reload data in the view
            controller._loadData().done(function () {
                //Reset Expand/Collapse button state
                var sourceTreeTableExpandCollapseButton = sap.ui.getCore().byId("sourceTreeTableExpandCollapseButton");
                if (sourceTreeTableExpandCollapseButton !== undefined) {
                    sourceTreeTableExpandCollapseButton.setIcon(sas.icons.HC.SHOWALL);
                    sourceTreeTableExpandCollapseButton.setTooltip(rb.getText("sourceTreeTable.expandCollapseButton.tip"));
                    view.sourceTreeTableExpanded = false;
                }
                var targetTreeTableExpandCollapseButton = sap.ui.getCore().byId("targetTreeTableExpandCollapseButton");
                targetTreeTableExpandCollapseButton.setIcon(sas.icons.HC.SHOWALL);
                targetTreeTableExpandCollapseButton.setTooltip(rb.getText("sourceTreeTable.expandCollapseButton.tip"));
                view.targetTreeTableExpanded = false;
            });
        });
    },

    _saveHierarchyMappingPage: function () {
        var deferred = $.Deferred();
        var view = this.getView();
        var controller = this;
        //Get mapped nodes (hierarchy and model).
        controller.mappedData = { hierarchyItemMapPairs: [], modelItemMaps: [] }; //Initialize, we'll use this var to store mapped nodes.
        var mappedData = controller._getMappedNodes(view.targetTreeTable.getModel().getData());
        delete controller.mappedData; //Done using this var.
        var hierarchyItemMaps = [];
        _.each(controller.hierarchyMapPairs, function (hierarchyMapPair) {
            //Get line item mappings for hierarchyMapPair.
            var hierarchyItemMapPairs = _.filter(mappedData.hierarchyItemMapPairs, function (hierarchyItemMapPair) {
                return hierarchyItemMapPair.hierarchyKey === hierarchyMapPair.getFromHierKey();
            });
            var itemMapPairs = [];
            _.each(hierarchyItemMapPairs, function (hierarchyItemMapPair) {
                itemMapPairs.push(new sas.stress.config.model.HierarchyItemMapDetail(hierarchyMapPair.getHierarchyMapKey(),
                    hierarchyItemMapPair.liKey, hierarchyItemMapPair.toLiKey));
            });
            hierarchyItemMaps.push(new sas.stress.config.model.HierarchyItemMap(hierarchyMapPair.getHierarchyMapKey(), itemMapPairs));
        });

        var hierarchyMapInfos = controller.hierarchyMapInfos;
        _.each(hierarchyMapInfos, function (hierarchyMapInfo) {
            if (hierarchyMapInfo.getHierarchyKey() === controller.targetHierarchyKey) {
                //Since there can be formula changes, we need to build list of line items from the target tree.
                var lineItems = [];
                controller._getLineItemsInTargetTree(view.targetTreeTable.getModel().getData(), lineItems);
                hierarchyMapInfo.setLineItems(lineItems);
            }
        });

        var modelHierItemMaps = [];
        _.each(mappedData.modelItemMaps, function (modelItemMap) {
            modelHierItemMaps.push(new sas.stress.config.model.ModelHierItemMap(modelItemMap.modelKey, controller.targetHierarchyKey, modelItemMap.toLiKey));
        });
        var hierarchyMaps = new sas.stress.config.model.HierarchyMaps([], hierarchyItemMaps, hierarchyMapInfos, modelHierItemMaps);
        sas.stress.config.StressAPI.updateTargetMappings(controller.targetHierarchyKey, hierarchyMaps).done(function () {
            deferred.resolve();
        }).fail(function (jqxhr) {
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });
        return deferred;
    },

    /* Gets nodes in the target tree having liKey and matching target hierarchy key and puts them in the lineItems array. */
    _getLineItemsInTargetTree: function (node, lineItems) {
        var controller = this;
        _.each(node.children, function (childNode) {
            if (childNode.liKey !== null && childNode.hierarchyKey === controller.targetHierarchyKey) {
                var hierarchyLineItem = new sas.stress.config.model.HierarchyLineItem(childNode.liKey, childNode.hierarchyKey,
                    childNode.liOrder, childNode.contra, childNode.aggregation, childNode.levelNames, childNode.memberName,
                    childNode.format, childNode.formula, childNode.levelNum, childNode.workgroupNames);
                lineItems.push(hierarchyLineItem);
            }
            if (childNode.children.length > 0) {
                controller._getLineItemsInTargetTree(childNode, lineItems);
            }
        });
    },

    /* Recursively searches for all mapped nodes under the given node. */
    _getMappedNodes: function (node) {
        var controller = this;
        _.each(node.children, function (childNode) {
            if (childNode.children.length === 0) {
                if (childNode.hierarchyKey !== undefined && childNode.hierarchyKey !== controller.targetHierarchyKey) { //Hierarchy node
                    controller.mappedData.hierarchyItemMapPairs.push(childNode);
                } else if (childNode.modelKey !== undefined) { //Model node
                    controller.mappedData.modelItemMaps.push(childNode);
                }
            } else {
                controller._getMappedNodes(childNode);
            }
        });
        return controller.mappedData;
    },

    /* Searches for mapped model nodes, under the given node and returns true if it finds at least 1 mapped model node. Returns false if no mapped model nodes found.
     * Searches recursively if recursive param is set to true. */
    _containsMappedModelNodes: function (node, recursive) {
        var controller = this;
        var mappedNodeExists = false;
        $.each(node.children, function (index, childNode) {
            if (childNode.children.length === 0) {
                mappedNodeExists = (childNode.modelKey !== undefined);
            } else if (recursive === true) {
                mappedNodeExists = controller._containsMappedModelNodes(childNode, true);
            }
            if (mappedNodeExists === true) {
                return false; //break for loop
            }
        });
        return mappedNodeExists;
    },

    breadcrumbClickHandler: function (event) {
        if (sas.stress.wkbench.StressUtil.confirmToContinue(this.navChecker.getDirty())) {
            switch (event.getParameter("item").getKey()) {
                case "Top":
                    // this view can be launched from both "Hierarchies" and "Hierarchy Map"
                    if (this.fromGraphLayout) {
                        sas.eventBus.publish(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.VIEW_CONFIG_ROOT, {});
                    }else {
                        sas.eventBus.publish(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.VIEW_HIERARCHIES, {});
                    }
                    break;
            }
        }
    },

    sourceTreeTableAddButtonPress: function (event) {
        var view = this.getView();
        var controller = this;
        var existingDialog = sap.ui.getCore().byId("sas.stress.config.view.AddHierarchyMappingDialog");
        if (existingDialog) {
            existingDialog.destroy();
        }
        var dialog = sap.ui.jsfragment("sas.stress.config.view.AddHierarchyMappingDialog", this);
        var sourceHierarchies = _.filter(sas.stress.config.model.Hierarchy.getCachedHierarchies(), function (hierarchy) {
            var hierarchyAlreadyAdded = _.find(view.sourceTreeTable.getModel().getData().children, function (sourceHierarchy) {
                return sourceHierarchy.hierarchyKey === hierarchy.hierarchyKey;
            });
            return hierarchy.getName().toLowerCase() !== controller.targetHierarchy.getName().toLowerCase() &&
                hierarchyAlreadyAdded === undefined;
        });
        var model = new sas.risk.common.models.RiskJSONModel({ sourceHierarchies: sourceHierarchies });
        dialog.setModel(model);
        dialog.open();

        var sourceHierarchiesTable = sap.ui.getCore().byId("AddHierarchyMappingDialog.sourceHierarchiesTable");

        // Default sort by Name asc.
        var sorters = [new sap.ui.model.Sorter("name", false)];
        if (sourceHierarchiesTable.getBinding("rows")) {
            sourceHierarchiesTable.getBinding("rows").sort(sorters);
        }
    },

    addHierarchyMappingDialogSearch: function (event) {
        var value = event.getParameter("value") || event.getParameter("newValue") || event.getParameter("query") || "";
        var filter1 = new sap.ui.model.Filter("name", sap.ui.model.FilterOperator.Contains, value);
        var filter2 = new sap.ui.model.Filter("description", sap.ui.model.FilterOperator.Contains, value);
        var filter3 = new sap.ui.model.Filter("categoryName", sap.ui.model.FilterOperator.Contains, value);
        var allFilters = new sap.ui.model.Filter([filter1, filter2, filter3]);
        var sourceHierarchiesTable = sap.ui.getCore().byId("AddHierarchyMappingDialog.sourceHierarchiesTable");
        var binding = sourceHierarchiesTable.getBinding("rows");
        binding.filter(allFilters);
    },

    sourceHierarchiesTableRowSelectionChange: function (event) {
        var sourceHierarchiesTable = sap.ui.getCore().byId("AddHierarchyMappingDialog.sourceHierarchiesTable");
        var saveButton = sap.ui.getCore().byId("AddHierarchyMappingDialog.saveButton");
        var enable = sourceHierarchiesTable.getSelectedIndices().length > 0;
        //if (enable) {
        //    // look through the selected hierarchies to see if there are multiple hierarchies with same name but different version
        //    _.each(sourceHierarchiesTable.getSelectedIndices(), function (selectedIndex) {
        //        var selectedContext = sourceHierarchiesTable.getContextByIndex(selectedIndex);
        //        var selectedHierarchy = selectedContext.getObject();
        //        var hierarchiesWithSameName =  _.filter(sourceHierarchiesTable.getSelectedIndices(), function (index) {
        //            var context = sourceHierarchiesTable.getContextByIndex(index);
        //            var hierarchy = context.getObject();
        //            return selectedHierarchy.getName().toLowerCase() === hierarchy.getName().toLowerCase();
        //        });
        //        if (hierarchiesWithSameName && hierarchiesWithSameName.length > 1) {
        //            enable = false;
        //        }
        //    });
        //}
        saveButton.setEnabled(enable);
    },

    addHierarchyMappingDialogSaveButtonPress: function (event) {
        var view = this.getView();
        var controller = this;
        var dialog = sap.ui.getCore().byId("sas.stress.config.view.AddHierarchyMappingDialog");
        dialog.setBusy(true);
        var sourceHierarchiesTable = sap.ui.getCore().byId("AddHierarchyMappingDialog.sourceHierarchiesTable");
        var hierarchyMapPairs = [];
        _.each(sourceHierarchiesTable.getSelectedIndices(), function (selectedIndex) {
            var context = sourceHierarchiesTable.getContextByIndex(selectedIndex);
            var obj = context.getObject();
            hierarchyMapPairs.push(new sas.stress.config.model.HierarchyMapPair(-1, parseInt(obj.hierarchyKey),
                parseInt(controller.targetHierarchyKey), ""));
        });
        if (hierarchyMapPairs.length > 0) {
            sas.stress.config.model.HierarchyMapPair.addHierarchyMapPairs(hierarchyMapPairs).done(function () {
                dialog.setBusy(false);
                dialog.close();
                controller._setDirty(true);
                view.sourceTreeTable.clearSelection();
                //Reload data in the view
                controller._loadData();
            }).fail(function (jqxhr) {
                dialog.setBusy(false);
                sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            });
        } else {
            dialog.setBusy(false);
            dialog.close();
        }
    },

    sourceTreeTableRemoveButtonPress: function (event) {
        var view = this.getView();
        var controller = this;
        var hierarchyMapKeys = [];
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        var removeMessage = "";
        _.each(view.sourceTreeTable.getSelectedIndices(), function (selectedIndex) {
            var selectedNodePath = view.sourceTreeTable.getContextByIndex(selectedIndex).getPath();
            var selectedNode = view.sourceTreeTable.getModel().getProperty(selectedNodePath);
            var hierarchyRootNode = {};
            //Check if selected node is the root node of the hierarchy. Ignore non-root nodes.
            hierarchyRootNode = _.find(view.sourceTreeTable.getModel().getData().children, function (data) {
                return selectedNode.hierarchyKey === data.hierarchyKey;
            });
            if (hierarchyRootNode !== undefined) {
                var hierarchyMapPair = _.find(controller.hierarchyMapPairs, function (hierarchyMapPair) {
                    return hierarchyMapPair.getFromHierKey() === hierarchyRootNode.hierarchyKey;
                });
                if (hierarchyMapPair !== null) {
                    hierarchyMapKeys.push(hierarchyMapPair.getHierarchyMapKey());
                    if (view.sourceTreeTable.getSelectedIndices().length === 1) {
                        removeMessage = rb.getText("hierarchyMapping.removeHierarchy.single.confirmation.msg.fmt.txt", [hierarchyRootNode.name]);
                    }
                }
            }
        });
        if (hierarchyMapKeys.length > 0) {
            if (hierarchyMapKeys.length > 1) {
                removeMessage = rb.getText("hierarchyMapping.removeHierarchy.multiple.confirmation.msg.fmt.txt", [hierarchyMapKeys.length]);
            }
            sas.hc.m.MessageDialog.warning({
                text: removeMessage,
                title: rb.getText("hierarchyMapping.removeHierarchy.window.title"),
                buttonActions: [sas.hc.m.MessageDialogAction.Delete, sas.hc.m.MessageDialogAction.Cancel],
                buttonPress: function (event) {
                    var action = event.getParameter("origin").getProperty("messageDialogAction");
                    if (action === sas.hc.m.MessageDialogAction.Delete) {
                        sas.stress.config.model.HierarchyMapPair.deleteHierarchyMapPairs(hierarchyMapKeys).done(function () {
                            controller._setDirty(true);
                            view.sourceTreeTable.clearSelection();
                            view.targetTreeTable.clearSelection();
                            //Reload data in the view
                            controller._loadData();
                        }).fail(function (jqxhr) {
                            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
                        });
                    }
                }
            });
        }
    },

    sourceTreeTableExpandButtonPress: function (event) {
        var view = this.getView();
        for (var i = 0; i < view.sourceTreeTable.getBinding("rows").getLength(); i++) {
            var nodePath = view.sourceTreeTable.getContextByIndex(i).getPath();
            var node = view.sourceTreeTable.getModel().getProperty(nodePath);
            if (node.children.length > 0) {
                view.sourceTreeTable.expand(i);
            }
        }
    },

    sourceTreeTableCollapseButtonPress: function (event) {
        var view = this.getView();
        view.sourceTreeTable.collapseAll();
    },

    targetTreeTableExpandButtonPress: function (event) {
        var view = this.getView();
        for (var i = 0; i < view.targetTreeTable.getBinding("rows").getLength(); i++) {
            var nodePath = view.targetTreeTable.getContextByIndex(i).getPath();
            var node = view.targetTreeTable.getModel().getProperty(nodePath);
            if (node.children.length > 0) {
                view.targetTreeTable.expand(i);
            }
        }
    },

    targetTreeTableCollapseButtonPress: function (event) {
        var view = this.getView();
        view.targetTreeTable.collapseAll();
    },

    sourceTreeTableRowSelectionChange: function (event) {
        var view = this.getView();
        var removeHierarchyButton = sap.ui.getCore().byId("sourceTreeTableRemoveButton");
        var selectedIndex = view.sourceTreeTable.getSelectedIndex();
        removeHierarchyButton.setEnabled(selectedIndex !== -1);
    },

    targetTreeTableFormulaLiveChange: function (event) {
        this._setDirty(true);
    },

    _setDirty: function (isDirty) {
        this.getView().saveButton.setEnabled(isDirty); //Enable Save button
        // Let Appswitcher know
        this.navChecker.setDirty(isDirty);
        //sas.applicationSwitcher.updateDirtyFlag(sas.config.getAppId(), isDirty);
    },

    targetTreeTableMaxButtonPress: function (event) {
        var view = this.getView();
        var maxButton = sap.ui.getCore().byId("targetTreeTableMaxButton");
        var content = view.splitter.getContentAreas()[0];
        if (view.targetTreeTableMaximized === false) {
            maxButton.setIcon(sas.icons.HC.NORMALVIEW);
            view.targetTreeTableMaximized = true;
            content.setLayoutData(new sas.hc.ui.layout.SplitterLayoutData({ size: "0%" }));
        } else {
            maxButton.setIcon(sas.icons.HC.MAXIMIZEVIEW);
            content.setLayoutData(new sas.hc.ui.layout.SplitterLayoutData());
            view.targetTreeTableMaximized = false;
        }
    }
});