jQuery.sap.require("sap.ui.model.Filter");
jQuery.sap.require("sap.ui.model.resource.ResourceModel");
jQuery.sap.require("sas.hc.m.Dialog");
jQuery.sap.require("sas.hc.m.MessageDialog");
jQuery.sap.require("sas.risk.common.models.RiskJSONModel");
jQuery.sap.require("sas.risk.common.utils.LayoutService");
jQuery.sap.require("sas.risk.common.widgets.ZeroStateLayout");
jQuery.sap.require("sas.stress.config.model.Hierarchy");
jQuery.sap.require("sas.stress.config.StressAPI");
jQuery.sap.require("sas.stress.wkbench.EventBusConstants");
jQuery.sap.require("sas.stress.wkbench.StressFixes");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.stress.wkbench.ViewManager");
jQuery.sap.require("sas.stress.wkbench.UserPrefs");
jQuery.sap.require("sas.stress.wkbench.model.SheetCategory");
jQuery.sap.require("sas.stress.wkbench.widget.AccessControlsDialog");
jQuery.sap.require("sas.stress.wkbench.CapabilityManager");

sap.ui.controller("sas.stress.config.view.SheetTemplateManager", {

    NO_VERSION: "__NO__VERSION__",

    onInit: function () {
        var view = this.getView();
        this.i18N = new sap.ui.model.resource.ResourceModel({ bundleName: "sas.stress.config.messagebundle" });
        view.setModel(this.i18N, "i18N");
        this.capabilityManager=sas.stress.wkbench.CapabilityManager.getInstance();

        //S1266048: refresh() is called onBeforeRendering
        //console.log("**********SheetTemplateManager.onInit");
        //this.refresh();

        //NavChecker in Configuration.controller should handle everything for the sub views.
        //this.navChecker = sas.stress.wkbench.StressUtil.createNavChecker(true, this.getView());

        new sas.risk.common.utils.LayoutService(this.getView().table, "SheetTemplateManagerTable");
        // Need to update the worksheet template table whenever a hierarchy is deleted
        sas.eventBus.subscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.HIERARCHY_DELETED, this.onHierarchyDeleted, this);
    },
    onBeforeRendering: function () {
        //console.log("**********SheetTemplateManager.onBeforeRendering");
        this.refresh();
    },
    onExit: function () {
        // Remove any subscriptions to ensure we don't get duplicates the next time the controller is initialized
        sas.eventBus.unsubscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.HIERARCHY_DELETED, this.onHierarchyDeleted, this);
        //this.navChecker.stop();
    },
    onHierarchyDeleted: function (channel, event, data) {
        this.refresh();
    },
    refresh: function () {
        var controller = this;
        var view = this.getView();
        view.table.setBusyIndicatorDelay(0);
        view.table.setBusy(true);
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        view.table.setNoData(rb.getText("sheetTemplates.templates.noData.loading.txt"));
        //console.log("**********SheetTemplateManager.refresh");
        $.when(sas.stress.config.model.Hierarchy.getCachedHierarchies(), sas.stress.wkbench.model.SheetCategory.getSheetCategories()).done(function (hierarchies,
                                                                                                                                                     sheetCategories) {
            //console.log("**********SheetTemplateManager.refresh done getSheetCategories ");
            var hierarchyNameMap = {};
            _.each(hierarchies, function (hierarchy) {
                hierarchyNameMap[hierarchy.getHierarchyKey()] = hierarchy.getName();
            });
            var category = controller._getPreferredCategory();
            var json = { categories: [], templates: [], selectedSheetCategory: category, selectedSheetTemplates: [], allowWorksheetCreation: false };
            json.categories.push({
                id: 0,
                label: controller.i18N.getResourceBundle().getText("categories.all.txt"),
                count: 0
            });
            _.each(sheetCategories, function (cat) {
                var templates = cat.getSheetTemplates();
                json.categories.push({ id: cat.getId(), label: cat.getName(), count: templates.length });
                _.each(templates, function (template) {
                    json.categories[0].count++; //Received from merging with new version;
                    json.templates.push({
                        id: template.getId(),
                        catId: template.getCategory().getId(),
                        name: template.getName(),
                        description: template.getDescription(),
                        version: (template.getVersion() === controller.NO_VERSION) ? "" : template.getVersion(),
                        nameVersion: template.nameVersion,
                        lastModifiedUser: template.getUpdatedByUser(),
                        lastModifiedTimestamp: template.getUpdatedTimestamp(),
                        history: template.numHistoryPeriods,
                        future: template.numFuturePeriods,
                        createdTimestamp: template.getCreatedTimestamp()
                    });
                });
            });
            // Only allow New Worksheets to be created if a worksheet category has been created
            if (json.categories.length > 1) { json.allowWorksheetCreation = true;}

            var model = view.getModel();
            if (model instanceof sap.ui.model.json.JSONModel) {
                controller._sortSheetCategories(json.categories);
                //console.log("**********SheetTemplateManager.refresh setData ");
                model.setData(json);
                view.categoryFilter.setSelectedCategory(category, true);
            } else {
                model = new sas.risk.common.models.RiskJSONModel(json);
                //console.log("**********SheetTemplateManager.refresh setModel ");
                view.setModel(model);
                view.categoryFilter.setSelectedCategory(category, true);
                //S1256573: Do sorting after creation in view so it won't overwrite LayoutService
                //var sorters = [new sap.ui.model.Sorter("name", false), new sap.ui.model.Sorter("version", false)];
                //var bindingInfo = view.table.getBinding("rows");
                //bindingInfo.sort(sorters);
            }
        }).fail(function(jqxhr) {
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        }).always(function() {
            view.table.setBusy(false);
            var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();
            view.table.setNoData(new sas.risk.common.widgets.ZeroStateLayout({
                icon: sas.icons.HC.ZEROSTATEADD,
                text: "{i18N>sheetTemplates.templates.noData.title}",
                buttons: [
                    new sas.hc.m.Button({
                        text: "{i18N>sheetCategory.new.window.title}",
                        ariaLabel: "{i18N>sheetCategory.new.window.title}",
                        press: $.proxy(controller.pressedCreateSheetCategory, controller)
                    }).addStyleClass("sasPrimaryButton"),
                    new sas.hc.m.Button({
                        text: "{i18N>sheetTemplates.templates.noData.button.txt}",
                        enabled: {
                            path: "/allowWorksheetCreation",
                            formatter: function (allowWorksheetCreation) {
                                return allowWorksheetCreation && capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.TEMPLATE.CREATE);
                            }
                        },
                        press: $.proxy(controller.pressedCreateSheetTemplate, controller)
                    })
                ]
            }));
        });
    },

    _sortSheetCategories: function (sheetCategories) {
        //Sort by Name ascending. Keep "All" at the top.
        var sortedSheetCategories =
            sheetCategories.slice(1).sort(function (a, b) {
                var aName = a.label.toLowerCase();
                var bName = b.label.toLowerCase();
                return ((aName < bName) ? -1 : ((aName > bName) ? 1 : 0));
            });
        sheetCategories.length = 1;
        sheetCategories.push.apply(sheetCategories, sortedSheetCategories);
    },
    _savePreferredCategory: function (category) {
        if (category === undefined) {
            var view = this.getView();
            category = view.categoryFilter.getSelectedCategory();
        }
        if (category) {
            sas.stress.wkbench.UserPrefs.save("sas.stress.config.templateCategory", category);
        } else {
            sas.stress.wkbench.UserPrefs.delete("sas.stress.config.templateCategory", category);
        }
    },
    _getPreferredCategory: function () {
        return sas.stress.wkbench.UserPrefs.get("sas.stress.config.templateCategory");
    },
    onRowSelectionChange: function (oEvent) {
        var view = this.getView();
        view.editSheetButton.setEnabled(oEvent.getSource().getSelectedIndices().length === 1 &&
            view.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.TEMPLATE.READ));
        view.permissionsSheetButton.setEnabled(oEvent.getSource().getSelectedIndices().length === 1 &&
            view.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.TEMPLATE.READ));
        view.deleteSheetButton.setEnabled(oEvent.getSource().getSelectedIndices().length !== 0 &&
            view.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.TEMPLATE.DELETE));
    },
    pressedSelectSheetCategory: function (e) {
        this._toggleCategoriesListToolbarButtons();
        this._savePreferredCategory();
        this.filterSheetTemplates();
    },

    _toggleCategoriesListToolbarButtons: function () {
        var view = this.getView();
        var selectedCategory = view.categoryFilter.getSelectedItem();
        if (selectedCategory !== null) {
            var categoryKey = this._getSelectedCategoryKey();
        }

        var canEdit = this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.TEMPLATE.UPDATE);
        var canDelete = this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.TEMPLATE.DELETE);
        // Enable edit and delete if category is not "All".

        view.byId("worksheetCategoriesList.editButton").setEnabled(canEdit && categoryKey !== 0);
        view.byId("worksheetCategoriesList.deleteButton").setEnabled(canDelete  && categoryKey !== 0);
    },

    filterSheetTemplates: function () {
        var view = this.getView();
        var catId = this._getSelectedCategoryKey();
        var search = view.searchField.getValue() || "";
        /*
         sas.stress.wkbench.UserPrefs.save({
                                              'sas.stress.config.sheetCategory_key': catId,
                                              'sas.stress.config.sheetTemplate_filter': search
                                          });
        */ // do not save for now.. we probably need to put this into local storage rather than the DB!!
        var table = this.getView().table;
        var filter1 = new sap.ui.model.Filter("catId", sap.ui.model.FilterOperator.EQ, catId);
        var filter2s = [];
        filter2s.push(new sap.ui.model.Filter("name", sap.ui.model.FilterOperator.Contains, search));
        filter2s.push(new sap.ui.model.Filter("description", sap.ui.model.FilterOperator.Contains, search));
        filter2s.push(new sap.ui.model.Filter("version", sap.ui.model.FilterOperator.Contains, search));
        filter2s.push(new sap.ui.model.Filter("lastModifiedUser", sap.ui.model.FilterOperator.Contains, search));
        filter2s.push(new sap.ui.model.Filter("hierarchy", sap.ui.model.FilterOperator.Contains, search));
        filter2s.push(new sap.ui.model.Filter("period", sap.ui.model.FilterOperator.Contains, search));
        var filter2 = new sap.ui.model.Filter(filter2s);
        var filtersToUse;
        // Are we in "ALL" categories?
        if (catId === 0) {
            // Skip the category filter
            filtersToUse = filter2;
        } else {
            // We need to include the category filter
            filtersToUse = new sap.ui.model.Filter([filter1, filter2], true/*ANDed*/);
        }
        var binding = table.getBinding("rows");
        binding.filter(filtersToUse);
    },
    _getSelectedCategoryKey: function () {
        var view = this.getView();
        var catItem = view.categoryFilter.getSelectedItem();
        if (catItem) {
            var path = catItem.getBindingContext().getPath();
            return catItem.getModel().getProperty(path + "/id");
        }
        return null;
    },
    _getSelectedTemplateKeys: function () {
        var view = this.getView();
        var table = view.table;
        var keys = [];
        _.each(table.getSelectedIndices(), function (index) {
            var selectedPath = table.getContextByIndex(index).getPath();
            keys.push(table.getModel().getProperty(selectedPath + "/id"));
        });
        return keys;
    },
    pressedCreateSheetCategory: function () {
        var controller = this;
        var view = sas.stress.wkbench.ViewManager.loadSheetCategoryEditor();
        var i18N = new sap.ui.model.resource.ResourceModel({ bundleName: "sas.stress.config.messagebundle" });
        var dialog = new sas.hc.m.Dialog({
            title: "{i18N>sheetCategory.new.window.title}",
            content: view,
            beginButton: view.okButton,
            endButton: view.cancelButton,
            afterClose: function () {
                dialog.destroy();
            },
            initialFocus: view.nameField
        });
        dialog.setModel(i18N, "i18N");
        view.attachClosing(function (e) {
            this.destroy();
            if (e.getParameter("action") !== 'CANCEL') {
                var newName = view.nameField.getValue().trim();
                controller._savePreferredCategory(newName);
                controller.refresh();
            }
        }, dialog);
        dialog.open();
    },
    pressedEditSheetCategory: function (e) {
        var controller = this;
        var catId = this._getSelectedCategoryKey();
        var sheetCat = sas.stress.wkbench.model.SheetCategory.getSheetCategory(catId);
        var view = sas.stress.wkbench.ViewManager.loadSheetCategoryEditor(sheetCat);
        var i18N = new sap.ui.model.resource.ResourceModel({ bundleName: "sas.stress.config.messagebundle" });
        var dialog = new sas.hc.m.Dialog({
            title: "{i18N>sheetCategory.edit.window.title}",
            content: view,
            beginButton: view.okButton,
            endButton: view.cancelButton,
            afterClose: function () {
                dialog.destroy();
            },
            initialFocus: view.nameField
        });
        dialog.setModel(i18N, "i18N");
        view.attachClosing(function (e) {
            this.destroy();
            if (e.getParameter("action") !== 'CANCEL') {
                var newName = view.nameField.getValue().trim();
                controller._savePreferredCategory(newName);
                controller.refresh();
            }
        }, dialog);
        dialog.open();
    },
    pressedDeleteSheetCategory: function (e) {
        var controller = this;
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        var categoryName = this.getView().categoryFilter.getSelectedCategory();
        sas.hc.m.MessageDialog.warning({
            text: rb.getText("sheetCategory.delete.confirmation.fmt.txt", categoryName),
            title: rb.getText("sheetCategory.delete.window.title"),
            buttonActions: [sas.hc.m.MessageDialogAction.Delete, sas.hc.m.MessageDialogAction.Cancel],
            buttonPress: function (event) {
                var action = event.getParameter("origin").getProperty("messageDialogAction");
                if (action === sas.hc.m.MessageDialogAction.Delete) {
                    var sheetCategoryKey = controller._getSelectedCategoryKey();
                    sas.stress.config.StressAPI.deleteSheetCategory(sheetCategoryKey).done(function () {
                        controller._deleteSheetLocalStorage(sheetCategoryKey);  // remove the sheets info from the local storage
                        controller._savePreferredCategory(null);
                        controller.refresh(sheetCategoryKey);
                    }).fail(function (jqxhr) {
                        sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
                    });
                }
            }
        });
    },
    _deleteSheetLocalStorage:function(sheetCategoryKey){
        var category=this._getSheetCategory(sheetCategoryKey);  // the selected category
        if (category){
            var templates=category.getSheetTemplates(); // the templates
            var multiDelete={};
            _.each(templates,function(template){
                // clear the sheet column widths.. the user would have refined, or updated the axis... cannot use old ones
                multiDelete["sas.stress.wkbench.sheet.template." + template.id + ".columns.widths"]=undefined;
            });
            // clear the sheet column widths.. the user would have refined, or updated the axis... cannot use old ones
            sas.stress.wkbench.UserPrefs.delete(multiDelete);

            delete templates;
        }
        delete category;
    },
    pressedCreateSheetTemplate: function (e) {
        var view = this.getView();
        var selectedCategoryKey = view.categoryFilter.getSelectedItem().getKey();
        sas.eventBus.publish(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.EDIT_WORKSHEET_TEMPLATE, { selectedCategoryKey: selectedCategoryKey });
    },

    _getSheetCategory: function (index) {
        var view = this.getView();
        // If index wasn't specified, then we are getting the currently selected index from the table.
        if (undefined === index) {
            index = view.table.getSelectedIndices()[0];
            if (undefined===index){
                return undefined;
            }
            var selected = view.table.getContextByIndex(index);
            var catId = selected.getObject().catId;
            return sas.stress.wkbench.model.SheetCategory.getSheetCategory(catId);
        }else{
            return sas.stress.wkbench.model.SheetCategory.getSheetCategory(index);
        }
    },

    pressedEditSheetTemplate: function (e) {
        var sheetCat = this._getSheetCategory();
        var templateId = this._getSelectedTemplateKeys()[0];
        var template = sheetCat.getSheetTemplate(templateId);

        sas.eventBus.publish(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.EDIT_WORKSHEET_TEMPLATE, {
            cat: sheetCat,
            template: template
        });
    },

    sheetTemplateAccessControlsButtonPress: function (event) {
        // Get the selected template's id
        var templateId = this._getSelectedTemplateKeys()[0];
        if (templateId !== undefined && templateId > 0) {
            this.accessControlsDialog = new sas.stress.wkbench.widget.AccessControlsDialog({});
            var controller = this;
            var view = this.getView();
            view.table.setBusy(true);
            sas.stress.config.StressAPI.getSheetTemplateAccessControls(templateId).done(function (accessControls) {
                view.table.setBusy(false);
                // Get the template information so we can pass the name along to the access controls dialog
                var sheetCat = controller._getSheetCategory();
                var template = sheetCat.getSheetTemplate(templateId);
                controller.accessControlsDialog.setup(template.name, accessControls, "template",
                    $.proxy(controller.sheetTemplateAccessControlsSaveButtonPress, controller));
            }).fail(function (jqxhr) {
                view.table.setBusy(false);
                sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            });
        }
    },

    sheetTemplateAccessControlsSaveButtonPress: function (workgroups) {
        var controller = this;
        // Get the selected template's id
        var templateId = this._getSelectedTemplateKeys()[0];
        if (templateId !== undefined && templateId > 0) {
            sas.stress.config.StressAPI.saveSheetTemplateAccessControls(templateId, workgroups).done(function () {
                controller.accessControlsDialog.setBusy(false);
                controller.accessControlsDialog.close();
            }).fail(function (jqxhr) {
                controller.accessControlsDialog.setBusy(false);
                sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            });
        }
    },
    pressedDeleteSheetTemplate: function (e) {
        var controller = this;
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        var keys = this._getSelectedTemplateKeys();
        //var message = rb.getText("sheetTemplates.delete.confirmations.msg.fmt.txt", keys.length);

        var sheetIds=[];  // a list of sheetIds whose local storage needs to be removed
        var nameVersions = [];
        var table = this.getView().table;
        _.each(table.getSelectedIndices(),function(selectedIndex){
            var sheet = table.getContextByIndex(selectedIndex).getObject();
            sheetIds.push(sheet.id);
            nameVersions.push(sheet.nameVersion);
        });
        //if (keys.length === 1) {
        //    var sheetName = sheetNames[0];
        //    message = rb.getText("sheetTemplates.delete.confirmation.msg.fmt.txt", sheetName);
        //}
        var message = sas.stress.wkbench.StressUtil.getConfirmationMessage(rb.getText("sheetTemplates.delete.confirmation.txt"), nameVersions);

        sas.hc.m.MessageDialog.warning({
            text: message,
            title: rb.getText("sheetTemplates.delete.window.title"),
            buttonActions: [sas.hc.m.MessageDialogAction.Delete, sas.hc.m.MessageDialogAction.Cancel],
            buttonPress: function (event) {
                var action = event.getParameter("origin").getProperty("messageDialogAction");
                if (action === sas.hc.m.MessageDialogAction.Delete) {
                    sas.stress.config.StressAPI.deleteSheetTemplates(keys).done(function () {
                        controller.refresh();
                        var multiDelete={};
                        _.each(sheetIds,function(sheetId){
                            // clear the sheet column widths.. the user would have refined, or updated the axis... cannot use old ones
                            multiDelete["sas.stress.wkbench.sheet.template." + sheetId + ".columns.widths"]=undefined;
                        });
                        sas.stress.wkbench.UserPrefs.delete(multiDelete);
                    }).fail(function (jqxhr) {
                        sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
                    });
                }
            }
        });
    },
    _updateSheetTemplate: function (sheetTemplate) {
        return sas.stress.config.StressAPI.updateSheetTemplate(sheetTemplate).done(function () {
        });
    }
});