jQuery.sap.require("sas.hc.m.MessageDialog");
jQuery.sap.require("sas.hc.m.MessageToast");
jQuery.sap.require("sap.ui.model.resource.ResourceModel");
jQuery.sap.require("sas.risk.common.models.RiskJSONModel");
jQuery.sap.require("sas.risk.common.utils.LayoutService");
jQuery.sap.require("sas.risk.common.widgets.ZeroStateLayout");
jQuery.sap.require("sas.stress.wkbench.CapabilityManager");
jQuery.sap.require("sas.stress.wkbench.LeftNavManager");
jQuery.sap.require("sas.stress.wkbench.StressAPI");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.stress.wkbench.ViewManager");

sap.ui.controller("sas.stress.config.view.SubledgerAccountingManager", {

    onInit: function (event) {
        var view = this.getView();
        this.rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        this.navChecker = sas.stress.wkbench.StressUtil.createNavChecker(true, view);
        this._init();
        new sas.risk.common.utils.LayoutService(this.getView().subledgerAccountingTable, "SubledgerAccountingTable");

    },

    _init:function(){
        var view = this.getView();
        this.i18N = new sap.ui.model.resource.ResourceModel({ bundleName: "sas.stress.config.messagebundle" });
        view.setModel(this.i18N, "i18N");
        view.setModel(new sas.risk.common.models.RiskJSONModel());
        //this.updateModel();
    },

    onBeforeRendering: function () {
        this.updateModel();
    },

    updateModel: function() {
        var view = this.getView();

        view.subledgerAccountingTable.setNoData(this.rb.getText("subledgerAccountingManager.subledgerAccountingTable.noData.loading.txt"));
        view.subledgerAccountingTable.setBusyIndicatorDelay(0);
        view.subledgerAccountingTable.setBusy(true);

        var controller = this;
        var model = view.getModel();
        //model.setProperty("/subledgerAccountings", []);

        sas.stress.wkbench.StressAPI.loadDomains().done(function (domains) {
            if (domains.length > 0 ) {
                model.setProperty("/domains", domains);
             //   controller.loadSubledgers(domains[0].key, 0);
                var subLedgerTableArr = [];
                subLedgerTableArr = controller.buildFlattenedDataStructure(domains);
                model.setProperty("/FlatDomains", subLedgerTableArr);
            }

        }).fail(function (jqxhr) {
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        }).always(function() {
            view.subledgerAccountingTable.setBusy(false);
            view.subledgerAccountingTable.setNoData(new sas.risk.common.widgets.ZeroStateLayout({
                icon: sas.icons.HC.ZEROSTATEADD,
                text: "{i18N>subledgerAccountingManager.subledgerAccountingTable.noData.txt}",
                buttons: [
                    new sas.hc.m.Button({
                        text: "{i18N>subledgerAccountingManager.subledgerAccountingTable.noData.newButton.txt}", /**/
                        press: $.proxy(controller.onCreateClick, controller),
                        enabled: true
                    }).addStyleClass("sasPrimaryButton")
                ]
            }));
        });
    },

    // loadSubledgersOnly: function (domId) {
    //     var that = this;
    //
    //     //var jsonModel = that.getView().getModel();
    //     sas.stress.wkbench.StressAPI.loadSubledgers(domId).done(function (subledgers) {
    //         that.getView().getModel().setProperty("/subledgers", subledgers);
    //         that.getView().getModel().refresh();
    //     });
    //
    // },

    loadSubledgers: function (domId, subIndex) {
        var that = this;

        //var jsonModel = that.getView().getModel();
        sas.stress.wkbench.StressAPI.loadSubledgers(domId).done(function (subledgers) {
            that.getView().getModel().setProperty("/subledgers", subledgers);
            // if (subledgers.length > 0) {
            //     that.loadSubledgerData(domId, subledgers[subIndex].key);
            //    // that.getView().getModel().setProperty("/subledgers", subledgers);
            //    // that.loadSubledgerData(domId, subledgers[subIndex].key);
            // } else {
            //     that.getView().getModel().setProperty("/subledgers", "");
            // }
           // console.log(subledgers);
        });
        that.getView().getModel().refresh();
    },

    loadSubledgerData: function (domId, subId) {
        var that = this;
        sas.stress.wkbench.StressAPI.loadSubledgerDetails(domId, subId).done(function (subledger) {
            var postings = [];
            var commentRules  = [];
            var mappingLevels = [];
            var eventMappings = [];

            //Build the Comment Rules data structure
            for (var i = 0; i < subledger.commentRules.length; i++) {
                var commentRule = [];
                commentRule.ruleName = subledger.commentRules[i].ruleName;
                commentRule.ruleDescription = subledger.commentRules[i].ruleDescription;
                commentRule.tag = subledger.commentRules[i].tag;
                commentRule.entity = subledger.commentRules[i].entity;
                commentRule.eventType = subledger.commentRules[i].eventType;
                commentRule.ruleLevel = subledger.commentRules[i].ruleLevel;
                commentRule.ruleName = subledger.commentRules[i].ruleName;
                commentRule.ruleOrder = subledger.commentRules[i].ruleOrder;
                //commentRule.entityHierarchy = {};
                //commentRule.entityHierarchy.hierarchyName = subledger.commentRules[i].entityHierarchy.hierarchyName;
                //commentRule.entityHierarchy.id = subledger.commentRules[i].entityHierarchy.id;

                commentRules.push(commentRule);
            }

            for (var i = 0; i < subledger.postingGroups.length; i++) {
                for (var j = 0; j < subledger.postingGroups[i].postings.length; j++) {
                    var post = subledger.postingGroups[i].postings[j];
                    post.postingGroupId = subledger.postingGroups[i].postingGroupId;
                    post.postingGroupDescription = subledger.postingGroups[i].postingGroupDescription;
                    postings.push(post);
                }
            }
            mappingLevels.push({"item": subledger.eventMappings[0].mappingLevel});
            for (var i = 1; i < subledger.eventMappings.length; i++) {
                if ( eventMappings.indexOf(subledger.eventMappings[i - 1].eventColumnName) == -1) {
                    eventMappings.push(subledger.eventMappings[i - 1].eventColumnName);
                }
            }

            for (var i = 1; i < subledger.eventMappings.length; i++) {
                if (subledger.eventMappings[i - 1].mappingLevel != subledger.eventMappings[i].mappingLevel)
                    mappingLevels.push({"item": subledger.eventMappings[i].mappingLevel});
            }

            console.log(subledger);
            that.getView().getModel().setProperty("/mappingLevels", mappingLevels);
            that.getView().getModel().setProperty("/postings", postings);
            that.getView().getModel().setProperty("/subledgerData", subledger);
            that.getView().getModel().setProperty("/eventMappings", eventMappings);
            //that.model.setProperty("/subledgerData/commentRules", commentRules);
            //that.model.setProperty("/commentRules", commentRules);
        });

    },


    buildFlattenedDataStructure: function(domains) {
        var tableArr = [];

        for (var i = 0; i < domains.length; i++) {
            for (var j = 0; j < domains[i].subledgers.length; j++) {
                var itemArr = {};
                itemArr = {
                    name: domains[i].name,
                    description: domains[i].description,
                    subledgerName: domains[i].subledgers[j].name,
                    version: domains[i].subledgers[j].version,
                    //creationDate: domains[i].createdTime,
                    creationDate: domains[i].subledgers[j].createdTime,
                    domainId: domains[i].id,
                    subledgerId: domains[i].subledgers[j].id,
                };
                tableArr.push(itemArr);
            }
        }
        return tableArr;
    },

    updateView: function() {
        var jsonModel = this.getView().getModel();
        if (jsonModel) {
            //jsonModel.checkUpdate(false);
            jsonModel.refresh(true);
        }
    },

    onExit: function () {

    //    this.navChecker.stop();
    },

    onCreateClick: function () {
        var view = this.getView();
        var controller = this;

        var SlamModel = new sas.risk.common.models.RiskJSONModel(controller.getView().getModel().getData());

        var dialog = sap.ui.jsfragment("sas.stress.config.view.CreateSubledgerAccountingDialog", controller);

        dialog.setModel(SlamModel);

        dialog.open();

    },

    onEditClick: function () {
        var subledgerAccounting = this.getSelectedSubledgerAccounting();
        var view = this.getView();

        sas.eventBus.publish(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.EDIT_SLAM_CONFIGURATION, {
                name: subledgerAccounting.name,
                domainId: subledgerAccounting.domainId,
                subledgerId: subledgerAccounting.subledgerId,
                subledgerName: subledgerAccounting.subledgerName,
                subledgerVersion: subledgerAccounting.version,
                editSubledger:1
            });
    },

    onRefreshClick: function () {
        this.updateModel();
    },

    // entityTreeTableExpandButtonPress: function() {
    //     var sourceTable = sap.ui.getCore().byId("subledgerEntitySelectDialog.entityTable");
    //
    //     var selectedNodePath = sourceTable.getContextByIndex(0).getPath();
    //     var selectedNode = sourceTable.getModel().getProperty(selectedNodePath);
    //
    //     for (var i = 0; i < sourceTable.getBinding("rows").getLength(); i++) {
    //         var nodePath = sourceTable.getContextByIndex(i).getPath();
    //         var node = sourceTable.getModel().getProperty(nodePath);
    //         if (node.hierarchyKey === selectedNode.hierarchyKey && node.children.length > 0) {
    //             sourceTable.expand(i);
    //         }
    //     }
    // },

    entityTreeTableCollapseButtonPress: function() {
        var sourceTable = sap.ui.getCore().byId("subledgerEntitySelectDialog.entityTable");
        if (sourceTable.getBinding("rows") !== undefined) {
            for (var i = sourceTable.getBinding("rows").getLength() - 1; i >= 0; i--) {
                sourceTable.collapse(i);
            }
        }
    },

    handleSubledgerVersionChange: function(){
       // Run validation to ensure similar version does not exist
        var that = this;
        var view = this.getView();

        var matchFound=false;

        var domId = sap.ui.getCore().byId("subledgerDialog.domainSelect").getSelectedItem().getText();
        var subledgerId = sap.ui.getCore().byId("subledgerDialog.subledgerSelect").getSelectedItem().getTooltip();
        var createButton = sap.ui.getCore().byId("subledgerAccountingManagerDialog.createButton");

        var versionInput = sap.ui.getCore().byId("subledgerDialog.versionInput").getValue();

        var subledger = _.find(view.getModel().getData()["FlatDomains"], function (item) {
            if (item.name === domId &&
                item.subledgerName === subledgerId &&
                item.version === versionInput) {
                matchFound=true;
                sap.ui.getCore().byId("subledgerDialog.versionInput").setValueStateText(that.rb.getText("subledgerAccountingManagerDialog.error.nameAlreadyInUse.txt"));
                sap.ui.getCore().byId("subledgerDialog.versionInput").setValueState(sap.ui.core.ValueState.Error);
                createButton.setEnabled(false);
            }
        });

        if (!matchFound) {
            sap.ui.getCore().byId("subledgerDialog.versionInput").setValueState(sap.ui.core.ValueState.None);
            createButton.setEnabled(true);
        }

    },

    getSelectedSubledgerAccounting: function() {
        var view = this.getView();
        var index = view.subledgerAccountingTable.getSelectedIndex();
        var subledger = null;
        if (index >= 0) {
            subledger = this._getSubledgerAccountingByIndex(index);
        }
        return subledger;
    },

    _getSubledgerAccountingByIndex: function (index) {
        var view = this.getView();
        // If index wasn't specified, then we are getting the currently selected index from the table.
        if (undefined === index) {
            index = view.subledgerAccountingTable.getSelectedIndex();
        }
        //var subledger = null;
        if (index >= 0) {
            var context = view.subledgerAccountingTable.getContextByIndex(index);
            return object = context.getObject();
            // subledger = _.find(view.getModel().getData()["FlatDomains"], function (item) {
            //     if (item.name === object.name &&
            //          item.subledgerName === object.subledgerName &&
            //          item.version === object.version) {
            //         return item;
            //     }
            // });
        }
        //return subledger;
    },


    onDeleteClick: function () {
        var view = this.getView();
        var table = view.subledgerAccountingTable;
        var selectedIndices = table.getSelectedIndices();
        if (selectedIndices.length === 1) {
            this._deleteSubledgerAccounting(this.getSelectedSubledgerAccounting());
        } else {
            this._deleteSubledgerAccountings(selectedIndices);
        }
    },

    _deleteSubledgerAccounting: function (subledgerAccounting) {
        var view=this.getView();
        var title = this.rb.getText("subledgerAccountingManager.delete.subledgerAccounting.window.title");
        var deleteConfirmation = this.rb.getText("subledgerAccountingManager.delete.subledgerAccounting.confirmation.msg.fmt", subledgerAccounting.name);
        var table = view.subledgerAccountingTable;
        var self=this;

        var model = table.getModel();

        sas.hc.m.MessageDialog.warning({
            title: title,
            text: deleteConfirmation,
            buttonActions: [sas.hc.m.MessageDialogAction.Delete, sas.hc.m.MessageDialogAction.Cancel],
            buttonPress: function (event) {
                var action = event.getParameter("origin").getProperty("messageDialogAction");
                if (action === sas.hc.m.MessageDialogAction.Delete) {
                   // if (subledgerAccounting.expanded) {
                   //     self._openBusyDialog(); // S1394037
                   // }

                    sas.stress.wkbench.StressAPI.deleteSubledgerAccountingConfiguration(subledgerAccounting.domainId, subledgerAccounting.subledgerId).done(function () {
                        var model = table.getModel();
                        var subledgerAccountings = model.getData()["FlatDomains"];
                        for (var i = subledgerAccountings.length - 1; i >= 0; i--) {
                            if (subledgerAccountings[i].name === subledgerAccounting.name &&
                                subledgerAccountings[i].subledgerName === subledgerAccounting.subledgerName &&
                                subledgerAccountings[i].version === subledgerAccounting.version) {
                                subledgerAccountings.splice(i, 1);
                                break;
                            }
                        }
                        model.refresh();
                        table.clearSelection();
                    //    self._closeBusyDialog();    // S1394037
                    }).fail(function (jqxhr) {
                    //    self._closeBusyDialog();    // S1394037
                        sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
                    });
                }
            }
        });
    },
    _deleteSubledgerAccountings: function (selectedIndices) {
        var self=this;
        var view=this.getView();
        var table = view.subledgerAccountingTable;
        var controller = this;
        var idList = [];
        var nameList = [];
        var isAnySubledgerAccountingExpanded=false; // S1394037
        _.each(selectedIndices, function (selectedIndex) {
            var subledgerAccounting = controller._getSubledgerAccountingByIndex(selectedIndex);
            idList.push(subledgerAccounting.subledgerAccountingId);
            nameList.push(subledgerAccounting.subledgerAccountingName);
            if (subledgerAccounting.expanded){
                isAnySubledgerAccountingExpanded=subledgerAccounting.expanded;
            }
        });
        var title = this.rb.getText("subledgerAccountingManager.delete.subledgerAccounting.window.title");
        var deleteConfirmation = this.rb.getText("subledgerAccountingManager.delete.subledgerAccounting.confirmation.msg.fmt", nameList[0]);

        if (idList.length > 1) {
            deleteConfirmation = sas.stress.wkbench.StressUtil.getConfirmationMessage(this.rb.getText("subledgerAccountingManager.delete.subledgerAccounting.confirmation.msg.txt"), nameList);
        }
        sas.hc.m.MessageDialog.warning({
            title: title,
            text: deleteConfirmation,
            buttonActions: [sas.hc.m.MessageDialogAction.Delete, sas.hc.m.MessageDialogAction.Cancel],
            buttonPress: function (event) {
                var action = event.getParameter("origin").getProperty("messageDialogAction");
                if (action === sas.hc.m.MessageDialogAction.Delete) {
                    if (isAnySubledgerAccountingExpanded) { // S1394037
                        self._openBusyDialog();
                    }
                    //Delete all the items in the selected list
                    sas.stress.subledgerAccounting.model.SubledgerAccounting.deleteSubledgerAccountings(idList).done(function () {
                        // or should i just refresh? What if other users have deleted the groups?
                        var model = table.getModel();
                        var subledgerAccountings = model.getData()["subledgerAccountings"];
                        for (var a = 0; a < idList.length; a++) {
                            for (var i = subledgerAccountings.length - 1; i >= 0; i--) {
                                if (subledgerAccountings[i].subledgerAccountingId === idList[a]) {
                                    subledgerAccountings.splice(i, 1);
                                    break;
                                }
                            }
                        }
                        model.refresh();
                        table.clearSelection();
                        self._closeBusyDialog();    // S1394037
                    }).fail(function (jqxhr) {
                        self._closeBusyDialog();    // S1394037
                        sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
                    });
                }
            }
        });
    },

    createSubledgerAccountingButtonPress: function () {

        var that = this;
        var view = this.getView();

        var name = sap.ui.getCore().byId("subledgerDialog.domainSelect").getSelectedItem().getText();
        var domainId = sap.ui.getCore().byId("subledgerDialog.domainSelect").getSelectedItem().getKey();

        var subledgerName = sap.ui.getCore().byId("subledgerDialog.subledgerSelect").getSelectedItem().getTooltip();
        var subledgerId = sap.ui.getCore().byId("subledgerDialog.subledgerSelect").getSelectedItem().getKey();
        var version = sap.ui.getCore().byId("subledgerDialog.versionInput").getValue();

        sas.eventBus.publish(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.EDIT_SLAM_CONFIGURATION, {
            name: name,
            domainId: domainId,
            subledgerId: subledgerId,
            subledgerName: subledgerName,
            subledgerVersion: version,
            editSubledger: 0
        });

        var dialog = sap.ui.getCore().byId("sas.stress.config.view.CreateSubledgerAccountingDialog");
        dialog.close();

    },

    onFilterSubledgerAccounting: function () {
        var search = this.getView().searchField.getValue() || "";
        var table = this.getView().subledgerAccountingTable;

        var filter0 = new sap.ui.model.Filter("name", sap.ui.model.FilterOperator.Contains, search);
        var filter1 = new sap.ui.model.Filter("subledgerName", sap.ui.model.FilterOperator.Contains, search);
        var filter2 = new sap.ui.model.Filter("version", sap.ui.model.FilterOperator.Contains, search);
        var allFilters = new sap.ui.model.Filter([filter0, filter1, filter2]);

        //filters.push(new sap.ui.model.Filter("name",sap.ui.model.FilterOperator.Contains,search));
        //var orFilters = new sap.ui.model.Filter(allFilters); // the filters on the visible columns OR'd
        var binding = table.getBinding("rows");

        binding.filter(allFilters);
    },

    onRowSelectionChange: function (oEvent) {
        var view = this.getView();

        var selected = oEvent.getSource().getSelectedIndices().length;

        view.deleteButton.setEnabled(selected === 1);
        view.editButton.setEnabled (selected === 1);
    },

    _openBusyDialog:function(){ // S1394037
        this.getView().setBusy(true);
        this._bzdlg = new sas.hc.m.BusyDialog({title: this.rb.getText("delete.subledgerAccountings.busy.dialog.title")});
        this._bzdlg.open();
    },
    _closeBusyDialog:function(){    // S1394037
        this.getView().setBusy(false);
        if (this._bzdlg){
            this._bzdlg.close();
            this._bzdlg = undefined;
        }
    }
});