jQuery.sap.require("sap.ui.model.Filter");
jQuery.sap.require("sap.ui.model.Sorter");
jQuery.sap.require("sas.hc.m.MessageDialog");
jQuery.sap.require("sas.hc.m.MessageToast");
jQuery.sap.require("sas.hc.ui.layout.SplitterLayoutData");
jQuery.sap.require("sas.risk.common.models.RiskJSONModel");
jQuery.sap.require("sas.stress.config.model.Hierarchy");
jQuery.sap.require("sas.stress.config.model.HierarchyMaps");
jQuery.sap.require("sas.stress.config.model.HierarchyMapPair");
jQuery.sap.require("sas.stress.models.Model");
jQuery.sap.require("sas.stress.wkbench.StressFixes");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.stress.wkbench.ViewManager");

sap.ui.controller("sas.stress.config.view.UserManager", {

    onInit: function (event) {
        this.currentUser = null;
        this._userLookup(sas.stress.wkbench.StressUtil.getUserName());
    },

    _searchUser: function(event) {
        var userName = event.getParameter("inputValue") || event.getParameter("newValue");
        this.currentUser = userName;
        if (userName) {
            this._userLookup(userName);
        } else {
            this._updateUserModel();
        }
    },

    _userLookup: function(userName) {
        var controller = this;
        var view = this.getView();
        view.permTable.setBusy(true);
        view.permTable.setBusyIndicatorDelay(0);
        sas.stress.config.StressAPI.getUserWorkGroupData(userName).done(function (userWorkGroupData) {
            controller._updateUserModel(userWorkGroupData);
            view.permTable.setBusy(false);
        }).fail(function(jqxhr){
            view.permTable.setBusy(false);
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            controller._updateUserModel();
        });
    },

    _expandAll: function() {
        var view = this.getView();
        view.permTable.expandToLevel(sas.risk.common.utils.CommonConstants.MAX_INTEGER);
    },

    _collapseAll: function() {
        var view = this.getView();
        view.permTable.collapseAll();
    },

    _updateUserModel: function(data) {
        var view = this.getView();
        model = new sap.ui.model.json.JSONModel([data]);
        view.permTable.setModel(model);
        view.permTable.bindRows("/");
        view.permTable.expandToLevel(sas.risk.common.utils.CommonConstants.MAX_INTEGER);
        this._filterWorkGroups();
    },

    _refreshUserData: function() {

        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        var controller = this;
        var view = controller.getView();
        view.permTable.setBusy(true);
        view.permTable.setBusyIndicatorDelay(0);

        sas.stress.config.StressAPI.refreshUserCache().done(function (data) {
            sas.hc.m.MessageToast.show(rb.getText("userPermissionsManager.refresh.toast.txt"));
            controller._userLookup(controller.currentUser);
        }).fail(function (jqXHR, status, error) {
            sas.stress.wkbench.StressUtil.handleServerFailure(jqXHR);
        });
    },

    _filterWorkGroups: function() {

        var view = this.getView();
        var search=view.searchField.getValue() || "";
        var table = view.permTable;
        var compatFilter = sap.ui.getCore().byId("UserManagaer.compatFilterCheckBox");
        var filters=[];

        if (search) {
            filters.push(new sap.ui.model.Filter("displayName", sap.ui.model.FilterOperator.Contains, search));
        }

        if (compatFilter && compatFilter.getSelected()) {
            filters.push(new sap.ui.model.Filter("usableByRfw", sap.ui.model.FilterOperator.EQ, true));
        }

        var orFilters = new sap.ui.model.Filter({
            filters: filters,
            and: true
        });

        var binding = table.getBinding("rows");

        if (filters.length > 0) {
            binding.filter(orFilters);
            this._expandAll();
        } else {
            binding.filter();
        }

    }

});