jQuery.sap.require("sas.hc.m.Button");
jQuery.sap.require("sas.hc.m.Text");
jQuery.sap.require("sas.hc.m.ResponsiveToolbar");
jQuery.sap.require("sas.hc.m.ToolbarSpacer");
jQuery.sap.require("sas.hc.ui.table.Column");
jQuery.sap.require("sas.hc.ui.table.TreeTable");
jQuery.sap.require("sas.risk.common.widgets.WidgetFactory");
jQuery.sap.require("sas.risk.common.widgets.RiskToolbar");
jQuery.sap.require("sas.hc.m.CompositeInput");
jQuery.sap.require("sas.hc.ui.layout.VerticalLayout");
jQuery.sap.require("sas.hc.m.SearchField");

sap.ui.jsview("sas.stress.config.view.UserManager", {

    getControllerName: function () {
        return "sas.stress.config.view.UserManager";
    },

    createContent: function (controller) {

        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");

        var refreshButton = new sas.hc.m.Button({
            text: rb.getText("userPermissionsManager.refreshButton.txt"),
            tooltip: rb.getText("userPermissionsManager.refreshButton.tip"),
            press: $.proxy(controller._refreshUserData, controller)
        });

        this.permTable = new sas.hc.ui.table.TreeTable({
            visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Auto,
            selectionMode: sas.hc.ui.table.SelectionMode.Single,
            selectionBehavior: sas.hc.ui.table.SelectionBehavior.RowOnly,
            columns: [
                new sas.hc.ui.table.Column({
                    label: rb.getText("userPermissionsManager.userTable.column.name.header.title"),
                    template: new sas.hc.ui.commons.Label({
                        text: {
                            parts: ["name", "displayName"],
                            formatter: function (name, displayName) {
                                return (displayName) ? displayName : name;
                            }
                        },
                        tooltip: "{name}"
                    }).addStyleClass("tableLabel"),
                    width: "10rem",
                    name: "displayName"
                }),
                new sas.hc.ui.table.Column({
                    label: "Type",
                    template: new sas.hc.m.Text({
                        text: {
                            path: 'type',
                            formatter: function (type) {
                                return type;
                            }
                        },
                        wrapping: false
                    }),
                    width: "10rem",
                    name: "type"
                }),
                new sas.hc.ui.table.Column({
                    label: rb.getText("userPermissionsManager.userTable.column.compat.header.title"),
                    template: new sas.hc.ui.core.Icon({
                        src: sas.icons.HC.CHECKMARK,
                        visible: {
                            path: 'usableByRfw',
                            formatter: function (usableByRfw) {
                                return usableByRfw;
                            }
                        }
                    }),
                    hAlign: sap.ui.core.HorizontalAlign.Center,
                    width: "2.5rem",
                    name: "usableByRfw"
                }),
                new sas.hc.ui.table.Column({
                    label: rb.getText("userPermissionsManager.userTable.column.filePath.header.title"),
                    template: new sas.hc.ui.core.Icon({
                        src: sas.icons.HC.FILE,
                        visible: {
                            path: 'fileSystemPath',
                            formatter: function (fileSystemPath) {
                                return !!fileSystemPath;
                            }
                        },
                        tooltip: {
                            path: 'fileSystemPath',
                            formatter: function (fileSystemPath) {
                                return fileSystemPath;
                            }
                        }
                    }),
                    hAlign: sap.ui.core.HorizontalAlign.Center,
                    width: "2.5rem",
                    name: "fileSystemPath"
                })
            ],
            toolbar: this._createTableToolbar(controller)
        });

        this.userInput = new sas.hc.m.CompositeInput({
            inputProperties: {
                label: rb.getText("userPermissionsManager.userSearch.label.txt"),
                labelPosition: sas.hc.m.InputLabelPosition.Top,
                type: sap.m.InputType.Text,
                value: sas.stress.wkbench.StressUtil.getUserName(),
                disableValidation: false,
                maxLength: 32,
                selectTextOnPress:false,
                validateOnBindingValueChange:false
            },
            buttonProperties: {
                text: rb.getText("userPermissionsManager.userSearch.button.txt")
            },
            customData: new sap.ui.core.CustomData({
                key: "membername",
                value: "{membername}"
            }),
            width: "300px"
        }).addStyleClass("sapUiSmallMarginBottom");

        this.userInput.attachChange(function(e) {
            controller._searchUser(e);
        });

        var tablePanel = sas.risk.common.widgets.WidgetFactory.buildPanel({
                content: new sas.hc.ui.layout.FixFlex({
                    fixContent: [
                        new sas.hc.m.ResponsiveToolbar({
                            content: [
                                new sas.hc.m.ToolbarSpacer(),
                                refreshButton
                            ]
                        }),
                        this.userInput
                    ],
                    flexContent: [
                        this.permTable
                    ]
                })
        });

        return sas.risk.common.widgets.WidgetFactory.buildPanel({
            content: tablePanel
        });

    },

    _createTableToolbar: function(controller) {

        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");

        return new sas.hc.m.ResponsiveToolbar({
            content: [
                this._createSearchField(controller),
                new sas.hc.m.CheckBox({
                    id: "UserManagaer.compatFilterCheckBox",
                    text: rb.getText("userPermissionsManager.userSearch.checkBox.label"),
                    selected: false,
                    select: $.proxy(controller._filterWorkGroups, controller)
                }),
                new sas.hc.m.ToolbarSpacer(),
                new sas.hc.m.Button({
                    icon: sas.icons.HC.SHOWALL,
                    press: $.proxy(controller._expandAll, controller)
                }),
                new sas.hc.m.Button({
                    icon: sas.icons.HC.HIDEALL,
                    press: $.proxy(controller._collapseAll, controller)
                })
            ]
        })
    },

    _createSearchField: function (controller) {
        this.searchField = new sas.hc.m.SearchField({
            liveChange: $.proxy(controller._filterWorkGroups, controller),
            search: $.proxy(controller._filterWorkGroups, controller),
            filterMode: true
        });
        return this.searchField;
    },

    destroy: function () {
        sap.ui.core.mvc.JSView.prototype.destroy.apply(this, arguments);
    }

});