jQuery.sap.require("sap.ui.core.format.FileSizeFormat");
jQuery.sap.require("sap.ui.core.format.DateFormat");
jQuery.sap.require("sap.ui.model.Sorter");
jQuery.sap.require("sas.risk.common.widgets.RiskToolbar");
jQuery.sap.require("sas.risk.common.widgets.RiskPage");
jQuery.sap.require("sas.risk.common.widgets.WidgetFactory");
jQuery.sap.require("sas.stress.wkbench.CapabilityManager");
jQuery.sap.require("sas.stress.wkbench.ViewManager");
jQuery.sap.require("sas.hc.m.IconTabBar");
jQuery.sap.require("sas.hc.m.IconTabFilter");
jQuery.sap.require("sas.risk.common.widgets.WidgetFactory");

sap.ui.jsview("sas.stress.data.view.DataEditor", {

    getControllerName: function () {
        return "sas.stress.data.view.DataEditor";
    },

    createContent: function (controller) {
        this.setHeight("100%");
        this.capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();
        this.globalDataTabKey = "GLOBALDATATAB";
        this.projectDataTabKey = "PROJECTTAB";
        this.dataLoadMapTabKey = "DATALOADMAPTAB";


        this.iconTabBar = new sas.hc.m.IconTabBar({
            id: "globalDataEditorTab",
            firstTabClosable: false,
            showOverflowPopOver: false,
            stretchContentHeight: true
        });

        this.globalDataTab = new sas.hc.m.IconTabFilter({
            key:  this.globalDataTabKey,
            text: "{i18N>dataLoadHistoryTable.title}",
            closable: false
        });
        this.globalDataTab.addContent(this._createGlobalDataPanel(controller));

        this.projectDataTab = new sas.hc.m.IconTabFilter({
            key:  this.projectDataTabKey,
            text: "{i18N>projectDataLoadHistoryTable.title}",
            closable: false
        });
        this.projectDataTab.addContent(this._createProjectDataPanel(controller));

        this.dataLoadMapTab = new sas.hc.m.IconTabFilter({
            key:  this.dataLoadMapTabKey,
            text: "{i18N>dataLoadMapTable.title}",
            closable: false
        });

        this.dataLoadMapTab.addContent(this._createDataLoadMapPage(controller));
        this.iconTabBar.addItem(this.globalDataTab);
        this.iconTabBar.addItem(this.projectDataTab);
        this.iconTabBar.addItem(this.dataLoadMapTab);

        this.iconTabBar.attachSelect(function(event){
            var tabId = event.getParameter('selectedKey');

            switch(tabId){
                case "GLOBALDATATAB":
                        controller.refreshGlobalData();
                    break;
                case "PROJECTTAB":
                        controller.refreshProjectData();
                    break;
                case "DATALOADMAPTAB":
                        controller.refreshDataLoadMap();
                    break;
            }
        });

        this.dataEditorLayout = sas.risk.common.widgets.WidgetFactory.buildPanel({
            type: sas.risk.common.widgets.WidgetFactory.Object,
            top: new sas.risk.common.widgets.RiskToolbar({
                title: new sas.hc.m.Label({text: "{i18N>dataEditor.breadcrumb.label}"})
            }),
            content: this.iconTabBar
        });

        return new sas.risk.common.widgets.RiskPage({
            mainContent: this.dataEditorLayout
        });

    },
    _createGlobalDataTableHeader: function (controller) {
        var loadGlobalDataButton = new sas.hc.m.Button({
            icon: sas.icons.HC.IMPORT,
            press: $.proxy(controller.loadGlobalDataButtonPress, controller),
            enabled: this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.DATA.IMPORT),
            tooltip: "{i18N>dataLoadHistoryTable.loadDataButton.tip}"
        });

        return new sas.hc.m.ResponsiveToolbar({
            content: [
                this._createGlobalDataSearchField(controller),
                new sas.hc.m.ToolbarSpacer(),
                loadGlobalDataButton,
                new sas.hc.m.Button({
                    icon: sas.icons.HC.REFRESH,
                    tooltip: "{i18N>dataLoadHistoryTable.refreshButton.tip}",
                    enabled: this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.DATA.READ),
                    press: $.proxy(controller.refreshGlobalData, controller)
                })
            ]
        });
    },

    _createGlobalDataSearchField: function (controller) {
        return new sas.hc.m.SearchField({
            id: "globalDataLoadHistoryTableSearchFld",
            liveChange: $.proxy(controller.onGlobalDataFilterChanged, controller),
            search: $.proxy(controller.onGlobalDataFilterChanged, controller),
            filterMode: true
        });
    },

    _createGlobalDataPanel: function (controller) {
        this.globalDataTable = new sas.hc.ui.table.Table({
            editable: false,
            width: "100%",
            visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Auto,
            minAutoRowCount: 1,
            selectionMode: sas.hc.ui.table.SelectionMode.Single,
            selectionBehavior: sas.hc.ui.table.SelectionBehavior.RowOnly,
            showTableOptions: false,
            toolbar: this._createGlobalDataTableHeader(controller),
            //noData: "{i18N>dataLoadHistoryTable.noData.loading.txt}",   // setNoData() in controller
            title: "{i18N>dataLoadHistoryTable.title}"
        }).addStyleClass("stressInnerTitle");

        this.globalDataTable.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("fileName"),
            label: new sas.hc.m.Label({text: "{i18N>dataLoadHistoryTable.column.filename.header.title}"}),
            template: new sas.hc.m.Text({
                text: "{globalDataLoadHistory>fileName}",
                wrapping: false,
                tooltip: "{globalDataLoadHistory>fileName}"
            }),
            sortProperty: "fileName",
            width: "15rem"
        }));
        this.globalDataTable.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("userName"),
            label: new sas.hc.m.Label({text: "{i18N>dataLoadHistoryTable.column.uploadedBy.header.title}"}),
            template: new sas.hc.m.Text({
                text: "{globalDataLoadHistory>userName}",
                wrapping: false,
                tooltip: "{globalDataLoadHistory>userName}"
            }),
            sortProperty: "userName"
        }));
        this.globalDataTable.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("loadDate"),
            label: new sas.hc.m.Label({text: "{i18N>dataLoadHistoryTable.column.loadDate.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: {
                    path: "{globalDataLoadHistory>uploadDate}",
                    formatter: function (dttm) {
                        if (!dttm) return;
                        var dateTimeFormat = sap.ui.core.format.DateFormat.getDateTimeInstance();
                        return dateTimeFormat.format(new Date(dttm));
                    }
                },
                text: {
                    path: "globalDataLoadHistory>uploadDate",
                    formatter: function (dttm) {
                        if (!dttm) return;
                        var dateTimeFormat = sap.ui.core.format.DateFormat.getDateTimeInstance();
                        return dateTimeFormat.format(new Date(dttm));
                    }
                }
            }),
            sortProperty: "uploadDate"
        }));
        this.globalDataTable.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("fileLocationType"),
            label: new sas.hc.m.Label({text: "{i18N>dataLoadHistoryTable.column.fileLocationType.header.title}"}),
            template: new sas.hc.m.Text({
                text: "{globalDataLoadHistory>fileLocationType}",
                wrapping: false,
                tooltip: "{globalDataLoadHistory>fileLocationType}"
            }),
            sortProperty: "fileLocationType"
        }));
        this.globalDataTable.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("fileSize"),
            label: new sas.hc.m.Label({text: "{i18N>dataLoadHistoryTable.column.fileSize.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: {
                    path: "{globalDataLoadHistory>fileSize}",
                    formatter: function (fileSizeInBytes) {
                        if (fileSizeInBytes < 0) {
                            return "";
                        }
                        var fileSizeFormatter = sap.ui.core.format.FileSizeFormat.getInstance({
                            binaryFilesize: false,
                            maxFractionDigits: 2
                        });
                        return fileSizeFormatter.format(fileSizeInBytes);
                    }
                },
                text: {
                    path: "globalDataLoadHistory>fileSize",
                    formatter: function (fileSizeInBytes) {
                        if (fileSizeInBytes < 0) {
                            return "";
                        }
                        var fileSizeFormatter = sap.ui.core.format.FileSizeFormat.getInstance({
                            binaryFilesize: false,
                            maxFractionDigits: 2
                        });
                        return fileSizeFormatter.format(fileSizeInBytes);
                    }
                }
            }),
            sortProperty: "fileSize"
        }));

        this.globalDataTable.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("dataFormat"),
            label: new sas.hc.m.Label({text: "{i18N>dataLoadHistoryTable.column.dataFormat.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: "{globalDataLoadHistory>dataFormat}",
                text: "{globalDataLoadHistory>dataFormat}"
            }),
            sortProperty: "dataFormat"
        }));

        this.globalDataTable.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("dataLoadMapName"),
            label: new sas.hc.m.Label({text: "{i18N>dataLoadHistoryTable.column.dataLoadMapName.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: "{globalDataLoadHistory>dataLoadMapName}",
                text: "{globalDataLoadHistory>dataLoadMapName}"
            }),
            sortProperty: "dataLoadMapName"
        }));

        this.globalDataTable.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("dataPointSource"),
            label: new sas.hc.m.Label({text: "{i18N>dataLoadHistoryTable.column.dataPointSource.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: "{globalDataLoadHistory>dataPointSource}",
                text: "{globalDataLoadHistory>dataPointSource}"
            }),
            sortProperty: "dataPointSource"
        }));

        this.globalDataTable.bindRows("globalDataLoadHistory>/");

        //S1256573: Default sort by Name/Version asc.
        var bindingInfo = this.globalDataTable.getBindingInfo("rows");
        if (bindingInfo) {
            var sorters = [
                new sap.ui.model.Sorter("uploadDate", true),
                new sap.ui.model.Sorter("fileName", false)];
            bindingInfo.sorter = sorters;
        }

        return sas.risk.common.widgets.WidgetFactory.buildPanel({
            bottomPadding: ".25rem",
            content: this.globalDataTable
        });
    },

    _createProjectDataTableHeader: function (controller) {
        var loadProjectDataButton = new sas.hc.m.Button({
            icon: sas.icons.HC.IMPORT,
            press: $.proxy(controller.loadProjectDataButtonPress, controller),
            enabled: this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.DATA.IMPORT),
            tooltip: "{i18N>dataLoadHistoryTable.loadDataButton.tip}"
        });

        return new sas.hc.m.ResponsiveToolbar({
            content: [
                this._createProjectDataSearchField(controller),
                new sas.hc.m.ToolbarSpacer(),
                loadProjectDataButton,
                new sas.hc.m.Button({
                    icon: sas.icons.HC.REFRESH,
                    tooltip: "{i18N>dataLoadHistoryTable.refreshButton.tip}",
                    enabled: this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.DATA.READ),
                    press: $.proxy(controller.refreshProjectData, controller)
                })
            ]
        });
    },

    _createProjectDataSearchField: function (controller) {
        return new sas.hc.m.SearchField({
            id: "allProjectsDataLoadHistoryTableSearchFld",
            liveChange: $.proxy(controller.onProjectDataFilterChanged, controller),
            search: $.proxy(controller.onProjectDataFilterChanged, controller),
            filterMode: true
        });
    },

    _createProjectDataPanel: function (controller) {
        this.projectDataTable = new sas.hc.ui.table.Table({
            editable: false,
            width: "100%",
            visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Auto,
            minAutoRowCount: 1,
            selectionMode: sas.hc.ui.table.SelectionMode.Single,
            selectionBehavior: sas.hc.ui.table.SelectionBehavior.RowOnly,
            showTableOptions: false,
            toolbar: this._createProjectDataTableHeader(controller),
            //noData: "{i18N>projectDataLoadHistoryTable.noData.loading.txt}",  // setNoData() in controller
            title: "{i18N>projectDataLoadHistoryTable.title}"
        }).addStyleClass("stressInnerTitle");

        this.projectDataTable.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("projectName"),
            label: new sas.hc.m.Label({text: "{i18N>dataLoadHistoryTable.column.project.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: "{projectDataLoadHistory>projectName}",
                text: "{projectDataLoadHistory>projectName}"
            }),
            sortProperty: "projectName",
            width: "15rem"
        }));
        this.projectDataTable.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("projectIteration"),
            label: new sas.hc.m.Label({text: "{i18N>dataLoadHistoryTable.column.projectIteration.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                textAlign: "Right",
                //tooltip: "{projectDataLoadHistory>projectIteration}",
                text: "{projectDataLoadHistory>projectIteration}"
            }),
            sortProperty: "projectIteration",
            hAlign: sap.ui.core.HorizontalAlign.Right,
            width: "6rem"
        }));
        this.projectDataTable.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("history-fileName"),
            label: new sas.hc.m.Label({text: "{i18N>dataLoadHistoryTable.column.filename.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: "{projectDataLoadHistory>fileName}",
                text: "{projectDataLoadHistory>fileName}"
            }),
            sortProperty: "fileName",
            width: "15rem"
        }));
        this.projectDataTable.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("uploadedBy"),
            label: new sas.hc.m.Label({text: "{i18N>dataLoadHistoryTable.column.uploadedBy.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: "{projectDataLoadHistory>userName}",
                text: "{projectDataLoadHistory>userName}"
            }),
            sortProperty: "userName"
        }));
        this.projectDataTable.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("uploadDate"),
            label: new sas.hc.m.Label({text: "{i18N>dataLoadHistoryTable.column.loadDate.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: {
                    path: "projectDataLoadHistory>uploadDate",
                    formatter: function (dttm) {
                        if (!dttm) return;
                        var dateTimeFormat = sap.ui.core.format.DateFormat.getDateTimeInstance();
                        return dateTimeFormat.format(new Date(dttm));
                    }
                },
                text: {
                    path: "projectDataLoadHistory>uploadDate",
                    formatter: function (dttm) {
                        if (!dttm) return;
                        var dateTimeFormat = sap.ui.core.format.DateFormat.getDateTimeInstance();
                        return dateTimeFormat.format(new Date(dttm));
                    }
                }
            }),
            sortProperty: "uploadDate"
        }));
        this.projectDataTable.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("history-fileLocationType"),
            label: new sas.hc.m.Label({text: "{i18N>projectDataLoadHistoryTable.column.fileLocationType.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: "{projectDataLoadHistory>fileLocationType}",
                text: "{projectDataLoadHistory>fileLocationType}"
            }),
            sortProperty: "fileLocationType"
        }));
        this.projectDataTable.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("history-fileSize"),
            label: new sas.hc.m.Label({text: "{i18N>projectDataLoadHistoryTable.column.fileSize.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: {
                    path: "projectDataLoadHistory>fileSize",
                    formatter: function (fileSizeInBytes) {
                        if (fileSizeInBytes < 0) {
                            return "";
                        }
                        var fileSizeFormatter = sap.ui.core.format.FileSizeFormat.getInstance({
                            binaryFilesize: false,
                            maxFractionDigits: 2
                        });
                        return fileSizeFormatter.format(fileSizeInBytes);
                    }
                },
                text: {
                    path: "projectDataLoadHistory>fileSize",
                    formatter: function (fileSizeInBytes) {
                        if (fileSizeInBytes < 0) {
                            return "";
                        }
                        var fileSizeFormatter = sap.ui.core.format.FileSizeFormat.getInstance({
                            binaryFilesize: false,
                            maxFractionDigits: 2
                        });
                        return fileSizeFormatter.format(fileSizeInBytes);
                    }
                }
            }),
            sortProperty: "fileSize"
        }));

        this.projectDataTable.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("history-dataFormat"),
            label: new sas.hc.m.Label({text: "{i18N>projectDataLoadHistoryTable.column.dataFormat.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: "{projectDataLoadHistory>dataFormat}",
                text: "{projectDataLoadHistory>dataFormat}"
            }),
            sortProperty: "dataFormat"
        }));

        this.projectDataTable.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("history-dataLoadMapName"),
            label: new sas.hc.m.Label({text: "{i18N>projectDataLoadHistoryTable.column.dataLoadMapName.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: "{projectDataLoadHistory>dataLoadMapName}",
                text: "{projectDataLoadHistory>dataLoadMapName}"
            }),
            sortProperty: "dataLoadMapName"
        }));

        this.projectDataTable.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("history-dataPointSource"),
            label: new sas.hc.m.Label({text: "{i18N>projectDataLoadHistoryTable.column.dataPointSource.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: "{projectDataLoadHistory>dataPointSource}",
                text: "{projectDataLoadHistory>dataPointSource}"
            }),
            sortProperty: "dataPointSource"
        }));

        this.projectDataTable.bindRows("projectDataLoadHistory>/");

        //S1256573: Default sort by Name/Version asc.
        var bindingInfo = this.projectDataTable.getBindingInfo("rows");
        if (bindingInfo) {
            var sorters = [
                new sap.ui.model.Sorter("uploadDate", true),
                new sap.ui.model.Sorter("projectName", false),
                new sap.ui.model.Sorter("fileName", false)];
            bindingInfo.sorter = sorters;
        }

        var contentPanel = sas.risk.common.widgets.WidgetFactory.buildPanel({
            topPadding: ".25rem",
            content: this.projectDataTable
        });
        contentPanel.setHeight("100%");
        return contentPanel;
    },

    _creatDataLoadMapTableHeader: function (controller) {
        var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();

        this.createNewDataLoadMapButton = new sas.hc.m.Button({
            icon: sas.icons.HC.NEWSTARBURST,
            id: "createMap",
            tooltip: "{i18N>dataLoadMapTable.create.button.tooltip}",
            press: $.proxy(controller.onOpenDataLoadMapDialog, controller),
            enabled: capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.DATA.CREATE_MAP)
        });

        this.editDataLoadMapButton = new sas.hc.m.Button({
            icon: sas.icons.HC.EDIT,
            id: "editMap",
            tooltip: "{i18N>dataLoadMapTable.edit.button.tooltip}",
            press: $.proxy(controller.onOpenDataLoadMapDialog, controller),
            enabled: false
        });

        this.deleteDataLoadMapButton = new sas.hc.m.Button({
            icon: sas.icons.HC.DELETE,
            tooltip: "{i18N>dataLoadMapTable.delete.button.tooltip}",
            press: $.proxy(controller.onDeleteDataLoadMapClick, controller),
            enabled: false
        });

        return new sas.hc.m.ResponsiveToolbar({
            content: [
                this._createDataLoadMapSearchField(controller),
                new sas.hc.m.ToolbarSpacer(),
                this.createNewDataLoadMapButton,
                this.editDataLoadMapButton,
                new sas.hc.m.Button({
                    icon: sas.icons.HC.REFRESH,
                    tooltip: "{i18N>dataLoadMapTable.refresh.button.tooltip}",
                    enabled: this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.DATA.READ),
                    press: $.proxy(controller.refreshDataLoadMap, controller)
                }),
                this.deleteDataLoadMapButton
            ]
        });
    },

    _createDataLoadMapSearchField: function (controller) {
        return new sas.hc.m.SearchField({
            id: "allDataLoadMapTableSearchFld",
            liveChange: $.proxy(controller.onDataLoadMapFilterChanged, controller),
            search: $.proxy(controller.onDataLoadMapFilterChanged, controller),
            filterMode: true
        });
    },

    _createDataLoadMapPage:function(controller){
        var contentPanel = sas.risk.common.widgets.WidgetFactory.buildPanel({
            topPadding: ".25rem",
            content: new sas.hc.ui.layout.Splitter({
                contentAreas: [
                    this._createDataLoadMapTablePanel(controller),
                    this._createDataLoadMapDetailTablePanel(controller)
                ],
                orientation: sap.ui.core.Orientation.Vertical,
                width: "100%",
                height: "100%"
            })
        });
        contentPanel.setHeight("100%");
        return contentPanel;
    },

    _createDataLoadMapTablePanel:function(controller){
        this.dataLoadMapTable = new sas.hc.ui.table.Table({
            editable: false,
            width: "100%",
            visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Auto,
            minAutoRowCount: 1,
            selectionMode: sas.hc.ui.table.SelectionMode.MultiToggle,
            showTableOptions: true,
            selectionBehavior: sas.hc.ui.table.SelectionBehavior.RowSelector,
            toolbar: this._creatDataLoadMapTableHeader(controller),
            title: "{i18N>dataLoadMapTable.title}",
            rowSelectionChange: $.proxy(controller.dataLoadMapTableRowSelectionChanged, controller)
        }).addStyleClass("stressInnerTitle");

        this.dataLoadMapTable.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("mapId"),
            label: new sas.hc.m.Label({text: "{i18N>dataLoadMapTable.id.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                text: "{dataLoadMap>mapId}"
            }),
            visible: false,
            sortProperty: "mapId"
        }));

        this.dataLoadMapTable.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("mapName"),
            label: new sas.hc.m.Label({text: "{i18N>dataLoadMapTable.name.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: "{dataLoadMap>mapName}",
                text: "{dataLoadMap>mapName}"
            }),
            sortProperty: "mapName"
        }));

        this.dataLoadMapTable.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("createdBy"),
            label: new sas.hc.m.Label({text: "{i18N>dataLoadMapTable.createdBy.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: "{dataLoadMap>createdBy}",
                text: "{dataLoadMap>createdBy}"
            }),
            sortProperty: "createdBy",
            width: "8rem"
        }));

        this.dataLoadMapTable.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("modifiedBy"),
            label: new sas.hc.m.Label({text: "{i18N>dataLoadMapTable.modifiedBy.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: "{dataLoadMap>modifiedBy}",
                text: "{dataLoadMap>modifiedBy}"
            }),
            sortProperty: "modifiedBy",
            width: "8rem"
        }));

        this.dataLoadMapTable.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("mapDesc"),
            label: new sas.hc.m.Label({text: "{i18N>dataLoadMapTable.desc.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: "{dataLoadMap>mapDesc}",
                text: "{dataLoadMap>mapDesc}"
            }),
            sortProperty: "mapDesc"
        }));

        this.dataLoadMapTable.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("dateCreated"),
            label: new sas.hc.m.Label({text: "{i18N>dataLoadMapTable.dateCreated.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: {
                    path: "dataLoadMap>dateCreated",
                    formatter: function (dttm) {
                        if (!dttm) return;
                        var dateTimeFormat = sap.ui.core.format.DateFormat.getDateTimeInstance();
                        return dateTimeFormat.format(new Date(dttm));
                    }
                },
                text: {
                    path: "dataLoadMap>dateCreated",
                    formatter: function (dttm) {
                        if (!dttm) return;
                        var dateTimeFormat = sap.ui.core.format.DateFormat.getDateTimeInstance();
                        return dateTimeFormat.format(new Date(dttm));
                    }
                }

            }),
            sortProperty: "dateCreated",
            width:"15rem"
        }));
        this.dataLoadMapTable.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("dateModified"),
            label: new sas.hc.m.Label({text: "{i18N>dataLoadMapTable.dateModified.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: {
                    path: "dataLoadMap>dateModified",
                    formatter: function (dttm) {
                        if (!dttm) return;
                        var dateTimeFormat = sap.ui.core.format.DateFormat.getDateTimeInstance();
                        return dateTimeFormat.format(new Date(dttm));
                    }
                },
                text: {
                    path: "dataLoadMap>dateModified",
                    formatter: function (dttm) {
                        if (!dttm) return;
                        var dateTimeFormat = sap.ui.core.format.DateFormat.getDateTimeInstance();
                        return dateTimeFormat.format(new Date(dttm));
                    }
                }
            }),
            sortProperty: "dateModified",
            width:"15rem"
        }));

        this.dataLoadMapTable.bindRows("dataLoadMap>/");

        var bindingInfo = this.dataLoadMapTable.getBindingInfo("rows");
        if (bindingInfo) {
            var sorters = [
                new sap.ui.model.Sorter("mapName", true),
                new sap.ui.model.Sorter("mapDesc", false)];
            bindingInfo.sorter = sorters;
        }

        var contentPanel = sas.risk.common.widgets.WidgetFactory.buildPanel({
            topPadding: ".25rem",
            content: this.dataLoadMapTable
        });

        return contentPanel;
    },

    _createDataLoadMapDetailTablePanel:function(controller){
        this.dataLoadMapDetailTable = this._createDataLoadMapDetailTable(controller);
        return sas.risk.common.widgets.WidgetFactory.buildPanel({
            topPadding: ".25rem",
            content: this.dataLoadMapDetailTable
        });

    },

    _createDataLoadMapDetailTable:function(controller){
        var table = new sas.hc.ui.table.Table({
            editable: false,
            width: "100%",
            visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Auto,
            minAutoRowCount: 1,
            selectionMode: sas.hc.ui.table.SelectionMode.single,
            selectionBehavior: sas.hc.ui.table.SelectionBehavior.RowOnly,
            showTableOptions: false,
            title: new sas.hc.m.Label({text: "{dataLoadMapDetail>mapName}"}).addStyleClass("largeTableTitle"),
            toolbar: new sas.hc.m.ResponsiveToolbar({
                content: [
                    this._createDataLoadMapDetailTableSearchField(controller)
                ]}
            ),
            noData: new sas.risk.common.widgets.ZeroStateLayout({
                text: "{i18N>dataLoadMapDetailTable.noData.title}"
            })
        }).addStyleClass("stressInnerTitle");

        table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("sourceVarName"),
            label: new sas.hc.m.Label({text: "{i18N>dataLoadMapDetailTable.sourceVariableName.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: "{dataLoadMapDetail>sourceVariableName}",
                text: "{dataLoadMapDetail>sourceVariableName}"
            }),
            sortProperty: "sourceVariableName"
        }));
        table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("dataType"),
            label: new sas.hc.m.Label({text: "{i18N>dataLoadMapDetailTable.dataType.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: "{dataLoadMapDetail>dataType}",
                text: "{dataLoadMapDetail>dataType}"
            }),
            sortProperty: "dataType"
        }));
        table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("targetDimName"),
            label: new sas.hc.m.Label({text: "{i18N>dataLoadMapDetailTable.targetDimName.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: "{dataLoadMapDetail>dimName}",
                text: "{dataLoadMapDetail>dimName}"
            }),
            sortProperty: "dimName"
        }));
        table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("mapping"),
            label: new sas.hc.m.Label({text: "{i18N>dataLoadMapDetailTable.mapping.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: "{dataLoadMapDetail>mapping}",
                text: "{dataLoadMapDetail>mapping}"
            }),
            sortProperty: "mapping"
        }));
        table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("hierarchyName"),
            label: new sas.hc.m.Label({text: "{i18N>dataLoadMapDetailTable.hierarchyName.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: "{dataLoadMapDetail>hierarchyName}",
                text: "{dataLoadMapDetail>hierarchyName}"
            }),
            sortProperty: "hierarchyName"
        }));
        table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("hierarchyVersion"),
            label: new sas.hc.m.Label({text: "{i18N>dataLoadMapDetailTable.hierarchyVersion.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: "{dataLoadMapDetail>hierarchyVersion}",
                text: "{dataLoadMapDetail>hierarchyVersion}"
            }),
            sortProperty: "hierarchyVersion"
        }));
        table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("levelName"),
            label: new sas.hc.m.Label({text: "{i18N>dataLoadMapDetailTable.levelName.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: "{dataLoadMapDetail>levelName}",
                text: "{dataLoadMapDetail>levelName}"
            }),
            sortProperty: "levelName"
        }));
        table.bindRows("dataLoadMapDetail>/");

        return table;

    },

    destroy: function () {
        sap.ui.core.mvc.JSView.prototype.destroy.apply(this, arguments);
    },

    _createDataLoadMapDetailTableSearchField: function (controller) {
        return new sas.hc.m.SearchField({
            id: "dataLoadMapDetailTableSearchFld",
            liveChange: $.proxy(controller.onDataLoadMapDetailTableFilterChanged, controller),
            search: $.proxy(controller.onDataLoadMapDetailTableFilterChanged, controller),
            filterMode: true
        });
    }
});