jQuery.sap.require("sas.hc.ui.commons.wizard.WizardStep");
jQuery.sap.require("sas.hc.ui.table.Table");
jQuery.sap.require("sas.hc.ui.table.TreeTable");
jQuery.sap.require("sas.hc.ui.table.Column");
jQuery.sap.require("sas.hc.m.Button");
jQuery.sap.require("sas.hc.m.Label");
jQuery.sap.require("sas.hc.m.Toolbar");
jQuery.sap.require("sas.hc.m.Text");
jQuery.sap.require("sas.hc.m.List");
jQuery.sap.require("sas.risk.common.widgets.RiskToolbar");
jQuery.sap.require("sas.risk.common.widgets.WidgetFactory");
jQuery.sap.require("sas.risk.common.widgets.RiskPage");
jQuery.sap.require("sas.risk.common.models.RiskJSONModel");
jQuery.sap.require("sas.stress.models.ModelGroupRunConstants");
jQuery.sap.require("sas.stress.wkbench.widget.StressObjStatus");
jQuery.sap.require("sas.stress.wkbench.SASStudioProxy");
jQuery.sap.require("sas.stress.wkbench.StressFixes");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.stress.wkbench.widget.CustomBreadcrumb");
jQuery.sap.require("sas.stress.wkbench.widget.StressIconBar");
jQuery.sap.require("sas.stress.wkbench.widget.RiskWizardComponent");
jQuery.sap.require({ modName: "sas.stress.models.view.execution.ModelGroupRunsCreate", type: "controller" });

sap.ui.jsview("sas.stress.models.view.execution.ModelGroupRunsCreate", (function () {

    var self,
        myController,
        breadcrumb,
        runsTable,
        searchField,
        runButton;

    var itemTemplate = null;
    var entityTree = null,
        _myController = null,
        wizardControl = null,
        step1 = null,
        step2 = null,
        step3 = null,
        step4 = null,
        STEP1_ID = 'sourcePage',
        STEP2_ID = 'mapPage',
        STEP3_ID = 'baPage',
        STEP4_ID = 'summaryPage',
        nameField = null,
        descriptionField = null,
        debugCheckbox = null,
        runTypeList = null,
        workGroupList = null,
        scenariosTree = null,
        modelGroupsTable = null,
        businessAssumptionsTable = null,
        summaryModel = new sas.risk.common.models.RiskJSONModel();

    function createScenariosModel() {
        var scenarioModel = new sas.risk.common.models.RiskJSONModel(sas.stress.models.ModelGroupRunAPI.getScenarios());
        return scenarioModel;
    }

    function createRunTypesModel() {
        var data = {
            "runTypes": [
                {
                    "id": "RunAsyncSerial",
                    "name": getResourceBundle().getText("modelGroupRuns.wizard.step1.runtype.series.txt")
                },
                {
                    "id": "RunAsyncParallel",
                    "name": getResourceBundle().getText("modelGroupRuns.wizard.step1.runtype.parallel.txt")
                }
            ]
        };
        var model = new sas.risk.common.models.RiskJSONModel();
        model.setData(data);
        return model;
    }

    function createWorkGroupsModel() {
        var workGroups = [];
        _.each(sas.stress.wkbench.StressUtil.getRiskWorkGroups(), function (workGroup) {
            workGroups.push({name:workGroup});
        });
        var model = new sas.risk.common.models.RiskJSONModel();
        model.setData({workGroups:workGroups}); // S1344032.. srini
        return model;
    }

    function createEntitiesModel() {
//		var rawItems = _myController.getEntities();
        var rawItems = sas.stress.models.ModelGroupRunAPI.getEntities();
        var items = buildHierarchy(rawItems);
        var model = new sas.risk.common.models.RiskJSONModel();
        model.setData(items);
        return model;
    }

    function buildHierarchy(items) {
        var roots = [],
            childMap = {},
            hierarchyArray = [],
            rootItem,
            rootItemKey,
            itemKey,
            item,
            parentId,
            children;

        for (itemKey in items) {
            item = items[itemKey];
            item.checked = false; // reset checked state
            parentId = item.parentLiKey;
            if (parentId) {
                children = childMap[parentId];
                if (children) {
                    children.push(item);
                } else {
                    children = [];
                    children.push(item);
                    childMap[parentId] = children;
                }
            } else {
                roots.push(item);
            }
        }
        if (roots.length===0){
            var key=(items.length > 0) ? items[0].parentLiKey : 0;
            // a dummy root created to indicate the main root is not found in the response
            //  however a root is required for processing to go further.. this item is never displayed
            //  and only exists if the there are no elements in the roots
            var dummyrootItem={name:"NotTheActualRoot", liKey:key,parentLiKey:null,checked:false};
            roots.push(dummyrootItem);
        }

        // Pull out all the roots and place in hiearachyArray
        for (rootItemKey in roots) {
            wireHierarchyTogether(roots[rootItemKey], childMap);
        }
        if (_.size(childMap)!==0){  // are there any children that are left un-processed
            var keys=_.keys(childMap);
            _.each(keys,function(key){
                roots.push(childMap[key][0]);  //store them as they are into the roots
            });
        }
        var dummyRoot=_.find(roots,function(root){
            // the dummy root
            return "NotTheActualRoot"===root.name;
        });
        if (dummyRoot){ // if the dummy root is found
            var children=[];
            // the children of the dummy root...
            children=dummyRoot.children;
            for (var i=1;i<roots.length;++i){   // assumption is dummy root is first
                children=_.union(children,roots[i]);
            }
            // return all the children.. these will be displayed as top-level items in the tree
            return children;
        }
        return roots;
    }

    function wireHierarchyTogether(currentItem, childMap) {
        var children, child, key, childKey;

        key = currentItem.liKey;
        children = childMap[key];
        if (children) {
            delete childMap[key];
            currentItem.children = children;
            for (childKey in children) {
                wireHierarchyTogether(children[childKey], childMap);
            }
        }
    }

    function createModelGroupsModel() {
        var items = sas.stress.models.ModelGroupRunAPI.getModelGroups();
        var model = new sas.risk.common.models.RiskJSONModel();
        model.setData({ 'modelGroups': items });
        return model;
    }

    function createBusinessAssumptionsModel() {
        var modelGroups = getSelectedModelGroups();
        var data={};
        if(businessAssumptionsTable){
            // the existing model data for the business assumptions table
            data=businessAssumptionsTable.getModel().getData();
        }
        var clonedBusinessAssumptions=[];   // this will be a union of all the assumptions for all selected model groups
        _.each(modelGroups,function(mg){
            // We need a copy of the originals so we can better use binding.

            // 1. get the list of the model groups assumptions
            var bizassumptions=JSON.parse(JSON.stringify(mg.defaultRuntimeBusinessAssumptions));
            // 2. add the model group name to the business assumptions
            _.each(bizassumptions,function(assumption){
                assumption.modelgroupname=mg.name;
            });
            // 3. unionize the existing with the new ones
            Array.prototype.push.apply(clonedBusinessAssumptions,bizassumptions);
        });
        // 4. set up the data
        if (clonedBusinessAssumptions.length>0) {
            data = {
                'businessAssumptions': clonedBusinessAssumptions
            };
        }else{
            data = {'businessAssumptions': []};
        }
        var model = new sas.risk.common.models.RiskJSONModel();
        model.setData(data);
        return model;
    }

// *************** END OF DATA

    /*
     * Builds a single model group run
     * @param name
     * @param description
     * @param modelGroupKey
     * @param runType
     * @param debugFlag
     * @param businessAssumptions
     * @param entityLIKey
     * @param scenarioKey - may be null
     * @returns {{modelGroupKey: *, name: *, description: *, runType: *, debug: *, overriddenAssumptions: *, runTypeStr: *, entityLIKey: *, scenarioKey: *}}
     */
    function createModelGroupRun(name,
                                 description,
                                 modelGroupKey,
                                 runType,
                                 debugFlag,
                                 workGroup,
                                 businessAssumptions,
                                 entityLIKey,
                                 scenarioKey) {
        return {
            'modelGroupKey': modelGroupKey,
            'name': name,
            'description': description,
            'runType': runType,
            'debug': debugFlag,
            'workGroupName': workGroup,
            'overriddenAssumptions': businessAssumptions,
            'runTypeStr': runType,
            'entityLIKey': entityLIKey,
            'scenarioKey': scenarioKey
        };
    }

    /*
     * Builds the list of runs to be sent to the midtier
     * @returns {{runSerially: boolean, runInputs: Array}}
     */
    function buildModelGroupRuns() {

        var name = nameField.getValue().trim();
        var description = descriptionField.getValue().trim();
        var debugFlag = debugCheckbox.getSelected() ? 1 : 0;
        var workGroup = getWorkGroup();
        var entities = getSelectedEntities();
        var scenarios = getSelectedScenarios();
        var modelGroups = getSelectedModelGroups();
        var businessAssumptions = [];
        //if (modelGroups.length === 1) {

        //}

        var modelGroupRuns = [];
        modelGroups.forEach(function (mg) {
            businessAssumptions=[]; //clear the array for the next model group run
            var currentBAs = businessAssumptionsTable.getModel().getProperty('/businessAssumptions');
            if (currentBAs) {
                currentBAs.forEach(function (ba) {
                    if (mg.name === ba.modelgroupname) {
                        var newBA = {
                            'name': ba.name,
                            'value': ba.value,
                            'description': ba.description
                        };
                        businessAssumptions.push(newBA);
                    }
                });
            }
            entities.forEach(function (entity) {
                if (scenarios !== null) {
                    scenarios.forEach(function (scenario) {
                        modelGroupRuns.push(createModelGroupRun(name, description, mg.key, "Standard", debugFlag, workGroup, businessAssumptions, entity.liKey, scenario.scenarioKey));
                    });
                } else {
                    modelGroupRuns.push(createModelGroupRun(name, description, mg.key, "Standard", debugFlag, workGroup, businessAssumptions, entity.liKey, null));
                }
            });
        });
        return {
            'runInputs': modelGroupRuns
        };
    }

    function updateSummaryModel(name, description) {
        var summary = {};
        var rb = getResourceBundle();

        // Name
        summary.name = name.trim();

        // description
        if (!description || description.trim().length === 0) {
            summary.description = '';
        } else {
            summary.description = description.trim();
        }

        // TODO: bind all this together
        // scenarios
        var scenarios = getSelectedScenarios();
        if (!scenarios || scenarios.length === 1) {
            summary.scenariosTitle = rb.getText('modelGroupRuns.summary.single.scenario.title');
        } else {
            summary.scenariosTitle = rb.getText('modelGroupRuns.summary.multiple.scenario.title');
        }
        summary.scenarios = _getNamedObjectsString(scenarios);
        summary.runtype = getRunTypeString();
        summary.workGroup = getWorkGroup();
        // Entities
        var entities = getSelectedEntities();
        if (entities.length === 1) {
            summary.entitiesTitle = rb.getText('modelGroupRuns.summary.single.entity.title');
        } else {
            summary.entitiesTitle = rb.getText('modelGroupRuns.summary.multiple.entity.title');
        }

        if (debugCheckbox.getSelected()) {
            summary.debugStatus = rb.getText('modelGroupRuns.summary.debug.included.txt');
        } else {
            summary.debugStatus = rb.getText('modelGroupRuns.summary.debug.notincluded.txt');
        }

        summary.entities = _getNamedObjectsString(entities);
        summary.modelGroups = _getNamedObjectsString(getSelectedModelGroups());
        summary.businessAssumptions = getBusinessAssumptionsString();
        summaryModel.setData({ 'summary': summary });
    }

    /**
     * Takes a collection of objects that are assumed to have a name attribute and builds a displayable string for them
     * @param {Array} collection The collection to iterate over,may be null
     * @returns {string} list of names
     * @private
     */
    function _getNamedObjectsString(collection) {
        var resultString = "";
        if (collection !== null) {
            collection.forEach(function (value) {
                if (resultString.length === 0) {
                    resultString = value.name;
                } else {
                    resultString = resultString + ', ' + value.name;
                }
            });
        }
        if (resultString.length === 0) {
            return getResourceBundle().getText("modelGroupRuns.wizard.nonamedobjects.txt");
        }
        return resultString;
    }

    /**
     * Returns an array of the selected scenarios
     * @returns {Array} selected scenarios
     */
    function getSelectedScenarios() {
        var selectedScenarios = [];
        var model = scenariosTree.getModel();

        $.each(scenariosTree.getSelectedIndices(), function (index, selectedIndex) {
            var path = scenariosTree.getContextByIndex(selectedIndex).getPath();
            var scenario = model.getProperty(path);
            selectedScenarios.push(scenario);
        });
        if (selectedScenarios.length === 0) {
            return null;
        }
        return selectedScenarios;
    }

    /**
     * Returns a String representation of the selected scenarios.
     * @returns {string} scenario string
     */
    function getSelectedScenariosString() {
        var selectedScenarios = getSelectedScenarios();
        return _getNamedObjectsString(selectedScenarios);
    }

    /**
     * Returns an array of the selected entities
     * @returns {Array} selected entities
     */
    function getSelectedEntities() {
        var rootEntityNodes = entityTree.getModel().getProperty('/');
        return _getSelectedEntitiesFromRoots(rootEntityNodes, null);
    }

    /**
     * Internal method to recursively walk the entity list and get the selected entities
     * @param {Array} nodes list of nodes
     * @param {Array} selectedNodes  list of selected
     * @returns {*} selectedNodes
     * @private
     */
    function _getSelectedEntitiesFromRoots(nodes, selectedNodes) {
        if (!selectedNodes) {
            selectedNodes = [];
        }
        if (!nodes) {
            return selectedNodes;
        }
        nodes.forEach(function (node) {
            if (node.checked) {
                selectedNodes.push(node);
            }
            _getSelectedEntitiesFromRoots(node.children, selectedNodes);
        });
        return selectedNodes;
    }

    /**
     * Returns an array of the selected model groups
     * @returns {Array} selected model groups
     */
    function getSelectedModelGroups() {
        var selectedModelGroups = [];
        var selectedIndices = modelGroupsTable.getSelectedIndices();
        for (var i = 0; i < selectedIndices.length; i++) {
            var context = modelGroupsTable.getContextByIndex(selectedIndices[i]);
            if (context) {
                var modelGroup = modelGroupsTable.getModel().getProperty(context.getPath());
                selectedModelGroups.push(modelGroup);
            }
        }
        return selectedModelGroups;
    }

    function getBusinessAssumptionsTable(){
        return businessAssumptionsTable;
    }

    /**
     * Returns a String representation of the selected runtype.
     * @returns {string}  item text
     */
    function getRunTypeString() {
        var item = runTypeList.getSelectedItem();
        if (item) {
            return item.getProperty('text');
        }
        return '';
    }

    /**
     * Returns a the selected workGroup.
     * @returns {string}  item text
     */
    function getWorkGroup() {
        var item = workGroupList.getSelectedItem();
        if (item) {
            return item.getProperty('key');
        }
        return '';
    }

    /**
     * Returns a String representation of the id of the selected runtype, suitable to
     * be sent to the midtier.
     * @returns {string}  run type id
     */
    function getRunTypeId() {
        var item = runTypeList.getSelectedItem();
        if (item) {
            return item.getKey();
        }
        return '';
    }

    function getBusinessAssumptionsString() {
        // Get the original list of business assumptions
        var rb= getResourceBundle();
        var modelGroups = getSelectedModelGroups();
        var data = {};
        /*if (modelGroups.length !== 1) {
         return getResourceBundle().getText('modelGroupRuns.summary.businessassumptions.multiplemodelgroups.txt');
         }*/

        // get the current list of business assumptions
        var currentBusinessAssumptions = businessAssumptionsTable.getModel().getProperty('/businessAssumptions');
        var stringToReturn = '';
        _.each(modelGroups,function(mg){
            var filteredassumptionsByGroup=_.filter(currentBusinessAssumptions,function(ba){
                    return ba.modelgroupname===mg.name;
                })||[];
            if (filteredassumptionsByGroup.length > 0) {
                if (stringToReturn.length===0) {
                    stringToReturn = mg.name + "\n";
                }else{
                    stringToReturn += "\n\n"+mg.name + "\n";
                }
                for (var i = 0; i < filteredassumptionsByGroup.length; i++) {
                    var newAssumption = filteredassumptionsByGroup[i];

                    switch (i){
                        case 0:
                            /*stringToReturn+="\u00A0 \u0009 \u00A0";*/
                            break;
                        default:
                            stringToReturn+="\n";   // add comma
                            break;
                    }
                    stringToReturn+="\u00A0 \u0009 \u00A0";
                    stringToReturn+=newAssumption.name + ": " + newAssumption.value;
                }
            }
        });
        /*for (var i = 0; i < currentBusinessAssumptions.length; i++) {
         var newAssumption = currentBusinessAssumptions[i];
         if (stringToReturn.length === 0) {
         stringToReturn =newAssumption.name + ":" + newAssumption.value + " ("+ newAssumption.modelgroupname+ ")";
         } else {
         stringToReturn += ', ' + newAssumption.name + ":" + newAssumption.value + " ("+ newAssumption.modelgroupname+ ")";
         }
         }*/
        // TODO: LEAVE THIS HERE, should we want to provide a list of the changed business assumptions instead of all
        // Anyone that is different add to the list
        //var stringToReturn = '';
        //var originalBusinessAssumptions = modelGroups[0].defaultRuntimeBusinessAssumptions;
        //for (var i=0; i<originalBusinessAssumptions.length; i++) {
        //    var originalAssumption = originalBusinessAssumptions[i];
        //    var newAssumption = currentBusinessAssumptions[i];
        //    if (originalAssumption.value != newAssumption.value) {
        //        if (stringToReturn.length == 0) {
        //            stringToReturn = originalAssumption.name+":"+newAssumption.value;
        //        }
        //        else {
        //            stringToReturn += ', ' + originalAssumption.name+":"+newAssumption.value;
        //        }
        //    }
        //}
        return stringToReturn;
    }

    /**
     * Isolates code to update the steps of the wizard control
     * @param {boolean} isCurrentStepValid - Is the step we are looking at valid
     * @param {String} currentStepId Id of the current step
     * @param {Object} nextStep Reference to the next step
     * @param {Object} stepStatuses A map of the complete and incomplete steps, will be created if null
     * @returns {*} The current map of complete and incomplete steps
     */
    function updateWizardSteps(isCurrentStepValid, currentStepId, nextStep, stepStatuses) {
        if (!stepStatuses) {
            stepStatuses = { completedSteps: [], incompleteSteps: [] };
        }

        if (isCurrentStepValid) {
            stepStatuses.completedSteps.push(currentStepId);
            if (!nextStep.getEnabled()) {
                wizardControl.enableNextStep();
            }
        } else {
            // Only mark step 1 as incomplete if the user has "opened-up" step 2 (by making step 1
            // valid at least once).
            if (nextStep.getEnabled()) {
                stepStatuses.incompleteSteps.push(currentStepId);
            }
        }
        return stepStatuses;
    }

    function _resetModel() {
        nameField.setValue("");
        nameField.setValueState(sap.ui.core.ValueState.None);

        descriptionField.setValue("");

        scenariosTree.setModel(createScenariosModel());
        scenariosTree.setSelectedIndex(-1);

        runTypeList.setSelectedItem("");

        // To reset the entityTree, we want to clear the selections like normal but we also
        // need to uncheck the checkboxes and the easiest way I found to do that was to just recreate
        // the model and reset it on the entity tree.
        entityTree.setModel(createEntitiesModel());
        entityTree.clearSelection();
        entityTree.bindRows("/");

        modelGroupsTable.clearSelection();
        modelGroupsTable.setModel(createModelGroupsModel());
        wizardControl.setCompleteSteps([]);
        wizardControl.setIncompleteSteps([STEP2_ID, STEP3_ID, STEP4_ID]);
        wizardControl.disableSteps([STEP2_ID, STEP3_ID, STEP4_ID]);
        wizardControl.setSelectedStepId(STEP1_ID);
        step1.setReviewed(false);
        step1.setState(sas.hc.ui.commons.wizard.WizardStep.State.ENABLED);
        wizardControl.setEnableSave(false);
        wizardControl.setEnableFinish(false);
        _myController.setDirty(false);
    }

    function updateAllowedScenariosAndModelGroups() {
        var selectedScenarios = getSelectedScenarios();
        var selectedPeriodTypes = "";
        if (selectedScenarios) {
            _.each(selectedScenarios, function (scenario) {
                selectedPeriodTypes = selectedPeriodTypes + scenario.periodType.toUpperCase();
            });
        }

        console.log('Selected Period Types: ' + selectedPeriodTypes);

        // Loop through all model groups and update the model accordingly to those that can be in the list
        var filteredModelGroups = sas.stress.models.ModelGroupRunAPI.getModelGroups();
        if (selectedPeriodTypes.length > 0) {
            filteredModelGroups = _.filter(sas.stress.models.ModelGroupRunAPI.getModelGroups(), function (modelGroup) {
                var periodType = modelGroup.periodType.toUpperCase();
                return (selectedPeriodTypes.indexOf(periodType) !== -1);
            });
        }

        var existingSelections=modelGroupsTable.getSelectedIndices();   // fixes for S1288352.. the currently selected modelgroups
        var model = new sas.risk.common.models.RiskJSONModel();
        model.setData({ 'modelGroups': filteredModelGroups });
        modelGroupsTable.setModel(model);

        // fixes for S1288352.. if there are selections, then set the selection interval
        if (existingSelections&&existingSelections.length>0){
            modelGroupsTable.setSelectionInterval(existingSelections[0],existingSelections[length-1]);
        }
    }

    /**
     * Does the actual work of validating the steps of the wizard.
     * @private
     */
    function _validateSteps() {

        var runName = nameField.getValue().trim();

        var runDescription = descriptionField.getValue().trim();

        var selectedScenarios = getSelectedScenarios();

        var selectedRunType = runTypeList.getSelectedItem();

        var selectedEntities = getSelectedEntities();

        var step1Valid = (runName.length > 0) && (selectedRunType !== undefined) && (selectedEntities.length > 0);
        var stepStatuses = updateWizardSteps(step1Valid, STEP1_ID, step2, null);

        var selectedModelGroups = getSelectedModelGroups();
        var step2Valid = selectedModelGroups.length > 0;
        stepStatuses = updateWizardSteps(step2Valid, STEP2_ID, step3, stepStatuses);

        // If step2 is valid then so is step3 and therefore step 4
        if (step2Valid) {
            wizardControl.enableSteps(STEP4_ID);
        }/*else{
            wizardControl.disableSteps(STEP3_ID);
            wizardControl.disableSteps(STEP4_ID);
        }*/

        var selectedStep = wizardControl.getSelectedStep();
        var selectedStepId = selectedStep.getProperty("stepId");
        if (selectedStepId === STEP4_ID) {
            updateWizardSteps(true, STEP3_ID, step4, stepStatuses);
        }

        wizardControl.setCompleteSteps(stepStatuses.completedSteps);
        wizardControl.setIncompleteSteps(stepStatuses.incompleteSteps);
        updateSummaryModel(runName, runDescription);
        var allValid = step1Valid && step2Valid;
        wizardControl.setEnableSave(allValid);
        wizardControl.setEnableFinish(allValid);

        _myController.setDirty(true);
    }

    /**
     * Builds the field to collect the model group run name
     *
     * @returns {sas.hc.m.Input} name field
     */
    function buildNameField() {
        nameField = new sas.hc.m.Input({
            value: "",
            disableValidation: false,
            placeholder: getResourceBundle().getText('modelGroupRuns.wizard.step1.name.placeHolder.txt'),
            required: true,
            liveChange: function (event) {
                var runName = nameField.getValue().trim();
                if (runName.length > 0) {
                    nameField.setValueState(sap.ui.core.ValueState.None);
                } else {
                    nameField.setValueState(sap.ui.core.ValueState.Error);
                }
                _validateSteps();
            }
        });
        return nameField;
    }

    /**
     * Builds the field to collect the model group run description
     *
     * @returns {sas.hc.m.Input} description field
     */
    function buildDescriptionField() {
        descriptionField = new sas.hc.m.TextArea({
            value: "",
            maxLength: 1024,
            rows:1,
            autoExpandable:true,
            placeholder: getResourceBundle().getText('modelGroupRuns.wizard.step1.description.placeHolder.txt'),
            change: _validateSteps
        });
        return descriptionField;
    }

    /**
     * Builds the list to collect the scenario(s) for the run
     *
     * @returns {sas.hc.ui.layout.VerticalLayout} scenario list layout
     */
    function buildScenarioList() {

        scenariosTree = new sas.hc.ui.table.Table({
            visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Auto,
            columnHeaderVisible: false,
            title: new sas.hc.m.Label({
                text: getResourceBundle().getText('modelGroupRuns.wizard.step1.scenario.title'),
                width: "100%"
            }),
            selectionMode: sas.hc.ui.table.SelectionMode.MultiToggle,
            rowSelectionChange: function () {
                updateAllowedScenariosAndModelGroups();
                _validateSteps();
            }
        });

        scenariosTree.addColumn(new sas.hc.ui.table.Column({
            template: new sas.hc.m.Text({ wrapping: false }).bindProperty("text", "name"),
            sortProperty: "name"
        }));

        scenariosTree.setModel(createScenariosModel());
        scenariosTree.bindRows("/");

        return scenariosTree.addStyleClass("sapUiMediumMarginRight");
    }

    /**
     * Builds the list to collect the Run Type for the run
     *
     * @returns {sas.hc.ui.layout.VerticalLayout} run type list layout
     */
    function buildRunTypeList() {
        /*var runTypeTitle = new sas.hc.m.Label({
            text: getResourceBundle().getText('modelGroupRuns.wizard.step1.runtype.title'),
            required: true
        }).addStyleClass("selectListLabel");*/
        runTypeList = new sas.hc.m.Select('runTypeList', {
            change: _validateSteps/*,
            maxWidth: "10rem"*/
        });
        var runTypeListTemplate = new sas.hc.ui.core.Item({
            key: '{id}',
            text: '{name}'
        });
        runTypeList.setModel(createRunTypesModel());
        runTypeList.bindAggregation("items", "/runTypes", runTypeListTemplate);

        return runTypeList;
    }

    /**
     * Builds the list to select work group for the run
     *
     * @returns {sas.hc.ui.layout.VerticalLayout} run type list layout
     */
    function buildWorkGroupList() {
        /*var workGroupTitle = new sas.hc.m.Label({
            text: getResourceBundle().getText('modelGroupRuns.wizard.step1.workGroup.title')
        }).addStyleClass("selectListLabel");*/
        workGroupList = new sas.hc.m.Select('workGroupList', {
            enableUnselected: true,
            change: _validateSteps,
            maxWidth: "100%"
        });
        var workGroupListTemplate = new sas.hc.ui.core.Item({
            key: '{name}',  // S1344032.. srini
            text: '{name}'  // S1344032.. srini
        });
        workGroupList.setModel(createWorkGroupsModel());
        workGroupList.bindAggregation("items", "/workGroups", workGroupListTemplate);

        /*var layout = new sas.hc.ui.layout.VerticalLayout({
            width: "100%",
            content: [workGroupTitle, workGroupList]
        });
        layout.addStyleClass("modelGroupRunRunTypePanel");
        return layout;*/

        return workGroupList;
    }

    /**
     * Builds the tree to collect the entities for the run
     *
     * @returns {sas.hc.ui.layout.VerticalLayout}   entities tree
     */
    function buildEntityTree() {

        var entityTitle = new sas.hc.m.Label({
            text: getResourceBundle().getText('modelGroupRuns.wizard.step1.entities.title'),
            required: true,
            width: "100%"
        });

        entityTree = new sas.hc.ui.table.TreeTable({
            visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Auto,
            columnHeaderVisible: false,
            title: entityTitle,
            columns: [new sas.hc.ui.table.Column({
                //label : "Select or deselect item for display:",
                //width: "10%",
                template: new sas.hc.m.CheckBox({
                    text: '{name}',
                    selected: '{checked}',
                    select: function (e) {
                        var cxt = e.getSource().getBindingContext();
                        var path = cxt.getPath();
//						validateChild(entityTree.getModel(), path);
                        path = path.substring(0, path.lastIndexOf('/'));
                        validateParent(entityTree.getModel(), path);
                        _validateSteps();
                    }
                })
            })
            ],
            //selectionMode: sas.hc.ui.table.SelectionMode.Single,
            cellClick: function () {
//                    console.error("cell clicked");
            },
            rowSelectionChange: function () {
//                    console.error("row selected changed");
            },
            expandFirstLevel: true,
            // layoutData:new sas.hc.m.FlexItemData({growFactor: 1000})
        });
        entityTree.setModel(createEntitiesModel());
        entityTree.bindRows("/");

        return entityTree;
    }

    function createCellWithContent(content, colSpan) {
        var cell = new sas.hc.ui.commons.layout.MatrixLayoutCell({
            vAlign: sap.ui.commons.layout.VAlign.Top,
            colSpan: colSpan
        });
        cell.addContent(content);
        return cell;
    }

    function createSourceInformationContentV2() {

        debugCheckbox=  new sas.hc.m.CheckBox({
            text: getResourceBundle().getText('modelGroupRuns.wizard.step1.debug.title'),
            selected: false
        }).addStyleClass("formElementWithoutLabel");

        // create a form
        var nameAndDescForm = new sas.hc.ui.layout.form.Form({
            layout: new sas.hc.ui.layout.form.ResponsiveGridLayout({
                columnsM: 3,
                columnsL: 3,
                labelSpanS: 12,
                labelSpanM: 12,
                labelSpanL: 12
            }),
            formContainers: [
                new sas.hc.ui.layout.form.FormContainer({
                    formElements: [
                        new sas.hc.ui.layout.form.FormElement({
                            label: new sas.hc.m.Label({
                                text: getResourceBundle().getText('modelGroupRuns.wizard.step1.name.title'),
                                required:true
                            }),
                            fields: [buildNameField()]
                        }),
                        new sas.hc.ui.layout.form.FormElement({
                            label: new sas.hc.m.Label({
                                text: getResourceBundle().getText('modelGroupRuns.wizard.step1.workGroup.title')
                            }).addStyleClass("selectListLabel"),
                            fields: [buildWorkGroupList()]
                        })
                    ]
                }),
                new sas.hc.ui.layout.form.FormContainer({
                    formElements: [
                        new sas.hc.ui.layout.form.FormElement({
                            label: new sas.hc.m.Label({
                                text: getResourceBundle().getText('modelGroupRuns.wizard.step1.description.title')
                            }),
                            fields: [buildDescriptionField()]
                        }),
                        new sas.hc.ui.layout.form.FormElement({
                            fields: [debugCheckbox]
                        })
                    ]
                }),
                new sas.hc.ui.layout.form.FormContainer({
                    formElements: [
                        new sas.hc.ui.layout.form.FormElement({
                            label: new sas.hc.m.Label({
                                text: getResourceBundle().getText('modelGroupRuns.wizard.step1.runtype.title'),
                                required: true
                            }),
                            fields: [buildRunTypeList()]
                        })
                    ]
                })
            ]
        });

        return new sas.hc.ui.layout.FixFlex({
            fixContent: [
                nameAndDescForm
            ],
            flexContent: [
                new sas.hc.ui.layout.Splitter({
                    contentAreas: [
                        buildScenarioList(),
                        buildEntityTree()
                    ],
                    width: "100%"
                })
            ]
        });

    }

    function setChildState(obj, state) {
        getChildren(obj).forEach(function (x) {
            x.checked = state;
            setChildState(x, state);
        });
    }

    function validateChild(model, path) {
        var cur = model.getProperty(path);
        setChildState(cur, cur.checked);
    }

    function getChildren(obj) {
        var children = [];
        if (obj.children) {
            children = obj.children;
        }
        return children;
    }

    function validateParent(model, path) {
        if (path === '') {
            return;
        }
        var obj = model.getProperty(path);
//            var state = false;
        var children = getChildren(obj);

        var selectedCount = children.filter(function (x) {
            return x.checked === true;
        }).length;

        if (selectedCount === children.length) {
            obj.checked = true;
        } else {
            var unselectedCount = children.filter(function (x) {
                return x.checked === false;
            }).length;

            if (unselectedCount) {
                obj.checked = false;
            }
        }

        model.setProperty(path, obj);
        path = path.substring(0, path.lastIndexOf('/children'));
        if (path !== '') {
            validateParent(model, path);
        }
    }

    function createModelGroupSelectionContent() {
        var rb = getResourceBundle();
        var table = new sas.hc.ui.table.Table("modelGroupMapTable", {
            width: "100%",
            editable: false,
            title: new sas.hc.m.Label({
                text: rb.getText("modelGroupRuns.modelGroupsTable.title"),
                required: true
            }),
            showTableOptions: false,
            visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Auto,
            selectionBehavior: sas.hc.ui.table.SelectionBehavior.RowSelector,
            selectionMode: sas.hc.ui.table.SelectionMode.MultiToggle,
            showSelectionControls: true,
            minAutoRowCount: 1,	// S1196469,
            noData: new sas.risk.common.widgets.ZeroStateLayout({
                icon: sas.icons.HC.ZEROSTATEALERT,
                title: rb.getText("modelGroupRuns.modelGroupsTable.nodata.title"),
                text: rb.getText("modelGroupRuns.modelGroupsTable.nodata.txt")
            }),
            //cellClick:$.proxy(controller.onCellClick,controller),
            rowSelectionChange: function () {
                businessAssumptionsTable.setModel(createBusinessAssumptionsModel());
                businessAssumptionsTable.setNoData(getZeroDataControlForBusinessAssumptions());

                _validateSteps();
            }
        });

        var idView = new sas.hc.m.Text({ wrapping: false, tooltip: "{id}" }).bindProperty("text", "key");
        table.addColumn(new sas.hc.ui.table.Column({
            name: "id",
            label: new sas.hc.m.Label({
                text: rb.getText("modelGroupRuns.ModelGroupTable.column.id.header.title")
            }),
            template: idView,
            sortProperty: "id",
            //width: "15rem",
            visible: false
        }));

        var nameView = new sas.hc.m.Text({ wrapping: false, tooltip: "{name}" }).bindProperty("text", "name");
        table.addColumn(new sas.hc.ui.table.Column({
            name: "name",
            label: new sas.hc.m.Label({
                text: rb.getText("modelGroupRuns.ModelGroupTable.column.name.header.title")
            }),
            template: nameView,
            sortProperty: "name",
            width: "15rem"
        }));
        var versionView = new sas.hc.m.Text({wrapping:false,tooltip:"{version}"}).bindProperty("text", "version");
        table.addColumn(new sas.hc.ui.table.Column({
            name:"version",
            label:new sas.hc.m.Label({
                text: rb.getText("modelGroupRuns.ModelGroupTable.column.version.header.title")
            }),
            template: versionView,
            sortProperty:"version",
            width:"15rem"
        }));
        var intervalView = new sas.hc.m.Text({
            wrapping: false,
            tooltip: {
                path: "periodType",
                formatter: sas.stress.models.ModelGroupRunUtils.intervalFormatter
            },
            text: {
                path: "periodType",
                formatter: sas.stress.models.ModelGroupRunUtils.intervalFormatter
            }
        });
        table.addColumn(new sas.hc.ui.table.Column({
            name: "periodType",
            label: new sas.hc.m.Label({
                text: rb.getText("modelGroupRuns.ModelGroupTable.column.interval.header.title")
            }),
            template: intervalView,
            sortProperty: "periodType",
            width: "10rem"
        }));

        var descriptionView = new sas.hc.m.Text({
            wrapping: false,
            tooltip: "{description}"
        }).bindProperty("text", "description");
        table.addColumn(new sas.hc.ui.table.Column({
            name: "description",
            label: new sas.hc.m.Label({
                text: rb.getText("modelGroupRuns.ModelGroupTable.column.description.header.title")
            }),
            template: descriptionView,
            sortProperty: "description"
            //width: "50%"
        }));

        // TODO: Add in a future version
        // var executionTypeView = new sas.hc.m.Text({wrapping:false,tooltip:"{executionType}"}).bindProperty("text", "executionType");
        // table.addColumn(new sas.hc.ui.table.Column({
        // 	name:"executionType",
        // 	label:new sas.hc.m.Label({
        // 		text: rb.getText("modelGroupRuns.ModelGroupTable.column.executionType.header.title")
        // 	}),
        // 	template: executionTypeView,
        // 	sortProperty:"executionType",
        // 	width:"15%"
        // }));
        //

        table.bindRows("/modelGroups");
        table.setModel(createModelGroupsModel());

        modelGroupsTable = table;
        return modelGroupsTable;
    }

    function getZeroDataControlForBusinessAssumptions() {
        var rb = getResourceBundle();
        var returnText = "";
        var selectedModelGroups = getSelectedModelGroups();
        if (selectedModelGroups) {
            switch (selectedModelGroups.length) {
                case 0:
                    returnText = "";
                    break;
                case 1:
                    returnText = rb.getText("modelGroupRuns.BusinessAssumptionsTable.nodata.singlemg.txt");
                    break;
                /*default:
                 returnText = rb.getText("modelGroupRuns.BusinessAssumptionsTable.nodata.multiplemg.txt");*/
            }
        }

        return new sas.risk.common.widgets.ZeroStateLayout({
            icon: sas.icons.HC.ZEROSTATEZERO,
            title: rb.getText("modelGroupRuns.BusinessAssumptionsTable.nodata.title"),
            text: returnText
        });
    }

    function createBusinessAssumptionsContent() {
        var rb = getResourceBundle();
        var table = new sas.hc.ui.table.Table({
            id: "businessassumptions-table",
            width: "100%",
            editable: true,
            title: new sas.hc.m.Label({
                text: rb.getText("modelGroupRuns.businessAssumptionsTable.title"),
                required: false
            }),
            showTableOptions: false,
            visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Auto,
            selectionBehavior: sas.hc.ui.table.SelectionBehavior.RowSelector,
            selectionMode: sas.hc.ui.table.SelectionMode.MultiToggle,
            showSelectionControls: false,
            minAutoRowCount: 1,	// S1196469,
            noData: getZeroDataControlForBusinessAssumptions()
//                cellClick:$.proxy(controller.onCellClick,controller),
//                rowSelectionChange:$.proxy(controller.onRowSelectionChange,controller),
        });

        var nameView = new sas.hc.m.Text({ wrapping: false, tooltip: "{name}" }).bindProperty("text", "name");
        table.addColumn(new sas.hc.ui.table.Column({
            name: "name",
            label: new sas.hc.m.Label({ text: rb.getText("modelGroupRuns.BusinessAssumptionsTable.column.name.header.title") }),
            template: nameView,
            sortProperty: "name",
            width: "15rem"
        }));
        var mgNameView = new sas.hc.m.Text({ wrapping: false, tooltip: "{modelgroupname}" }).bindProperty("text", "modelgroupname");
        table.addColumn(new sas.hc.ui.table.Column({
            name: "modelgroupname",
            label: new sas.hc.m.Label({ text: rb.getText("modelGroupRuns.BusinessAssumptionsTable.column.model.group.name.header.title") }),
            template: mgNameView,
            sortProperty: "modelgroupname",
            width: "15rem"
        }));

        var valueView = new sas.hc.m.Input({
            type: sap.m.InputType.Text,
            placeholder: rb.getText("modelGroupRuns.BusinessAssumptionsTable.column.value.placeholder.txt"),
            value: "{value}"
        });

        table.addColumn(new sas.hc.ui.table.Column({
            name: "value",
            label: new sas.hc.m.Label({ text: rb.getText("modelGroupRuns.BusinessAssumptionsTable.column.value.header.title") }),
            template: valueView,
            sortProperty: "value",
            width: "15rem"
        }));

        var descriptionView = new sas.hc.m.Text({
            wrapping: false,
            tooltip: "{description}"
        }).bindProperty("text", "description");
        table.addColumn(new sas.hc.ui.table.Column({
            name: "description",
            label: new sas.hc.m.Label({ text: rb.getText("modelGroupRuns.BusinessAssumptionsTable.column.description.header.title") }),
            template: descriptionView,
            sortProperty: "description"
            //width: "50%"
        }));

        table.bindRows("/businessAssumptions");
        table.setModel(createBusinessAssumptionsModel());
        businessAssumptionsTable = table;

        return table;
    }

    function createSummaryContent() {
        var contentToShow = [];
        var UTILS = sas.stress.models.ModelGroupRunUtils;
        // Source info section
        contentToShow.push(UTILS.createSectionTitle('modelGroupRuns.summary.header.title'));
        contentToShow.push(UTILS.createNameValueRow('modelGroupRuns.summary.name.title', '/summary/name', summaryModel));
        contentToShow.push(UTILS.createNameValueRow('modelGroupRuns.summary.description.title', '/summary/description', summaryModel));
        contentToShow.push(UTILS.createBoundNameValueRow("/summary/scenariosTitle", "/summary/scenarios", summaryModel));
        contentToShow.push(UTILS.createNameValueRow('modelGroupRuns.summary.runtype.title', '/summary/runtype', summaryModel));
        contentToShow.push(UTILS.createNameValueRow('modelGroupRuns.summary.debug.title', '/summary/debugStatus', summaryModel));
        contentToShow.push(UTILS.createNameValueRow('modelGroupRuns.summary.workGroup.title', '/summary/workGroup', summaryModel));
        contentToShow.push(UTILS.createBoundNameValueRow("/summary/entitiesTitle", "/summary/entities", summaryModel));

        // Model Group section
        contentToShow.push(UTILS.createSectionTitle('modelGroupRuns.summary.modelgroups.header.title'));
        contentToShow.push(UTILS.createBoundValueOnlyRow("/summary/modelGroups", summaryModel));

        // Business Assumptions
        contentToShow.push(UTILS.createSectionTitle('modelGroupRuns.summary.businessassumptions.header.title'));
        contentToShow.push(UTILS.createBoundValueOnlyRow("/summary/businessAssumptions", summaryModel));

        // Final layout
        var layout = new sas.hc.ui.layout.VerticalLayout({
            content: contentToShow
        });
        layout.addStyleClass('modelGroupRunWizardSummarySection');
        return layout;
    }

    function getResourceBundle() {
        return sas.stress.models.ModelGroupRunUtils.getResourceBundle();
    }

    return {
        getControllerName: function () {
            return "sas.stress.models.view.execution.ModelGroupRunsCreate";
        },

        createContent: function (controller) {

            self = this;
            _myController = controller;
            var rb = getResourceBundle();

            this.setHeight("100%");

            var step1Content = createSourceInformationContentV2();
            step1 = new sas.hc.ui.commons.wizard.WizardStep({
                content: [step1Content],
                stepId: STEP1_ID,
                stepTitle: rb.getText("modelGroupRuns.wizard.step1.title"),
                stepNumber: 1,
                nextStepId: STEP2_ID,
                previousStepId: '-1'
            });

            // Step 2 - Model Group Selection Table
            var step2Content = createModelGroupSelectionContent();
            step2 = new sas.hc.ui.commons.wizard.WizardStep({
                content: [step2Content],
                stepId: STEP2_ID,
                stepTitle: rb.getText("modelGroupRuns.wizard.step2.title"),
                stepNumber: 2,
                nextStepId: STEP3_ID,
                previousStepId: STEP1_ID,
                enabled: false
            });

            // Step 3 - Modify Business Assumptions Table
            var step3Content = createBusinessAssumptionsContent();
            step3 = new sas.hc.ui.commons.wizard.WizardStep({
                content: [step3Content],
                stepId: STEP3_ID,
                stepTitle: rb.getText("modelGroupRuns.wizard.step3.title"),
                stepNumber: 3,
                nextStepId: STEP4_ID,
                previousStepId: STEP2_ID,
                enabled: false
            });

            var step4Content = createSummaryContent();
            step4 = new sas.hc.ui.commons.wizard.WizardStep({
                content: [step4Content],
                stepId: STEP4_ID,
                stepTitle: rb.getText("modelGroupRuns.wizard.step4.title"),
                stepNumber: 4,
                nextStepId: '-1',
                previousStepId: STEP3_ID,
                enabled: false
            });

            wizardControl = new sas.stress.wkbench.widget.RiskWizardComponent({
                name: "",
                numberOfSteps: 4,
                finishLabel: rb.getText("modelGroupRuns.wizard.button.finish.txt"),
                closeLabel: rb.getText("modelGroupRuns.wizard.button.cancel.txt"),
                showBackButton: false,
                showSaveButton: true,
                showStepNumbers: false,
                showMOfNSteps: false,
                stepChanged: _validateSteps,
                steps: [step1, step2, step3, step4]
            }).addStyleClass("modelGroupWizard");

            wizardControl.attachClose(function (oEvent) {
                _myController.stopNavChecker(); //S1265212
                sas.stress.wkbench.ViewManager.loadModelGroupRuns(sas.stress.wkbench.model.Project.getCurrentProject());
            });

            wizardControl.attachSave(function (oEvent) {
                var projectId = _myController.projectId;
                var modelGroupRuns = buildModelGroupRuns();
                modelGroupRuns.runMode = "DoNotRun";
                if (self.getBusy() === false) { // This prevents double clicks causing multiple executions
                    self.setBusy(true);
                    sas.stress.models.ModelGroupRunAPI.executeModelGroupRuns(projectId, modelGroupRuns, getWorkGroup())
                        .done(function (data) {
                            _myController.stopNavChecker();    //S1265212
                            sas.stress.wkbench.ViewManager.loadModelGroupRuns(sas.stress.wkbench.model.Project.getCurrentProject());
                        })
                        .fail(function (jqXHR) {
                            _myController.onServerFailure(jqXHR);
                        })
                        .always(function () {
                            self.setBusy(false);
                        });
                }
            });

            wizardControl.attachFinish(function (oEvent) {
                var projectId = _myController.projectId;
                var modelGroupRuns = buildModelGroupRuns();
                modelGroupRuns.runMode = getRunTypeId();
                if (self.getBusy() === false) {  // This prevents double clicks causing multiple executions
                    self.setBusy(true);
                    sas.stress.models.ModelGroupRunAPI.executeModelGroupRuns(projectId, modelGroupRuns, getWorkGroup())
                        .done(function (data) {
                            _myController.stopNavChecker();    //S1265212
                            sas.stress.wkbench.ViewManager.loadModelGroupRuns(sas.stress.wkbench.model.Project.getCurrentProject());
                        })
                        .fail(function (jqXHR) {
                            _myController.onServerFailure(jqXHR);
                        })
                        .always(function () {
                            self.setBusy(false);
                        });
                }
            });

            wizardControl.setSaveLabel(rb.getText("modelGroupRuns.wizard.button.save.txt"));

            this.breadCrumb = new sas.stress.wkbench.widget.CustomBreadcrumb({
                itemSelected: $.proxy(controller.onBreadCrumbClick, controller),
                customData: new sap.ui.core.CustomData({key: "size", value: "3rem"})
            }).addStyleClass("innerBreadCrumb");

            this.viewLayout = new sas.risk.common.widgets.RiskPage({
                mainContent: sas.risk.common.widgets.WidgetFactory.buildPanel(
                    {
                        type: sas.risk.common.widgets.WidgetFactory.OBJECT,
                        top: new sas.risk.common.widgets.RiskToolbar({
                            homeFunction: function () {
                                sas.stress.wkbench.ViewManager.loadProjectManager();
                            },
                            title: new sas.hc.m.Label({ text: sas.stress.wkbench.model.Project.getCurrentProject().getName()}),
                            headerContent: [
                                new sas.hc.m.Label({
                                    text: sas.stress.wkbench.StressUtil.getBundle().getText("iteration.label.txt")
                                }).addStyleClass("toolbarLabel"),
                                new sas.hc.m.Label({
                                    text: "{/currentIterationNo}"
                                }).addStyleClass("toolbarValue"),
                                new sas.hc.m.Label({
                                    text: sas.stress.wkbench.StressUtil.getBundle().getText("workbench.toolbar.asOfDate.label")
                                }).addStyleClass("toolbarLabel"),
                                new sas.hc.m.Label({
                                    text: {
                                        path: "/asOfDate",
                                        formatter: function (asOfDate) {
                                            var dateFormat = new sap.ui.model.type.Date({source: {pattern: "yyyy-MM-dd"}}, {target: {pattern: "MMM dd, yyyy"}}, asOfDate);
                                            return dateFormat.formatValue(asOfDate, "string");
                                        }
                                    }
                                }).addStyleClass("toolbarValue"),
                                sas.stress.notification.NotificationUtil.createNotificationButton()
                            ]
                        }),
                        content: new sas.risk.common.widgets.RiskPage({
                            mainContent: sas.risk.common.widgets.WidgetFactory.buildPanel({
                                top: this.breadCrumb,
                                content: wizardControl
                            }),
                            iconBar: new sas.stress.wkbench.widget.StressIconBar({
                                section: sas.stress.wkbench.widget.StressIconBar.MODELGROUPS
                            })
                        })
                    }
                )
            });

            return this.viewLayout;
        },

        reset: function () {
            _resetModel();
        },

        getBreadcrumb: function () {
            return breadcrumb;
        },

        destroy:function(){
            // destroy everything related to the scenario editor
            sap.ui.core.mvc.JSView.prototype.destroy.apply(this, arguments);
        }
    };

}()));