jQuery.sap.require("sap.ui.model.Filter");
jQuery.sap.require("sas.risk.common.utils.LayoutService");
jQuery.sap.require("sas.risk.common.widgets.ZeroStateLayout");
jQuery.sap.require("sas.risk.common.models.RiskJSONModel");
jQuery.sap.require("sas.stress.models.ModelGroupRunAPI");
jQuery.sap.require("sas.stress.models.ModelGroupRunUtils");
jQuery.sap.require("sas.stress.wkbench.CapabilityManager");
jQuery.sap.require("sas.stress.wkbench.LeftNavManager");
jQuery.sap.require("sas.stress.wkbench.StressAPI");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.stress.wkbench.StressFixes");
jQuery.sap.require("sas.stress.wkbench.ViewManager");
jQuery.sap.require("sas.stress.wkbench.model.Project");
jQuery.sap.require("sas.stress.wkbench.widget.AccessControlsDialog");
jQuery.sap.require("sas.risk.common.msa.RiskModelSensitivityAnalysisPanel");
jQuery.sap.require("sas.hc.m.MessageToast");
jQuery.sap.require("sas.hc.m.MessageSummary");

sap.ui.controller("sas.stress.models.view.execution.ModelGroupRunsManager", {
    onInit: function () {
        this.router = sap.ui.core.routing.Router.getRouter("router");
        this.router.attachExactRouteMatched("ModelGroupRunsManager", this._init, this);
        this.modelGroupCache = {};
        this.scenarioCache = {};
        this.entityCache = {};
        this._pollIntervalIDs = [];
        this.loadResults = [];
        this.CONSTANTS = sas.stress.models.ModelGroupRunConstants;

        new sas.risk.common.utils.LayoutService(this.getView().runsTable, "ModelGroupRunsManagerTable"); // version this layout preference S1382451

        // show groove bar
        sas.stress.wkbench.LeftNavManager.getInstance().showGrooveBar(sas.stress.wkbench.LeftNavManager.PROJECTS);
    },

    refreshModel: function (projectId) {
        var controller = this;
        var view = controller.getView();

        controller.projectId = projectId;
        $.when(sas.stress.models.ModelGroupRunAPI.getModelGroupRuns(projectId),
               sas.stress.models.ModelGroupRunAPI.getScenariosForRuns(projectId),
               sas.stress.models.ModelGroupRunAPI.getEntityLineItemsForProject(projectId),
               sas.stress.models.ModelGroupRunAPI.getModelGroupsForRuns(projectId)).done(
            function (modelGroupRuns, scenarios, entities, modelGroups) {
                controller.scenarioCache = {};
                controller.entityCache = {};
                controller.modelGroupCache = {};

                // To maintain any sort already applied on the table, if model exists, set data on the model.
                // view.setModel recreates the binding to the model by the Table and the previously applied sort information is lost.
                var model = view.getModel();
                if (!(model instanceof sas.risk.common.models.RiskJSONModel)) {  // Lives in StressFixes
                    model = new sas.risk.common.models.RiskJSONModel();
                    view.setModel(model);
                    //S1256573: Do sorting after creation in view so it won't overwrite LayoutService
                    //var sorters = [new sap.ui.model.Sorter("name", false)];
                    //view.getTable().getBinding("rows").sort(sorters);
                }

                // Warning: This code is order specific
                // sas.stress.models.ModelGroupRunConstants.scenariosCache = scenarios;
                // sas.stress.models.ModelGroupRunConstants.entitiesCache = entities;
                // sas.stress.models.ModelGroupRunConstants.modelGroupsCache = modelGroups;
                model.setProperty(controller.CONSTANTS.PATH_SCENARIOS, scenarios);
                model.setProperty(controller.CONSTANTS.PATH_ENTITIES, entities);
                model.setProperty(controller.CONSTANTS.PATH_MODEL_GROUPS, modelGroups);
                controller.updateModelGroupRunsWithAssociatedNames(controller, modelGroupRuns);
                model.setProperty(controller.CONSTANTS.PATH_MODEL_GROUP_RUNS, modelGroupRuns);
                if (controller.project) {
                    model.setProperty("/projectName", controller.project.getName());
                    model.setProperty("/currentIterationNo", controller.project.getCurrentIterationNo());
                    model.setProperty("/asOfDate", controller.project.getAsOfDate());
                    model.setProperty("/projectIsLocked", controller.project.isLocked());
                }

                view.setBusy(false);
                view.fireSelectionEvent();
                controller._initController();

            }
        ).fail(function (jqxhr) {
            controller._clearInterval();
            view.setBusy(false);
            controller.onServerFailure(jqxhr);
        }).always(function() {
            var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();
            view.runsTable.setNoData(new sas.risk.common.widgets.ZeroStateLayout({
                icon: sas.icons.HC.ZEROSTATEADD,
                text: "{i18N>modelGroupRunsTable.noData.title}",
                buttons: [
                    new sas.hc.m.Button({
                        text: "{i18N>modelGroupRunsTable.noData.button.txt}",
                        tooltip: "{i18N>modelGroupRunsTable.button.tooltip}",
                        enabled: !controller.project.isLocked() && capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.MGR.CREATE),
                        press: $.proxy(controller.navigateToCreateView, controller)
                    }).addStyleClass("sasPrimaryButton")
                ]
            }));
        });
    },

    updateModelGroupRunsWithAssociatedNames: function (controller, modelGroupRuns) {
        // For sorting on scenarios, entities, or model groups we have to add a sort property for each
        // to our list of runs, so here goes.
        _.each(modelGroupRuns, function (run) {
            var scenario = controller.getScenario(run.scenarioKey);
            if (scenario) {
                run.scenarioName = scenario.name;
            }
            var entity = controller.getEntity(run.entityLIKey);
            if (entity) {
                run.entityName = entity.memberName;
            }
            var modelGroup = controller.getModelGroup(run.modelGroupKey);
            if (modelGroup) {
                run.modelGroupName = modelGroup.name;
                run.modelGroupVersion = modelGroup.version;
            }
        });

    },
    getModelGroup: function (key) {
        if (!key) {
            return null;
        }
        var modelGroup = this.modelGroupCache[key];
        if (!modelGroup) {
            var modelGroups = this.getView().getModel().getProperty("/modelGroupsForRuns");
            if (modelGroups) {
                modelGroup = _.find(modelGroups, function (modelGroup) {
                    return modelGroup.key === key;
                });
                if (modelGroup) {
                    this.modelGroupCache[key] = modelGroup;
                }
            }
        }
        return modelGroup;
    },

    getScenario: function (key) {
        if (!key) {
            return null;
        }
        var scenario = this.scenarioCache[key];
        if (!scenario) {
            var scenarios = this.getView().getModel().getProperty("/scenariosForRuns");
            if (scenarios) {
                for (var i = 0; i < scenarios.length; i++) {
                    if (scenarios[i].scenarioKey === key) {
                        scenario = scenarios[i];
                        this.scenarioCache[key] = scenario;
                        break;
                    }
                }
            }
        }
        return scenario;
    },

    getEntity: function (key) {
        if (!key) {
            return null;
        }
        var normalizedKey = key.toUpperCase();
        var entity = this.entityCache[normalizedKey];
        if (!entity) {
            var entities = this.getView().getModel().getProperty("/entitiesForRuns");
            if (entities) {
                for (var i = 0; i < entities.length; i++) {
                    if (entities[i].liKey.toUpperCase() === normalizedKey) {
                        entity = entities[i];
                        this.entityCache[normalizedKey] = entity;
                        break;
                    }
                }
            }
        }
        return entity;
    },

    _init: function (event) {
        var controller = this;
        var view = this.getView();

        /* these are the time out IDs for multiple model runs and checking */
        controller._pollTimeOutIDs = [];
        /* the following are the variables needed for selected scenarios and entities for model run */
        controller.aScenarioIsSelected = false;
        controller.anEntityIsSelected = false;
        controller.scenariolist = [];
        controller.entitylist = [];

        view.setBusy(true);
        /*
         * this is a map of the visible columns on the table
         * key = column name from json, value=(visible)true/false
         */
        // Not sure how this is supposed to work so pulled out. MCA
//        controller._mdlVisibleColumns = {};

        controller.i18N = new sap.ui.model.resource.ResourceModel({ bundleName: "sas.stress.models.messagebundle" });
        view.setModel(controller.i18N, "i18N");

        var projectId = event.getParameter("arguments").projectId;

        controller.projectId = projectId;
        // S1273684: don't use the cached project since it might be stale or deleted by others
        //controller.project = sas.stress.wkbench.model.Project.getProject(projectId);
        //if (!controller.project) { // If we don't, load it into memory
        $.when(sas.stress.wkbench.model.Project.loadProject(projectId, true)).done(function (proj) {
            controller.project = proj;
            //Load data in the table
            controller.refreshModel(controller.projectId);
            //ctrl._initcontroller(view, executing);
        }).fail(function (jqxhr) {
            controller.onServerFailure(jqxhr);
            view.setBusy(false);
            sas.stress.wkbench.ViewManager.loadProjectManager();

        });
        //} else { //
        //    // update breadcrumb.. do it here because there could be a failure and the bread crumb will nenver showup
        //    controller.refreshModel(controller.projectId);
        //    //controller._updateBreadCrumb();
        //}
        this.navChecker = sas.stress.wkbench.StressUtil.createNavChecker(true, this.getView());
    },
    onExit: function () {
        this._clearInterval();
        this.router.detachExactRouteMatched("ModelGroupRunsManager", this._init, this);
        // clear the View on the router otherwise the next router.navTo() would fail with a blank page
        this.router.setView("sas.stress.models.view.execution.ModelGroupRunsManager", null);
        this.navChecker.stop();
    },

    _areRunsRunning: function () {
        var model = this.getView().getModel();
        if (!model) return false;
        var data = model.getProperty("/modelGroupRuns");
        for (var i = 0; i < data.length; i++) {
            if (data[i].status.toUpperCase() === 'RUNNING' || data[i].status.toUpperCase() === 'READYTORUN') {
                return true;
            }
        }
        return false;
    },

    _initController: function () {
        this.poll(true);
        //this._updateBreadCrumb();
    },

    poll: function (atLeastOnce) {
        var controller = this;
        // if there is an interval handle started, do not add another!!
        if (this._pollIntervalIDs.length === 0) {
	        if (controller._areRunsRunning() || atLeastOnce) {
	            controller._pollIntervalIDs.push(setInterval(function () {
	                sas.stress.models.ModelGroupRunAPI.getModelGroupRuns(controller.projectId).done(function (modelGroupRuns) {
	                    var model = controller.getView().getModel();
	                    controller.updateModelGroupRunsWithAssociatedNames(controller, modelGroupRuns);
	                    var sortedMGs= _.sortBy(modelGroupRuns,function(mg){
	                        return mg.id;
	                    })
	                    model.setProperty("/modelGroupRuns", sortedMGs);
	                    if (!controller._areRunsRunning()) {
	                        controller._clearInterval();
	                    }
	                }).fail(function (jqxhr) {
	                    controller._clearInterval();
	                    controller.onServerFailure(jqxhr);
	                }).always(function () {
	                    controller.getView().setBusy(false);
	                });
	            }, 10000/*2500*/));	// increase the time to 10 secs
	        } else {
	            controller._clearInterval();
	        }
        } else {
        	console.log("TEST FILE ONLY::: no need to poll!!");
        }
    },

    _clearInterval: function () {
        // clear the interval ID
        _.each(this._pollIntervalIDs, function (intervalId) {
            clearInterval(intervalId);
        });
        this._pollIntervalIDs = [];
        var that=this;
        setTimeout(function () {
            var rows=that.getView().runsTable.getBinding("rows");
            if (rows){
                var index=-1;
                var found=_.find(rows.getContexts(),function(context,idx){
                    index=idx;
                    return that.lastSelected&&(that.lastSelected.id===context.getObject().id);
                });
                if (found&&index>-1) {
                    that.getView().runsTable.setSelectedIndex(index);
                }
            }
        }, 0);
    },

    onLoadSelected: function (e) {

        var selectedRuns = this.getView().getSelectedRuns();
        var controller = this;
        var view = controller.getView();
        var rb = this.i18N.getResourceBundle();

        busyDialog = new sas.hc.m.BusyDialog({
            title: rb.getText("modelsTable.noData.loading.txt")
        });

        busyDialog.open();

        var completeLoads = [];
        var partialLoads = [];
        var emptyLoads = [];
        var failures = [];

        // We have to put the loads in a timeout because they're running synchronously and block our busy dialog
        // FIXME: This needs to be replaced with a multi-load API from the midtier
        setTimeout(function() {

            _.each(selectedRuns, function (run) {
                var result = null;
                if (run.results && run.results[1] && run.results[0].files) result = run.results[1].files[0];
                if (!_.isNull(result)) {
                    sas.stress.models.ModelGroupRunAPI.loadDataFromRun(result, false).done(
                        function (data) {
                            var numLoaded = data.run.countOfDataValuesLoaded;
                            if (numLoaded == 0) {
                                emptyLoads.push({name: run.name});
                            } else if (data.run.partialDataLoad == true) {
                                partialLoads.push({name: run.name + ": " + rb.getText("results.dialog.batch.success.label.fmt", [numLoaded])});
                            } else {
                                completeLoads.push({name: run.name + ": " + rb.getText("results.dialog.batch.success.label.fmt", [numLoaded])})
                            }
                            controller.poll(true);
                        }
                    ).fail(
                        function (jqxhr) {
                            failures.push({name: run.name + ": " + jqxhr.responseJSON.message});
                        }
                    );
                }
            });

            var summary = new sas.hc.m.MessageSummary({
                showToolbar: true,
                layoutMode: sas.hc.m.MessageSummaryLayoutModeType.WindowShade,
                showInformationFilter: false,
                showExportButton: true
            });

            var messages = [];

            if (completeLoads.length !== 0) {
                messages.push({
                    type: sas.hc.m.MessageSummaryFilterButtonType.Success,
                    title: rb.getText("results.dialog.batch.success.title.txt"),
                    associatedObjects: completeLoads
                })
            }
            if (partialLoads.length !== 0) {
                messages.push({
                    type: sas.hc.m.MessageSummaryFilterButtonType.Warning,
                    title: rb.getText("results.dialog.batch.partial.title.txt"),
                    text: rb.getText("results.dialog.batch.partial.details.txt"),
                    associatedObjects: partialLoads
                })
            }
            if (emptyLoads.length !== 0) {
                messages.push({
                    type: sas.hc.m.MessageSummaryFilterButtonType.Warning,
                    title: rb.getText("results.dialog.batch.empty.title.txt"),
                    text: rb.getText("results.dialog.batch.empty.details.txt"),
                    associatedObjects: emptyLoads
                })
            }
            if (failures.length !== 0) {
                messages.push({
                    type: sas.hc.m.MessageSummaryFilterButtonType.Error,
                    title: rb.getText("results.dialog.batch.failed.title.txt"),
                    text: rb.getText("results.dialog.batch.failed.details.txt"),
                    associatedObjects: failures
                })
            }
            summary.setModel(new sap.ui.model.json.JSONModel({messages: messages}));

            var layout=new sas.hc.ui.layout.VerticalLayout({
                width: "100%",
                content: [
                    new sas.hc.m.Text({text: rb.getText("results.dialog.batch.label.txt")}).addStyleClass("sapUiSmallMarginBottom"),
                    summary
                ]
            });

            var dialog = new sas.hc.m.Dialog({
                title: rb.getText("results.dialog.title.txt"),
                contentWidth: sas.stress.wkbench.StressUtil.dialogMedium,
                content: layout,
                endButton: new sas.hc.m.Button({
                    text: rb.getText("results.dialog.button.close.txt"),
                    press: function () {
                        dialog.close();
                    },
                    enabled: true
                }),
                afterClose: function () {
                    busyDialog.close();
                    dialog.destroy();
                }
            }).addStyleClass("stressFormMedium");

            busyDialog.close();

            dialog.open();

        }, 100);

    },

    onRunSelected: function (e) {
        var rb = this.i18N.getResourceBundle();
        var selectedRuns = this.getView().getSelectedRuns();
        var controller = this;

        _.each(selectedRuns, function (run) {
            run.status = "READYTORUN";
        });
        this.getView().getTable().getModel().updateBindings(false);

        var runIds = _.map(selectedRuns, function (run) {
            return run.id;
        });
        sas.stress.models.ModelGroupRunAPI.rerunRuns(runIds).done(
            function (run) {
            	var model = controller.getView().getModel();
                var modelGroupRuns = model.getProperty("/modelGroupRuns");
                var modelGroupRun = _.find(modelGroupRuns, function(modelGroupRun) {
                	return modelGroupRun.id === run.id; 
                });
                if (modelGroupRun) {
                	modelGroupRun.status = "running";	// force this to a running state so that the UI responds faster
                	model.setProperty("/modelGroupRuns", modelGroupRuns);
                }
                controller.poll(true);
            }
        ).fail(
            function (jqxhr) {
                controller.poll(true);
                controller.onServerFailure(jqxhr);
            }
        );
    },
    openLoadDialog: function (data, controller) {
        var rb = this.i18N.getResourceBundle();
        var dialogModel = new sas.risk.common.models.RiskJSONModel();
        // fix type of value

        // All of this is totally lame but I needed to get it done in the most expediant way.
        // TODO: Should either use binding all the way or not!!
        var valueString = data.sampleDataPointLoaded.value;
        data.valueAsFloat = Number(valueString);
        var formatter = sap.ui.core.format.NumberFormat.getFloatInstance({ maxFractionDigits: 2 });
        data.valueForDisplay = formatter.format(data.sampleDataPointLoaded.value);
        controller.partiallyLoaded = data.partialDataLoad;
        controller.numSuccessfullyLoaded = data.run.countOfDataValuesLoaded;    // S1367281
        var startDate = data.sampleDataPointLoaded.intervalStartDate;
        var endDate = data.sampleDataPointLoaded.intervalEndDate;
        var interval = data.sampleDataPointLoaded.interval;
        controller.intervalDisplayString = rb.getText("results.dialog.dateinterval.value.fmt", [
            sas.stress.models.ModelGroupRunUtils.dateFormatter(startDate),
            sas.stress.models.ModelGroupRunUtils.dateFormatter(endDate),
            sas.stress.models.ModelGroupRunUtils.intervalFormatter(interval)
        ]);

        dialogModel.setData(data);
        var existingDialog = sap.ui.getCore().byId("LoadResultsDialog");
        if (existingDialog) {
            existingDialog.open();
            existingDialog.setModel(dialogModel);
        } else {
            var dialog = sap.ui.jsfragment("sas.stress.models.view.execution.LoadResultsDialog", controller);
            dialog.setModel(dialogModel);
            dialog.setBindingContext("/");
            dialog.open();
        }

    },

    onExecuteLoad: function (resultsFile) {
        var rb = this.i18N.getResourceBundle();
        var controller = this;
        this.loadResults = [];
        this.getView().setBusy(true);

        sas.stress.models.ModelGroupRunAPI.loadDataFromRun(resultsFile, true).done(
            function (data) {
                controller.getView().setBusy(false);
                var numLoaded = data.run.countOfDataValuesLoaded;   // fixes for S1367281
                if (numLoaded > 0) {
                    controller.openLoadDialog(data, controller);
                } else {
                    var loadResultsLogName = rb.getText('modelGroupRun.outputfile.log.loadresults.txt');
                    var nonloadableDataSetName = rb.getText('modelGroupRun.outputfile.sas.nonloadabledata.txt');
                    sas.stress.models.ModelGroupRunUtils.showErrorDialog(rb.getText("modelGroupRuns.load.nothingloaded.error.fmt", [loadResultsLogName, nonloadableDataSetName]), rb.getText("modelGroupRuns.load.nothingloaded.error.title"));
                }

                controller.poll(true);
            }
        ).fail(
            function (jqxhr) {
                controller.getView().setBusy(false);
                controller.poll(true);
                controller.onServerFailure(jqxhr);
            }
        );
    },

    onSensitivitySelected: function(howToRun) {
        var view = this.getView();
        var table = view.getTable();
        var context = table.getContextByIndex(table.getSelectedIndex());
        var item = table.getModel().getProperty(context.getPath());
        var runIds = _.map(table.getSelectedIndices(), function (index) {
            var modelGroupRun = table.getModel().getProperty(table.getContextByIndex(index).getPath());
            return modelGroupRun.id;
        });
        this.openModelSensitivityAnalysis(item.scenarioKey, runIds, howToRun, item.workGroup);
    },

    openModelSensitivityAnalysis: function (scenarioId, runIds, howToRun, workGroup) {
        var ctrl = this;
        sas.stress.wkbench.ViewManager.loadModelGroupRunsSensitivity(sas.stress.wkbench.model.Project.getCurrentProject(), scenarioId, runIds, howToRun, workGroup);
    },

    onDeleteSelected: function () {
        var rb = this.i18N.getResourceBundle();
        var controller = this;
        sas.stress.models.ModelGroupRunUtils.showDeleteConfirmationDialog(
            rb.getText("modelGroupRuns.confirm.delete.title"),
            rb.getText("modelGroupRuns.confirm.delete.txt"),
            null,
            $.proxy(controller._handleDeleteRunInstance, controller)
        );
    },

    navigateToCreateView: function () {
        sas.stress.wkbench.ViewManager.loadModelGroupRunsCreate(sas.stress.wkbench.model.Project.getCurrentProject());
    },

    /**
     * Handler for the delete run instance button. Deletes the currently selected
     * run instance.
     */
    _handleDeleteRunInstance: function () {
        var rb = this.i18N.getResourceBundle();
        var view = this.getView();
        var controller = this;

        view.setBusy(true);
        var selectedRuns = view.getSelectedRuns();
        var runIds = _.map(selectedRuns, function (run) {
            return run.id;
        });
        sas.stress.models.ModelGroupRunAPI.deleteRuns(runIds).done(
            function () {
                // remove the instance from the table model
                var runsModel = view.getTable().getModel();
                view.getTable().setSelectedIndex(-1);
                var runs = runsModel.getProperty("/modelGroupRuns");

                _.each(runIds, function (id) {
                    // find the index in the model/runs array
                    var deletedIndex = sas.stress.models.ModelGroupRunUtils.findIndex(runs, function (run) {
                        return run.id === id;
                    });
                    if (deletedIndex !== -1) {
                        runs.splice(deletedIndex, 1);
                    }
                });

                // set the updated runs on the model.. poll may not or may not update immediately
                runsModel.setProperty("/modelGroupRuns", runs);
                controller.poll(true);
                view.setBusy(false);
            }
        ).fail(
            function (restError) {
            	view.setBusy(false);
                sas.stress.models.ModelGroupRunUtils.showErrorDialog(rb.getText("modelGroupRuns.error.delete.txt"), rb.getText("modelGroupRuns.error.delete.title"));
            }
        );
    },

    onRefreshButton: function (e) {
    },

    /*
     * common method used between loading the view initially and for execution statuses during RUN, REFRESH and navigate out-in
     */
    _setOrUpdateViewModel: function (json, executing) {
        var view = this.getView();

        // To maintain any sort already applied on the table, if model exists, set data on the model.
        // view.setModel recreates the binding to the model by the Table and the previously applied sort information is lost.
        sas.stress.models.Model._json = json;
        if (view.getModel() instanceof sas.risk.common.models.RiskJSONModel) {
            view.getModel().setData(json);
        } else {
            var model = new sas.risk.common.models.RiskJSONModel(json);
            view.setModel(model);
        }
        return executing;
    },

    _clearTimeout: function () {
        // clear the timeout ID
        _.each(this._pollTimeOutIDs, function (timeoutid) {
            clearTimeout(timeoutid);
        });
        this._pollTimeOutIDs = [];
    },

    /*
     * we will loop through the visible columns map to create filters for each
     */
    filterModelGroupRuns: function () {
        var search = this.getView().getSearchField().getValue().trim() || "";
        var table = this.getView().getTable();
        var nameFilter = new sap.ui.model.Filter("name", sap.ui.model.FilterOperator.Contains, search);
        var scenarioFilter = new sap.ui.model.Filter("scenarioName", sap.ui.model.FilterOperator.Contains, search);
        var entityFilter = new sap.ui.model.Filter("entityName", sap.ui.model.FilterOperator.Contains, search);
        var descriptionFitler = new sap.ui.model.Filter("description", sap.ui.model.FilterOperator.Contains, search);
        var modelGroupFilter = new sap.ui.model.Filter("modelGroupName", sap.ui.model.FilterOperator.Contains, search);
        var userFilter = new sap.ui.model.Filter("runByUserName", sap.ui.model.FilterOperator.Contains, search);
        var allFilters = new sap.ui.model.Filter([nameFilter, scenarioFilter, entityFilter, descriptionFitler, modelGroupFilter, userFilter]);
        var binding = table.getBinding("rows");
        if (binding) {
            binding.filter(allFilters);
        }
    },

    onRowSelectionChange: function (oEvent) {
        var controller = this;
        var view = this.getView();
        var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();
        var selectedIndices = oEvent.getSource().getSelectedIndices();
        var runsTable = view.getTable();

        var sensEnabled = selectedIndices.length !== 0;
        var sensScenario;
        var sensWorkGroup;

        // For MSA, make sure all MGRs have same base scenario, workgroup, standard type, and are in a Completed state
        _.each(selectedIndices, function (selectedIdx) {
            var context = runsTable.getContextByIndex(selectedIdx);
            if (!_.isUndefined(context)) {
                var path = context.sPath;
                var item = runsTable.getModel().getProperty(path);
                if (_.isUndefined(sensScenario)) { sensScenario = item.scenarioName; sensWorkGroup = item.workGroup; }
                if (item.scenarioName !== sensScenario || item.scenarioName === "" || _.isUndefined(item.scenarioName)) { sensEnabled = false; }
                if (item.workGroup !== sensWorkGroup) { sensEnabled = false; }
                if (item.runType !== "Standard") { sensEnabled = false; }
                if (item.status !== "Completed") { sensEnabled = false; }
            }
        });

        view.getSensitivityButton().setEnabled(sensEnabled);

        var loadEnabled = selectedIndices.length !== 0;

        _.each(selectedIndices, function (selectedIdx) {
            var context = runsTable.getContextByIndex(selectedIdx);
            if (!_.isUndefined(context)) {
                var path = context.sPath;
                var item = runsTable.getModel().getProperty(path);
                if (item.status !== "Completed") { loadEnabled = false; }
            }
        });

        view.getLoadButton().setEnabled(loadEnabled);

        switch (selectedIndices.length) {
            case 0:
                view.getRunButton().setEnabled(false);
                view.getDeleteButton().setEnabled(false);
                this.lastSelected=undefined;
                break;
            case 1:
                // do not let user execute Run if the mgRun is already running or scheduled to run.. not sure about delete
            	var mgRunObject = view.runsTable.getContextByIndex(selectedIndices[0]).getObject();
            	var isInRunningState = (mgRunObject.status.toUpperCase() === "READYTORUN" || mgRunObject.status.toUpperCase() === "RUNNING" || mgRunObject.status.toUpperCase() === "QUEUEDTORUN");
                view.getRunButton().setEnabled(
                    capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.MGR.RUN) &&
                    (controller.project && !controller.project.isLocked()) && !isInRunningState);
                view.getDeleteButton().setEnabled(
                    capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.MGR.DELETE) &&
                    (controller.project && !controller.project.isLocked()));

                this.lastSelected = mgRunObject;
                break;
            default:
                // For now we only support running one at a time, not optimal user experience, so leaving
                // this code in.
                // var allRunnable = true;
                // _.each(selectedIndices, function (selectedIndex) {
                //     var context = oEvent.getSource().getContextByIndex(selectedIndex);
                //     var obj = context.getObject();
                //     if (obj.actionControls !== undefined && obj.actionControls.runnable !== undefined) {
                //         allRunnable = allRunnable && obj.actionControls.runnable;
                //     }
                // });
                view.getRunButton().setEnabled(false);
                view.getDeleteButton().setEnabled(
                    capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.MGR.DELETE) &&
                    (controller.project && !controller.project.isLocked()));

                this.lastSelected=undefined;
                break;
        }


    },
    onCellClick: function (event) {
    },

    onServerFailure: function (jqxhr) {
        sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
    }

});