jQuery.sap.require("sap.ui.model.Filter");
jQuery.sap.require("sas.hc.m.MessageToast");
jQuery.sap.require("sas.hc.m.MessageDialog");
jQuery.sap.require("sas.risk.common.models.RiskJSONModel");
jQuery.sap.require("sas.risk.common.utils.LayoutService");
jQuery.sap.require("sas.risk.common.widgets.ZeroStateLayout");
jQuery.sap.require("sas.stress.reports.model.ReportConfig");
jQuery.sap.require("sas.stress.reports.model.ReportHistory");
jQuery.sap.require("sas.stress.reports.model.ReportConfigCategory");
jQuery.sap.require("sas.stress.reports.StressAPI");
jQuery.sap.require("sas.stress.wkbench.CapabilityManager");
jQuery.sap.require("sas.stress.wkbench.LeftNavManager");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.stress.wkbench.model.Project");
jQuery.sap.require("sas.stress.wkbench.widget.StatusLabel");
jQuery.sap.require("sas.stress.wkbench.widget.StressIconBar");
jQuery.sap.require("sas.stress.wkbench.ViewManager");
jQuery.sap.require({ modName: "sas.stress.wkbench.view.CategoryDialog", type: "fragment" });

sap.ui.controller("sas.stress.reports.view.ProjectReportsEditor", {

    onInit: function () {
        this.router = sap.ui.core.routing.Router.getRouter("router");
        this.router.attachExactRouteMatched("Reports", this._init, this);
        this.navChecker = sas.stress.wkbench.StressUtil.createNavChecker(true, this.getView());
        this.capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();
        //S1297506 do it at the beginning so that it will not overwrite the category filter
        new sas.risk.common.utils.LayoutService(this.getView().reportConfigsTable, "ReportConfigsTable");
        new sas.risk.common.utils.LayoutService(this.getView().reportHistoryTable, "ReportHistoryTable");

        // show groove bar
        sas.stress.wkbench.LeftNavManager.getInstance().showGrooveBar(sas.stress.wkbench.LeftNavManager.PROJECTS);
    },
    _init: function (event) {
        var controller = this;
        controller._pollIntervalIDs = [];
        var view = this.getView();
        var i18N = new sap.ui.model.resource.ResourceModel({ bundleName: "sas.stress.reports.messagebundle" });
        view.setModel(i18N, "i18N");
        var model = new sas.risk.common.models.RiskJSONModel({});
        view.setModel(model);
        var projectId = event.getParameter("arguments").projectId;

        // S1273684: don't use the cached project since it might be stale or deleted by others
        //controller.project = sas.stress.wkbench.model.Project.getProject(projectId);
        //if (!controller.project) {	// no it is not in the cache
        sas.stress.wkbench.model.Project.loadProject(projectId, true, false).done(function (project) {
            controller.project = sas.stress.wkbench.model.Project.getProject(projectId);
            controller.afterProjectIsLoaded();
        }).fail(function (jqxhr) {
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            controller.getView().setBusy(false);
            sas.stress.wkbench.ViewManager.loadProjectManager();
            //controller.getView().destroy();
        });
        //} else {
        //    controller.afterProjectIsLoaded();
        //}
    },

    afterProjectIsLoaded: function () {
        var controller = this;
        var view = this.getView();
        var projectKey = this.project.getId();
        view.getModel().setProperty("/projectName", this.project.getName());
        view.getModel().setProperty("/currentIterationNo", this.project.getCurrentIterationNo());
        view.getModel().setProperty("/asOfDate", this.project.getAsOfDate());
        view.getModel().setProperty("/projectIsLocked",this.project.isLocked());

        view.reportConfigsTable.setBusyIndicatorDelay(0);
        view.reportConfigsTable.setBusy(true);

        //this._addBreadCrumbItems();
        var reportConfigCategories = this._getReportConfigCategories(this.project.getId());
        //Load reports
        var reportConfigs = this.getReportConfigs(projectKey);

        $.when(reportConfigCategories, reportConfigs).done(function (cats, cfgs, running) {
            //Set reportConfigsCount for each category.
            controller._setReportConfigsCountInCategoriesList();

            //Select first category and filter data in the report config table.
            var categoriesListItems = controller._getCategoriesListItems();
            if (categoriesListItems.length > 0) {
                controller._selectPreferredCategory();
            }
            controller.handleProcessOrchestration();
            if (running || controller._areReportConfigsRunning()){
                controller._poll(true);
            }
        }).fail(function (jqxhr) {
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);

            view.reportConfigsTable.setBusy(false);
        });

        //Initialize model on History table
        this._setReportHistoryData();
    },

    /**
     * Select the reports in the table that match the given names or keys.
     * @param {Array} reportNames list of report names
     * @param {Array} reportKeys optional list of report keys
     */
    selectReports: function (reportNames, reportKeys) {
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.reports");
        var view = this.getView();
        // Ensure no selections exist in the table already.
        view.reportConfigsTable.clearSelection();
        var numFoundReports = 0;
        // We have to loop through the model because the rows we want to select might not have been loaded yet
        var allReports = view.reportConfigsTable.getModel("reportConfigs").getData();
        for (var reportIndex = 0; reportIndex < reportNames.length; ++reportIndex) {
            var foundReport = false;
            var reportName = reportNames[reportIndex];
            for (var i = 0; i < allReports.length; i++) {
                var tableContext = view.reportConfigsTable.getContextByIndex(i);
                if (tableContext) {
                    var path = tableContext.getPath();
                    var report = view.reportConfigsTable.getModel("reportConfigs").getProperty(path);
                    if (report.reportName.toLocaleLowerCase() === reportName.toLocaleLowerCase()
                            // S1300977: when launching from package viewer (WorkSheetReportViewer.controller.navigateToReport), name don't match
                            || (reportKeys && reportKeys[reportIndex] === report.key)) {
                        // Report found. Add it to the selection.
                        view.reportConfigsTable.addSelectionInterval(i, i);
                        // We found the current report, break out of the inner for-loop.
                        foundReport = true;
                        ++numFoundReports;
                        if (1 === numFoundReports) {
                            // We want to ensure at least the first report is visible, so we scroll to it.
                            view.reportConfigsTable.setFirstVisibleRow(i);
                        }
                        break;
                    }
                }
            }
            if (!foundReport) {
                // Specified report wasn't found, throw an error.
                throw new Error(rb.getText("reports.processOrchestration.invalidReport.fmt.txt", [reportName]));
            }
        }
    },
    onExit: function () {
        this._clearInterval();
        this.router.detachExactRouteMatched("Reports", this._init, this);
        // clear the View on the router otherwise the next router.navTo() would fail with a blank page
        this.router.setView("sas.stress.reports.view.ProjectReportsEditor", null);
        this.navChecker.stop();
    },

    _getReportConfigCategories: function (projectKey) {
        var deferred = $.Deferred();
        var view = this.getView();
        var controller = this;
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.reports");
        sas.stress.reports.model.ReportConfigCategory.listReportConfigCategories(projectKey).done(function (reportConfigCategories) {
            // Add category "All" to the top of the list.
            reportConfigCategories.unshift(new sas.stress.reports.model.ReportConfigCategory(projectKey, 0, rb.getText("categories.all.txt"), false));
            var json = [];
            jQuery.each(reportConfigCategories, function (index, reportConfigCategory) {
                json.push(sas.stress.reports.model.ReportConfigCategory.toJson(reportConfigCategory));
            });
            controller._sortReportConfigCategoriesJson(json);
            var jsonModel = new sas.risk.common.models.RiskJSONModel({ categories: json });
            if (view.categoryFilter)
                view.categoryFilter.setModel(jsonModel);
            else
                view.setModel(jsonModel);
            deferred.resolve(reportConfigCategories);
        }).fail(function (jqxhr) {
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });
        return deferred;
    },

    _sortReportConfigCategoriesJson: function (reportConfigCategories) {
        //Sort by Name ascending. Keep "All" at the top.
        var sortedReportConfigCategories =
            reportConfigCategories.slice(1).sort(function (a, b) {
                var aName = a.name.toLocaleLowerCase();
                var bName = b.name.toLocaleLowerCase();
                return ((aName < bName) ? -1 : ((aName > bName) ? 1 : 0));
            });
        reportConfigCategories.length = 1;
        reportConfigCategories.push.apply(reportConfigCategories, sortedReportConfigCategories);
    },

    _setReportConfigsCountInCategoriesList: function () {
        var view = this.getView();
        var categoriesListModelData = this._getCategoriesInModel();
        //reset reportConfigsCount.
        _.each(categoriesListModelData, function (category) {
            category.reportConfigsCount = 0;
        });
        var reportConfigs = view.reportConfigsTable.getModel("reportConfigs").getData();
        //set reportConfigsCount
        categoriesListModelData[0].reportConfigsCount = reportConfigs.length; //Count for category "All".
        _.each(reportConfigs, function (reportConfig) {
            var category = _.find(categoriesListModelData, function (category) {
                return category.key === reportConfig.reportConfigCategoryKey     //S1312516: use key since category name is bogus for packages
                    || category.name === reportConfig.reportConfigCategory; //S1293977: numeric key may not match string key
            });
            if (category !== undefined) {
                category.reportConfigsCount++;
            }
        });
        view.categoryFilter.getModel().refresh();
    },

    getReportConfigs: function (projectKey, enablePolling) {
        var deferred = $.Deferred();
        var view = this.getView();
        var controller = this;
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.reports");
        view.reportConfigsTable.setNoData(rb.getText("reportConfigsTable.noData.loading.txt"));

        sas.stress.reports.model.ReportConfig.getReportConfigs(projectKey).done(function (reportConfigs) {
            var json = [];
            var hasRunningReports = false;
            jQuery.each(reportConfigs, function (index, reportConfig) {
                if (reportConfig.getReportStatus() === "Executing") {
                    hasRunningReports = true;
                }
                var reportConfigJson = sas.stress.reports.model.ReportConfig.toJson(reportConfig);
                json.push(reportConfigJson);
            });

            // To maintain any sort already applied on the table, if model exists, set data on the model.
            // view.setModel recreates the binding to the model by the Table and the previously applied sort information is lost.
            var model = view.reportConfigsTable.getModel("reportConfigs");
            if (model instanceof sap.ui.model.json.JSONModel) {
                model.setData(json);
            } else {
                var newModel = new sas.risk.common.models.RiskJSONModel(json);
                view.reportConfigsTable.setModel(newModel, "reportConfigs");

                //S1256573: Do sorting after creation in view so it won't overwrite LayoutService
                //var reportConfigSorters = [new sap.ui.model.Sorter("reportName", false)];
                //if (view.reportConfigsTable.getBinding("rows")) {
                //    view.reportConfigsTable.getBinding("rows").sort(reportConfigSorters);
                //}
            }
            //Select first category and filter data in the report config table.
            //var categoriesListItems = controller._getCategoriesListItems();
            //if (categoriesListItems.length > 0) {
            //    controller._selectPreferredCategory();
            //    //view.categoryFilter.setSelectedItem(categoriesListItems[0], true);
            //    //controller.categoriesListSelectionChange();
            //}

            controller._toggleReportConfigToolbarButtons();
            /*if (enablePolling && hasRunningReports) {
                controller._poll(hasRunningReports);
            }*/
            deferred.resolve(reportConfigs, hasRunningReports);
        }).fail(function (jqxhr) {
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            controller.getView().setBusy(false);
            sas.stress.wkbench.ViewManager.loadProjectManager();
            //controller.getView().destroy();
        }).always(function() {
            view.reportConfigsTable.setBusy(false);
            view.reportConfigsTable.setNoData(new sas.risk.common.widgets.ZeroStateLayout({
                icon: sas.icons.HC.ZEROSTATEADD,
                text: "{i18N>reportConfigsTable.noData.title}",
                buttons: [
                    new sas.hc.m.Button({
                        text: "{i18N>addCategoryDialog.window.title}",
                        ariaLabel: "{i18N>addCategoryDialog.window.title}",
                        press: $.proxy(controller.categoriesListCreateButtonPress, controller)
                    }).addStyleClass("sasPrimaryButton"),
                    new sas.hc.m.Button({
                        text: "{i18N>reportConfigsTable.noData.button.txt}",
                        press: $.proxy(controller.createReportButtonPress, controller),
                        enabled: {
                            path: "reportConfigs>/allowNewReportCreation",
                            formatter: function (enabled) {
                                return enabled;
                            }
                        }
                    })
                ]
            }));
        });
        return deferred;
    },

    _poll: function (executing) {
        var controller = this;
        var projectKey = this.project.getId();

     // if there is an interval handle started, do not add another!!
        if (this._pollIntervalIDs.length === 0) {
	        if (executing) {
	            controller._pollIntervalIDs.push(setInterval(function () {
	                controller.getReportConfigs(projectKey, false).done(function (reportConfigs, hasRunningReports) {
	                    if (!hasRunningReports) {
	                        controller._donePolling();
	                    }
	                    controller._loadReportHistory();
	                }).fail(function (jqxhr) {
	                    controller._clearInterval();
	                    sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
	                });
	            }, 10000/*2500*/));	// increase the time to 10 secs
	        } else {
	            controller._donePolling();
	        }
        } else {
        	console.log("TEST FILE ONLY::: no need to poll!!");
        }
    },

    _donePolling: function () {
        this._clearInterval();
        sas.hc.m.MessageToast.show(sap.ui.getCore().getLibraryResourceBundle("sas.stress.reports").getText("reports.execution.complete.message.txt"));
    },

    _clearInterval: function () {
        // clear the interval ID
        _.each(this._pollIntervalIDs, function (intervalId) {
            clearInterval(intervalId);
        });
        this._pollIntervalIDs = [];
    },

    handleProcessOrchestration: function () {
        var view = this.getView();
        var controller = this;
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.reports");
        // handle process orchestration commands, if they exist
        if (sap.ui.getCore().getModel("CreateReportParams")) {
            controller.createReportParams = sap.ui.getCore().getModel("CreateReportParams").getData();
            if (view.newReportButton.getEnabled()) {
                view.newReportButton.firePress();
            } else {
                sas.hc.m.MessageDialog.error({text: rb.getText("reports.processOrchestration.createReportNotAllowed.txt"), title: rb.getText("error.window.title")});
                delete controller.createReportParams;
            }
            sap.ui.getCore().setModel(undefined, "CreateReportParams");
        } else if (sap.ui.getCore().getModel("RunReportParams")) {
            try {
                controller.runReportParams = sap.ui.getCore().getModel("RunReportParams").getData();
                if (controller.runReportParams) {
                    if (controller.runReportParams.reportNames && controller.runReportParams.reportNames.length > 0) {
                        controller.selectReports(controller.runReportParams.reportNames, controller.runReportParams.reportKeys);
                        if (view.runReportsButton.getEnabled()) {
                            view.runReportsButton.firePress();
                        } else {
                            // Problem with the selections, display an error to the user.
                            sas.hc.m.MessageDialog.error({text: rb.getText("reports.processOrchestration.runReportsNotAllowedInvalidSelection.txt"), title: rb.getText("error.window.title")});
                            delete controller.runReportParams;
                        }
                    } else {
                        // At least one report must be specified in order to run, display an error to the user.
                        sas.hc.m.MessageDialog.error({text: rb.getText("reports.processOrchestration.runReportsNotAllowedNoSelection.txt"), title: rb.getText("error.window.title")});
                        delete controller.runReportParams;
                    }
                }
            } catch (ex) {
                sas.hc.m.MessageDialog.error({text: ex.message, title: rb.getText("error.window.title")});
            } finally {
                sap.ui.getCore().setModel(undefined, "RunReportParams");
            }
        }
    },

    _toggleReportConfigToolbarButtons: function () {
        var view = this.getView();
        var selectedIndices = view.reportConfigsTable.getSelectedIndices();
        var hasSelection = false;
        var packageReportSelected = false;
        var aReportIsRunning = false;
        var reportConfigCanBeEdited=true;
        var nonLandingAreaReport = false;
        var landingAreaReport = false;
        var differentReportsSelected = false;

        if (selectedIndices.length > 0) {
            hasSelection = true;
            for (var n = 0; n < selectedIndices.length; ++n) {
                var index = selectedIndices[n];
                var reportConfig = this._getReportConfig(index);
                if (reportConfig !== null) {
                    // Has a report belonging to a package been selected?
                    packageReportSelected = packageReportSelected || reportConfig.getTaxonomyPackageOwners().length > 0;
                    // Is a report currently running?
                    aReportIsRunning = aReportIsRunning || reportConfig.getReportStatus() === "Executing";
                    if (selectedIndices.length=== 1) { // in case of multi-elect.. we might see if the flag is false and keep it false
                        /*
                         project locking enforcement
                         1.	Reports created before locking cannot be deleted.
                         3.	Cannot modify report definition created before locking.
                         */
                        reportConfigCanBeEdited = reportConfig.canBeEditedIfProjectIsLocked;
                    }
                    // Check if the report is landingAreaFile report or not
                    if (reportConfig.requiresFileFrom !== undefined && reportConfig.requiresFileFrom !== null) {
                        landingAreaReport = true;
                    } else {
                        nonLandingAreaReport = true;
                    }
                }
            }
        }
        var canEdit = this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.REPORT.UPDATE) && reportConfigCanBeEdited;
        var canDelete = this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.REPORT.DELETE) && reportConfigCanBeEdited;
        var canRun = this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.REPORT.RUN);

        if (landingAreaReport && nonLandingAreaReport) {
            differentReportsSelected = true;
        }
        // A single report must be selected and it cannot be a package report or currently running.
        canEdit = canEdit && !packageReportSelected && !aReportIsRunning;
        view.deleteReportButton.setEnabled(canDelete && selectedIndices.length === 1); //TODO: allow multi delete hasSelection);
        view.editReportButton.setEnabled(canEdit && selectedIndices.length === 1);
        // At least one report must be selected and all must be runnable (but not currently running).
        view.runReportsButton.setEnabled(canRun && hasSelection && !aReportIsRunning && !differentReportsSelected);
    },

    onReportConfigsTableRowSelectionChanged: function (event) {
        this._toggleReportConfigToolbarButtons();

        this._loadReportHistory();
        this._clearReportHistoryTableFilter();
    },

    _clearReportHistoryTableFilter: function () {
        //Clear filter and text in Search field of History table
        var reportHistorySearchFld = this.getView().byId("reportHistorySearchFld");
        reportHistorySearchFld.setValue("");
        this._filterReportHistoryTable("");
    },

    _getReportConfig: function (index) {
        var view = this.getView();
        // If index wasn't specified, then we are getting the currently selected index from the table.
        if (undefined === index) {
            var selections = view.reportConfigsTable.getSelectedIndices();
            if (selections !== null && selections.length > 0) {
                index = selections[0];
            } else {
                index = -1;
            }
        }
        var reportConfig = null;
        if (index >= 0) {
            var context = view.reportConfigsTable.getContextByIndex(index);
            var reportConfigObj = context.getObject();
            reportConfig = sas.stress.reports.model.ReportConfig.getReportConfig(reportConfigObj.key);
        }
        return reportConfig;
    },

    _loadReportHistory: function () {
        var controller = this;
        var view = this.getView();
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.reports");
        var reportConfig = this._getReportConfig();
        if (reportConfig !== null) {
            view.reportHistoryTableTitle.setText(rb.getText("reportHistoryTable.reportHeader.title.fmt.txt", [reportConfig.getReportName()]));
            //view.reportHistoryTableHeaderTitle.setText(rb.getText("reportHistoryTable.reportHeader.title.fmt.txt", [reportConfig.getReportName()]));
            sas.stress.reports.model.ReportConfig.getLatestDataForReportConfig(reportConfig.getProjectKey(), reportConfig.getKey()).done(function (reportConfig) {
                var json = [];
                $.each(reportConfig.getHistory(), function (index, history) {
                    var reportHistoryJson = sas.stress.reports.model.ReportHistory.toJson(history);
                    json.push(reportHistoryJson);
                });

                controller._setReportHistoryData(json);
            }).fail(function (jqxhr) {
                sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            });
        } else {
            controller._setReportHistoryData();
        }
    },

    _setReportHistoryData: function (json) {
        var view = this.getView();
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.reports");
        if (undefined === json) {
            // No report is selected. Do not display the option to run the report within the zero state layout.
            view.reportHistoryTable.setNoData(new sas.risk.common.widgets.ZeroStateLayout({
                text: "{i18N>reportHistoryTable.noData.title}"
            }));
            // Use the default title for the table.
            view.reportHistoryTableTitle.setText(rb.getText("reportHistoryTable.defaultHeader.title"));
            //view.byId("reportHistoryTableHeaderTitle").setText(rb.getText("reportHistoryTable.defaultHeader.title"));
        } else if (json.length === 0) {
            // A report is selected but has no history. Display the run button within the zero state layout.
            view.reportHistoryTable.setNoData(new sas.risk.common.widgets.ZeroStateLayout({
                text: "{i18N>reportHistoryTable.zeroData.title}",
                buttons: [
                    new sas.hc.m.Button({
                        text: "{i18N>reportHistoryTable.runReport.button.txt}",
                        ariaLabel: "{i18N>reportHistoryTable.runReport.button.txt}",
                        press: $.proxy(this.runReportsButtonPress, this)
                    })
                ]
            }));
        }
        var model = view.reportHistoryTable.getModel("reportHistory");
        if (model instanceof sap.ui.model.json.JSONModel) {
            model.setData(json);
        } else {
            var newModel = new sas.risk.common.models.RiskJSONModel(json);
            view.reportHistoryTable.setModel(newModel, "reportHistory");

            //S1256573: Do sorting after creation in view so it won't overwrite LayoutService
            //var reportHistorySorters = [new sap.ui.model.Sorter("createdDTTM", true)];
            //if (view.reportHistoryTable.getBinding("rows")) {
            //    view.reportHistoryTable.getBinding("rows").sort(reportHistorySorters);
            //}
        }
    },

    downloadReportButtonPress: function () {
        var reportHistory = this._getReportHistory();
        if (null !== reportHistory) {
            var view = this.getView();
            var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.reports");
            var attachmentKey = reportHistory.attachmentKey;
            if (attachmentKey && attachmentKey > 0) {
                view.setBusy(true);
                var downloadReportCall = sas.stress.reports.StressAPI.downloadReport(attachmentKey);
                $.when(downloadReportCall).done(function () {
                    view.setBusy(false);
                }).fail(function (jqxhr) {
                    sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
                });

            }
        }
    },

    publishReportButtonPress: function () {
        var reportHistory = this._getReportHistory();
        if (null !== reportHistory) {
            var view = this.getView();
            var controller = this;
            var projectKey = this.project.getId();
            view.setBusy(true);
            var publishReportCall = sas.stress.reports.StressAPI.publishReport(projectKey, reportHistory.key);
            $.when(publishReportCall).done(function () {
                view.setBusy(false);
                controller._loadReportHistory();
            }).fail(function (jqxhr) {
                view.setBusy(false);
                sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            });
        }
    },

    onReportHistoryTableRowSelectionChanged: function (event) {
        var reportHistory = this._getReportHistory();
        var attachmentExists = false;
        var isPackageReport = false;
        if (reportHistory !== null) {
            var reportConfig = sas.stress.reports.model.ReportConfig.getReportConfig(reportHistory.getReportConfigKey());
            // Does the report belong to a package?
            isPackageReport = reportConfig.getTaxonomyPackageOwners().length > 0;
            // Does the attachment key exist and is it greater than zero?
            attachmentExists = (reportHistory !== null && reportHistory.getAttachmentKey() > 0);
        }
        var canDownload = this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.REPORT.DOWNLOAD);
        var canPublish = this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.REPORT.PUBLISH);

        this.getView().downloadButton.setEnabled(canDownload && attachmentExists);
        // Only reports belonging to a package can be published.
        this.getView().publishButton.setEnabled(canPublish && attachmentExists && isPackageReport);
    },

    _getReportHistory: function (index) {
        var view = this.getView();
        // If index wasn't specified, then we are getting the currently selected index from the table.
        if (undefined === index) {
            index = view.reportHistoryTable.getSelectedIndex();
        }
        var reportHistory = null;
        if (index >= 0) {
            var context = view.reportHistoryTable.getContextByIndex(index);
            var reportHistoryObj = context.getObject();
            reportHistory = sas.stress.reports.model.ReportHistory.getReportHistory(reportHistoryObj.key);
        }
        return reportHistory;
    },

    createReportButtonPress: function (evt) {
        var controller = this;
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.reports");
        var view = this.getView();
        var existingDialog = view.byId("sas.stress.reports.view.EditReportDialog");
        if (existingDialog) {
            existingDialog.destroy();
        }
        var dialog = sap.ui.jsfragment("sas.stress.reports.view.EditReportDialog", this);
        var availableCategories = [];
        var allCategories = controller._getCategoriesInModel().slice(1); // Skip "ALL"
        for (var categoryIndex = 0; categoryIndex < allCategories.length; ++categoryIndex) {
            // Only add categories that are not read-only
            if (allCategories[categoryIndex].readOnly !== true) {
                availableCategories.push(allCategories[categoryIndex]);
            }
        }
        if (availableCategories.length === 0) {
            // No editable categories available, do not open the create dialog.
            sas.hc.m.MessageDialog.error({text: rb.getText("error.noCategoriesAvailable.txt"), title: rb.getText("newReportDialog.window.title")});
            return;
        }
        var newReportJSON = {
            title: rb.getText("newReportDialog.window.title"),
            name: sas.stress.reports.model.ReportConfig.getNewReportName(),
            templates: [],
            dependentTemplates: [],
            categories: availableCategories
        };
        var model = new sas.risk.common.models.RiskJSONModel(newReportJSON);
        dialog.setModel(model);
        dialog.open();

        // handle report name parameter for process orchestration
        if (controller.createReportParams && controller.createReportParams.reportName) {
            var nameInput = view.byId("editReportNameInput");
            nameInput.setValue(controller.createReportParams.reportName);
            nameInput.fireLiveChange({ value: controller.createReportParams.reportName });
        }

        if (controller.createReportParams && controller.createReportParams.categoryName && controller.createReportParams.categoryName.length > 0) {
            var categorySelect = view.byId("editReportCategorySelect");
            //var categoryModel = dialog.getModel().getData().categories;
            var foundCategory = false;
            for (var i = 0; i < availableCategories.length; ++i) {
                if (availableCategories[i].name.toLocaleLowerCase() === controller.createReportParams.categoryName.toLocaleLowerCase()) {
                    categorySelect.setSelectedKey(availableCategories[i].key);
                    categorySelect.fireChange({ selectedKey: availableCategories[i].key });
                    foundCategory = true;
                    break;
                }
            }
            if (!foundCategory) {
                // Specified category does not exist.
                sas.hc.m.MessageDialog.error({text: rb.getText("reports.processOrchestration.invalidCategory.fmt.txt", [controller.createReportParams.categoryName]), title: rb.getText('newReportDialog.window.title')});
            }

        } else {
            // Get report config category obj for the selected category.
            var categoryToSelect = sas.stress.reports.model.ReportConfigCategory.getReportConfigCategory(this._getSelectedCategoryKey());
            if (categoryToSelect !== undefined) {
                view.byId("editReportCategorySelect").setSelectedKey(categoryToSelect.getKey());
            }
        }

        dialog.setBusy(true);
        //get templates
        sas.stress.reports.StressAPI.listReportTemplates().done(function (templates) {
            if (templates.length > 0) {
                _.each(templates.sort(), function (template) {
                    dialog.getModel().getData().templates.push({ "name": template });
                    if (!controller._endsWith(template, ".plusdelim") && !controller._endsWith(template, ".csv")) { //check if template does not end with .plusdelim or .csv
                        dialog.getModel().getData().dependentTemplates.push({ "name": template });
                    }
                });
                dialog.getModel().refresh();
            }

            //Since the first template is selected by default, pass this template to decide if the Dependent template needs to be shown.
            var reportTemplateSelect = view.byId("reportTemplateSelect");
            controller.toggleDependentReportTemplateFields(reportTemplateSelect.getFirstItem());

            // handle process orchestration commands, if they exist
            if (controller.createReportParams) {
                if (controller.createReportParams.template && controller.createReportParams.template.length > 0) {
                    var templateSelect = view.byId("reportTemplateSelect");
                    var templateModel = dialog.getModel().getData().templates;
                    var foundTemplate = false;
                    for (var i = 0; i < templateModel.length; ++i) {
                        if (templateModel[i].name.toLocaleLowerCase() === controller.createReportParams.template.toLocaleLowerCase()) {
                            templateSelect.setSelectedKey(templateModel[i].name);
                            templateSelect.fireChange({ selectedKey: templateModel[i].name });
                            foundTemplate = true;
                            break;
                        }
                    }
                    if (!foundTemplate) {
                        // Specified template does not exist.
                        sas.hc.m.MessageDialog.error({text: rb.getText("reports.processOrchestration.invalidTemplate.fmt.txt", [controller.createReportParams.template]), title: rb.getText('newReportDialog.window.title')});
                    }
                    if (controller.createReportParams.dependentTemplate && controller.createReportParams.dependentTemplate.length > 0) {
                        var dependentTemplateSelect = view.byId("reportDependentTemplateSelect");
                        var dependentTemplateModel = dialog.getModel().getData().dependentTemplates;
                        var foundDependentTemplate = false;
                        for (var i = 0; i < dependentTemplateModel.length; ++i) {
                            if (dependentTemplateModel[i].name.toLocaleLowerCase() === controller.createReportParams.dependentTemplate.toLocaleLowerCase()) {
                                dependentTemplateSelect.setSelectedKey(dependentTemplateModel[i].name);
                                dependentTemplateSelect.fireChange({ selectedKey: dependentTemplateModel[i].name });
                                foundDependentTemplate = true;
                                break;
                            }
                        }
                        if (!foundDependentTemplate) {
                            // Specified dependent template does not exist.
                            sas.hc.m.MessageDialog.error({text: rb.getText("reports.processOrchestration.invalidDependentTemplate.fmt.txt", [controller.createReportParams.dependentTemplate]), title: rb.getText('newReportDialog.window.title')});
                        }
                    }
                } else if (controller.createReportParams && controller.createReportParams.dependentTemplate && controller.createReportParams.dependentTemplate.length > 0) {
                    // Can't specify a dependent template without specifying a template.
                    sas.hc.m.MessageDialog.error({text: rb.getText("reports.processOrchestration.dependentWithoutTemplate.txt"), title: rb.getText('newReportDialog.window.title')});
                }
                delete controller.createReportParams;
            }

            dialog.setBusy(false);
        }).fail(function (jqxhr) {
            dialog.setBusy(false);
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });
    },

    editReportButtonPress: function (evt) {
        var controller = this;
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.reports");
        var view = this.getView();
        var reportConfig = this._getReportConfig();
        if (null === reportConfig) {
            return;
        }

        var existingDialog = view.byId("sas.stress.reports.view.EditReportDialog");
        if (existingDialog) {
            existingDialog.destroy();
        }
        var dialog = sap.ui.jsfragment("sas.stress.reports.view.EditReportDialog", this);
        var editReportJSON = {
            title: rb.getText("editReportDialog.window.title"),
            key: reportConfig.key,
            projectKey: reportConfig.projectKey,
            name: reportConfig.reportName,
            templates: [],
            dependentTemplates: [],
            categories: controller._getCategoriesInModel().slice(1) // Skip "ALL"
        };
        var model = new sas.risk.common.models.RiskJSONModel(editReportJSON);
        dialog.setModel(model);
        dialog.open();

        //Get report config category obj for the selected category.
        var categoryToSelect = sas.stress.reports.model.ReportConfigCategory.getReportConfigCategory(reportConfig.reportConfigCategoryKey);
        if (categoryToSelect !== undefined) {
            view.byId("editReportCategorySelect").setSelectedKey(categoryToSelect.getKey());
        }

        dialog.setBusy(true);
        //get templates
        sas.stress.reports.StressAPI.listReportTemplates().done(function (templates) {
            if (templates.length > 0) {
                _.each(templates.sort(), function (template) {
                    dialog.getModel().getData().templates.push({ "name": template });
                    //check if template does not end with .plusdelim or .csv
                    if (!controller._endsWith(template, ".plusdelim") && !controller._endsWith(template, ".csv")) {
                        dialog.getModel().getData().dependentTemplates.push({ "name": template });
                    }
                });
                dialog.getModel().refresh();
            }

            // Since the first template is selected by default, pass this template to decide if the Dependent template needs to be shown.
            var reportTemplateSelect = view.byId("reportTemplateSelect");
            controller.toggleDependentReportTemplateFields(reportTemplateSelect.getFirstItem());

            // Select the template.
            var templateSelect = view.byId("reportTemplateSelect");
            var templateModel = dialog.getModel().getData().templates;
            var foundTemplate = false;
            for (var i = 0; i < templateModel.length; ++i) {
                if (templateModel[i].name.toLocaleLowerCase() === reportConfig.template.toLocaleLowerCase()) {
                    templateSelect.setSelectedKey(templateModel[i].name);
                    templateSelect.fireChange({ selectedKey: templateModel[i].name });
                    foundTemplate = true;
                    break;
                }
            }
            if (!foundTemplate) {
                // Specified template does not exist.
                sas.hc.m.MessageDialog.error({text: rb.getText("editReportDialog.invalidTemplate.fmt.txt", [reportConfig.template]), title: rb.getText('reportConfigsTable.editReportButton.tip')});
            }

            // If a dependent template was saved before, select it.
            if (reportConfig.dependentTemplate && reportConfig.dependentTemplate.length > 0) {
                var dependentTemplateSelect = view.byId("reportDependentTemplateSelect");
                var dependentTemplateModel = dialog.getModel().getData().dependentTemplates;
                var foundDependentTemplate = false;
                for (var i = 0; i < dependentTemplateModel.length; ++i) {
                    if (dependentTemplateModel[i].name.toLocaleLowerCase() === reportConfig.dependentTemplate.toLocaleLowerCase()) {
                        dependentTemplateSelect.setSelectedKey(dependentTemplateModel[i].name);
                        dependentTemplateSelect.fireChange({ selectedKey: dependentTemplateModel[i].name });
                        foundDependentTemplate = true;
                        break;
                    }
                }
                if (!foundDependentTemplate) {
                    // Specified dependent template does not exist.
                    sas.hc.m.MessageDialog.error({text: rb.getText("editReportDialog.invalidDependentTemplate.fmt.txt", [reportConfig.dependentTemplate]), title: rb.getText('reportConfigsTable.editReportButton.tip')});
                }
            }
            dialog.setBusy(false);
        }).fail(function (jqxhr) {
            dialog.setBusy(false);
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });
    },

    editReportDialogSaveButtonPress: function (event) {
        var editReportDialog = this.getView().byId("sas.stress.reports.view.EditReportDialog");
        var controller = this;
        var view = this.getView();
        var reportTemplateSelectedItem = view.byId("reportTemplateSelect").getSelectedItem();
        if (reportTemplateSelectedItem === null) {
            return;
        }
        var newReportName = view.byId("editReportNameInput").getValue();
        var selectedCategory = view.byId("editReportCategorySelect").getSelectedItem();
        var reportDependentTemplateSelect = view.byId("reportDependentTemplateSelect");
        var dependentTemplate = null;
        var selectedReportTemplateName = reportTemplateSelectedItem.getText();
        //check if template name ends with .plusdelim or .csv
        var dependentTemplateRequired = controller._endsWith(selectedReportTemplateName, ".plusdelim") ||
                                        controller._endsWith(selectedReportTemplateName, ".csv");
        if (dependentTemplateRequired === true && reportDependentTemplateSelect.getSelectedItem() !== null) {
            dependentTemplate = reportDependentTemplateSelect.getSelectedItem().getText();
        }

        var reportConfigJson = editReportDialog.getModel().getData();
        var editReportConfig = new sas.stress.reports.model.ReportConfig.getReportConfig(reportConfigJson.key);
        editReportConfig.reportName = newReportName;
        var categoryChanged = editReportConfig.reportConfigCategoryKey !== selectedCategory.getKey();
        if (categoryChanged) {
            editReportConfig.reportConfigCategoryKey = selectedCategory.getKey();
            editReportConfig.reportConfigCategory = selectedCategory.getText();
        }
        editReportConfig.template = selectedReportTemplateName;
        editReportConfig.dependentTemplate = dependentTemplate;

        sas.stress.reports.model.ReportConfig.updateReportConfig(editReportConfig).done(function () {
            var reportConfigsTableModel = view.reportConfigsTable.getModel("reportConfigs");
            var reportConfigToUpdate = _.find(reportConfigsTableModel.getData(), function (reportConfigObj) {
                return reportConfigObj.key === editReportConfig.getKey();
            });
            if (reportConfigToUpdate) {
                reportConfigToUpdate.key = editReportConfig.getKey();
                reportConfigToUpdate.projectKey = editReportConfig.getProjectKey();
                reportConfigToUpdate.reportName = editReportConfig.getReportName();
                reportConfigToUpdate.template = editReportConfig.getTemplate();
                reportConfigToUpdate.dependentTemplate = editReportConfig.getDependentTemplate();
                reportConfigToUpdate.reportConfigCategory = editReportConfig.getCategoryName();
                reportConfigToUpdate.reportConfigCategoryKey = editReportConfig.getCategoryKey();

                reportConfigsTableModel.refresh();
                editReportDialog.close();
            }

            // Check if the category has changed
            if (categoryChanged) {
                // Select the category that was used
                view.categoryFilter.setSelectedCategory(editReportConfig.getCategoryName(), true);
                //view.categoryFilter.setSelectedItem(_.find(controller._getCategoriesListItems(), function (category) {
                //    return category.getTitle() === editReportConfig.getCategoryName();
                //}), true);
                //controller.categoriesListSelectionChange();
                controller._setReportConfigsCountInCategoriesList();
            }
        }).fail(function (jqxhr) {
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });
    },

    _getFilesOnServer: function () {
        var deferred= $.Deferred();
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.data");
        var view = this.getView();
        var projectKey = this.project.getId();
        var controller = this;
        $.when(sas.stress.reports.StressAPI.listRequiredFilesFrom(projectKey)).done(function (landingAreaFiles) {
            _.each(landingAreaFiles,function(file){
                // update the name to be the typelessSurrogatePath
                file.actualName=file.name;
                file.name=file.typelessSurrogatePath;
            });
            var fileSort = function (a, b) {
                var aName = a.name.toLowerCase();
                var bName = b.name.toLowerCase();
                return ((aName < bName) ? -1 : ((aName > bName) ? 1 : 0));
            };
            var sortedLandingAreaFiles = landingAreaFiles.sort(fileSort);
            deferred.resolve(sortedLandingAreaFiles);

        }).fail(function (jqxhr) {
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            //deferred.reject(jqxhr);
        });

        return deferred;
    },

    landingAreaDataListChange: function (event) {
        var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();
        var canRun = capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.REPORT.RUN);

        this.runReportButton.setEnabled(canRun);
    },

    runReportsButtonPress: function (evt) {
        var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();
        var controller = this;
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.reports");
        var view = this.getView();
        var projectKey = this.project.getId();
        var selectedIndices = view.reportConfigsTable.getSelectedIndices();
        if (selectedIndices.length > 0) {
            var keys = [];
            this.reportHasFilters = false;
            this.isTaxonomyPackageReport = false;
            this.reportHasLandingAreaFile = false;
            var reportConfigObj;
            for (var n = 0; n < selectedIndices.length; ++n) {
                var index = selectedIndices[n];
                reportConfigObj = this._getReportConfig(index);
                this.reportHasLandingAreaFile = (this.reportHasLandingAreaFile || !! reportConfigObj.requiresFileFrom);
                this.reportHasFilters = (this.reportHasFilters || (reportConfigObj.getDataName() !== "com.sas.risk.output.data.NoData" && 0 >= reportConfigObj.getTaxonomyPackageOwners().length));
                this.isTaxonomyPackageReport = (this.isTaxonomyPackageReport || 0 < reportConfigObj.getTaxonomyPackageOwners().length);
                keys.push(reportConfigObj.getKey());
            }
            //Show dialog to take a comment.
            var existingDialog = this.getView().byId("sas.stress.reports.view.RunReportDialog");
            if (existingDialog) {
                existingDialog.destroy();
            }
            var dialog = sap.ui.jsfragment("sas.stress.reports.view.RunReportDialog", this);
            var title = rb.getText("runReportDialog.window.runOneReport.title.fmt.txt", [reportConfigObj.getReportName()]);
            if (keys.length > 1) {
                title = rb.getText("runReportDialog.window.runMultipleReports.title.fmt.txt", [keys.length]);
            }
            dialog.setTitle(title);

            //reportConfigObj.requiresFileFrom = "landing_area";
            //var sortedLandingAreaFiles = {};
            var newReportJSON = {
                projectKey: projectKey, keys: keys,
                landingAreaFile: {},
                reportHasFilters:this.reportHasFilters,
                reportHasComments:true,
                reportHasLandingAreaFile:this.reportHasLandingAreaFile
            };
            var model = new sas.risk.common.models.RiskJSONModel(newReportJSON);
            if ( this.reportHasLandingAreaFile ) {
                this._getFilesOnServer().done(function(sortedLandingAreaFiles){
                    model.setProperty("/reportHasFilters", false);
                    model.setProperty("/landingAreaFiles", sortedLandingAreaFiles );
                    dialog.setModel(model);
                    // Set whether the run buttons are enabled initially. If all reports have no data or were generated by a package,
                    // then no filters are to be displayed and the buttons can be enabled initially. Otherwise the user must
                    // select one or more filters, in which case the buttons must not be enabled initially.
                    //var canRun = capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.REPORT.RUN);
                    //controller.runReportButton.setEnabled(canRun && controller.reportHasNoFilters);
                    //controller.runAndExportReportButton.setEnabled(canRun && controller.reportHasNoFilters && !controller.reportHasLandingAreaFile);
                    controller.enableRunButtons(false);
                    dialog.open();
                });

            } else {
                dialog.setModel(model);
                // Set whether the run buttons are enabled initially. If all reports have no data or were generated by a package,
                // then no filters are to be displayed and the buttons can be enabled initially. Otherwise the user must
                // select one or more filters, in which case the buttons must not be enabled initially.
                //var canRun = capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.REPORT.RUN);
                //controller.runReportButton.setEnabled(canRun && controller.reportHasNoFilters);
                //controller.runAndExportReportButton.setEnabled(canRun && controller.reportHasNoFilters && !controller.reportHasLandingAreaFile);
                this.enableRunButtons(true);
                dialog.open();
            }
        }
    },
    // Set whether the run buttons are enabled.
    enableRunButtons: function (enable) {
        // If all reports have no data with sas template or were generated by a package, then no filters
        // are to be displayed and the buttons can be enabled initially.  Otherwise the user must select
        // one or more filters, in which case the buttons must be enabled when all filters are selected.
        var allFiltersSet = !this.reportHasFilters || !this.validFiltersWindowShade.hasUnsetFilters();
        var canRun = this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.REPORT.RUN);
        this.runReportButton.setEnabled(enable && canRun && allFiltersSet
            && !this.reportHasLandingAreaFile);
        var canExport = this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.REPORT.EXPORT);  //defect
        // this.runAndExportReportButton.setEnabled(enable && canExport && allFiltersSet
        //     && !this.reportHasLandingAreaFile && !this.isTaxonomyPackageReport);
    },

    runReportDialogFilterSelectHandler: function () {
        this.enableRunButtons(true);
        //var enableRunButtons = false;
        //if (this.reportHasNoFilters) {
        //    enableRunButtons = true;
        //} else {
        //    enableRunButtons = !this.validFiltersWindowShade.hasUnsetFilters();
        //}
        //var canRun = this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.REPORT.RUN);
        //this.runReportButton.setEnabled(canRun && enableRunButtons);
        //var canExport = this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.REPORT.EXPORT);
        //this.runAndExportReportButton.setEnabled(canExport && enableRunButtons && !this.isTaxonomyPackageReport);
    },

    runReportDialogRunButtonPress: function (event) {
        var controller = this;
        var runReportDialog = this.getView().byId("sas.stress.reports.view.RunReportDialog");
        runReportDialog.setBusy(true);
        var comment = this.getView().byId("runReportCommentInput").getValue();
        var projectKey = this.project.getId();
        var runReportDialogModelData = runReportDialog.getModel().getData();
        var reportConfigIds = runReportDialogModelData.keys;
        var filters = this.validFiltersWindowShade.getSelectedFilters();
        var fileId = null;
//@@@
        if (true === this.reportHasLandingAreaFile) {
            if (this.landingAreaDataList.getListItems().length > 0) {
                var selectedItem = this.landingAreaDataList.getSelectedItem();
                fileId = selectedItem.id || selectedItem.getProperty("key");
            }
        }

        sas.stress.reports.StressAPI.startReportGeneration(projectKey, filters, comment, reportConfigIds, fileId).done(function () {
            controller._clearReportHistoryTableFilter();
            controller.getReportConfigs(projectKey, true).done(function(cfgs, running){
                controller._loadReportHistory();
                controller._poll(true);
            }).fail (function(jqxhr){
                runReportDialog.setBusy(false);
                sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            });
            runReportDialog.setBusy(false);
            runReportDialog.close();
        }).fail(function (jqxhr) {
            runReportDialog.setBusy(false);
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });
    },

    runReportDialogRunAndExportButtonPress: function (event) {
        var controller = this;
        var runReportDialog = this.getView().byId("sas.stress.reports.view.RunReportDialog");
        runReportDialog.setBusy(true);
        var comment = this.getView().byId("runReportCommentInput").getValue();
        var projectKey = this.project.getId();
        var runReportDialogModelData = runReportDialog.getModel().getData();
        var reportConfigIds = runReportDialogModelData.keys;
        var filters = this.validFiltersWindowShade.getSelectedFilters();

        sas.stress.reports.StressAPI.startFinalReportGeneration(projectKey, filters, comment, reportConfigIds).done(function () {
            controller._clearReportHistoryTableFilter();
            controller.getReportConfigs(projectKey).done(function(){
                controller._loadReportHistory();
                controller._poll(true);
            }).fail (function(jqxhr){
                runReportDialog.setBusy(false);
                sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            });

            runReportDialog.setBusy(false);
            runReportDialog.close();
        }).fail(function (jqxhr) {
            runReportDialog.setBusy(false);
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });
    },

    _endsWith: function (str, suffix) {
        return str.indexOf(suffix, str.length - suffix.length) !== -1;
    },

    reportTemplateSelectChange: function () {
        var reportTemplateSelect = this.getView().byId("reportTemplateSelect");
        this.toggleDependentReportTemplateFields(reportTemplateSelect.getSelectedItem());
    },

    /* Shows Dependent Template Label and Select if selected template is .plusdelim. */
    toggleDependentReportTemplateFields: function (selectedReportTemplate) {
        var dependentFieldsVisible = false;
        if (selectedReportTemplate) {
            var selectedReportTemplateName = selectedReportTemplate.getText();
            //check if template name ends with .plusdelim or .csv
            dependentFieldsVisible = this._endsWith(selectedReportTemplateName, ".plusdelim") ||
                                     this._endsWith(selectedReportTemplateName, ".csv");
        }
        var dependentTemplateLabel = this.getView().byId("dependentTemplateLabel");
        dependentTemplateLabel.toggleStyleClass("ui-state-disabled", !dependentFieldsVisible);
        var reportDependentTemplateSelect = this.getView().byId("reportDependentTemplateSelect");
        reportDependentTemplateSelect.setEnabled(dependentFieldsVisible);
        var reportDependentTemplateLabel = this.getView().byId("dependentTemplateLabel");
        reportDependentTemplateLabel.setRequired(dependentFieldsVisible);
    },

    handleNameInputLiveChange: function (event) {
        var isEdit = this._isEditReport();
        this.nameInputLiveChange(event.getSource(), isEdit);
    },

    nameInputLiveChange: function (nameInput, isEdit) {
        var view = this.getView();
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.reports");
        var isValid = true;
        var newName = nameInput.getValue();

        if (isEdit) {
            var reportKey = view.byId("sas.stress.reports.view.EditReportDialog").getModel().getData().key;
            var reportBeingEdited = sas.stress.reports.model.ReportConfig.getReportConfig(reportKey);
        }

        //Check if name is empty.
        if (newName === null || newName.trim() === "") {
            isValid = false;
            nameInput.setValueStateText(rb.getText("error.requiredField.txt"));
            nameInput.setValueState(sap.ui.core.ValueState.Error);
        } else {
            // Check if the name is available to use
            isValid = sas.stress.reports.model.ReportConfig.isNameAvailable(newName, reportBeingEdited);
            if (!isValid) {
                nameInput.setValueStateText(rb.getText("error.nameAlreadyInUse.txt"));
                nameInput.setValueState(sap.ui.core.ValueState.Error);
            } else {
                //Name is good.
                nameInput.setValueStateText("");
                nameInput.setValueState(sap.ui.core.ValueState.None);
            }
        }

        // Check if the Category field is not empty
        if (isValid) {
            var editReportCategorySelectedItem = view.byId("editReportCategorySelect").getSelectedItem();
            if (editReportCategorySelectedItem === null) {
                isValid = false;
            }
        }
        // Check if the Template field is not empty
        if (isValid) {
            var reportTemplateSelectedItem = view.byId("reportTemplateSelect").getSelectedItem();
            if (reportTemplateSelectedItem === null) {
                isValid = false;
            }
        }

        view.byId("editReportDialogSaveReportButton").setEnabled(isValid);
    },

    handleReportDialogSaveButtonPress: function (event) {
        var isEdit = this._isEditReport();
        if (isEdit) {
            this.editReportDialogSaveButtonPress(event);
        } else {
            this.newReportDialogSaveButtonPress(event);
        }
    },

    newReportDialogSaveButtonPress: function (event) {
        var newReportDialog = this.getView().byId("sas.stress.reports.view.EditReportDialog");
        var controller = this;
        var view = this.getView();
        var projectKey = this.project.getId();
        var reportTemplateSelectedItem = view.byId("reportTemplateSelect").getSelectedItem();
        if (reportTemplateSelectedItem === null) {
            return;
        }
        var newReportName = view.byId("editReportNameInput").getValue();
        var selectedCategory = view.byId("editReportCategorySelect").getSelectedItem();
        var reportDependentTemplateSelect = view.byId("reportDependentTemplateSelect");
        var dependentTemplate = null;
        var selectedReportTemplateName = reportTemplateSelectedItem.getText();
        //check if template name ends with .plusdelim or .csv
        var dependentTemplateRequired = controller._endsWith(selectedReportTemplateName, ".plusdelim") ||
                                        controller._endsWith(selectedReportTemplateName, ".csv");
        if (dependentTemplateRequired === true && reportDependentTemplateSelect.getSelectedItem() !== null) {
            dependentTemplate = reportDependentTemplateSelect.getSelectedItem().getText();
        }

        var newReportConfig = new sas.stress.reports.model.ReportConfig(0, projectKey, selectedCategory.getKey(),
            selectedCategory.getText(), 0, newReportName, selectedReportTemplateName, [], null, null,
            dependentTemplate, 0, null, []);

        sas.stress.reports.model.ReportConfig.addReportConfig(newReportConfig).done(function (reportConfig) {
            var reportConfigsTableModel = view.reportConfigsTable.getModel("reportConfigs");
            reportConfigsTableModel.getData().push(sas.stress.reports.model.ReportConfig.toJson(reportConfig));
            reportConfigsTableModel.refresh();
            newReportDialog.close();

            // Select the category that was used for the new report config
            view.categoryFilter.setSelectedCategory(reportConfig.getCategoryName(), true);
            //view.categoryFilter.setSelectedItem(_.find(controller._getCategoriesListItems(), function (category) {
            //    return category.getTitle() === reportConfig.getCategoryName();
            //}), true);
            //controller.categoriesListSelectionChange();
            controller._setReportConfigsCountInCategoriesList();
        }).fail(function (jqxhr) {
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });
    },

    deleteReportButtonPress: function () {
        var view = this.getView();
        var controller = this;
        var index = view.reportConfigsTable.getSelectedIndex();
        if (index >= 0) {
            var reportConfigObj = this._getReportConfig(index);
            var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.reports");

            sas.hc.m.MessageDialog.warning({
                text: rb.getText("reportConfigsTable.delete.confirmation.fmt.txt", [reportConfigObj.getReportName()]),
                title: rb.getText("reportConfigsTable.delete.window.title"),
                buttonActions: [sas.hc.m.MessageDialogAction.Delete, sas.hc.m.MessageDialogAction.Cancel],
                buttonPress: function (event) {
                    var action = event.getParameter("origin").getMessageDialogAction();
                    if (action === sas.hc.m.MessageDialogAction.Delete) {
                        sas.stress.reports.model.ReportConfig.deleteReportConfig(reportConfigObj.getProjectKey(), reportConfigObj.getKey()).done(function () {
                            var reportConfigsTableModel = view.reportConfigsTable.getModel("reportConfigs");
                            var reportConfigsTableModelData = reportConfigsTableModel.getData();
                            for (var i = reportConfigsTableModelData.length - 1; i >= 0; i--) {
                                if (reportConfigsTableModelData[i].key === reportConfigObj.getKey()) {
                                    reportConfigsTableModelData.splice(i, 1);
                                    break;
                                }
                            }
                            reportConfigsTableModel.refresh();
                            view.reportConfigsTable.clearSelection();

                            controller._setReportConfigsCountInCategoriesList();
                        }).fail(function (jqxhr) {
                            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
                        });
                    }
                }
            });

        }
    },

    onReportConfigsTableFilterChanged: function (e) {
        var value = e.getSource().getValue() || "";
        this._filterReportTable(value);
    },

    _filterReportTable: function (value) {
        var view = this.getView();
        if (!value) {
            value = view.searchField.getValue() || "";
        }
        var nameFilter = new sap.ui.model.Filter("reportName", sap.ui.model.FilterOperator.Contains, value);
        var templateFilter = new sap.ui.model.Filter("template", sap.ui.model.FilterOperator.Contains, value);
        var dependentTemplateFilter = new sap.ui.model.Filter("dependentTemplate", sap.ui.model.FilterOperator.Contains, value);
        var statusFilter = new sap.ui.model.Filter("reportStatus", sap.ui.model.FilterOperator.Contains, value);

        var searchFilter = new sap.ui.model.Filter([nameFilter, templateFilter, dependentTemplateFilter, statusFilter]);     // OR by default

        //S1297730 apply category filter at the same time
        var categoryKey = this._getSelectedCategoryKey();
        if (categoryKey === 0) {
            var allFilters = searchFilter;
        } else {
            var categoryFilter = new sap.ui.model.Filter("reportConfigCategoryKey", sap.ui.model.FilterOperator.EQ, categoryKey);
            allFilters = new sap.ui.model.Filter([searchFilter, categoryFilter], true);   //AND together
        }
        var binding = this.getView().reportConfigsTable.getBinding("rows");
        binding.filter(new sap.ui.model.Filter(allFilters));
    },

    onReportHistoryTableFilterChanged: function (e) {
        var value = e.getSource().getValue() || "";
        this._filterReportHistoryTable(value);
    },

    _filterReportHistoryTable: function (value) {
        var createdByFilter = new sap.ui.model.Filter("createdBy", sap.ui.model.FilterOperator.Contains, value);
        var commentFilter = new sap.ui.model.Filter("comment", sap.ui.model.FilterOperator.Contains, value);
        var statusFilter = new sap.ui.model.Filter("reportStatus", sap.ui.model.FilterOperator.Contains, value);
        var iterationNumFilter = new sap.ui.model.Filter("iterationNum", sap.ui.model.FilterOperator.EQ, value);
        var allFilters = new sap.ui.model.Filter([createdByFilter, commentFilter, statusFilter, iterationNumFilter]);
        var binding = this.getView().reportHistoryTable.getBinding("rows");
        if (binding) binding.filter(allFilters);
    },

    onCellClick: function (event) {
        var ctx = event.getSource().getContextByIndex(event.getParameter("rowIndex"));
        var cell = event.getParameter("cellControl");
        // Check if the click was in the messages cell, if not then we want to ignore it.
        if ((cell.getId().indexOf("history-messages")>=0) || (cell.getId().indexOf("config-messages")>=0)) {
            var obj = ctx.getObject();
            // Only acknowledge the click if at least one message exists.
            if (obj.messages.length > 0) {
                var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.wkbench");
                var existingDialog = this.getView().byId("sas.stress.reports.view.MessagesDialog");
                if (existingDialog) {
                    existingDialog.destroy();
                }
                var dialog = sap.ui.jsfragment("sas.stress.reports.view.MessagesDialog", this);

                var messagesStr="";
                if (obj.messages.length > 1) {
                    messagesStr = rb.getText("list.bullet.txt") + obj.messages[0];
                    for (var index = 1; index < obj.messages.length; ++index) {
                        messagesStr += "\n"+ rb.getText("list.bullet.txt") + obj.messages[index];
                    }
                }else{
                    messagesStr = obj.messages[0];
                }

                var messagesDialogJSON = {
                    messages: messagesStr
                };
                var model = new sas.risk.common.models.RiskJSONModel(messagesDialogJSON);
                dialog.setModel(model);
                dialog.open();
                // Prevent message click event from also selecting row
                event.preventDefault();
            }
        }
    },

    categoriesListCreateButtonPress: function (event) {
        var existingDialog = this.getView().byId("sas.stress.wkbench.view.CategoryDialog");
        if (existingDialog) {
            existingDialog.destroy();
        }
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.reports");
        var dialog = sap.ui.jsfragment("sas.stress.wkbench.view.CategoryDialog", this);
        var newCategoryJSON = {
            title: rb.getText("addCategoryDialog.window.title"),
            name: sas.stress.reports.model.ReportConfigCategory.getNewCategoryName()
        };
        var jsonModel = new sas.risk.common.models.RiskJSONModel(newCategoryJSON);
        dialog.setModel(jsonModel);
        dialog.open();
    },

    addCategoryDialogSaveButtonPress: function (event) {
        var controller = this;
        var view = this.getView();
        var addCategoryDialog = view.byId("sas.stress.wkbench.view.CategoryDialog");
        addCategoryDialog.setBusy(true);
        var categoryName = view.byId("CategoryDialog.categoryNameInput").getValue();
        var categoryKey = 0; //New Category
        var projectKey = this.project.getId();
        var newReportConfigCategory = new sas.stress.reports.model.ReportConfigCategory(projectKey, categoryKey, categoryName.trim(), false);
        sas.stress.reports.model.ReportConfigCategory.addReportConfigCategory(projectKey, newReportConfigCategory).done(function (reportConfigCategory) {
            var categoriesModelData = controller._getCategoriesInModel();
            categoriesModelData.push(sas.stress.reports.model.ReportConfigCategory.toJson(reportConfigCategory));
            controller._sortReportConfigCategoriesJson(categoriesModelData);
            view.categoryFilter.getModel().refresh();
            //Select the new category
            view.categoryFilter.setSelectedCategory(reportConfigCategory.getName(), true);
            //view.categoryFilter.setSelectedItem(_.find(controller._getCategoriesListItems(), function (category) {
            //    return category.getTitle() === reportConfigCategory.getName();
            //}), true);
            //controller.categoriesListSelectionChange();
            addCategoryDialog.setBusy(false);
            addCategoryDialog.close();
        }).fail(function (jqxhr) {
            addCategoryDialog.setBusy(false);
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });
    },

    /* Called when the user enters a value in the Name field in the Add/Edit Category dialog. Validates the name by
     * checking if it is empty or duplicate of an existing category.
     * If validation fails, displays an error message around the Name field and disables the Save button. */
    handleCategoryNameInputChange: function (event) {
        var view = this.getView();
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.reports");
        var isValid = true;
        var nameInput = event.getSource();
        var newName = nameInput.getValue().trim();

        var categoryKey = view.byId("sas.stress.wkbench.view.CategoryDialog").getModel().getData().key;
        var categoryBeingEdited = sas.stress.reports.model.ReportConfigCategory.getReportConfigCategory(categoryKey);

        //Check if name is empty.
        if (newName === null || newName === "") {
            isValid = false;
            nameInput.setValueStateText(rb.getText("error.requiredField.txt"));
            nameInput.setValueState(sap.ui.core.ValueState.Error);
        } else {
            isValid = sas.stress.reports.model.ReportConfigCategory.isNameAvailable(newName, categoryBeingEdited);
            if (!isValid) {
                nameInput.setValueStateText(rb.getText("error.nameAlreadyInUse.txt"));
                nameInput.setValueState(sap.ui.core.ValueState.Error);
            } else {
                //Name is good.
                nameInput.setValueStateText("");
                nameInput.setValueState(sap.ui.core.ValueState.None);
            }
        }
        view.byId("CategoryDialog.saveCategoryButton").setEnabled(isValid);
    },

    handleCategoryDialogSaveButtonPress: function (event) {
        var isEdit = this._isEditCategory();
        if (isEdit) {
            this.editCategoryDialogSaveButtonPress(event);
        } else {
            this.addCategoryDialogSaveButtonPress(event);
        }
    },

    categoriesListEditButtonPress: function (event) {
        var view = this.getView();
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.reports");
        var selectedCategory = view.categoryFilter.getSelectedCategory();
        if (selectedCategory === null) {
            return;
        }
        var reportConfigCategory = sas.stress.reports.model.ReportConfigCategory.getReportConfigCategory(this._getSelectedCategoryKey());
        var existingDialog = view.byId("sas.stress.wkbench.view.CategoryDialog");
        if (existingDialog) {
            existingDialog.destroy();
        }
        var dialog = sap.ui.jsfragment("sas.stress.wkbench.view.CategoryDialog", this);
        var reportConfigCategoryJSON = {
            title: rb.getText("editCategoryDialog.window.title"),
            key: reportConfigCategory.getKey(),
            name: reportConfigCategory.getName()
        };
        var jsonModel = new sas.risk.common.models.RiskJSONModel(reportConfigCategoryJSON);
        dialog.setModel(jsonModel);
        dialog.open();
    },

    editCategoryDialogSaveButtonPress: function (event) {
        var view = this.getView();
        var controller = this;
        var editCategoryDialog = view.byId("sas.stress.wkbench.view.CategoryDialog");
        var categoryKey = editCategoryDialog.getModel().getData().key;
        var projectKey = this.project.getId();
        var categoryName = view.byId("CategoryDialog.categoryNameInput").getValue();
        var reportConfigCategory = new sas.stress.reports.model.ReportConfigCategory(projectKey, categoryKey, categoryName.trim(), false);
        editCategoryDialog.setBusy(true);
        sas.stress.reports.model.ReportConfigCategory.updateReportConfigCategory(projectKey, reportConfigCategory).done(function (updatedCategory) {
            //Update the report config category in the model.
            var category = _.find(controller._getCategoriesInModel(), function (category) {
                return category.key === updatedCategory.getKey();
            });
            if (category) {
                category.name = updatedCategory.getName();
                controller._sortReportConfigCategoriesJson(controller._getCategoriesInModel());
                view.categoryFilter.getModel().refresh();
                //Select the new category
                view.categoryFilter.setSelectedCategory(updatedCategory.getName(), true);
                //view.categoryFilter.setSelectedItem(_.find(controller._getCategoriesListItems(), function (reportConfigCategory) {
                //    return reportConfigCategory.getTitle() === category.name;
                //}));
            }
            editCategoryDialog.setBusy(false);
            editCategoryDialog.close();
        }).fail(function (jqxhr) {
            editCategoryDialog.setBusy(false);
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });
    },

    categoriesListDeleteButtonPress: function (event) {
        var view = this.getView();
        var controller = this;
        var selectedCategory = view.categoryFilter.getSelectedCategory();
        if (!selectedCategory) {
            return;
        }
        var selectedCategoryKey = controller._getSelectedCategoryKey();
        var projectKey = this.project.getId();
        //Check if there are report configs belonging to this category
        var reportConfigs = _.filter(view.reportConfigsTable.getModel("reportConfigs").getData(), function (reportConfig) {
            return reportConfig.reportConfigCategoryKey === selectedCategoryKey;
        });
        var runningReports= _.filter(reportConfigs,function(reportConfig){
            return "Executing"===reportConfig.reportStatus;
        });
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.reports");
        var message = rb.getText("categoriesList.delete.category.confirmation.msg.fmt.txt", [selectedCategory]);
        if (runningReports.length>0){
            var reports=[];
            _.each(runningReports,function(report){
                reports.push(report.reportName);
            });
            message=sas.stress.wkbench.StressUtil.getConfirmationMessage(rb.getText("categoriesList.delete.category.reports.running.msg.txt"), reports);
            sas.hc.m.MessageDialog.warning(message, rb.getText("categoriesList.delete.category.reports.running.title.txt"));

            return;
        }
        if (reportConfigs.length > 0) {
            /*
            project locking enforcement for reports
             Report categories may be deleted only if they are empty (not allowing reports to be deleted).
             */
            var configs=_.some(reportConfigs,function(reportConfig){
                return !reportConfig.canBeEditedIfProjectIsLocked;
            });
            if (configs&&configs.length>0){
                // cannot delete because there are configs created before project locked
                return;
            }
            message += "\n" + rb.getText("categoriesList.reportConfigsWillBeDeleted.msg.fmt.txt", [reportConfigs.length]);
        }

        sas.hc.m.MessageDialog.warning({
            text: message,
            title: rb.getText("categoriesList.delete.category.window.title"),
            buttonActions: [sas.hc.m.MessageDialogAction.Delete, sas.hc.m.MessageDialogAction.Cancel],
            buttonPress: function (event) {
                var action = event.getParameter("origin").getMessageDialogAction();
                if (action === sas.hc.m.MessageDialogAction.Delete) {
                    sas.stress.reports.model.ReportConfigCategory.deleteReportConfigCategory(projectKey, selectedCategoryKey).done(function () {
                        var categoriesListModel = view.categoryFilter.getModel();
                        $.each(categoriesListModel.getData().categories, function (index, category) {
                            if (category.key === selectedCategoryKey) {
                                categoriesListModel.getData().categories.splice(index, 1);
                                return false; //break for loop
                            }
                        });
                        categoriesListModel.refresh();
                        var categoriesListItems = controller._getCategoriesListItems();
                        if (categoriesListItems.length > 0) {
                            view.categoryFilter.setSelectedItem(categoriesListItems[0], true);
                            //controller.categoriesListSelectionChange();
                        }
                        // If any report configs were deleted, refresh list of report configs and update the count in the categories list
                        if (reportConfigs.length > 0) {
                            controller.getReportConfigs(projectKey, true).done(function () {
                                controller._setReportConfigsCountInCategoriesList();
                            });
                        }
                    }).fail(function (jqxhr) {
                        sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
                    });
                }
            }
        });

    },

    categoriesListSelectionChange: function () {
        this._toggleCategoriesListToolbarButtons();
        this._applyCategoryFilter();
        this._savePreferredCategory();
    },

    _toggleCategoriesListToolbarButtons: function () {
        var view = this.getView();
        var selectedCategory = view.categoryFilter.getSelectedItem();
        if (selectedCategory !== null) {
            var categoryKey = this._getSelectedCategoryKey();
            var reportConfigCategory = _.find(this._getCategoriesInModel(), function (data) {
                return data.key === categoryKey;
            });
            if (reportConfigCategory !== undefined) {
                var readOnly = reportConfigCategory.readOnly;
            }
        }
        var canCreate = this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.REPORT.CREATE);
        var canEdit = this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.REPORT.UPDATE);
        var canDelete = this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.REPORT.DELETE);
        // Enable edit and delete if category is not read-only and not "All".
        view.byId("reportCategoriesList.createButton").setEnabled(canCreate);
        view.byId("reportCategoriesList.editButton").setEnabled(canEdit && readOnly === false && categoryKey !== 0);
        view.byId("reportCategoriesList.deleteButton").setEnabled(canDelete && readOnly === false && categoryKey !== 0);
        this._toggleCreateReportConfigButton();
    },

    _toggleCreateReportConfigButton: function () {
        var view = this.getView();
        // A report configuration cannot be created without a valid category.
        // Check if at least 1 category exists which is not "All" and not read-only.
        var reportConfigCategory = _.find(this._getCategoriesInModel(), function (category) {
            return category.readOnly === false && category.key !== 0;
        });
        // Get the read-only status of the selected category.
        var selectedCategory = view.categoryFilter.getSelectedItem();
        if (selectedCategory !== null) {
            var categoryKey = this._getSelectedCategoryKey();
            var selectedReportConfigCategory = _.find(this._getCategoriesInModel(), function (data) {
                return data.key === categoryKey;
            });
            if (selectedReportConfigCategory !== undefined) {
                var readOnly = selectedReportConfigCategory.readOnly;
            }
        }
        var canCreate = this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.REPORT.CREATE);
        var allowNewReportCreation = canCreate && (reportConfigCategory !== undefined && readOnly === false);
        view.byId("reportConfigsTable.createReportButton").setEnabled(allowNewReportCreation);
        view.reportConfigsTable.getModel("reportConfigs").setProperty("/allowNewReportCreation", allowNewReportCreation);
    },

    _getSelectedCategoryKey: function () {
        var view = this.getView();
        var categoryKey = null;
        var selectedCategory = view.categoryFilter.getSelectedCategory();
        if (selectedCategory !== null) {
            var category = _.find(this._getCategoriesInModel(), function (data) {
                return data.name === selectedCategory;
            });
            if (category !== undefined) {
                categoryKey = category.key;
            }
        }
        return categoryKey;
    },

    /* Filters rows in the report config table based on the selected category. */
    _applyCategoryFilter: function () {
        this._filterReportTable();  //S1297730 apply search at the same time
        //var view = this.getView();
        //var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.reports");
        //var selectedCategory = view.categoryFilter.getSelectedCategory();
        //if (selectedCategory) {
        //    if (selectedCategory === rb.getText("categories.all.txt")) {
        //        view.reportConfigsTable.getBinding("rows").filter([]);
        //    } else {
        //        //Filter rows in the report configs table
        //        var categoryFilter = new sap.ui.model.Filter("reportConfigCategoryKey", sap.ui.model.FilterOperator.EQ, this._getSelectedCategoryKey());
        //        view.reportConfigsTable.getBinding("rows").filter([categoryFilter]);
        //    }
        //}
    },

    _savePreferredCategory: function (category) {
        if (category === undefined) {
            var view = this.getView();
            category = view.categoryFilter.getSelectedCategory();
        }
        if (category) {
            sas.stress.wkbench.UserPrefs.save("sas.stress.config.reportCategory", category);
        } else {
            sas.stress.wkbench.UserPrefs.delete("sas.stress.config.reportCategory", category);
        }
    },

    _selectPreferredCategory: function () {
        var view = this.getView();
        var preferredCategory = sas.stress.wkbench.UserPrefs.get("sas.stress.config.reportCategory");
        view.categoryFilter.setSelectedCategory(preferredCategory, true);
    },

    _getCategoriesListItems: function() {
        var view = this.getView();
        return view.categoryFilter.getListItems();
    },

    _getCategoriesInModel: function() {
        var view = this.getView();
        return view.categoryFilter.getModel().getData().categories;
    },

    _isEditCategory: function () {
        var view = this.getView();
        var categoryKey = view.byId("sas.stress.wkbench.view.CategoryDialog").getModel().getData().key;
        return (categoryKey != null);
    },

    _isEditReport: function () {
        var view = this.getView();
        var reportKey = view.byId("sas.stress.reports.view.EditReportDialog").getModel().getData().key;
        return (reportKey != null);
    },

    _areReportConfigsRunning: function(){
        var model = this.getView().reportConfigsTable.getModel("reportConfigs");
        if (!model) return false;

        var data = model.getProperty("/");
        var running=_.filter(data,function(each){
            return (each&&"Executing"===each.reportStatus);
        });
        if (running&&running.length>0){
            model.setProperty("/areReportConfigsRunning", true);
            return true;
        }
        model.setProperty("/areReportConfigsRunning", false);
        return false;
    }
});