/**
 * Created by sinsrj on 2/22/2016.
 */

jQuery.sap.declare("sas.stress.scripts.model.Script");
jQuery.sap.require("sas.stress.scripts.ScriptsAPI");
jQuery.sap.require("sas.stress.scripts.model.Parameters");
jQuery.sap.require("sas.hc.ui.commons.codeEditor.CodeEditor");
jQuery.sap.require("sas.stress.wkbench.StressUtil");

/**
 *
 */
sap.ui.base.Object.extend("sas.stress.scripts.model.Script",{
    /*
     *
     * @param id
     * @param name
     * @param description
     * @param dateInterval
     * @param gluecode
     */
    constructor:function(id,name,description,code,created,createdByUserId,updated,updatedByUserId,version,developer){
        this.id = id;
        this.name = name;
        this.description = description;
        this.version = version;
        this.scriptCode=code;

        this.created=created;
        this.createdByUserId=createdByUserId;
        this.updated=updated;
        this.updatedByUserId=updatedByUserId;

        this._parameters={};

        this.nameVersion=sas.stress.wkbench.StressUtil.getNameVersion(this.name,this.version);

        this.developer=developer||sas.stress.wkbench.StressUtil.getUserDisplayName();
    }
});

sas.stress.scripts.model.Script.prototype.getId = function(){
    return this.id;
};
sas.stress.scripts.model.Script.prototype.getName = function(){
    return this.name;
};
sas.stress.scripts.model.Script.prototype.setName = function(name){
    this.name=name;
};
sas.stress.scripts.model.Script.prototype.getDescription = function(){
    return this.description;
};
sas.stress.scripts.model.Script.prototype.setDescription = function(desc){
    this.description=desc;
};
sas.stress.scripts.model.Script.prototype.getUpdatedByUser = function(){
    return this.updatedByUserId;
};
sas.stress.scripts.model.Script.prototype.getUpdatedTimestamp = function(){
    return this.updatedTimestamp;
};
sas.stress.scripts.model.Script.prototype.getCreatedByUser = function(){
    return this.createdByUserId;
};
sas.stress.scripts.model.Script.prototype.getCreatedTimestamp = function(){
    return this.createdTimestamp;
};
sas.stress.scripts.model.Script.prototype.getNameVersion=function(){
    return this.nameVersion;
};
sas.stress.scripts.model.Script.prototype.getVersion=function(){
    return this.version;
};

sas.stress.scripts.model.Script.prototype.getScriptCode=function(){
    return this.scriptCode;
};

sas.stress.scripts.model.Script.prototype.getDeveloper=function(){
    return this.developer;
};

/*
 *
 * @param model
 */
sas.stress.scripts.model.Script.prototype.addParameter = function (parameter) {
    var name=parameter.getName();

    this._parameters[name] = parameter;
};
/*
 *
 * @returns {Array}
 */
sas.stress.scripts.model.Script.prototype.getParameters = function () {
    var parameters = [];
    var that = this;
    var all= _.size(this._parameters);
    var idx=0;
    _.each(this._parameters, function (parameter,key,i) {
        if (idx===(all-1)){
            parameter.setIsLast(true);
        } else {
            parameter.setIsLast(false);
        }
        parameters.push(parameter);
        idx++;
    });
    return parameters;
};

/*
 *
 * @param parameter
 */
sas.stress.scripts.model.Script.prototype.removeParameter = function (parameter) {
    var name=parameter.getName();
    delete this._parameters[name];
};
/*
 *
 * @param name
 */
sas.stress.scripts.model.Script.prototype.getParameterByName=function(name){
    return _.findWhere(this._parameters, {name: name});
};
/*
 *
 */
sas.stress.scripts.model.Script.commitChanges = function (model,scriptcode,isCreate) {
    var deferred= $.Deferred();
    var that=this;
    var script=model.getProperty("/script");
    var updatedParameters=model.getProperty("/parameters");

    var parameters=[];
    _.each(updatedParameters,function(parameter){
        var value=parameter.value,
            definition=parameter.value,
            type=parameter.parameterType;
        switch(parameter.parameterType.toLowerCase()){
            case "simple":
                type="FreeForm";
                value=parameter.value;
                definition=parameter.value;

                break;

            case "fixedlist":
                value=parameter.parameterDefaultValue;
                break;

            case "systemobject":
                definition=parameter.objectTypeParameterValues.objectTypeCode;
                break;

            case "sascode":
                value="";
                break;

            default:
                value="";
                break;
        }
        parameters.push({
            name:parameter.name,
            description:parameter.description,

            value:value,
            typeCd:type,
            definition:definition
            //definition:parameter.value,
            //defaultValue:parameter.defaultValue
        });
    });

    if (isCreate) {
        var commitjson = {
            name: script.getName(),
            description: script.getDescription(),
            version: script.getVersion(),
            code: scriptcode,
            defaultParameters: parameters,
            developerName: script.getDeveloper()
        };

        sas.stress.scripts.ScriptsAPI.createScript(commitjson)
            .done(function (scriptjson) {
                var script = sas.stress.scripts.model.Script._fromJSON(scriptjson);
                script.commit();
                //sas.stress.scripts.model.Script._modelGroups[modelGroup.id] = modelGroup;
                deferred.resolve(script);
            })
            .fail(function (jqxhr) {
                deferred.reject(jqxhr);
            });
    }else{
        var commitjson = {
            name: script.getName(),
            description: script.getDescription(),
            version: script.getVersion(),
            code: scriptcode,
            defaultParameters: parameters,
            developerName: script.getDeveloper()
        };

        sas.stress.scripts.ScriptsAPI.editScript(script.getId(),commitjson)
            .done(function(scriptjson) {
                var script = sas.stress.scripts.model.Script._fromJSON(scriptjson);
                script.commit();
                deferred.resolve(script);
            })
            .fail(function (jqxhr) {
                deferred.reject(jqxhr);
            });
    }
    return deferred;
};

sas.stress.scripts.model.Script.deleteScripts = function (scriptIdList) {
    var deferred = $.Deferred();
    sas.stress.scripts.ScriptsAPI.deleteScripts(scriptIdList).done(function () {
            _.each(scriptIdList, function (scriptId) {
                sas.stress.scripts.model.Script._scripts[scriptId] = undefined;
                delete sas.stress.scripts.model.Script._scripts[scriptId];
            });
            deferred.resolve();
        })
        .fail(function (jqxhr, textStatus, error) {
            deferred.reject(jqxhr);
        });
    return deferred;
};
/**
 *
 */
sas.stress.scripts.model.Script.prototype.commit = function () {
    sas.stress.scripts.model.Script._scripts[this.id] = this;
};
/*
 *
 * @param modelGroupId
 */
sas.stress.scripts.model.Script.prototype.clearScript = function (scriptId) {
    sas.stress.scripts.model.Script._scripts[scriptId] = undefined;
    delete sas.stress.scripts.model.Script._scripts[scriptId];
};

/*
 * STATICS
*/
sas.stress.scripts.model.Script._scripts = {};
sas.stress.scripts.model.Script._currentId = null;
sas.stress.scripts.model.Script.getScripts = function(refresh) {
    var deferred = $.Deferred();
    var scripts = [];
    if (!refresh && _.size(sas.stress.scripts.model.Script._scripts)>0){
        _.each(sas.stress.scripts.model.Script._scripts,function(script){
            scripts.push(script);
        });
        deferred.resolve(scripts);
    }else {
        sas.stress.scripts.ScriptsAPI.loadScripts().done(function (scriptsJSON) {
                sas.stress.scripts.model.Script._scripts = {};
                _.each(scriptsJSON, function (json) {
                    var script = sas.stress.scripts.model.Script._fromJSON(json);
                    script.commit();
                    //sas.stress.scripts.model.Script._modelGroups[modelGroup.id] = modelGroup;
                    scripts.push(script);
                });
                deferred.resolve(scripts);
            })
            .fail(function (jqxhr) {
                deferred.reject(jqxhr);
            });
    }
    return deferred;
};

sas.stress.scripts.model.Script.getScriptsHeader = function(refresh) {
    var deferred = $.Deferred();
    var scripts = [];
    if (!refresh && _.size(sas.stress.scripts.model.Script._scripts)>0){
        _.each(sas.stress.scripts.model.Script._scripts,function(script){
            scripts.push(script);
        });
        deferred.resolve(scripts);
    }else {
        sas.stress.scripts.ScriptsAPI.loadScriptsHeader().done(function (scriptsJSON) {
            sas.stress.scripts.model.Script._scripts = {};
            _.each(scriptsJSON, function (json) {
                var script = sas.stress.scripts.model.Script._fromJSON(json);
                script.commit();
                //sas.stress.scripts.model.Script._modelGroups[modelGroup.id] = modelGroup;
                scripts.push(script);
            });
            deferred.resolve(scripts);
        })
            .fail(function (jqxhr) {
                deferred.reject(jqxhr);
            });
    }
    return deferred;
};

sas.stress.scripts.model.Script.loadScriptById = function(scriptId) {
    var deferred = $.Deferred();
    sas.stress.scripts.ScriptsAPI.loadScript(scriptId)
        .done(function (runTimescriptJSON) {
            sas.stress.scripts.model.Script._scripts[scriptId] = sas.stress.scripts.model.Script._fromJSON(runTimescriptJSON);
            deferred.resolve(sas.stress.scripts.model.Script._scripts[scriptId]);
        })
        .fail(function (jqxhr) {
            deferred.reject(jqxhr);
        });

    return deferred;
};

sas.stress.scripts.model.Script.loadScriptDetailsByID = function(scriptId,projectId){
    var deferred = $.Deferred();
    sas.stress.scripts.ScriptsAPI.loadScriptDetailsByID(scriptId,projectId)
        .done(function (scriptJSON) {
            deferred.resolve(scriptJSON);
        })
        .fail(function (jqxhr) {
            deferred.reject(jqxhr);
        });

    return deferred;
};

/*
 *
 * @param modelGroupId
 * @returns {*}
 */
sas.stress.scripts.model.Script.getScript = function (scriptId) {
    //var deferred = $.Deferred();
    sas.stress.scripts.model.Script._currentId = scriptId;
    //if (sas.stress.scripts.model.Script._scripts[scriptId] === undefined) {
    /*sas.stress.scripts.ScriptsAPI.loadScript(scriptId).done(function (script) {
            sas.stress.scripts.model.Script._scripts[scriptId] = sas.stress.scripts.model.Script._fromJSON(script);
            deferred.resolve(sas.stress.scripts.model.Script._scripts[scriptId]);
        }).fail(function (jqxhr) {
            //sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            deferred.reject(jqxhr)
        });
    //} else {
        //deferred.resolve(sas.stress.scripts.model.Script._scripts[scriptId]);
    }*/

    return sas.stress.scripts.model.Script._scripts[scriptId];
    //return deferred;
};

/*
 * Used to initially set the Model group Name input with a unique name.
 * @returns {string}
 */
sas.stress.scripts.model.Script.getNewScriptName = function (type) {
    var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.scripts")/*.getBundle()*/;
    var type = type || "model.group";
    var newScriptName = "";
    var count = 1;
    while (newScriptName === "") {
        var tempScriptName = rb.getText(type + ".name.fmt", [count++]);
        //check if project with this name exists
        var exists = _.find(sas.stress.scripts.model.Script._scripts, function (script) {
            return script.getName().toLocaleLowerCase() === tempScriptName.toLocaleLowerCase();
        });
        //if not, use this name
        if (exists === undefined) {
            newScriptName = tempScriptName;
        }
    }
    return newScriptName;
};

/**
 * creates a copy, an exact replica of the existing model group
 *
 * @param {object} existingScript
 * @param {String} newName
 * @param {String} newDesc
 * @param {Array} updatedParameters
 * @param {Object} updatedoutputVars
 * @returns {sas.stress.scripts.model.Script}
 */
// This method is being called from 2 distinct locations therefore, we need to be careful
sas.stress.scripts.model.Script.makeACopy=function(existingScript,newName,newVersion, updatedParameters){
    //var updatedmodels=model.getProperty("/models");   // fixes for S1265692
    var newScript=new sas.stress.scripts.model.Script("", newName, existingScript.description, existingScript.scriptCode,
        existingScript.created,existingScript.createdByUserId,existingScript.updated,existingScript.updatedByUserId,newVersion,
        existingScript.developer);

    _.each(updatedParameters,function(parameter){
        var parameter=sas.stress.scripts.model.Parameters.clone(parameter);

        parameter.setScript(newScript);
        newScript.addParameter(parameter);
    });

    return newScript;
};
sas.stress.scripts.model.Script._fromJSON = function (json) {
    var script = new sas.stress.scripts.model.Script(json.key, json.name, json.description, json.code,
                        json.created,json.createdByUserId,json.updated,json.updatedByUserId,json.version,json.developerName);

    /*var sortedparameters= _.sortBy(json.defaultParameters,function(parameter){
        return parameter.name.toLocaleLowerCase();
    });*/
    _.each(/*sortedparameters*/json.defaultParameters,function(parameterjson){
        var definition, type,defaultValue,objectTypeParameterValues;
        type=parameterjson.typeCd?parameterjson.typeCd:"simple";
        type=(type==="FreeForm"?"simple":type);
        definition = parameterjson.definition;
        switch (type.toLowerCase()){
            case "simple":
                definition=parameterjson.value;
                defaultValue=parameterjson.value;
                break;
            case "systemobject":
                objectTypeParameterValues={id:"",name:"",version:"",objectTypeCode:""};
                objectTypeParameterValues.objectTypeCode=definition;  // FIXME
                objectTypeParameterValues.name=parameterjson.value;  // FIXME
                defaultValue=parameterjson.value;

                break;
            case "serverpath":
                break;
            default:
                defaultValue=parameterjson.value;
                break;
        }
        var parameter=new sas.stress.scripts.model.Parameters(parameterjson.name,definition,parameterjson.description,type,defaultValue,objectTypeParameterValues, parameterjson.id);

        parameter.setAllowableValues(parameterjson.allowableValues);
        parameter.setLastRunValue(parameterjson.lastUsedValue);

        parameter.setScript(script);
        script.addParameter(parameter);
    });

    return script;
};
sas.stress.scripts.model.Script.createScript=function(type){
    var newName=sas.stress.scripts.model.Script.getNewScriptName(type);
    var script = new sas.stress.scripts.model.Script("-1",newName,"");
    return script;
};
sas.stress.scripts.model.Script.tempScriptFromExisting=function(existingScript){
    var script = sas.stress.scripts.model.Script.makeACopy(existingScript,existingScript.getName(),existingScript.getVersion(),existingScript.getParameters());
    script.id=existingScript.getId();   //fixes for S1282519..
    return script;
};

sas.stress.scripts.model.Script.createParameter=function(script){
    var newName=sas.stress.scripts.model.Parameters.getNewBizAssumptionName();
    var parameter=new sas.stress.scripts.model.Parameters(newName,-1,undefined,script);
    script.addBizAssumptions(parameter);
    return parameter;
};
sas.stress.scripts.model.Script.getScriptByName=function(name){
    return _.findWhere(sas.stress.scripts.model.Script._scripts, {name:name});
};
sas.stress.scripts.model.Script.getScriptByNameAndVersion=function(name, version){
    var exists= _.find(sas.stress.scripts.model.Script._scripts,function(group){
        if (version.length) {
            var nameVersion=sas.stress.wkbench.StressUtil.getNameVersion(name,version);
            return group.nameVersion.toLocaleLowerCase() === nameVersion.toLocaleLowerCase();
        }else{
            return group.name.toLocaleLowerCase() === name.toLocaleLowerCase();
        }
    });
    if (exists&&!version.length){    // if version not provided we checked for name.. check for version also
        if(exists.version===version){
            return exists;
        }else{
            return undefined;
        }
    }
    return exists;
    //return _.findWhere(sas.stress.scripts.model.Script._modelGroups, {name:name, version: version});
};

sas.stress.scripts.model.Script.getTaskScriptExecutionLog=function(scriptId,runKey){
    var deferred= $.Deferred();
    sas.stress.scripts.ScriptsAPI.getScriptExecutionLog(scriptId,runKey)
        .done(function(log){
            deferred.resolve(log);
        })
        .fail(function(jqxhr){
            deferred.reject(jqxhr);
        });

    return deferred;
};

sas.stress.scripts.model.Script.getParameter=function(id){
    return sas.stress.scripts.model.Script._parameters[id];
};