jQuery.sap.require("sas.hc.m.MessageToast");
jQuery.sap.require("sas.risk.common.utils.LayoutService");
jQuery.sap.require("sas.risk.common.widgets.ZeroStateLayout");
jQuery.sap.require("sas.risk.common.models.RiskJSONModel");
jQuery.sap.require("sas.stress.scripts.model.Script");
jQuery.sap.require("sas.stress.wkbench.LeftNavManager");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.stress.wkbench.ViewManager");

sap.ui.controller("sas.stress.scripts.view.CreateScript",{

    onInit : function(event) {
        var view = this.getView();

        this.capabilityManager=sas.stress.wkbench.CapabilityManager.getInstance();
        this.i18N = new sap.ui.model.resource.ResourceModel({ bundleName: "sas.stress.scripts.messagebundle" });
        view.setModel(this.i18N, "i18N");
        this.router = sap.ui.core.routing.Router.getRouter("router");
        var viewId=event.getParameter("id");
        //var viewKey=viewId.substring(0,viewId.indexOf("View"));
        this.router.attachExactRouteMatched("ScriptDetails", this._init, this);
        this.navChecker = sas.stress.wkbench.StressUtil.createNavChecker(true, view);

        new sas.risk.common.utils.LayoutService(this.getView().bizVariablesTable, "ScriptDetailsParametersTable");

        // show groove bar
        sas.stress.wkbench.LeftNavManager.getInstance().showGrooveBar(sas.stress.wkbench.LeftNavManager.SCRIPTS);
    },

    _init: function(event) {
        var scriptId = Number(event.getParameter("arguments").scriptId);
        this.refresh(scriptId);
    },

    refresh: function (scriptId) {
        var view = this.getView();
        var that = this;

        this.json = {};
        this.modelGroup = undefined;
        this._saveAsVisible = false;

        this.json = {
            listEnabled: true
        };
        if (view.getModel() instanceof sap.ui.model.json.JSONModel) {
            var modelExists = true;
        } else {
            var modelCreateModel = new sas.risk.common.models.RiskJSONModel(this.json);
            view.setModel(modelCreateModel);
        }
        var model = view.getModel();
        model.setProperty("/scriptId",scriptId);
        model.setProperty("/scriptName","");
        // the various properties of the model
        model.setProperty("/script",null);
        model.setProperty("/parameters",[]);
        model.setProperty("/modelsexist",false);

        // if it's an edit, the template is provided
        if (scriptId !== 0){
            /*sas.stress.scripts.model.Script.getScript(scriptId).done(function(scriptObj) {
                var script = scriptObj;
                that.template=script;
                that._makeMGViewModel(script);
                model.setProperty("/scriptName",script.name);
                that.navChecker.setDirty(false);
                that._toggleSaveCreateModelGroupButton(false);
                that._toggleSaveAsModelGroupButton(true,true);
            }).fail(function(jqxhr){
                sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            });*/
            var script = sas.stress.scripts.model.Script.getScript(scriptId);
            that.template=script;
            that._makeMGViewModel(script);
            model.setProperty("/scriptName",script.name);
            that.navChecker.setDirty(false);
            that._toggleSaveCreateModelGroupButton(false);
            that._toggleSaveAsModelGroupButton(true,true);
        } else {
            that.template=undefined;
            that.modelGroup=sas.stress.scripts.model.Script.createScript("script");
            // create, the button is ON.. Per Don/Dan
            that._toggleSaveCreateModelGroupButton(true);
            that._toggleSaveAsModelGroupButton(false);

            // we are dirty.. why because the save button is enabled
            that._setDirty(true);
            that._setupBizAssumptionsModel();
        }
        model.setProperty("/script", that.modelGroup);
        model.setProperty("/script/displayForEdit", scriptId !== 0);
    },
    onSave:function(){
        var view=this.getView();
        view.setBusy(true);
        view.setBusyIndicatorDelay(0);

        var model=view.getModel();
        var controller = this;
        var mg=model.getProperty("/script");
        var isCreate=(mg.id==="-1");

        sas.stress.scripts.model.Script.commitChanges(model, view.udlCodeEditor.getText(),isCreate).done(function(group){
            controller._setDirty(false);
            controller._makeMGViewModel(group);
            // we need to set the property on the view's model to use the new group
            model.setProperty("/script",group);
            model.setProperty("/scriptId",group.id);
            model.setProperty("/scriptName",group.name);
            var rb=controller.i18N.getResourceBundle();
            if (isCreate) {
                sap.m.MessageToast.show(rb.getText("script.create.success.msg.fmt",group.name), {at: sap.ui.core.Popup.Dock.CenterTop});
            }else{
                sap.m.MessageToast.show(rb.getText("script.save.success.msg.fmt",group.name), {at: sap.ui.core.Popup.Dock.CenterTop});
            }
            view.setBusy(false);
        }).fail(function (jqxhr) {
            view.setBusy(false);
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });
    },
    onSaveAs:function(){
        var view=this.getView();
        var model=view.getModel();
        var controller = this;

        var existingDialog = sap.ui.getCore().byId("sas.stress.scripts.view.ScriptSaveAsDialog");
        if (existingDialog) {
            existingDialog.destroy();
        }
        var dialog = sap.ui.jsfragment("sas.stress.scripts.view.ScriptSaveAsDialog", this);
        var saveAsJSON = {name: sas.stress.scripts.model.Script.getNewScriptName("script"),existingname:model.getData().script.name};
        var jsonModel = new sas.risk.common.models.RiskJSONModel(saveAsJSON);
        dialog.setModel(jsonModel);
        dialog.open();
    },
    _makeMGViewModel:function(template){
        var view=this.getView();
        if (template){
            this.modelGroup=sas.stress.scripts.model.Script.tempScriptFromExisting(template);
             view.getModel().setProperty("/script", this.modelGroup);
        }
        // fixes for S1255041
        // if the modelgroup has been created, the template is undefined. however because the save was successful, we need to update this
        this.template=template;
        view.udlCodeEditor.setText(this.template.getScriptCode());
        this.json.deleteEnabled=true;

        // re-create the bizassumptions and output variables model
        this._setupBizAssumptionsModel();

        // assume we are good
        this._toggleSaveCreateModelGroupButton(false);
        // save as is also enabled and visible
        this._saveAsVisible=true;
        this._toggleSaveAsModelGroupButton(true);
    },
    saveAsModelGroupButtonPress:function(){
        var controller = this;
        var view = this.getView();
        var model=view.getModel();
        var rb=controller.i18N.getResourceBundle();

        var saveAsDialog = sap.ui.getCore().byId("sas.stress.scripts.view.ScriptSaveAsDialog");
        saveAsDialog.setBusy(true);
        var modelGroupSaveAsName = sap.ui.getCore().byId("ScriptSaveAsDialog.newNameInput").getValue();
        var modelGroupSaveAsVersion = sap.ui.getCore().byId("ScriptSaveAsDialog.newVersionInput").getValue();
        var existinggroup=model.getProperty("/script");
        var newGroup=sas.stress.scripts.model.Script.makeACopy(existinggroup,modelGroupSaveAsName,modelGroupSaveAsVersion,model.getProperty("/parameters"));
        model.setProperty("/script",newGroup);
        sas.stress.scripts.model.Script.commitChanges(model, view.udlCodeEditor.getText(),true).done(function(group){
            saveAsDialog.setBusy(false);
            saveAsDialog.close();
            sap.m.MessageToast.show(rb.getText("script.save.success.msg.fmt",group.name), {at: sap.ui.core.Popup.Dock.CenterTop});
            controller._makeMGViewModel(group);
            model.setProperty("/script",group); // fixes for
            model.setProperty("/scriptId",group.id);
            model.setProperty("/scriptName",group.name);
            controller._setDirty(false);
        }).fail(function (jqxhr) {
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            saveAsDialog.setBusy(false);
        });
    },
    createOrEditModelGroupNameVersionLiveChange:function(e){
        var nameInput = sap.ui.getCore().byId("createScriptNameInput");
        var newName = nameInput.getValue().trim();
        var versionInput = sap.ui.getCore().byId("createScriptVersionInput");
        var newVersion = versionInput.getValue().trim();

        var isValid = true;
        if (newName == null || newName.trim() === ""){
            nameInput.setValueState(sap.ui.core.ValueState.Error);
            nameInput.setValueStateText(this.i18N.getProperty("error.requiredField.txt"));
            isValid=false;
        } else {
            var exists = sas.stress.scripts.model.Script.getScriptByNameAndVersion(newName, newVersion);
            if (exists) {
                if (exists.id===this.modelGroup.id){    // fixes for S1254608 and now S1298567 in 2.3 after version introduced
                    //same Name is not save.
                    nameInput.setValueStateText("");
                    nameInput.setValueState(sap.ui.core.ValueState.None);
                    if (exists.name!==newName) {
                        //same Name different case... therefore, save
                        isValid = true;
                    }else{
                        isValid=this.navChecker.getDirty()||false;  // current dirty state
                    }
                }else {
                    //isValid = false;
                    nameInput.setValueStateText(this.i18N.getProperty("script.nameAndVersion.inUse.txt"));
                    nameInput.setValueState(sap.ui.core.ValueState.Error);
                    versionInput.setValueStateText(this.i18N.getProperty("script.nameAndVersion.inUse.txt"));
                    versionInput.setValueState(sap.ui.core.ValueState.Error);
                    isValid = false;
                }
            } else {
                //Name is good.
                nameInput.setValueStateText("");
                nameInput.setValueState(sap.ui.core.ValueState.None);
                versionInput.setValueStateText("");
                versionInput.setValueState(sap.ui.core.ValueState.None);
            }
        }
        //Toggle Save button based on the result of validation.
        this._setDirty(isValid);

        return isValid;
    },
    createOrEditModelGroupDescLiveChange:function(e){
        var newDesc= e.getParameter("newValue");
        var dirty=true;
        if (this.modelGroup&&this.modelGroup.description!==newDesc) {
            dirty=true;
        }
        this.navChecker.setDirty(dirty);
        this._toggleSaveCreateModelGroupButton(dirty);
        this._toggleSaveAsModelGroupButton(dirty);
    },
    createOrEditModelGroupDeveloperLiveChange:function(e){
        var newName= e.getParameter("newValue");
        var dirty=true;
        if (this.modelGroup&&this.modelGroup.developer!==newName) {
            dirty=true;
        }
        this.navChecker.setDirty(dirty);
        this._toggleSaveCreateModelGroupButton(dirty);
        this._toggleSaveAsModelGroupButton(dirty);
    },
    /*
     * We should make sure there is no model group that has the same name because this is a Save As
     * @param e {Event} event
     * @returns {boolean} isValid
     */
    saveAsModelGroupNameLiveChange:function(e){
        //var newName= e.getParameter("newValue");
        //var nameInput=e.getSource();

        var nameInput = sap.ui.getCore().byId("ScriptSaveAsDialog.newNameInput");
        var newName = nameInput.getValue().trim();
        var versionInput = sap.ui.getCore().byId("ScriptSaveAsDialog.newVersionInput");
        var newVersion = versionInput.getValue().trim();

        var isValid = true;
        if (newName == null || newName.trim() === ""){
            nameInput.setValueState(sap.ui.core.ValueState.Error);
            nameInput.setValueStateText(this.i18N.getProperty("error.requiredField.txt"));
            isValid=false;
        } else {
            var exists = sas.stress.scripts.model.Script.getScriptByNameAndVersion(newName, newVersion);
            if (exists) {
                //same Name is not save as.
                nameInput.setValueStateText(this.i18N.getProperty("script.nameAndVersion.inUse.txt"));
                nameInput.setValueState(sap.ui.core.ValueState.Error);
                versionInput.setValueStateText(this.i18N.getProperty("script.nameAndVersion.inUse.txt"));
                versionInput.setValueState(sap.ui.core.ValueState.Error);
                isValid=false;
            } else {
                //Name is good.
                nameInput.setValueStateText("");
                nameInput.setValueState(sap.ui.core.ValueState.None);
                versionInput.setValueStateText("");
                versionInput.setValueState(sap.ui.core.ValueState.None);
            }
        }
        return isValid;
    },

    onExit: function (e) {
        this.router.detachExactRouteMatched("ScriptDetails", this._init, this);
        // clear the View on the router otherwise the next router.navTo() would fail with a blank page
        this.router.setView("sas.stress.scripts.view.CreateScript", null);
        this.navChecker.stop();
    },

    _onAddBizAssumption:function(e){
        var that=this;
        var view=this.getView();
        var model=view.getModel();
        var assumptionsjson=model.getData()["parameters"];

        var uiAssumption={id:0,name:"",value:"",description:"",committed:false};
        uiAssumption.id=_.uniqueId();
        uiAssumption.name=sas.stress.scripts.model.Parameters.getNewParameterName(this.modelGroup,assumptionsjson);
        if (assumptionsjson.length) {
            assumptionsjson.splice(0, 0, uiAssumption);
        } else{
            assumptionsjson.push(uiAssumption);
        }

        view.assumptionValue.fireLiveChange({id:view.assumptionValue.getId(),newValue:"",value:""});
        model.setProperty("/parameters",assumptionsjson);

        //the navchecker should be dirty.
        this._setDirty(true);
    },
    _onRemoveBizAssumption:function(e){
        var that=this;
        var view=this.getView();
        var model=view.getModel();
        var committedDeleted=false;

        var table= view.bizVariablesTable;
        var selectedIndices=table.getSelectedIndices();
        var parametersjson=model.getData()["parameters"];
        _.each(selectedIndices,function(selectedIndex){
            var parameterIndex = selectedIndex;
            var ctx = table.getContextByIndex(selectedIndex);
            if (ctx&&ctx.getObject()) {
                var id = ctx.getProperty("id");
                var parameter = _.find(parametersjson, function(parm, idx){
                    if (parm !== undefined && parm.id === id) {
                        parameterIndex = idx;
                        return true;
                    }
                    return false;
                });
            }
            if (parameter) {
                if (!committedDeleted){
                    committedDeleted=parameter.committed;
                }
                // delete the assumption from the json
                delete parametersjson[parameterIndex];
            }
        });
        // remove the undefineds'
        parametersjson=_.without(parametersjson,undefined);
        model.setProperty("/parameters",parametersjson);
        table.setSelectedIndex(-1);
        view.deletBizVarsButton.setEnabled(parametersjson.length>0&&
            this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.SCRIPT.UPDATE));

        // fixes for S1252750.. un-saved changes notification
        this._setDirty(this.template&&committedDeleted);
    },
    _bizAssumptionNameLiveChange:function(e){
        var view=this.getView();
        var newName= e.getParameter("newValue");
        var nameInput=e.getSource();
        var isValid=true;
        var model=view.getModel();
        var parametersjson=model.getData()["parameters"];
        var input= e.getSource();   // fixes for S1268012

        if (newName == null||newName.length===0){
            nameInput.setValueState(sap.ui.core.ValueState.Error);
            nameInput.setValueStateText(this.i18N.getProperty("error.requiredField.txt"));
            isValid=false;
        } else if(!sas.stress.wkbench.StressUtil._isValidName(newName)){    // fixes for S1255068
            nameInput.setValueState(sap.ui.core.ValueState.Error);
            nameInput.setValueStateText(this.i18N.getProperty("error.invalidname.txt"));
            isValid=false;
        } else {
            //Check if Biz Assumption with this name already exists.
            /* fixes for S1268012 */
            var customData = _.find(input.getCustomData(), function (customData) {
                return customData.getKey() === "id";
            });
            var id = customData.getValue();
            /* fixes for S1268012 end */
            var exists = _.find(parametersjson,function(parameter){
                // fixes for S1261257.. case insensitive compare please
                return (parameter.name.toLowerCase()===newName.toLowerCase()&&id!==parameter.id);
            });
            if (exists) {
                //isValid = false;
                nameInput.setValueStateText(this.i18N.getProperty("error.nameAlreadyInUse.txt"));
                nameInput.setValueState(sap.ui.core.ValueState.Error);
                isValid=false;
            } else {
                //Name is good.
                nameInput.setValueStateText("");
                nameInput.setValueState(sap.ui.core.ValueState.None);
                isValid=true;
            }
        }
        //Toggle Save button based on the result of validation.
        this._toggleSaveAsModelGroupButton(isValid);
        this._toggleSaveCreateModelGroupButton(isValid);
        this.navChecker.setDirty(true);
    },
    _bizAssumptionValueLiveChange:function(e){
        var newValue= e.getParameter("newValue");
        var valueInput=e.getSource();
        var isValid=true;
        if (newValue == null/* || newValue.trim() === ""*/){
            valueInput.setValueState(sap.ui.core.ValueState.Error);
            valueInput.setValueStateText(this.i18N.getProperty("error.requiredField.txt"));
            isValid=false;
        } else {
            //Name is good.
            valueInput.setValueStateText("");
            valueInput.setValueState(sap.ui.core.ValueState.None);
        }
        //Toggle Save button based on the result of validation.
        this._toggleSaveAsModelGroupButton(isValid);
        this._toggleSaveCreateModelGroupButton(isValid);
        this.navChecker.setDirty(true);
    },
    _bizAssumptionDescLiveChange:function(/*e*/){
        this._setDirty(true);
    },
    _onBizAssumptionsSelectionChange:function(e){
        var view=this.getView();

        switch(e.getSource().getSelectedIndices().length){
            case 0: // nothing selected in the table.. disabled
                view.deletBizVarsButton.setEnabled(false);
                view.editScriptParameterBtn.setEnabled(false);
                break;
            case 1: // 1 selected in the table.. enabled
                view.deletBizVarsButton.setEnabled(true&&
                    this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.SCRIPT.DELETE));
                view.editScriptParameterBtn.setEnabled(true&&
                    this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.SCRIPT.UPDATE));
                break;
            default: // more than 1 selected in the table.. enabled and disabled
                view.deletBizVarsButton.setEnabled(true&&
                    this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.SCRIPT.DELETE));
                view.editScriptParameterBtn.setEnabled(false);
                break;
        }
    },

    /**
     * Toggle Save button based on the result of validation.
     * @param {boolean} isValid valid or not
     * @private
     */
    _toggleSaveCreateModelGroupButton: function (isValid) {
        this.getView().saveButton.setEnabled(isValid&&
            this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.SCRIPT.CREATE));
    },
    /**
     *
     * @param {boolean} isValid valid or not
     * @private
     */
    _toggleSaveAsModelGroupButton: function (isValid) {
        this.getView().saveAsButton.setEnabled(isValid&&
            this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.SCRIPT.CREATE));
        this.getView().saveAsButton.setVisible(this._saveAsVisible);
    },
    _setupBizAssumptionsModel:function(){
        var view=this.getView();
        var model=view.getModel();
        var parameters=model.getProperty("/parameters")||[];
        var parameter;
        if (parameters&&parameters.length){
            parameters=[];
            model.setProperty("/parameters",[]);
        }
        _.each(this.modelGroup.getParameters(),function(param){
            parameter={id:0,name:"",value:"",description:"",committed:true,parameterType:""};

            parameter.id=param.getId();
            parameter.name=param.getName();
            parameter.value=param.getValue(); // FIXME?
            parameter.description=param.getDescription();
            parameter.parameterType=param.getParameterType();
            parameter.parameterDefaultValue=param.getDefaultValue();
            parameter.objectTypeParameterValues=param.getObjectTypeParameterValues();
            //parameter.parameterObjectTypeCode=

            parameters.push(parameter);
        });
        model.setProperty("/parameters",parameters);
    },
    _setDirty: function (isDirty) {
        //This needs to be changed
        this.getView().saveButton.setEnabled(isDirty);  //Enable Save button
        this.getView().saveAsButton.setEnabled(true);   //Save As button is always enabled

        // Let Appswitcher know
        //var dirty=this.navChecker.getDirty(); // fixes for S1282173
        this.navChecker.setDirty(/*dirty||*/isDirty);   // make sure the navchecker's dirtiness is taken into accoun
    },
    _onUDLCodeTabClick:function(){  // fixes for S1266309
        // the buttons are now enabled because we are in the UDL editor
        this._toggleModelHeaderButtons();
    },
    _enableSaveButtons:function(){
        var that=this;
        var view=this.getView();
        var model=view.getModel();
        var modelsjson=model.getProperty("/models");
        /*var isValid=true;

        _.each(modelsjson,function(model){
            var search=that._searchForModelNameInUDLCode(model);
            if (search&&search.length) {    // if the search returns something? fixes for S1255041
                if (isValid) {  // if it's false in the loop keep it at false
                    isValid = search.length > 0;
                }
            }
        });*/
        // code for model inserted.. therefore, valid?
        this._toggleSaveAsModelGroupButton(true);
        this._toggleSaveCreateModelGroupButton(true);

    },
    _onPrint:function(e) {
        var view=this.getView();
        var sourceBtn= e.getSource();
        var codeeditor;
        switch(sourceBtn.getId()){
            case "codeEditorPrint":
                codeeditor=view.udlCodeEditor;
                break;
        }
        var printDialog = window.open('about:blank', "printWindow", 'resizable=1,scrollbars=0,width=800,height=600');
        printDialog.document.write('<link type="text/css" rel="stylesheet" href="resources/themes/sas/hc/ui/commons/themes/sas_corporate/library.css">');
        printDialog.document.write(codeeditor.getHTML({lineNumber: true})); // fixes for S1250303
        printDialog.document.close();
        window.setTimeout(function(){
            printDialog.print();
        }, 100);
    },
    _udlUndoPressed:function() {
        var view=this.getView();
        var model=view.getModel();
        var modelsjson=model.getProperty("/models");

        view.udlCodeEditor.undo();
        var text=view.udlCodeEditor.getSelectedText();
        var searchstring="model_name=";
        var index=text.indexOf(searchstring);
        var modelname=text.substring(index+searchstring.length,text.length-3);  //
        var existsInUDL=false;
        _.each(modelsjson,function(model){
            if (model.name===modelname){
                existsInUDL=true;
            }
        });
        if (existsInUDL){
            //view.validationLabel.setText("The UDL code contains reference to a model " +modelname+ " that is not part of the model group");
            var rb=this.i18N.getResourceBundle();
            sas.hc.m.MessageDialog.warning({text: rb.getText("modelgroup.codeeditor.undo.model.warning.fmt", modelname)}); // fixes for S1266279
        } else {
            view.validationLabel.setText("");
        }

        // code for model inserted.. therefore, valid?
        this._toggleSaveAsModelGroupButton(!existsInUDL);
        this._toggleSaveCreateModelGroupButton(!existsInUDL);
    },
    _udlRedoPressed:function() {
        var view=this.getView();
        var model=view.getModel();

        view.udlCodeEditor.redo();
        // code for model inserted.. therefore, valid?
        this._toggleSaveAsModelGroupButton(true);
        this._toggleSaveCreateModelGroupButton(true);
    },
    _isValidName:function(name){
        var isValid=name.match(/[^\w]/gi);
        if (isValid&&isValid.length){
            return false;
        }
        return true;
    },
    _udlTextChanged:function(e){
        console.info(e);
        var origScriptText=this.getView().getModel().getData().script?this.getView().getModel().getData().script.scriptCode:"";  // fixes for S1316269
        var text = e.getParameter("text");
        if (text===origScriptText){ // fixes for S1316269 aftermath of S1256396
            return false;
        }
        var removedText = e.getParameter("removedText");
        if ((removedText.length || text.length) && (removedText !== text)) {    // fixes for S1256396
            // something has changed
            this._setDirty(true);
        }
    },
    _showEditScriptParameterDialog: function (e,rowIndex) {
        var that = this;
        var _rb=this.i18N.getResourceBundle();
        var parameter = undefined;
        var title;
        if (e&&e.getSource().getId().indexOf("parameter-add")>=0){
            title = _rb.getText("script.add.parameter.title")
        }else{
            parameter = this._getParameter(rowIndex);
            title = _rb.getText("script.edit.parameter.title")
        }
        var editScriptParameterView=sas.stress.wkbench.ViewManager.loadEditScriptParameterView(parameter,that.getView().getModel().getProperty("/parameters"));
        var editScriptParameterDlg = new sas.hc.m.Dialog({
            title: title,
            content: editScriptParameterView,
            /*beginButton:editScriptParameterView.beginbutton,
            endButton: editScriptParameterView.endbutton,*/
            afterClose: function () {
                this.destroy();
            },
            contentHeight:"48.3rem",
            contentWidth:"45rem",
            //initialFocus: editScriptParameterView.endbutton,
            defaultButton:editScriptParameterView.beginbutton,
            buttons:[
                editScriptParameterView.beginbutton,
                editScriptParameterView.endbutton
            ],
            resizable:true
        });
        editScriptParameterView.attachClosing(function(e){
            editScriptParameterDlg.close();
            if (e.getParameter("action")==="OK"){   //fixes for S1255041
                // on Add we need to set the view to dirty and enable the buttons
                var param = e.getParameter("data");
                var parameters=that.getView().getModel().getProperty("/parameters");
                var index=-1;
                var parameter= _.find(parameters,function(parameter,idx){
                    index=idx;
                    return Number(parameter.id)===Number(param.id);
                });
                if (parameter===undefined) {
                    parameter = new sas.stress.scripts.model.Parameters(
                        param.name, param.definition, param.description, param.type, param.defaultValue, param.objectTypeParameterValues);

                    parameters.push(parameter);
                }else{
                    parameter.name=(param.name);
                    parameter.description=(param.description);
                    parameter.parameterType=(param.type);

                    //parameter.setCode(param.code);
                    parameter.parameterDefaultValue=(param.defaultValue);
                    //parameter.setAllowableValues(param.list);
                    if (param.type.toLowerCase()==="systemobject"&&(param.objectTypeParameterValues&&param.objectTypeParameterValues.objectTypeCode)) {
                        parameter.objectTypeParameterValues=(param.objectTypeParameterValues);
                        parameter.value="";
                    }else{
                        parameter.objectTypeParameterValues=(undefined);
                        parameter.value=param.definition;
                    }
                    parameter.parameterObjectTypeCode=param.objectTypeParameterValues?param.objectTypeParameterValues.objectTypeCode:"";
                    //update
                    parameters[index]=parameter;
                }
                //that.getView().getModel().setProperty("/parameters",[]);
                that.getView().getModel().setProperty("/parameters",parameters);
                that._setDirty(true);
            }
        },editScriptParameterDlg);

        editScriptParameterDlg.open();
    },
    onEditClick: function (e) {
        this._showEditScriptParameterDialog(e);
    },
    onCellClick: function (e){
        var that=this;
        var rowIndex=e.getParameter("rowIndex");
        // do in timeout because the table needs to select the row
        // unlike other cellclicks, we are not using the event to get to the row context
        setTimeout(function() {
            if (that._getParameter(rowIndex)) {
                that._showEditScriptParameterDialog(undefined,rowIndex);
            }
            //that.getView().bizVariablesTable.removeSelectionInterval(0,0);
        },0);
    },
    _getParameter: function (index) {
        var view = this.getView();
        // If index wasn't specified, then we are getting the currently selected index from the table.
        if (undefined === index) {
            index = view.bizVariablesTable.getSelectedIndex();
        }
        var parameter = null;
        if (index >= 0) {
            var context = view.bizVariablesTable.getContextByIndex(index);
            if (context&&context.getObject()) {
                var parameterName = context.getObject().name;
                parameter = _.find(view.getModel().getData()["parameters"], function (parameter) {
                    if (parameter.name === parameterName) {
                        return parameter;
                    }
                });
            }
        }
        return parameter;
    }
});