jQuery.sap.declare("sas.stress.wkbench.model.Task");
jQuery.sap.require("sas.stress.wkbench.StressAPI");
jQuery.sap.require("sas.stress.wkbench.model.Sheet");
jQuery.sap.require("sas.stress.wkbench.model.TaskAction");
jQuery.sap.require("sas.stress.wkbench.model.TaskComment");

sap.ui.base.Object.extend("sas.stress.wkbench.model.Task", {
    constructor: function (project,
                           id,
                           name,
                           description,
                           users,
                           owner,
                           dueDays,
                           status,
                           type,
                           impl,
                           parameters,
                           iterationNum,
                           commentCount,
                           script,
                           parentNodeId,
                           plannedStartDate,
                           plannedEndDate,
                           taskGroup,
                           canUpdate) {
        this.project = project;
        this.id = id;
        this.name = name;
        this.description = description;
        this.owner = owner;
        this.users = users;
        this.dueDays = dueDays;
        this.attachments = null;
        this.comments = null;
        this.comment = status.comment;
        this.commentCount = commentCount;
        this.value = status.value;
        this.iterationNum = iterationNum;
        this.date = status.date;
        this.label = status.label || (status.value === "ACTIVE" ? "task.status.inprogress.txt" : "task.status.notstarted.txt");
        this.icon = this.getTaskIcon(status);
        this.type = type;
        this.implementationType = impl;
        this.parms = parameters;
        this.taskGroup = taskGroup;
        this.parentNodeId = parentNodeId;
        this.plannedStartDate = plannedStartDate;
        this.plannedEndDate = plannedEndDate;
        this.script=script;
        this.canUpdate=canUpdate;
        this.actions = [];
        this.status = {
            date: this.date,
            icon: this.icon,
            label: this.label
        };
        if (status.value === "ACTIVE") {
            this.addAction(new sas.stress.wkbench.model.TaskAction(this, "ITERATE", "home.mytasks.iterate.txt", "NEWSTARBURST", project));
        }
        if (this.implementationType) {
            switch (this.implementationType.toUpperCase()) {
                case "WORKBENCH":
                    this.addAction(new sas.stress.wkbench.model.TaskAction(this, "LAUNCH", "home.mytasks.launch.txt", "FORMULA", [sas.stress.wkbench.ViewManager.loadWorkbench, project, parameters]));
                    break;
                case "REPORTS":
                    this.addAction(new sas.stress.wkbench.model.TaskAction(this, "LAUNCH", "home.mytasks.launch.txt", "REPORT", [sas.stress.wkbench.ViewManager.loadProjectReportsEditor, project]));
                    break;
                case "REPORTS.CREATE":
                    this.addAction(new sas.stress.wkbench.model.TaskAction(this, "LAUNCH", "home.mytasks.launch.txt", "REPORT", [sas.stress.wkbench.ViewManager.loadProjectReportsCreate, project, parameters]));
                    break;
                case "REPORTS.RUN":
                    this.addAction(new sas.stress.wkbench.model.TaskAction(this, "LAUNCH", "home.mytasks.launch.txt", "REPORT", [sas.stress.wkbench.ViewManager.loadProjectReportsRun, project, parameters]));
                    break;
                case "SCENARIOS":
                    this.addAction(new sas.stress.wkbench.model.TaskAction(this, "LAUNCH", "home.mytasks.launch.txt", "SCENARIO", [sas.stress.wkbench.ViewManager.loadScenarioPage, project]));
                    break;
                case "SCENARIOS.ADD":
                    this.addAction(new sas.stress.wkbench.model.TaskAction(this, "LAUNCH", "home.mytasks.launch.txt", "SCENARIO", [sas.stress.wkbench.ViewManager.loadScenarioPageAdd, project, parameters]));
                    break;
                case "SCENARIOS.IMPORT":
                    this.addAction(new sas.stress.wkbench.model.TaskAction(this, "LAUNCH", "home.mytasks.launch.txt", "SCENARIO", [sas.stress.wkbench.ViewManager.loadScenarioPageImport, project, parameters]));
                    break;
                case "DATALOADER":
                    this.addAction(new sas.stress.wkbench.model.TaskAction(this, "LAUNCH", "home.mytasks.launch.txt", "DATALOADER", [sas.stress.wkbench.ViewManager.loadProjectDataEditor, project]));
                    break;
                case "DATALOADER.LOAD":
                    this.addAction(new sas.stress.wkbench.model.TaskAction(this, "LAUNCH", "home.mytasks.launch.txt", "DATALOADER", [sas.stress.wkbench.ViewManager.loadProjectDataEditorLoadData, project, parameters]));
                    break;
                case "MODELGROUPS":
                    this.addAction(new sas.stress.wkbench.model.TaskAction(this, "LAUNCH", "home.mytasks.launch.txt", "MODELS", [sas.stress.wkbench.ViewManager.loadModelGroupRuns, project]));
                    break;
                case "MODELGROUPS.CREATE":
                    this.addAction(new sas.stress.wkbench.model.TaskAction(this, "LAUNCH", "home.mytasks.launch.txt", "MODELS", [sas.stress.wkbench.ViewManager.loadModelGroupRunsCreate, project]));
                    break;
                case "PACKAGES":
                    this.addAction(new sas.stress.wkbench.model.TaskAction(this, "LAUNCH", "home.mytasks.launch.txt", "PACKAGES", [sas.stress.wkbench.ViewManager.loadFilingPackagePage, project]));
                    break;
                case "PACKAGES.CREATE":
                    this.addAction(new sas.stress.wkbench.model.TaskAction(this, "LAUNCH", "home.mytasks.launch.txt", "PACKAGES", [sas.stress.wkbench.ViewManager.loadProjectPackagesCreate, project, parameters]));
                    break;
                case "PACKAGES.VALIDATE":
                    this.addAction(new sas.stress.wkbench.model.TaskAction(this, "LAUNCH", "home.mytasks.launch.txt", "PACKAGES", [sas.stress.wkbench.ViewManager.loadProjectPackagesValidate, project, parameters]));
                    break;
                case "SCRIPT.RUN":
                    var parms = this.getParms();
                    var displayLabel = "home.mytasks.execute.txt";
                    if (parms.displayLabel !== undefined) {
                        displayLabel = parms.displayLabel;
                    }
                    this.addAction(new sas.stress.wkbench.model.TaskAction(this, "EXECUTE", displayLabel, "TASKS", [project, this, parameters]));
                    break;
                case "URL":
                    this.addAction(new sas.stress.wkbench.model.TaskAction(this, "LAUNCH", "home.mytasks.launch.txt", "URL", [project, this, parameters]));
                    break;
            }
        }
    }
});

sas.stress.wkbench.model.Task.prototype.getTaskIcon = function (status) {
    var icon;
    if (status && status.iconName !== "CHECKMARK") {
        icon = (status.iconName === "CLOSE") ? "INVALIDSTATUS" : status.iconName;
    }
    if (!icon) {
        icon = (status.value === "ACTIVE") ? 'REVIEWSTATUS' : (this.date ? 'COMPLETESTATUS' : 'NEUTRALSTATUS');
    }
    return icon;
};

sas.stress.wkbench.model.Task.prototype.getIterationNum = function () {
    return this.iterationNum;
};
sas.stress.wkbench.model.Task.prototype.getDueDays = function () {
    return this.dueDays;
};
sas.stress.wkbench.model.Task.prototype.getProject = function () {
    return this.project;
};
sas.stress.wkbench.model.Task.prototype.getId = function () {
    return this.id;
};
sas.stress.wkbench.model.Task.prototype.getName = function () {
    return this.name;
};
sas.stress.wkbench.model.Task.prototype.getDescription = function () {
    return this.description;
};
sas.stress.wkbench.model.Task.prototype.getUsers = function () {
    return this.users;
};
sas.stress.wkbench.model.Task.prototype.getTaskGroup = function () {
    return this.taskGroup;
};
sas.stress.wkbench.model.Task.prototype.getOwner = function () {
    return this.owner;
};
sas.stress.wkbench.model.Task.prototype.getStatusLabel = function () {
    return this.label;
};
sas.stress.wkbench.model.Task.prototype.getStatus = function () {
    return this.status;
};
sas.stress.wkbench.model.Task.prototype.getParentNodeId = function () {
    return this.parentNodeId;
};
sas.stress.wkbench.model.Task.prototype.getPlannedStartDate = function () {
    return this.plannedStartDate;
};
sas.stress.wkbench.model.Task.prototype.getPlannedEndDate = function () {
    return this.plannedEndDate;
};
sas.stress.wkbench.model.Task.prototype.getStatusIcon = function () {
    return this.icon;
};
sas.stress.wkbench.model.Task.prototype.getStatusDate = function () {
    return this.date;
};
sas.stress.wkbench.model.Task.prototype.getStatusValue = function () {
    return this.value;
};
sas.stress.wkbench.model.Task.prototype.getType = function () {
    return this.type;
};
sas.stress.wkbench.model.Task.prototype.getStatusComment = function () {
    return this.comment;
};
sas.stress.wkbench.model.Task.prototype.getCommentCount = function () {
    return this.commentCount;
};
sas.stress.wkbench.model.Task.prototype.getParms = function () {
    return this.parms;
};
sas.stress.wkbench.model.Task.prototype.getCanUpdate = function () {
    return this.canUpdate;
};
sas.stress.wkbench.model.Task.prototype.getComments = function () {
    var def = $.Deferred();
    var that = this;
    if (!this.comments) {//check cache
        sas.stress.wkbench.StressAPI.loadTaskComments(this).done(function (json) {
            that.comments = [];
            _.each(json, function (cmnt) {
                that.comments.push(new sas.stress.wkbench.model.TaskComment(that, cmnt.commentKey, cmnt.user,
                    cmnt.timestamp, cmnt.comment, cmnt.currentIterationNum));
            });
            def.resolve(that.comments);
        }).fail(function (jqxhr) {  // S1391333
            def.reject(jqxhr);
        });
    } else {
        def.resolve(this.comments);
    }
    return def;
};

sas.stress.wkbench.model.Task.prototype.deleteComment = function (commentId) {
    var def = $.Deferred();
    var that = this;
    sas.stress.wkbench.StressAPI.deleteTaskComment(this, commentId).done(function () {
        that.comments = _.reject(that.comments, function (comment) {
            return comment.getId() === commentId;
        });
        that.commentCount = that.comments.length;
        def.resolve();
    }).fail(function (jqxhr) {  // S1391333
        def.reject(jqxhr);
    });
    return def;
};

sas.stress.wkbench.model.Task.prototype.editComment = function (commentId, text) {
    var def = $.Deferred();
    var that = this;
    sas.stress.wkbench.StressAPI.editTaskComment(this, commentId, text).done(function (result) {
        var comment = _.find(that.comments, function (comment) {
            return comment.getId() === commentId;
        });
        comment.setText(text);
        comment.setAuthor(result.user);
        comment.setDate(result.timestamp);
        def.resolve();
    }).fail(function (jqxhr) {  // S1391333
        def.reject(jqxhr);
    });
    return def;
};

sas.stress.wkbench.model.Task.prototype.createComment = function (text) {
    var def = $.Deferred();
    var that = this;
    sas.stress.wkbench.StressAPI.createTaskComment(this, text).done(function (json) {
        var comment = new sas.stress.wkbench.model.TaskComment(that, json.commentKey, json.user, json.timestamp,
            json.comment, json.currentIterationNum);
        that.comments.push(comment);
        that.commentCount = that.comments.length;
        def.resolve();
    }).fail(function (jqxhr) {  // S1391333
        def.reject(jqxhr);
    });
    return def;
};

sas.stress.wkbench.model.Task.prototype.deleteAttachment = function (attachmentId) {
    var def = $.Deferred();
    var that = this;
    sas.stress.wkbench.StressAPI.deleteTaskAttachment(attachmentId).done(function () {
        that.attachments = _.reject(that.attachments, function (attachment) {
            return attachment.getId() === attachmentId;
        });
        def.resolve();
    }).fail(function (jqxhr) {  // S1391333
        def.reject(jqxhr);
    });
    return def;
};

sas.stress.wkbench.model.Task.prototype.editAttachment = function (attachmentId, description) {
    var def = $.Deferred();
    var that = this;
    sas.stress.wkbench.StressAPI.editTaskAttachment(attachmentId, description).done(function () {
        var attachment = _.find(that.attachments, function (attachment) {
            return attachment.getId() === attachmentId;
        });
        attachment.setDescription(description);
        def.resolve();
    }).fail(function (jqxhr) {  // S1391333
        def.reject(jqxhr);
    });
    return def;
};

sas.stress.wkbench.model.Task.prototype.createAttachment = function (attachmentFile, name, text) {
    var def = $.Deferred();
    var that = this;
    sas.stress.wkbench.StressAPI.createTaskAttachment(this, attachmentFile, name, text).done(function (json) {
        var attachment = new sas.stress.wkbench.model.TaskAttachment(that, json.attachmentKey, json.updatedByUser,
            json.updatedTimestamp, json.name, json.description, json.currentIterationNum);
        that.attachments.push(attachment);
        def.resolve();
    }).fail(function (jqxhr) {  // S1391333
        def.reject(jqxhr);
    });
    return def;
};

sas.stress.wkbench.model.Task.prototype.getAttachments = function () {
    var def = $.Deferred();
    var that = this;
    if (!this.attachments) {//check cache
        sas.stress.wkbench.StressAPI.loadTaskAttachments(this).done(function (json) {
            that.attachments = [];
            _.each(json, function (att) {
                that.attachments.push(new sas.stress.wkbench.model.TaskAttachment(that, att.attachmentKey, att.updatedByUser,
                    att.updatedTimestamp, att.name, att.description, att.currentIterationNum));
            });
            def.resolve(that.attachments);
        }).fail(function (jqxhr) {  // S1391333
            def.reject(jqxhr);
        });
    } else {
        def.resolve(this.attachments);
    }
    return def;
};

sas.stress.wkbench.model.Task.prototype.getActions = function () {
    return this.actions;
};

sas.stress.wkbench.model.Task.prototype.addAction = function (action) {
    this.actions.push(action);
};

sas.stress.wkbench.model.Task._fromJSON = function (json) {
    var project = sas.stress.wkbench.model.Project.getProject(json.projectKey);
    var task = new sas.stress.wkbench.model.Task(project, json.id, json.name, json.description, json.users, json.owner,
        json.dueInDays, json.status, json.type, json.impl, json.parms, json.iterationNum, json.commentCount,json.script,json.parentNodeId,json.plannedStartDate,json.plannedEndDate,json.taskGroup,json.canUpdate);
    _.each(json.validActions, function (action) {
        task.addAction(new sas.stress.wkbench.model.TaskAction(task, "STATUS_UPDATE", action.actionLabel, action.actionIcon,
            action.statusKey));
    });
    return task;
};

sas.stress.wkbench.model.Task.getSummaryTasks = function (project) {
    var deferred = $.Deferred();
    sas.stress.wkbench.StressAPI.loadSummaryTasks(project.getProcessId()).done(function (tasksJSON) {
        var tasks = [];
        _.each(tasksJSON, function (json) {
            var task = sas.stress.wkbench.model.Task._fromJSON(json);
            tasks.push(task);
        });
        deferred.resolve(tasks);
    }).fail(function (jqxhr) {
        deferred.reject(jqxhr);
    });
    return deferred;
};

sas.stress.wkbench.model.Task.getMyTasks = function (project) {
    var deferred = $.Deferred();
    sas.stress.wkbench.StressAPI.loadMyTasks(project.getProcessId()).done(function (tasksJSON) {
        var tasks = [];
        _.each(tasksJSON, function (json) {
            var task = sas.stress.wkbench.model.Task._fromJSON(json);
            tasks.push(task);
        });
        deferred.resolve(tasks);
    }).fail(function (jqxhr) {
        deferred.reject(jqxhr);
    });
    return deferred;
};

sas.stress.wkbench.model.Task.getAllMyTasks = function (project) {
    var deferred = $.Deferred();
    sas.stress.wkbench.StressAPI.loadAllMyTasks(project && project.getProcessId()).done(function (tasksJSON) {
        var tasks = [];
        _.each(tasksJSON, function (json) {
            var task = sas.stress.wkbench.model.Task._fromJSON(json);
            tasks.push(task);
        });
        deferred.resolve(tasks);
    }).fail(function (jqxhr) {
        deferred.reject(jqxhr);
    });
    return deferred;
};

sas.stress.wkbench.model.Task.getHistoricalTasks = function (project) {
    var deferred = $.Deferred();
    sas.stress.wkbench.StressAPI.loadHistoricalTasks(project.getProcessId()).done(function (json) {
        var tasks = [];
        _.each(json, function (task) {
            var type = task.type === 2 ? "MANUAL" : (task.type === 1 ? "END" : "START");
            var status = {
                comment: task.comment,
                date: task.timestamp,
                iconName: "COMPLETESTATUS",
                label: task.statusLabel === "ACTIVE" ? "task.status.started.fmt.txt" : task.statusLabel,
                value: task.statusId
            };
            tasks.push(new sas.stress.wkbench.model.Task(project, task.nodeId, task.label, task.description,
                [task.user], task.userRole, task.dueInDays, status, type, task.impl, task.parms, task.iterationNum,
                task.scriptStatus,task.scriptLog,task.parentNodeId,task.plannedStartDate,task.plannedEndDate,task.taskGroup));
        });
        deferred.resolve(tasks);
    }).fail(function (jqxhr) {
        deferred.reject(jqxhr);
    });
    return deferred;
};

sas.stress.wkbench.model.Task.prototype.isActive = function () {
    return this.value === "ACTIVE";
};

sas.stress.wkbench.model.Task.prototype.getScript=function(){
    return this.script;
};

sas.stress.wkbench.model.Task.prototype.isRunning = function () {
    return this.scriptStatus === "RUNNING" || this.scriptstatus==="READYTORUN" || this.scriptstatus==="Executing";
};

sas.stress.wkbench.model.Task.prototype.getScriptLog=function(){
    return this.scriptLog;
};