/**
 * Created by sinsrj on 1/24/2018.
 */
jQuery.sap.require("sas.stress.wkbench.model.Project");
jQuery.sap.require("sas.stress.config.StressAPI");
jQuery.sap.require("sas.stress.wkbench.StressUtil");

sap.ui.controller("sas.stress.wkbench.view.AddRemoveSheetTemplates",{
    onInit: function () {
        var rb = sas.stress.wkbench.StressUtil.getBundle();
        var self = this;
        var viewData = this.getView().getViewData();
        var project = sas.stress.wkbench.model.Project.getCurrentProject();
        var model = new sas.risk.common.models.RiskJSONModel({
            id: project.getId(),
            name: project.getName(),
            asOfDate: project.getAsOfDate(),
            description: project.getDescription(),

            processDefinitionKey:project.getProcessDefId(),
            processDefinitionName:project.getProcessDefinitionName(),
            processStartDate: project.getProcessStartDate(),
            processNotificationType: project.getProcessNotificationType(),
            sheetDefinitions: project.getSheetDefinitions()/*,
            projectIsLocked:project.isLocked()*/,
            requireComments:project.doesRequireComments(),
            dimensions:project.getDimensions(),
            dimensionCrossingName: project.getDimensionCrossingName(),
            worksheets: [],

            categories: {}
        });

        var _categories = {};
        self.getView().setModel(model);

        this.sheetDefinitions = project.getSheetDefinitions();
        var associatedSheets = [];

        _.each(this.sheetDefinitions, function (sheetDefinition) {
            associatedSheets.push(sheetDefinition.sheetTemplateKey);
        });
        var projectIsLocked=project.isLocked();
        $.when(sas.stress.config.StressAPI.loadSheetTemplates(false), sas.stress.config.StressAPI.loadSheetCategories()).done(function (sheetTemplates, categories) {
            //var sortedDefs = _.sortBy(sheetTemplates, "name");
            var workSheets = [];
            var packageSheets = [];
            // the all category will contain the total count and it goes first
            _categories[rb.getText("filter.category.view.all.txt")] = {
                    count: sheetTemplates.length,		// the count of "All" category should be the size of all the sheet templates
                    key: 0,
                    name: rb.getText("filter.category.view.all.txt")
            };
            _.each(sheetTemplates, function (workSheet) {
                //S1312525: get all the worksheets, including the ones from packages.
                var isPackageSheet = workSheet.taxonomyPackageOwners && workSheet.taxonomyPackageOwners.length > 0;
                var isSelected = associatedSheets.indexOf(workSheet.id) !== -1;
                workSheet.selected = isSelected;
                workSheet.editable = !projectIsLocked && !isPackageSheet;
                workSheet.nameVersion = sas.stress.wkbench.StressUtil.getNameVersion(workSheet.name, workSheet.version);
                var categoryKey,
                    categoryName,
                    sheetCategoryCount,
                    category;
                if (!isPackageSheet) {
                    // keep the normal worksheets on the top
                    workSheets.push(workSheet);

                    categoryKey = workSheet.categoryKey;
                    category = _.find(categories, function(cat){
                        return cat.categoryKey === categoryKey;
                    });
                    if (category) {
                        categoryName = category.name;

                        sheetCategoryCount = _categories[categoryName] || {count: 0, key: "0", name: ""};
                        // increment the count &&
                        sheetCategoryCount.count++;
                        _categories[categoryName] = {count: sheetCategoryCount.count, key: categoryKey, name: categoryName};
                    }
                } else if (isSelected) {
                    //only include the package sheets if they belong to the current project.
                    packageSheets.push(workSheet);

                    categoryKey = workSheet.getSheetCategory().getId();
                    category = _.find(categories, function(cat){
                        return cat.categoryKey === categoryKey;
                    });
                    if (category) {
                        categoryName = category.name;

                        sheetCategoryCount = _categories[categoryName] || {count: 0, key: "0", name: ""};
                        // increment the count &&
                        sheetCategoryCount.count++;
                        _categories[categoryName] = {count: sheetCategoryCount.count, key: categoryKey, name: categoryName};
                    }
                }
            });
            model.setProperty("/worksheets", workSheets.concat(packageSheets));
            model.setProperty("/categories", _categories);
            var selectedCategoryItem=_.find(_categories, function(categoryItem){
                return Number(categoryItem.key) === Number(viewData.selectedCategoryKey);
            });
            // the selected category
            model.setProperty("/selectedCategoryName", selectedCategoryItem.name);
            model.setProperty("/selectedCategoryItem", selectedCategoryItem);
            
            self._refreshView();
        }).fail(function (jqxhr) {  // S1391333
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });
    },
    onExit: function () {
        this.itemsTable=undefined;
        this.categoriesFilter=undefined;
    },

    toggleEditProjectDialogSaveButton: function(e) {
        /*var selectedItems=this.getView().workSheetList.getSelectedItems();
        if (selectedItems.length === 0){
            this.getView().okButton.setEnabled(true);
            return;
        }
        var areEqual = sas.stress.wkbench.model.Project._areSheetDefinitionsEqual(selectedItems);
        this.getView().okButton.setEnabled(!areEqual);*/
        var selectedItems= e.getParameter("selectedItems")||this.getView().workSheetList.getSelectedSheetTemplates();
        var isValid = (e.getParameter("selectedItemsLength") || selectedItems.length) !==0;
        var areEqual = false;
        if (isValid){
            areEqual = sas.stress.wkbench.model.Project._areSheetDefinitionsEqual(selectedItems);
        }
        this.getView().okButton.setEnabled(isValid&&!areEqual);
    },

    handleOKButtonPress: function() {
        // save the project
        var view = this.getView();
        var model = view.getModel();
        var sheetDefinitions = [];
        _.each(view.workSheetList.getSelectedSheetTemplates(), function (workSheet) {
            sheetDefinitions.push({sheetTemplateKey: workSheet.key});
        });
        var editedProject = {
            projectKey: model.getProperty("/id"),
            projectName: model.getProperty("/name"),
            asOfDateStr: model.getProperty("/asOfDate"),
            sheetDefinitions: sheetDefinitions,

            description: model.getProperty("/description"),
            processDefinitionKey: model.getProperty("/processDefinitionKey"),
            processDefinitionName: model.getProperty("/processDefinitionName"),
            processStartDateStr: model.getProperty("/processStartDate"),
            processNotificationType: model.getProperty("/processNotificationType")/*,
             projectIsLocked:project.isLocked()*/,
            requireComments: model.getProperty("/requireComments"),
            dimensions: model.getProperty("/dimensions"),
            dimensionCrossingName: model.getProperty("/dimensionCrossingName")
        };
        // are the sheet definitions equal
        var dfnsEqual = sas.stress.wkbench.model.Project._areSheetDefinitionsEqual(sheetDefinitions);
        sas.stress.wkbench.model.Project.editProject(editedProject).done(function(updatedProject){
            if (!dfnsEqual){
                // if not equal, we need to delete the default sheet key user preference
                sas.stress.wkbench.UserPrefs.delete("sas.stress.wkbench.sheet_key", updatedProject.getId());
            }
            view.fireEvent(view.getId() + "_CLOSING", { "action": "DONE", "response": "Done", "data": updatedProject });
        }).fail(function(e){
            sas.stress.wkbench.StressUtil.handleServerFailure(e);
        });
    },

    cancelAddRemoveSheetTemplates: function() {
        var view = this.getView();
        view.fireEvent(view.getId() + "_CLOSING", { "action": "CANCEL", response: "Cancel" });
    },
    //Kept this callback outside the widget since the matching comparision may differ based on the data.
    handleTestWidgetReady: function(table, list){
        this.widgetRendered=true;
        this.itemsTable=table;
        this.categoriesFilter=list;
    },

    _refreshView: function(){
        var self = this;
        var model = self.getView().getModel();
        if (this.widgetRendered) {
            // select the default or already present items
            var inputTemplates = model.getProperty("/sheetDefinitions");
            var tableRowsBinding = self.itemsTable.getBinding("rows");
            var rowsData = tableRowsBinding.getModel().getProperty(tableRowsBinding.getPath());
            var firstVisibleRow = 0;	// for scrolling to the first selected sheet template
            _.each(inputTemplates, function (template) {
                var match = _.find(rowsData, function (row) {
                    return row.key === template.sheetTemplateKey;
                });
                if (match) {
                    var index = rowsData.indexOf(match);
                    self.itemsTable.addSelectionInterval(index, index);
                    if (firstVisibleRow === 0)
                        firstVisibleRow=index;
                    else {
                    	// we need to be @ the first selection. if the index is < use else, we are good
                        if (index <= firstVisibleRow)
                            firstVisibleRow=index;
                    }
                }
            });

            // select the default after all events processed
            setTimeout(function(){
                // select the default category
                if (model.getProperty("/selectedCategoryItem")){
                    self.categoriesFilter.setSelectedItem(model.getProperty("/selectedCategoryItem"), true);
                    self.itemsTable.setFirstVisibleRow(firstVisibleRow);	// scroll to that row
                }
            }, 0);

        }

        this.widgetRendered=false;  // reset once done
    }
});