jQuery.sap.require("sap.ui.core.routing.Router");
jQuery.sap.require("sap.ui.model.resource.ResourceModel");
jQuery.sap.require("sas.risk.common.utils.LayoutService");
jQuery.sap.require("sas.risk.common.models.RiskJSONModel");
jQuery.sap.require("sas.stress.wkbench.CapabilityManager");
jQuery.sap.require("sas.stress.wkbench.EventBusConstants");
jQuery.sap.require("sas.stress.wkbench.LeftNavManager");
jQuery.sap.require("sas.stress.wkbench.StressAPI");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.stress.wkbench.ViewManager");
jQuery.sap.require("sas.stress.wkbench.model.Project");
jQuery.sap.require("sas.stress.wkbench.model.TaskAttachment");
jQuery.sap.require("sas.stress.wkbench.widget.StatusLabel");
jQuery.sap.require("sas.stress.wkbench.widget.CommentDialog");
jQuery.sap.require("sas.stress.scripts.model.Script");
jQuery.sap.require("sas.stress.wkbench.StressGoTemplates");
jQuery.sap.require("sas.stress.wkbench.widget.ProcessDiagram");
jQuery.sap.require("sas.stress.scripts.ScriptsAPI");

sap.ui.controller("sas.stress.wkbench.view.HomePage", {
    onInit: function () {
        this.router = sap.ui.core.routing.Router.getRouter("router");
        this.router.attachExactRouteMatched("HomePage", this._init, this);
        this.navChecker = sas.stress.wkbench.StressUtil.createNavChecker(true, this.getView());

        // save the table
        new sas.risk.common.utils.LayoutService(this.getView().taskTable, "HomePageTaskTable");

        // show groove bar
        sas.stress.wkbench.LeftNavManager.getInstance().showGrooveBar(sas.stress.wkbench.LeftNavManager.PROJECTS);
    },
    _init: function (event) {
        //var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.wkbench");
        this.taskMap = {};
        this._pollIntervalIDs = [];
        var projectId = event.getParameter("arguments").projectId;
        var view = this.getView();
        view.setBusyIndicatorDelay(0);
        view.setBusy(true);
        view.setModel(new sap.ui.model.resource.ResourceModel({bundleName: "sas.stress.wkbench.messagebundle"}), "i18N");
        this._loadProject(projectId,true);

        sas.eventBus.subscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.TASK_DIAGRAM_CLICKED, this.diagramClickHandler, this);
        sas.eventBus.subscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.TASK_DIAGRAM_DOUBLECLICKED, this.diagramDoubleClickHandler, this);

    },
    _loadProject: function(projectId,poll) {

        var that = this;
        var view = this.getView();

        // S1273684: don't use the cached project since it might be stale or deleted by others
        //that.project = sas.stress.wkbench.model.Project.getProject(projectId);
        //if (!that.project) {    // no it is not in the cache
        sas.stress.wkbench.model.Project.loadProject(projectId, true).done(function (project) {
            // store it
            that.project = sas.stress.wkbench.model.Project.getProject(projectId);
            // init the controller
            that.refreshModel(poll);    //S1314295

            view.diagramBreadCrumb.removeAllItems();
            view.diagramBreadCrumb.addBreadcrumbItem("Root", that.project.processDefinitionName);

        }).fail(function (jqxhr) {
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            that.getView().setBusy(false);
            sas.stress.wkbench.ViewManager.loadProjectManager();
            //S1272357: the following seems to help the case when launching app with a preferred project that is no longer available
            //setTimeout(function () {
            //    sas.stress.wkbench.ViewManager.loadProjectManager();
            //}, 0);
            //that.getView().destroy();
        });

    },
    refreshModel:function(poll){    //S1314295
        var that = this;
        var p = this.project;
        var json = {};
        json.project = this.project;
        json.selectedTask = -1;

        this.setColumnVisibilityFromPrefs();

        if (p.getProcessId() > 0 && p.getProcessDefId() > 0) { //Only ids > 0 are valid.
            $.when(p.getSummaryTasks(), p.getAllMyTasks(), p.getProcess(), p.getHistoricalTasks()).done(function (summaryTasks,
                                                                                                                  myTasks,
                                                                                                                  process,
                                                                                                                  historyTasks) {
                that.process = process;
                json.historyTasks = that._buildTaskJson(historyTasks, process);
                json.summaryTasks = that._buildTaskJson(summaryTasks, process);
                that.initialTaskSelected = -1;
                json.myTasks = that._buildTaskJson(myTasks, process);
                json.process = {};
                json.process.nodes = process.getNodes();
                json.process.links = process.getLinks();
                json.isProjectLocked= p.isLocked();
                var model = new sas.risk.common.models.RiskJSONModel(json);
                that.getView().setModel(model);

                setTimeout(function () {
                    that.getView().taskTable.setSelectedIndex(that.initialTaskSelected);
                    json.selectedTask = that.initialTaskSelected;   // S1379945.. the table listens to this attribute
                    if (!that._pollIntervalIDs.length&&that.initialTaskSelected>=0) {    // Srini and S1379945
                        that.getView().taskTable.setFirstVisibleRow(that.initialTaskSelected);
                    }
                }, 0);
                if (poll) {
                    that._initcontroller();    // srini
                }else{
                    if (that.getView().searchField.getValue().length) {
                        that.onFilterChanged();
                    }
                }
            }).fail(function (jqxhr) {
                sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            }).always(function () {
                that.getView().setBusy(false);
                var rb = sas.stress.wkbench.StressUtil.getBundle();
                that.getView().taskTable.setNoData(rb.getText("task.tile.alltasks.nodata.txt"));
            });
        } else {
            var model = new sas.risk.common.models.RiskJSONModel(json);
            that.getView().setModel(model);
            that.getView().setBusy(false);
            var rb = sas.stress.wkbench.StressUtil.getBundle();
            that.getView().taskTable.setNoData(rb.getText("task.tile.alltasks.nodata.txt"));
        }
    },

    _initcontroller: function () {
        this.poll();    // srini
    },
    layoutDiagram: function () {
        if (this.getView().workflowDiagram) {
            this.getView().workflowDiagram.layoutDiagram();
        }
    },

    diagramDoubleClickHandler: function (channel, event, data) {

        var nodeData = data.nodeData;
        var view = this.getView();
        var that = this;

        if (!nodeData) {
            return;
        }

        view.diagramBreadCrumb.addBreadcrumbItem(nodeData.key, nodeData.text);
        this.loadDiagramSubProcess(nodeData.key, false);

    },

    loadDiagramSubProcess: function(nodeId, updateBreadCrumb) {

        var that = this;
        // SRINI: S1421522.. if the nodeId == "Root", we need to set it to undefined so that the process is invoked
        //nodeId = (nodeId === "Root" ? undefined: nodeId);
        // TODO: Cleanup similar calls
        if ("Root" !== nodeId) {
            sas.stress.wkbench.StressAPI.loadSubProcess(this.project.getProcessId(), nodeId).done(function (json) {
                var process = new sas.stress.wkbench.model.Process(that.project, json.id, json.name, json.description, json.startNodeId, json.endNodeId);
                _.each(json.sequenceFlow, function (trans) {
                    var transition = new sas.stress.wkbench.model.Transition(trans.id, trans.sourceId, trans.targetId, trans.syncGroup, trans.expression);
                    process.addTransition(transition);
                });
                _.each(json.tasks, function (tsk) {
                    var task = sas.stress.wkbench.model.Task._fromJSON(tsk);
                    process.addTask(task);
                });
                if (updateBreadCrumb) {
                    sas.stress.wkbench.StressUtil._updateDiagramBreadCrumb(that.getView().diagramBreadCrumb, nodeId, that.project.processDefinitionName);  // S1421622
                }

                that.updateDiagram(process);
                that._selectActiveTaskAfterNavigation(json.tasks);  // S1421622
            });
        }
        else {
            sas.stress.wkbench.StressAPI.loadProcess(this.project.getProcessId()).done(function (json) {
                var process = new sas.stress.wkbench.model.Process(that.project, json.id, json.name, json.description, json.startNodeId, json.endNodeId);
                _.each(json.sequenceFlow, function (trans) {
                    var transition = new sas.stress.wkbench.model.Transition(trans.id, trans.sourceId, trans.targetId, trans.syncGroup, trans.expression);
                    process.addTransition(transition);
                });
                _.each(json.tasks, function (tsk) {
                    var task = sas.stress.wkbench.model.Task._fromJSON(tsk);
                    process.addTask(task);
                });

                sas.stress.wkbench.StressUtil._updateDiagramBreadCrumb(that.getView().diagramBreadCrumb, nodeId, that.project.processDefinitionName);  // S1421622

                that.updateDiagram(process);
                that._selectActiveTaskAfterNavigation(json.tasks);  // S1421622
            });
        }
    },

    updateDiagram: function(process) {
        process = process?process:this.process; // process = (_.isNull(process)) ? this.process : process; S1387940
        this.getView().getModel().setProperty("/process/nodes", process.getNodes());
        this.getView().getModel().setProperty("/process/links", process.getLinks());
        this.getView().getModel().checkUpdate(true);

        var that = this;
        setTimeout(function () {
            that.recenterDiagram();
        }, 150);
    },

    diagramClickHandler: function (channel, event, data) {
        var table = this.getView().taskTable;
        var nodeData = data.nodeData;

        table.setSelectedIndex(-1);

        if (!nodeData) {
            return;
        }

        var currentContent = this.getView().getModel().getProperty("/summaryTasks");

        // We have to loop through the model because the row we want to select might not have been loaded yet
        for (var i = 0; i < currentContent.length; i++) {
            var context = table.getContextByIndex(i);
            if (context) {
                var path = context.getPath();
                if (table.getModel().getProperty(path).id === nodeData.key) {
                    // Scrolling to a row and setting index at the same time was causing flashing
                    (function(e) {
                        setTimeout(function(){
                            table.setFirstVisibleRow(e);
                        }, 10);
                    })(i);
                    table.setSelectedIndex(i);
                    break;
                }
            }
        }
    },
    _handleTaskSwitch: function () {
        var path = this.getView().taskSelect.getSelectedKey();
        this.getView().taskTable.bindRows("/" + path);
        this.getView().getModel().setProperty("/selectedTask", -1);

        this.onFilterChanged();
    },

    _buildTaskJson: function (tasks, process) {
        var currentIteration = process.getProcessProject().getCurrentIterationNo();
        var json = [];
        for (var i = 0; i < tasks.length; i++) {
            var task = tasks[i];
            var parms = null;
            var iteration = task.getIterationNum();
            var value = task.getStatusValue();
            if (value === "ACTIVE" && this.initialTaskSelected !== undefined && this.initialTaskSelected < 0) {
                // save this, we will call it in a timeout at the end of initialization to ensure the correct row in the table is selected
                this.initialTaskSelected = i;
            }
            var status = {
                label: task.getStatusLabel(),
                icon: task.getStatusIcon(),
                date: task.getStatusDate()
            };
            if (task.getParms()&&(task.getParms() !== "" && task.getParms() !== null )) {
                if (task.getParms().parmType === "URL") {
                    parms = {
                        url: task.getParms().resolvedURL
                    };
                }
            }
            var statusLabel = sas.stress.wkbench.widget.StatusLabel.calculateStatusLabel(status);
            var calcIteration = iteration;
            if (iteration === 0) {
                calcIteration = currentIteration;
            }
            json.push({
                id: task.getId(),
                name: task.getName(),
                description: task.getDescription(),
                duration: task.getDueDays(),
                plannedStart: task.getPlannedStartDate(),
                plannedEnd: task.getPlannedEndDate(),
                process: this.project.getProcessDefinitionName(),
                iteration: calcIteration,
                commentCount: task.getCommentCount(),
                taskGroup: task.getTaskGroup(),
                users: task.getUsers() || [],
                owner: task.getOwner(),
                status: status,
                parentNodeId: task.getParentNodeId(),
                statusLabel: statusLabel,
                parms: parms,
                actions: task.getActions() || [],
                script: task.getScript()    // Srini
            });
            this.taskMap[task.getId()] = task;
        }
        return json;
    },
    onExit: function () {
        // clear the intervals for polling..
        this._clearInterval();  // S1408264

        // Remove any subscriptions to ensure we don't get duplicates the next time the controller is initialized
        sas.eventBus.unsubscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.TASK_DIAGRAM_CLICKED, this.diagramClickHandler, this);
        sas.eventBus.unsubscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.TASK_DIAGRAM_DOUBLECLICKED, this.diagramClickHandler, this);
        this.router.detachExactRouteMatched("HomePage", this._init, this);
        // clear the View on the router otherwise the next router.navTo() would fail with a blank page
        this.router.setView("sas.stress.wkbench.view.HomePage", null);

        this.navChecker.stop();
    },
    _getSelectedTaskId: function () {
        var view = this.getView();
        var table = view.taskTable;

        var index = table.getSelectedIndex() < 0? this.initialTaskSelected : table.getSelectedIndex();
        var context = table.getContextByIndex(index);
        // S1401661.. if the context is not found.. use the table binding ( i believe this returns the rows based on filters)
        if (context===undefined){
            var binding = table.getBinding("rows");
            var contexts=binding.getContexts();
            context=_.find(contexts, function(context){
                var items=context.getModel().getProperty(binding.getPath());
                return items[index];
            });
        }
        if (context) {
            var selectedPath = context.getPath();
            return table.getModel().getProperty(selectedPath + "/id");
        }
    },
    _getSelectedTask: function () {
        var taskId = this._getSelectedTaskId();
        if (taskId) {
        	return this.taskMap[taskId];
        }else{
            return this.taskMap[_.keys(this.taskMap)[0]];
        }
    },
    /**
     * return the active task
     *
     * @returns {*}
     * @private
     */
    _getActiveTask: function () {
        var table = this.getView().taskTable;
        // use table binding
        var binding = table.getBinding("rows");
        var contexts = binding.getContexts();
        // the items from the first context
        if (contexts.length > 0) {
            var item = _.find(contexts[0].getModel().getProperty(binding.getPath()), function (item) {
                if (item.status.label === "task.status.inprogress.txt") {
                    return true;
                } else {
                    return false;
                }
            });

            return item;
        } else {
            return undefined;
        }
    },
    enableTaskActions: function (selectedTaskId) {
        var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();
        var controller = this;
        var view = this.getView();
        var i = 0;

        if (selectedTaskId > -1 && capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.PROCESS.ACTION)
            && !this.project.isLocked() /*&& !this._areTasksRunning()*/) {  //S1511358 allow multiple tasks to run in parallel
            // S1401661.. make sure there are no tasks running
            var task = this._getSelectedTask();
            /*
                Srini...the following is to stop a task from re-running an executing script.. once executing it cannot be completed too...
                If ever, it can be, we can add the enabled to the MenuItem below
             */
            var taskScript=task?task.getScript()||task.script:undefined;
            if (taskScript!==undefined&&taskScript.status!==undefined){
                if (taskScript.status.toUpperCase()==="EXECUTING"){
                    return false;
                }
            }
            if (task.getCanUpdate() === false || task.getTaskGroup() === true) {return false;}  // S1419643
            // every task has iterate (now separate) and other actions. IF parent of a group, no other actiosn available
            var isANonIterateAction = _.find(task.getActions(), function (action) {
                return "ITERATE" !== action.type;
            });
            if (isANonIterateAction === undefined) { return false; } // parent tasks do not have actions.. disable button

            view.actions.removeAllItems();
            _.each(task.getActions(), function (action) {
                /*
                    Srini... cannot start new iteration if there are SAS Script tasks running
                 */
                //var iterationMenuItemEnabled=true;
                if ("ITERATE"!==action.type/*&&that._areTasksRunning()*/) {
                var item = new sas.hc.m.MenuItem({
                    key: i++,
                    text: "{i18N>" + action.getLabel() + "}",
                    press: [controller.performTaskAction, controller],
                    enabled: true
                });
                view.actions.addItem(item);
                }else{
                    // RQSSTR-331... increment the key to go to the next action item the first is always iterate that we are ignoring
                    i++;
                }
            });

            return task.isActive();
        }
        return false;
    },
    onFilterChanged: function (e) {
        var that=this;
        var value = /*e.getSource().getValue() ||*/ this.getView().searchField.getValue() || "";
        var table = this.getView().taskTable;
        var nameFilter = new sap.ui.model.Filter("name", sap.ui.model.FilterOperator.Contains, value);
        var descFilter = new sap.ui.model.Filter("description", sap.ui.model.FilterOperator.Contains, value);
        var idFilter = new sap.ui.model.Filter("id", sap.ui.model.FilterOperator.Contains, value);
        var ownerFilter = new sap.ui.model.Filter("owner", sap.ui.model.FilterOperator.Contains, value);
        var statusFilter = new sap.ui.model.Filter("statusLabel", sap.ui.model.FilterOperator.Contains, value);
        var iterationFilter = new sap.ui.model.Filter("iteration", sap.ui.model.FilterOperator.EQ, value);
        var allFilters = new sap.ui.model.Filter([nameFilter, descFilter, idFilter, ownerFilter, statusFilter, iterationFilter]);
        var binding = table.getBinding("rows");
        binding.filter(allFilters);

        setTimeout(function(){
            that._setSelectedTaskAfterFiltering();
        }, 0);
    },
    _setSelectedTaskAfterFiltering: function(){
        var view = this.getView();

        var selectedTaskIndex = this._getSelectedTaskIndexAfterFiltering();
        view.taskTable.setSelectedIndex(selectedTaskIndex);
        if (!this._pollIntervalIDs.length&&this.initialTaskSelected>=0) {    // Srini and S1379945
            view.taskTable.setFirstVisibleRow(selectedTaskIndex);
        }
    },
    performTaskAction: function (event) {
        var rb = sas.stress.wkbench.StressUtil.getBundle();
        var task = this._getSelectedTask();
        var item = event.getSource();
        var taskAction = task?task.getActions()[item.getKey()]:new sas.stress.wkbench.model.TaskAction(undefined, "ITERATE", "home.mytasks.iterate.");
        var project = this.project;
        var controller = this;
        if (taskAction) {
            switch (taskAction.getType()) {
                case "STATUS_UPDATE":
                    var callBack = function (comment) {
                        sas.stress.wkbench.StressAPI.saveTaskStatus(taskAction, comment).done(function () {
                            //S1276875: no need to REFRESH PAGE
                            controller._loadProject(project.getId(),true);
                            // Clear our project cache
                            //sas.stress.wkbench.model.Project.clearProject(project.getId());
                            //hasher.setHash(hasher.getHash() + "?refresh");//tweak the hasher so that it reloads the page
                            //sas.stress.wkbench.ViewManager.loadHomePage(project);
                        }).fail(function (jqxhr) {
                            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
                        });
                    };
                    // call REST API
                    sas.stress.wkbench.StressAPI.loadTaskTransitionChangeSummary(taskAction).done(function(changes){
                        sas.stress.wkbench.widget.CommentDialog.show(rb.getText(taskAction.getLabel()), callBack, undefined, changes);
                    }).fail(function(jqxhr){
                        sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
                    });
                    break;
                case "LAUNCH":
                    if (task.implementationType !== undefined && task.implementationType !==  "url") {
                        var payload = taskAction.getPayload();
                        var func = payload.shift();
                        func.apply(null, payload);
                    } else if (task.implementationType !== undefined && task.implementationType ===  "url") {
                        var payload = taskAction.getPayload();
                        var resolvedURL = payload[2].resolvedURL;
                        window.open(resolvedURL, "_blank");
                    }
                    break;
                case "ITERATE":
                    if (task) { // RQSSTR-331
                        var possibleStartingTasks = this.getPossibleStartingTasksForNewIteration();
                        var newProjectIteration = sas.stress.wkbench.ViewManager.loadNewProjectIteration({
                            action: taskAction,
                            process: this.process,
                            possibles: possibleStartingTasks
                        });

                        var dialog = new sas.hc.m.Dialog({
                            title: sas.stress.wkbench.StressUtil.getBundle().getText("iteration.title.txt"),
                            content: newProjectIteration,
                            beginButton: newProjectIteration.okButton,
                            endButton: newProjectIteration.cancelButton,
                            afterClose: function () {
                                dialog.destroy();
                            },
                            initialFocus: newProjectIteration.cancelButton
                        });
                        newProjectIteration.attachClosing(function (e) {
                            if (e.getParameter("action") === "CANCEL") {
                                this.destroy();
                            } else {
                                var response = e.getParameter("response");
                                if (response !== "Done") {
                                    sas.stress.wkbench.StressUtil.handleServerFailure(response);
                                    newProjectIteration.setBusy(false);
                                } else {
                                    //S1276875: no need to REFRESH PAGE
                                    controller._loadProject(project.getId());
                                    // Clear our project cache
                                    //sas.stress.wkbench.model.Project.clearProject(project.getId());
                                    //hasher.setHash(hasher.getHash() + "?refresh");//tweak the hasher so that it reloads the page
                                    //sas.stress.wkbench.ViewManager.loadHomePage(project);
                                    newProjectIteration.setBusy(false);
                                    this.destroy();
                                }
                            }
                        }, dialog);
                        dialog.open();
                    }else{  // RQSSTR-331
                        sas.hc.m.MessageDialog.warning({
                            text: rb.getText("home.tasks.start.new.iteration.confirm.txt"),
                            title: sas.stress.wkbench.StressUtil.getBundle().getText("new.iteration.title.txt"),
                            buttonActions: [sas.hc.m.MessageDialogAction.Ok, sas.hc.m.MessageDialogAction.Cancel],
                            buttonPress: function (event) {
                                var action = event.getParameter("origin").getMessageDialogAction();
                                if (action === sas.hc.m.MessageDialogAction.Ok) {
                                    sas.stress.wkbench.StressAPI.createProjectIteration(project).done(function () {
                                        //S1276875: no need to REFRESH PAGE
                                        controller._loadProject(project.getId());
                                    }).fail(function (e) {
                                        sas.stress.wkbench.StressUtil.handleServerFailure(e);
                                    });
                                } else {

                                }
                            }
                        });
                    }
                    break;
                case "EXECUTE":
                    var payload = taskAction.getPayload();
                    this.processSASScriptTask(payload[0],payload[1],payload[2])
                        .done(function(script){
                            var task=payload[1];
                            var parametersDlgView=sas.stress.wkbench.ViewManager.loadTaskSASScriptParameters({task:payload[1],script:script});
                            var parametersdialog=new sas.hc.m.Dialog({
                                id:"ScriptParameters",
                                title: rb.getText("script.parameters.dialog.title.txt"),
                                content: parametersDlgView,
                                beginButton:parametersDlgView.okButton,
                                endButton: parametersDlgView.cancelButton,
                                resizable:true,
                                afterClose: function () {
                                    parametersdialog.destroy();
                                    // set the selection as it was
                                    controller.getView().taskTable.setSelectedIndex(controller.initialTaskSelected);
                                },
                                contentHeight: "30rem",
                                contentWidth: "70rem",
                                initialFocus: parametersDlgView.okButton
                            });
                            parametersDlgView.attachClosing(function(e){
                                // set the selection as it was
                                controller.getView().taskTable.setSelectedIndex(controller.initialTaskSelected);
                                parametersdialog.close();
                                this.destroy();
                                var response= e.getParameter("response");
                                if(e.getParameter("action")==="CANCEL"){
                                    //... nothing to do
                                }else{
                                    response=e.getParameter("response");
                                    if (response!=="Done") {
                                        sas.stress.wkbench.StressUtil.handleServerFailure(response);
                                    } else {
                                        var responsedata= e.getParameter("data");
                                        // start polling
                                        // need to setup the progress indicator on the table
                                        var path = controller.getView().taskSelect.getSelectedKey();
                                        var data = controller.getView().getModel().getProperty("/"+path);
                                        var selectedTask= _.find(data,function(mytask){
                                            return mytask.id===task.id;
                                        });
                                        if (selectedTask){
                                            // make sure the binding contexts on the table are updated
                                            controller.onFilterChanged();   // S1401661

                                            selectedTask.script=responsedata.script;
                                            // S1401661.. the above will update the data
                                            // controller.getView().getModel().setProperty("/"+path,data);
                                        }
                                        controller.poll(true);
                                    }
                                }
                            },parametersdialog);
                            //setTimeout(function(){
                            parametersdialog.open();
                            //}, 1000);
                        })
                        .fail(function(jqxhr){
                            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
                        });
                    break;
            }
        }
    },
    getPossibleStartingTasksForNewIteration: function () {
        var taskInfos = {};
        var task = this._getActiveTask() || this._getSelectedTask();    // make sure we have a task;
        _.each(this.getView().getModel().getProperty("/historyTasks"), function (taskJson) {
            // S1404588.. if a task does not have any default actions, ignore such task as a possible starting task for new iteration
            if (taskJson.actions && taskJson.actions.length > 0) {
                taskInfos[taskJson.id] = {name: taskJson.name, isActive: ((task === undefined) ? false : (task.id === taskJson.id)) };
            }
        });
        return taskInfos;
    },
    enableComments: function (selectedTaskId) {
        if (selectedTaskId > -1) {
            this.refreshDetailPanel();
            this.recenterDiagram();
            return true;
        }
        return false;
    },
    _startNewIteration: function(){ // RQSSTR-331
        var rb = sas.stress.wkbench.StressUtil.getBundle();
        var task = this._getSelectedTask();
        var taskAction = task&&task.getActions()[0]?task.getActions()[0]:new sas.stress.wkbench.model.TaskAction(task, "ITERATE", "home.mytasks.iterate."); // S1404588
        var project = this.project;
        var controller = this;

        if (task) { // RQSSTR-331
            var possibleStartingTasks = this.getPossibleStartingTasksForNewIteration();
            var newProjectIteration = sas.stress.wkbench.ViewManager.loadNewProjectIteration({
                action: taskAction,
                process: this.process,
                possibles: possibleStartingTasks
            });

            var dialog = new sas.hc.m.Dialog({
                title: sas.stress.wkbench.StressUtil.getBundle().getText("new.iteration.title.txt"),
                content: newProjectIteration,
                beginButton: newProjectIteration.okButton,
                endButton: newProjectIteration.cancelButton,
                afterClose: function () {
                    dialog.destroy();
                },
                initialFocus: newProjectIteration.cancelButton
            });
            newProjectIteration.attachClosing(function (e) {
                if (e.getParameter("action") === "CANCEL") {
                    this.destroy();
                } else {
                    var response = e.getParameter("response");
                    if (response !== "Done") {
                        sas.stress.wkbench.StressUtil.handleServerFailure(response);
                        newProjectIteration.setBusy(false);
                    } else {
                        //S1276875: no need to REFRESH PAGE
                        controller._loadProject(project.getId());
                        // Clear our project cache
                        //sas.stress.wkbench.model.Project.clearProject(project.getId());
                        //hasher.setHash(hasher.getHash() + "?refresh");//tweak the hasher so that it reloads the page
                        //sas.stress.wkbench.ViewManager.loadHomePage(project);
                        newProjectIteration.setBusy(false);
                        this.destroy();
                    }
                }
            }, dialog);
            dialog.open();
        }else{  // RQSSTR-331
            sas.hc.m.MessageDialog.information({
                text: rb.getText("home.tasks.start.new.iteration.confirm.txt"),
                title: sas.stress.wkbench.StressUtil.getBundle().getText("new.iteration.title.txt"),
                buttonActions: [sas.hc.m.MessageDialogAction.Ok, sas.hc.m.MessageDialogAction.Cancel],
                buttonPress: function (event) {
                    var action = event.getParameter("origin").getMessageDialogAction();
                    if (action === sas.hc.m.MessageDialogAction.Ok) {
                        sas.stress.wkbench.StressAPI.createProjectIteration(project).done(function () {
                            //S1276875: no need to REFRESH PAGE
                            controller._loadProject(project.getId());
                        }).fail(function (e) {
                            sas.stress.wkbench.StressUtil.handleServerFailure(e);
                        });
                    } else {

                    }
                }
            });
        }
    },
    recenterDiagram: function () {
        this.getView().workflowDiagram.centerOnNode(this._getSelectedTask());
    },
    refreshDetailPanel: function () {
        var task = this._getSelectedTask();
        var view = this.getView();
        var isActive = task.isActive() && !this.project.isLocked();
        var json = [];
        view.getModel().setProperty("/comments", []);
        $.when(task.getAttachments(), task.getComments()).done(function (attachments, comments) {
            _.each(comments, function (comment) {
                json.push({
                              author: comment.getAuthor(),
                              date: comment.getDate(),
                              text: comment.getText(),
                              editable: isActive,
                              number: comment.getIterationNum(),
                              entryId: comment.getId()
                          });
            });
            _.each(attachments, function (attachment) {
                json.push({
                              author: attachment.getAuthor(),
                              date: attachment.getDate(),
                              text: attachment.getDescription(),
                              editable: isActive,
                              number: attachment.getIterationNum(),
                              entryId: attachment.getId(),
                              attachmentName: attachment.getName()
                          });
            });
            var sortedJson=_(json).chain().sortBy("date").sortBy("number").reverse().value();  // S1334887 sorted descending by date and then iteration
            view.getModel().setProperty("/comments", sortedJson);
        }).fail(function (jqxhr) {  // S1391333
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });
    },
    isCommentsEditable: function (selectedTaskId) {
        if (selectedTaskId > -1 && !this.project.isLocked()) {
            return true;//ALLOW ALL FOR EA
//			var task=this._getSelectedTask();
//			return task.isActive();
        }
        return false;
    },
    saveComment: function (e) {
        var that = this;
        var task = this._getSelectedTask();
        var entry = e.getParameter("entry");
        var text = entry.getText();
        var path = entry.getBindingContext().getPath();
        var entryId = entry.getModel().getProperty(path + "/entryId");
        if (entry.getAttachmentName()) {
            task.editAttachment(entryId, text).done(function () {
                that.getView().getModel().checkUpdate(true);
                // Unlike with add and delete, don't update diagram on save.
            }).fail(function (jqxhr) {  // S1391333
                sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            });
        } else {
            task.editComment(entryId, text).done(function () {
                that.getView().getModel().checkUpdate(true);
                // Unlike with add and delete, don't update diagram on save.
            }).fail(function (jqxhr) {  // S1391333
                sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            });
        }
    },
    deleteComment: function (e) {
        var that = this;
        var task = this._getSelectedTask();
        var entry = e.getParameter("entry");
        var path = entry.getBindingContext().getPath();
        var entryId = entry.getModel().getProperty(path + "/entryId");
        if (entry.getAttachmentName()) {
            task.deleteAttachment(entryId).done(function () {
                that.updateTask(task);
            }).fail(function (jqxhr) {  // S1391333
                sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            });
        } else {
            task.deleteComment(entryId).done(function () {
                that.updateTask(task);
            }).fail(function (jqxhr) {  // S1391333
                sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            });
        }
    },
    addComment: function (e) {
        var text = e.getParameter("text");
        var attachmentFile = e.getParameter("attachment");
        var name = e.getParameter("attachmentName");
        var task = this._getSelectedTask();
        var that = this;
        if (attachmentFile) {
            task.createAttachment(attachmentFile, name, text).done(function () {
                that.updateTask(task);
            }).fail(function (jqxhr) {  // S1391333
                sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            });
        } else {
            task.createComment(text).done(function () {
                that.updateTask(task);
            }).fail(function (jqxhr) {
                sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            });
        }
    },
    updateTask: function (task) {
        // Replace the existing task in the process definition with the updated one.
        if (this.process.getTasks()[task.getId()]) {
            this.process.addTask(task);
            this.updateDiagram();
        }else{
            this.getView().getModel().checkUpdate(true);
        }
    },
    downloadAttachment: function (e) {
        var entry = e.getParameter("entry");
        var path = entry.getBindingContext().getPath();
        var attachmentId = entry.getModel().getProperty(path + "/entryId");
        // fixes for S1188266
        var that=this;
        var view=this.getView();
        var rb = sas.stress.wkbench.StressUtil.getBundle();
        view.setBusyIndicatorDelay(0);
        view.setBusy(true);

        sas.stress.wkbench.StressAPI.downloadTaskAttachment(attachmentId)
            .done(function(){
                view.setBusy(false);
            })
            .fail(function (error) {
                view.setBusy(false);
                sas.hc.m.MessageDialog.error({text: rb.getText("download.attachment.failed.message.txt"), title: rb.getText("download.attachment.failed.title.txt")});
            });
    },
    taskTableRowSelectionChange: function (e) {
        var view = this.getView();
        var model = view.getModel();

        model.setProperty("/selectedTask", view.taskTable.getSelectedIndex());
    },

    handleColumnVisibilityChange: function (e) {
        var view = this.getView();
        // var that = this;
        var visibilitySettings = {};
        _.each(view.taskTable.getColumns(), function (column) {
            var name = column.getProperty("name");
            if (name && name.length) {
                visibilitySettings[name] = column.getVisible();
            }
        });
        var vizSettingsStr = JSON.stringify(visibilitySettings);
        sas.stress.wkbench.UserPrefs.save("sas.stress.wkbench.visible_task_columns", vizSettingsStr, this.project.getId());
    },
    setColumnVisibilityFromPrefs: function () {
        var view = this.getView();
        var visibilityPrefs = sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.visible_task_columns", this.project.getId());
        // Do user preferences for the task table in this project already exist?
        if (visibilityPrefs) {
            var visibilitySettings = JSON.parse(visibilityPrefs);
            if (visibilitySettings) {
                _.each(view.taskTable.getColumns(), function (column) {
                    var name = column.getProperty("name");
                    if (name && name.length > 0 && undefined !== visibilitySettings[name]) {
                        column.setVisible(visibilitySettings[name]);
                    }
                });
            }
        }
    },
    processSASScriptTask:function(project,task,parameters){
        var deferred= $.Deferred();
        var that=this;
        this.getView().setBusy(true);
        this.getView().setBusyIndicatorDelay(0);
        var scriptId=parameters.scriptId;
        sas.stress.scripts.model.Script.loadScriptDetailsByID(scriptId,project.getId())
            .done(function(script){
                that.getView().setBusy(false);
                deferred.resolve(script);
            })
            .fail(function(jqxhr){
                that.getView().setBusy(false);
                deferred.reject(jqxhr);
            });

        return deferred;
    },

    // --------- Srini start -----------
    poll: function () {
        var controller = this;
        if (this._pollIntervalIDs.length === 0) {	// if there is polling occuring do not add to the interval
	        if (controller._areTasksRunning()) {
	            var model = controller.getView().getModel();

	            controller._pollIntervalIDs.push(setInterval(function () {
	                var p=controller.project;
	                controller.initialTaskSelected=-1;
	                $.when(p.getSummaryTasks(), p.getAllMyTasks(), p.getProcess(), p.getHistoricalTasks()).done(function (summaryTasks,
	                                                                                                                      myTasks,
	                                                                                                                      process,
	                                                                                                                      historyTasks) {
	                    console.info("Tasks: Polling...");
	                    // update the model properties with the respective task JSONs
	                    var json;
	                    // do these the first they do not have the task.script associated with them
	                    json=controller._buildTaskJson(historyTasks,process);
	                    model.setProperty("/historyTasks",json);
	                    json=controller._buildTaskJson(summaryTasks,process);
	                    model.setProperty("/summaryTasks",json);
	                    json=controller._buildTaskJson(myTasks,process);
	                    model.setProperty("/myTasks",json);

	                    if (!controller._areTasksRunning()) {
	                        controller._clearInterval();
	                        // polling done.. therefore, reload.. this is what is done when we do a "STATUS_UPDATE"
	                        controller._loadProject(p.getId(),false);
	                    }
	                }).fail(function (jqxhr) {
	                    controller._clearInterval();
	                    controller.onServerFailure(jqxhr);
	                }).always(function () {
	                    controller.getView().setBusy(false);
	                });
	            }, 20000/*2500*/));	// increase the time out to 20 secs
	        } else {
	            controller._clearInterval();
	            // S1401661.. make sure the table is filtered based on user preference
	            if (controller.getView().searchField.getValue().length) {
	                controller.onFilterChanged();
	            }
	        }
        } else {
        	console.log("TEST FILE ONLY::: no need to poll!!");
        }
    },

    _clearInterval: function () {
        // clear the interval ID
        _.each(this._pollIntervalIDs, function (intervalId) {
            clearInterval(intervalId);
        });
        this._pollIntervalIDs = [];
    },
    _areTasksRunning: function () {
        var model = this.getView().getModel();
        if (!model) return false;
        var key = this.getView().taskSelect.getSelectedKey();
        var data = model.getProperty("/"+key);
        var running=_.filter(data,function(each){
            return (each.script&&"Executing"===each.script.status);
        });
        if (running&&running.length>0){
            model.setProperty("/areTasksRunning", true);
            return true;
        }
        model.setProperty("/areTasksRunning", false);
        return false;
    },
    // -------  srini end -------------
    onServerFailure: function (jqxhr) {
        sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
    },
    onOpenLog:function(e){
        var controller=this;
        var view=this.getView();
        var model=view.getModel();

        //var input= e.getSource();   // fixes for S1268012
        var input =  sap.ui.getCore().byId(e.getParameters().id);
        var customData = _.find(input.getParent().getParent().getCustomData(), function (customData) {
            return customData.getKey() === "id";
        });
        var id = customData.getValue();
        var path = view.taskSelect.getSelectedKey();
        var data = model.getProperty("/"+path);
        var selectedTask= _.find(data,function(mytask){
            return mytask.id===id;
        });

        if (selectedTask&&selectedTask.script){
            var script=selectedTask.script;
            var rb=sas.stress.wkbench.StressUtil.getBundle();
            sas.stress.scripts.model.Script.getTaskScriptExecutionLog(script.key,script.runKey).done(function(log){
                var executionLogDlgView=sas.stress.wkbench.ViewManager.loadExecutionLogDialogView(selectedTask.name, selectedTask.script.status, log.logText, rb);
                var logDialog = new sas.hc.m.Dialog({
                    id:"ExecutionLog",
                    title: rb.getText("task.execution.log.dialog.header.title"),
                    content: executionLogDlgView,
                    endButton: executionLogDlgView.endbutton,
                    afterClose: function () {
                        logDialog.destroy();
                        // set the selection as it was
                        view.taskTable.setSelectedIndex(controller.initialTaskSelected);
                        view.taskTable.setFirstVisibleRow(controller.initialTaskSelected);
                    },
                    contentHeight: "30rem",
                    contentWidth: "84rem",
                    initialFocus: executionLogDlgView.endbutton,
                    resizable:true
                });
                executionLogDlgView.attachClosing(function(){
                    // set the selection as it was
                    view.taskTable.setSelectedIndex(controller.initialTaskSelected);
                    view.taskTable.setFirstVisibleRow(controller.initialTaskSelected);
                    logDialog.close();
                    this.destroy();
                },logDialog);
                //setTimeout(function(){
                logDialog.open();
                //}, 1000);
            }).fail(function(jqxhr){
                sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            });
        }
    },

    onDownloadLog:function(e){
        var controller=this;
        var view=this.getView();
        var model=view.getModel();

        var input =  sap.ui.getCore().byId(e.getParameters().id);
        var customData = _.find(input.getParent().getParent().getCustomData(), function (customData) {
            return customData.getKey() === "id";
        });
        var id = customData.getValue();
        var path = view.taskSelect.getSelectedKey();
        var data = model.getProperty("/"+path);
        var selectedTask= _.find(data,function(mytask){
            return mytask.id===id;
        });

        if (selectedTask&&selectedTask.script){
            var script=selectedTask.script;
            //var rb=sas.stress.wkbench.StressUtil.getBundle();
            var file = {};
            file.runId = script.runKey;
            file.filename="model_group_execution_sas.log";
            sas.stress.scripts.ScriptsAPI.downloadFile(file);
        }
    },

    onOKButton:function(){

    },
    _getSelectedTaskIndexAfterFiltering : function(){
        var view = this.getView();
        var table = view.taskTable;
        var rows = table.getRows();

        var index = table.getSelectedIndex() < 0? this.initialTaskSelected : table.getSelectedIndex();
        var context = table.getContextByIndex(index);
        // S1401661.. if the context is not found.. use the table binding ( i believe this returns the rows based on filters)
        if (context===undefined){
            var binding = table.getBinding("rows");
            var contexts=binding.getContexts();
            context=_.find(contexts, function(context){
                var items=context.getModel().getProperty(binding.getPath());
                return items[index];
            });
        }
        if (context) {
            var selectedTaskIndex = -1;
            _.find(rows, function(row, index){
                if (row.getBindingContext().getPath() === context.getPath()){
                    selectedTaskIndex = index;
                    return true;
                }else{
                    return false;
                }
            });

            return selectedTaskIndex;
        }
        return -1;
    },

    _selectActiveTaskAfterNavigation: function (tasks) {
        var that = this;
        for (var i = 0; i < tasks.length; i++) {
            var task =  tasks[i];
            var value = task.statusValue;
            if (value === "ACTIVE") {
                // save this, we will call it in a timeout at the end of initialization to ensure the correct row in the table is selected
                that.initialTaskSelected = i;
                break;
            }
        }
        setTimeout(function () {
            that.getView().taskTable.setSelectedIndex(that.initialTaskSelected);
            if (!that._pollIntervalIDs.length&&that.initialTaskSelected>=0) {    // Srini and S1379945
                that.getView().taskTable.setFirstVisibleRow(that.initialTaskSelected);
            }
        }, 0);
    }
});