jQuery.sap.require("sas.hc.ui.commons.windowshade.WindowShade");
jQuery.sap.require("sas.hc.ui.commons.windowshade.WindowShadeSection");
jQuery.sap.require("sas.hc.ui.layout.VerticalLayout");
jQuery.sap.require("sas.hc.m.Button");
jQuery.sap.require("sas.hc.m.Dialog");
jQuery.sap.require("sas.hc.m.ScrollContainer");
jQuery.sap.require("sas.hc.m.TextArea");
jQuery.sap.require("sas.hc.m.ResponsiveToolbar");
jQuery.sap.require("sas.risk.common.utils.LayoutService");
jQuery.sap.require("sas.risk.common.widgets.ZeroStateLayout");
jQuery.sap.require("sas.risk.common.models.RiskJSONModel");
jQuery.sap.require("sas.stress.wkbench.CapabilityManager");
jQuery.sap.require("sas.stress.wkbench.LeftNavManager");
jQuery.sap.require("sas.stress.wkbench.StressAPI");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.stress.wkbench.ViewManager");
jQuery.sap.require("sas.stress.wkbench.model.Project");
jQuery.sap.require("sas.stress.wkbench.model.ProjectTemplate");
jQuery.sap.require("sas.stress.wkbench.widget.AccessControlsDialog");

sap.ui.controller("sas.stress.wkbench.view.ProjectManager", {
    onInit: function () {
        this.router = sap.ui.core.routing.Router.getRouter("router");
        this.router.attachExactRouteMatched(["ProjectManager", "Default"], this._init, this);
        //TODO: need to find out why the view cannot be destroyed by NavChecker on exit
        this.navChecker = sas.stress.wkbench.StressUtil.createNavChecker(true, this.getView());
        this.capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();

        new sas.risk.common.utils.LayoutService(this.getView().table, "ProjectManagerHomeTable");
        new sas.risk.common.utils.LayoutService(this.getView().taskTable, "ProjectActiveTasksTable");

        // show groove bar
        sas.stress.wkbench.LeftNavManager.getInstance().showGrooveBar(sas.stress.wkbench.LeftNavManager.PROJECTS);
    },

    _init: function (event) {
        var view = this.getView();
        var i18N = new sap.ui.model.resource.ResourceModel({ bundleName: "sas.stress.wkbench.messagebundle" });
        var oArgs = event.getParameter("arguments"),
            oQuery;

        view.setModel(i18N, "i18N");
        /*
         * this is a map of the visible columns on the table
         * key = column name from json, value=(visible)true/false
         */
        this._projectsTableVisibleColumns = {}; // S1389396

        this._loadProjects();
        oQuery = oArgs["?query"];
        if (oQuery && "true"===oQuery.clearProjectUserPreference){
            // delete the last used project preference of the user
            sas.stress.wkbench.UserPrefs.delete("sas.stress.wkbench.project_key", undefined);
        }
        //this._loadTasks();
    },

    onExit: function () {
        this.router.detachExactRouteMatched(["ProjectManager", "Default"], this._init, this);
        // clear the View on the router otherwise the next router.navTo() would fail with a blank page
        this.router.setView("sas.stress.wkbench.view.ProjectManager", null);
        this.navChecker.stop();
    },

    _loadProjects:function() {
        var view = this.getView();
        var that = this;
        view.table.setBusyIndicatorDelay(0);
        view.table.setBusy(true);
        var rb = sas.stress.wkbench.StressUtil.getBundle();
        view.table.setNoData(rb.getText("project.noData.loading.txt"));
        sas.stress.wkbench.model.Project.getProjects().done(function (projects) {
            if (view.table.getModel() instanceof sap.ui.model.json.JSONModel) {
                view.table.getModel().setData(projects);
            } else {
                var projectsModel = new sas.risk.common.models.RiskJSONModel(projects);
                view.table.setModel(projectsModel);

                //S1354153: Do sorting after creation in view so it won't overwrite LayoutService
                //var sorters = [new sap.ui.model.Sorter("name", false)];
                //view.table.getBinding("rows").sort(sorters);
            }

            // store the columns and their visibility status     // S1389396
            _.each(view.table.getColumns(), function (column) {
                var name = column.getProperty("name");
                if (name && name.length) {
                    that._projectsTableVisibleColumns[name] = column.getVisible();
                }
            });
            // select the previous selected project by default
            setTimeout(function () {
                // the current project
                var project=sas.stress.wkbench.model.Project.getCurrentProject();
                if (project) {
                    // look for the index== project with the same ID in the list    // S1382441
                    // the idea is the rows are now sorted according to user prefs and we use them to get to our project
                    var rows=view.table.getBinding("rows");
                    if (rows) {
                        var index = -1;
                        var lastSelectedProjectContext = _.find(rows.getContexts(), function (context, idx) {
                            index = idx;
                            var obj = context.getObject();    // everything is now inside a context
                            return obj.getName().localeCompare(project.getName()) === 0;  // compare by name
                        });
                        //var lastSelectedProject = lastSelectedProjectContext ? lastSelectedProjectContext.getObject() : undefined;
                        // if found.. select it and scroll to it
                        if (index >= 0 /*&& (lastSelectedProject && lastSelectedProject.getId() === project.getId())*/) {   // do not worry about ID
                            that.getView().table.setSelectedIndex(index);
                            that.getView().table.setFirstVisibleRow(index);
                        }
                    }
                }
            }, 0);

            that._loadTasks();
        }).fail(function (jqxhr) {
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        }).always(function() {
            view.table.setBusy(false);
            view.table.setNoData(new sas.risk.common.widgets.ZeroStateLayout({
                icon: sas.icons.HC.ZEROSTATETABLE,
                text: "{i18N>project.noData.title}",
                buttons: [
                    new sas.hc.m.Button({
                        text: "{i18N>project.noData.button.txt}",
                        tooltip: "{i18N>project.createButton.tip}",
                        enabled: that.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.PROJECT.CREATE),
                        press: $.proxy(that.newProjectButtonPress, that)
                    }).addStyleClass("sasPrimaryButton")
                ]
            }));
        });
    },

    _loadProjectTemplates:function() {
        var view = this.getView();
        var that = this;
        view.table.setBusyIndicatorDelay(0);
        view.table.setBusy(true);
        var rb = sas.stress.wkbench.StressUtil.getBundle();
        view.table.setNoData(rb.getText("project.noData.loading.txt"));
        sas.stress.wkbench.model.ProjectTemplate.getProjectTemplates().done(function (projects) {
            if (view.table.getModel() instanceof sap.ui.model.json.JSONModel) {
                view.table.getModel().setData(projects);
            } else {
                var projectsModel = new sas.risk.common.models.RiskJSONModel(projects);
                view.table.setModel(projectsModel);

                //S1354153: Do sorting after creation in view so it won't overwrite LayoutService
                //var sorters = [new sap.ui.model.Sorter("name", false)];
                //view.table.getBinding("rows").sort(sorters);
            }

            // store the columns and their visibility status     // S1389396
            _.each(view.table.getColumns(), function (column) {
                var name = column.getProperty("name");
                if (name && name.length) {
                    that._projectTemplatesTableVisibleColumns[name] = column.getVisible();
                }
            });
            // select the previous selected project by default
            setTimeout(function () {
                // the current project
                var project=sas.stress.wkbench.model.ProjectTemplate.getCurrentProject();
                if (project) {
                    // look for the index== project with the same ID in the list    // S1382441
                    // the idea is the rows are now sorted according to user prefs and we use them to get to our project
                    var rows=view.table.getBinding("rows");
                    if (rows) {
                        var index = -1;
                        var lastSelectedProjectContext = _.find(rows.getContexts(), function (context, idx) {
                            index = idx;
                            var obj = context.getObject();    // everything is now inside a context
                            return obj.getName().localeCompare(project.getName()) === 0;  // compare by name
                        });
                        var lastSelectedProject = lastSelectedProjectContext ? lastSelectedProjectContext.getObject() : undefined;
                        // if found.. select it and scroll to it
                        if (index >= 0 && (lastSelectedProject && lastSelectedProject.getId() === project.getId())) {
                            that.getView().table.setSelectedIndex(index);
                            that.getView().table.setFirstVisibleRow(index);
                        }
                    }
                }
            }, 0);

            //that._loadTasks();
        }).fail(function (jqxhr) {
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        }).always(function() {
            view.table.setBusy(false);
            view.table.setNoData(new sas.risk.common.widgets.ZeroStateLayout({
                icon: sas.icons.HC.ZEROSTATETABLE,
                text: "{i18N>project.noData.title}",
                buttons: [
                    new sas.hc.m.Button({
                        text: "{i18N>project.noData.button.txt}",
                        tooltip: "{i18N>project.createButton.tip}",
                        enabled: that.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.PROJECT.CREATE),
                        press: $.proxy(that.newProjectTemplateButtonPress, that)
                    }).addStyleClass("sasPrimaryButton")
                ]
            }));
        });
    },

    _loadTasks:function() {
        var view = this.getView();
        //var deferred = $.Deferred();
        view.taskTable.setBusyIndicatorDelay(0);
        view.taskTable.setBusy(true);
        var rb = sas.stress.wkbench.StressUtil.getBundle();
        view.taskTable.setNoData(rb.getText("home.mytasks.noData.loading.txt"));

        sas.stress.wkbench.StressAPI.loadAllMyTasks().done(function (tasksJSON) {
            // the active tasks
            var projectActiveTasks= _.filter(tasksJSON,function(task){
                return task.status.value === "ACTIVE";
            });

            var taskModel = new sas.risk.common.models.RiskJSONModel(projectActiveTasks);
            view.taskTable.setModel(taskModel);
            //deferred.resolve();
        }).fail(function(jqxhr) {
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        }).always(function() {
            view.taskTable.setBusy(false);
            view.taskTable.setNoData(new sas.risk.common.widgets.ZeroStateLayout({
                icon: sas.icons.HC.ZEROSTATEZERO,
                text: "{i18N>home.mytasks.noData.title}"
            }));
        });

        //return deferred;
    },

    taskTableClick:function(event){
        var view = this.getView();
        var context = view.taskTable.getContextByIndex(event.getParameter("rowIndex"));
        if (context !== null && this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.PROJECT.READ)) {
            var taskObj = context.getObject();
            var project = sas.stress.wkbench.model.Project.getProject(taskObj.projectKey);
            sas.stress.wkbench.ViewManager.loadHomePage(project);
        }
    },

    projectsTableRowSelectionChange: function (event) {
        var view = this.getView();
        var selectedIndicesCnt = view.table.getSelectedIndices().length;
        var rb = sas.stress.wkbench.StressUtil.getBundle();
        var project;

        if (selectedIndicesCnt !== 0) {
            var context = view.table.getContextByIndex(view.table.getSelectedIndices()[0]);
            project = context.getObject();

            view.lockProjectMenu.setText((!project.isLocked()) ?
                rb.getText("lockProjectDialog.lock.window.title") : rb.getText("lockProjectDialog.unlock.window.title"));

        }

        view.editProjectButton.setEnabled(selectedIndicesCnt === 1 &&
            this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.PROJECT.UPDATE)/* &&
            project && !project.isLocked()*/);
        view.projectAccessControlsMenu.setEnabled(selectedIndicesCnt === 1 &&
            this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.PROJECT.UPDATE) &&
            project && !project.isLocked());
        view.openProjectMenu.setEnabled(selectedIndicesCnt === 1 &&
            this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.PROJECT.READ));
        view.deleteProjectButton.setEnabled(selectedIndicesCnt >= 1 &&
            this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.PROJECT.DELETE) &&
            project && !project.isLocked());
        view.lockProjectMenu.setEnabled(selectedIndicesCnt === 1 &&
            this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.PROJECT.LOCK));
    },

    projectsTableCellClick: function (event) {
        var project = this._getProject(event.getParameter("rowIndex"));
        if (null !== project && this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.PROJECT.READ)) {
            // HomePage now handles missing project by redirecting back ProjectManager
            sas.stress.wkbench.ViewManager.loadHomePage(project);
            // the following would also work
            //sas.stress.wkbench.StressAPI.loadProject(project.getId()).done(function () {
            //    sas.stress.wkbench.ViewManager.loadHomePage(project);
            //}).fail(function (jqxhr) {
            //    sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            //    that._loadProjects();
            //});
        }
    },

    projectTemplatesTableRowSelectionChange: function (event) {
        var selectedIndicesCnt = sap.ui.getCore().byId("ProjectTemplateDialog.ProjectTemplateTable").getSelectedIndices().length;

        sap.ui.getCore().byId("ProjectTemplateDialog.editProjectTemplateButton").setEnabled(selectedIndicesCnt === 1 &&
            this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.PROJECT.UPDATE));
        sap.ui.getCore().byId("ProjectTemplateDialog.deleteProjectTemplateButton").setEnabled(selectedIndicesCnt >= 1 &&
            this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.PROJECT.DELETE));
    },

    //projectTemplatesTableCellClick: function (event) {
    //    var project = this._getProject(event.getParameter("rowIndex"));
    //    if (null !== project && this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.PROJECT.READ)) {
    //        // HomePage now handles missing project by redirecting back ProjectManager
    //        sas.stress.wkbench.ViewManager.loadHomePage(project);
    //        // the following would also work
    //        //sas.stress.wkbench.StressAPI.loadProject(project.getId()).done(function () {
    //        //    sas.stress.wkbench.ViewManager.loadHomePage(project);
    //        //}).fail(function (jqxhr) {
    //        //    sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
    //        //    that._loadProjects();
    //        //});
    //    }
    //},

    filterProjects: function (e) {
        var view=this.getView();
        var searchField=view.byId("projectsSearchFld")||sap.ui.getCore().byId("projectsSearchFld");
        var search = searchField?(searchField.getValue()||""):"";
        var table = view.table;
        var filters = [];
        _.each(this._projectsTableVisibleColumns, function (value, key) {   // S1389396
            switch (value) {
                case true:
                    switch (key){
                        case "id":
                            filters.push(new sap.ui.model.Filter({path: key, operator: sap.ui.model.FilterOperator.EQ, value1: search}));
                            break;

                        default:
                            filters.push(new sap.ui.model.Filter(key, sap.ui.model.FilterOperator.Contains, search));
                            break;
                    }
                    break;

                default:
                    break;
            }
        });
        var allFilters=new sap.ui.model.Filter(filters); // the filters on the visible columns OR'd
        var binding = table.getBinding("rows");
        binding.filter(allFilters);
    },


    filterProjectTemplates: function (e) {
        var view=this.getView();
        var searchField=view.byId("projectTemplateSearchFld")||sap.ui.getCore().byId("projectTemplateSearchFld");
        var search = searchField?(searchField.getValue()||""):"";
        var table = sap.ui.getCore().byId("ProjectTemplateDialog.ProjectTemplateTable");
        var filters = [];

        var _projectTemplateTableVisibleColumns = {};
        _.each(table.getColumns(), function (column) {
            var name = column.getProperty("name");
            if (name && name.length) {
                _projectTemplateTableVisibleColumns[name] = column.getVisible();
            }
        });

        _.each(_projectTemplateTableVisibleColumns, function (value, key) {   // S1389396
            switch (value) {
                case true:
                    switch (key){
                        case "id":
                            filters.push(new sap.ui.model.Filter({path: key, operator: sap.ui.model.FilterOperator.EQ, value1: search}));
                            break;

                        default:
                            filters.push(new sap.ui.model.Filter(key, sap.ui.model.FilterOperator.Contains, search));
                            break;
                    }
                    break;

                default:
                    break;
            }
        });
        var allFilters=new sap.ui.model.Filter(filters); // the filters on the visible columns OR'd
        var binding = table.getBinding("rows");
        binding.filter(allFilters);
    },

    openProjectButtonPress: function (event) {
        var project = this._getProject();
        var that = this;
        if (null !== project && this.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.PROJECT.READ)) {
            sas.stress.wkbench.model.Project.loadProject(project.getId(), true).done(function () {
                sas.stress.wkbench.ViewManager.loadHomePage(project);
            }).fail(function (jqxhr) {
                sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
                that._loadProjects();
                that.getView().table.clearSelection();
            });
        }

    },


    newProjectTemplateFragmentButtonPress: function (event) {
        var existingDialog = sap.ui.getCore().byId("sas.stress.wkbench.view.ProjectTemplateDialog");
        if (existingDialog) {
            existingDialog.destroy();
        }
        var rb = sas.stress.wkbench.StressUtil.getBundle();
        var dialog = sap.ui.jsfragment("sas.stress.wkbench.view.ProjectTemplateDialog", this);

        var dateFormat = sap.ui.core.format.DateFormat.getDateInstance({ pattern: "yyyy-MM-dd" });
        var defaultDate = dateFormat.format(new Date(), true);

        sas.stress.wkbench.model.ProjectTemplate.getProjectTemplates().done(function (projectTemplates) {
            var projectTemplatesModel = new sas.risk.common.models.RiskJSONModel(projectTemplates);
            dialog.setModel(projectTemplatesModel);
        });

        dialog.open();
    },

    newProjectTemplateButtonPress: function (event) {
        var existingDialog = sap.ui.getCore().byId("sas.stress.wkbench.view.CreateOrEditProjectTemplate");
        if (existingDialog) {
            existingDialog.destroy();
        }
        var rb = sas.stress.wkbench.StressUtil.getBundle();
        var dialog = sap.ui.jsfragment("sas.stress.wkbench.view.CreateOrEditProjectTemplate", this);

        var dateFormat = sap.ui.core.format.DateFormat.getDateInstance({ pattern: "yyyy-MM-dd" });
        var defaultDate = dateFormat.format(new Date(), true);
        var newProjectJSON = {
            title: rb.getText("newProjectTemplateDialog.window.title"),
            name: sas.stress.wkbench.model.ProjectTemplate.getNewProjectTemplateName(),
            description: "",
            sheetTemplateKeys: [],
            processStartDateStr: defaultDate,
            worksheets: [],
            dimensionKeys: []
        };

        var model = new sas.risk.common.models.RiskJSONModel(newProjectJSON);
        dialog.setModel(model);
        dialog.open();
        //S1477186 disabling project dimensions
        // sas.stress.config.StressAPI.getDimensions().done(function (dimensions) {
        //     var sortedDimensionList = _.sortBy(dimensions, "name");
        //     model.setProperty("/dimensionKeys", sortedDimensionList);
        // });



        sas.stress.config.StressAPI.loadSheetTemplates(true).done(function (worksheets) {
            var sortedDefs = _.sortBy(worksheets, "name");
            _.each(sortedDefs, function (worksheet) {
                worksheet.nameVersion =  sas.stress.wkbench.StressUtil.getNameVersion(worksheet.name, worksheet.version);
            });
            model.setProperty("/worksheets", sortedDefs);
        }).fail(function (jqxhr) {  // S1391333
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });

        sas.stress.wkbench.StressAPI.loadProcessDefinitions().done(function (definitions) {
            var sortedDefs = _.sortBy(definitions, "name");
            var rb = sas.stress.wkbench.StressUtil.getBundle();
            sortedDefs.unshift({ id: 0, name: rb.getText("none.txt") });
            model.setProperty("/definitions", sortedDefs);
        }).fail(function (jqxhr) {  // S1391333
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });
    },

    newProjectButtonPress: function (event) {
        var existingDialog = sap.ui.getCore().byId("sas.stress.wkbench.view.EditProjectDialog");
        if (existingDialog) {
            existingDialog.destroy();
        }
        var rb = sas.stress.wkbench.StressUtil.getBundle();
        var dialog = sap.ui.jsfragment("sas.stress.wkbench.view.EditProjectDialog", this);
        var dateFormat = sap.ui.core.format.DateFormat.getDateInstance({ pattern: "yyyy-MM-dd" });
        var defaultDate = dateFormat.format(new Date(), true);
        var newProjectJSON = {
            title: rb.getText("newProjectDialog.window.title"),
            name: sas.stress.wkbench.model.Project.getNewProjectName(),
            description: "",
            projectTemplate: null,
            definitions: [],
            asOfDate: defaultDate,
            processStartDate: defaultDate,
            processNotificationType: false,
            currentIterationNo: 1,
            worksheets: [],
            dimensionKeys: []
        };
        var model = new sas.risk.common.models.RiskJSONModel(newProjectJSON);
        dialog.setModel(model);
        dialog.open();
        //S1477186 disabling project dimensions
        // sas.stress.config.StressAPI.getDimensions().done(function (dimensions) {
        //     var sortedDimensionList = _.sortBy(dimensions, "name");
        //     model.setProperty("/dimensionKeys", sortedDimensionList);
        // });



        sas.stress.config.StressAPI.loadSheetTemplates(true).done(function (worksheets) {
            var sortedDefs = _.sortBy(worksheets, "name");
            _.each(sortedDefs, function (worksheet) {
                worksheet.nameVersion =  sas.stress.wkbench.StressUtil.getNameVersion(worksheet.name, worksheet.version);
            });
            model.setProperty("/worksheets", sortedDefs);
        }).fail(function (jqxhr) {  // S1391333
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });

        sas.stress.wkbench.StressAPI.loadProcessDefinitions().done(function (definitions) {
            var sortedDefs = _.sortBy(definitions, "name");
            var rb = sas.stress.wkbench.StressUtil.getBundle();
            sortedDefs.unshift({ id: 0, name: rb.getText("none.txt") });
            model.setProperty("/definitions", sortedDefs);
        }).fail(function (jqxhr) {  // S1391333
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });

        // enable the save button if everything is filled
        this.toggleEditProjectDialogSaveButton();
    },

    newProjectFromTemplateButtonPress: function (event) {
        var existingDialog = sap.ui.getCore().byId("sas.stress.wkbench.view.EditProjectDialog");
        if (existingDialog) {
            existingDialog.destroy();
        }
        var rb = sas.stress.wkbench.StressUtil.getBundle();
        var dialog = sap.ui.jsfragment("sas.stress.wkbench.view.EditProjectDialog", this);
        var dateFormat = sap.ui.core.format.DateFormat.getDateInstance({ pattern: "yyyy-MM-dd" });
        //DD: need work here
        var projectTemplateList = {Key:1, Text: "one"};
        var defaultDate = dateFormat.format(new Date(), true);
        var newProjectJSON = {
            title: rb.getText("newProjectDialog.window.title"),
            name: sas.stress.wkbench.model.Project.getNewProjectName(),
            description: "",
            definitions: [],
            asOfDate: defaultDate,
            projectTemplate: projectTemplateList,
            processStartDate: defaultDate,
            processNotificationType: false,
            currentIterationNo: 1,
            worksheets: [],
            dimensionKeys: []
        };
        var model = new sas.risk.common.models.RiskJSONModel(newProjectJSON);
        dialog.setModel(model);
        dialog.open();
        //S1477186 disabling project dimensions
        // sas.stress.config.StressAPI.getDimensions().done(function (dimensions) {
        //     var sortedDimensionList = _.sortBy(dimensions, "name");
        //     model.setProperty("/dimensionKeys", sortedDimensionList);
        // });


        sas.stress.wkbench.StressAPI.loadProjectTemplates().done(function (projectTemplates) {
            var sortedProjectTemplates = _.sortBy(projectTemplates, "name");
            sortedProjectTemplates.unshift({ id: 0, name: rb.getText("none.txt") });
            model.setProperty("/projectTemplateList", sortedProjectTemplates);

            //var processDefnNameSelect = sap.ui.getCore().byId("EditProjectDialog.processDefnNameSelect");
            //processDefnNameSelect.setSelectedKey(project.getProcessDefId());
            //
            //var processNotification = sap.ui.getCore().byId("EditProjectDialog.processNotification");
            ////processNotification.setSelected(project.getProcessNotificationType());
            //var processStartDatePicker = sap.ui.getCore().byId("EditProjectDialog.processStartDatePicker");
            //var processStartDatePickerLabel = sap.ui.getCore().byId("EditProjectDialog.processStartDatePickerLabel");
            //
            //if (project.getProcessDefId() > 0) {
            //    processStartDatePicker.setEnabled(true && !projectIsLocked);
            //    processStartDatePickerLabel.toggleStyleClass("ui-state-disabled", false);
            //    processNotification.setEnabled(true && !projectIsLocked);
            //} else {
            //    processNotification.setSelected(false);
            //    //S1279351: default to today
            //    var dateFormat = sap.ui.core.format.DateFormat.getDateInstance({pattern: "yyyy-MM-dd"});
            //    processStartDatePicker.setValue(dateFormat.format(new Date(), true));
            //}
        }).fail(function (jqxhr) {  // S1391333
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });



        sas.stress.config.StressAPI.loadSheetTemplates(true).done(function (worksheets) {
            var sortedDefs = _.sortBy(worksheets, "name");
            _.each(sortedDefs, function (worksheet) {
                worksheet.nameVersion =  sas.stress.wkbench.StressUtil.getNameVersion(worksheet.name, worksheet.version);
            });
            model.setProperty("/worksheets", sortedDefs);
        }).fail(function (jqxhr) {  // S1391333
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });

        sas.stress.wkbench.StressAPI.loadProcessDefinitions().done(function (definitions) {
            var sortedDefs = _.sortBy(definitions, "name");
            var rb = sas.stress.wkbench.StressUtil.getBundle();
            sortedDefs.unshift({ id: 0, name: rb.getText("none.txt") });
            model.setProperty("/definitions", sortedDefs);
        }).fail(function (jqxhr) {  // S1391333
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });

        // enable the save button if everything is filled
        this.toggleEditProjectDialogSaveButton();
    },

    editProjectButtonPress: function (event) {
        // get the project that is currently selected
        var project = this._getProject();
        var that = this;
        if (null !== project) {
            //S1273684: make sure the project is there before continue
            sas.stress.wkbench.model.Project.loadProject(project.getId(), true).done(function () {
                that.editProject(project);
            }).fail(function (jqxhr) {
                sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
                that._loadProjects();
                that.getView().table.clearSelection();
            });
        }
    },

    onProjectTemplateNameSelect: function (event) {
        var projectTemplateIndex = sap.ui.getCore().byId("EditProjectDialog.projectTemplateNameSelect").getSelectedIndex();

        var editProjectDialog = sap.ui.getCore().byId("sas.stress.wkbench.view.EditProjectDialog");
        var projectTemplateList = editProjectDialog.getModel().getProperty("/projectTemplateList");

        var projectTemplate = projectTemplateList[projectTemplateIndex];

        if (projectTemplate.processDefinitionKey > 0) {
            sas.stress.wkbench.StressAPI.loadProcessDefinitions().done(function (definitions) {
                var sortedDefs = _.sortBy(definitions, "name");
                var rb = sas.stress.wkbench.StressUtil.getBundle();
                sortedDefs.unshift({id: 0, name: rb.getText("none.txt")});
                editProjectDialog.getModel().setProperty("/definitions", sortedDefs);
                var processDefnNameSelect = sap.ui.getCore().byId("EditProjectDialog.processDefnNameSelect");
                processDefnNameSelect.setSelectedKey(projectTemplate.processDefinitionKey);

                var processStartDatePicker = sap.ui.getCore().byId("EditProjectDialog.processStartDatePicker");
                var processStartDatePickerLabel = sap.ui.getCore().byId("EditProjectDialog.processStartDatePickerLabel");

                if (projectTemplate.processDefinitionKey > 0) {
                    processStartDatePicker.setEnabled(true);
                    //processStartDatePicker.setValue(dateFormat.format(new Date(), true));
                    processStartDatePicker.setValue(projectTemplate.processStartDateStr);
                    processStartDatePickerLabel.toggleStyleClass("ui-state-disabled", false);
                } else {
                    var dateFormat = sap.ui.core.format.DateFormat.getDateInstance({pattern: "yyyy-MM-dd"});
                    processStartDatePicker.setValue(dateFormat.format(new Date(), true));
                }

            }).fail(function (jqxhr) {  // S1391333
                sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            });
        }
        //S1477186 disabling project dimensions
        //Reset the selections
       // var dimensionKeysList = editProjectDialog.getModel().getProperty("/dimensionKeys");
        var worksheetList = editProjectDialog.getModel().getProperty("/worksheets");

        // _.each(dimensionKeysList, function (dimension) {
        //     dimension.selected = false;
        // });
        _.each(worksheetList, function (worksheet) {
            worksheet.selected=false;
        });
        //S1477186 disabling project dimensions
        // // Set Dimension list if its on the project Template
        // if (projectTemplate.dimensionKeys.length > 0 ) {
        //
        //     _.each(projectTemplate.dimensionKeys, function (dimensionKey) {
        //         _.find(dimensionKeysList, function (dimension) {
        //             if (dimension.id === dimensionKey) {
        //                 dimension.selected = true;
        //             }
        //         });
        //     });
        //     editProjectDialog.getModel().setProperty("/dimensionKeys", dimensionKeysList);
        // }

        if (projectTemplate.sheetTemplateKeys.length > 0) {

            _.each(projectTemplate.sheetTemplateKeys, function (sheetTemplateKey) {
                _.find(worksheetList, function (sheetTemplate) {
                    if (sheetTemplate.id === sheetTemplateKey) {
                        sheetTemplate.selected = true;
                       // sheetTemplate.nameVersion =  sas.stress.wkbench.StressUtil.getNameVersion(worksheet.name, worksheet.version)
                    }
                });
            });
            editProjectDialog.getModel().setProperty("/worksheets", worksheetList);
        }

        //Set the Demension crossing file name, if any
        if (projectTemplate.dimensionCrossingFilepath.length > 0) {
            editProjectDialog.getModel().setProperty("/dimensionCrossingName", projectTemplate.dimensionCrossingFilepath);
        }

    },

    editProjectTemplateButtonPress: function (event) {
        // get the project that is currently selected
        var projectTemplate = this._getProjectTemplate(event.getParameter("rowIndex"));
        var that = this;
        if (null !== projectTemplate) {
            that.editProjectTemplate(projectTemplate);
        }
    },

    editProjectTemplate: function(projectTemplate) {
        var rb = sas.stress.wkbench.StressUtil.getBundle();
        var existingDialog = sap.ui.getCore().byId("sas.stress.wkbench.view.CreateOrEditProjectTemplate");

        if (existingDialog) {
            existingDialog.destroy();
        }
        var dialog = sap.ui.jsfragment("sas.stress.wkbench.view.CreateOrEditProjectTemplate", this);

        var editProjectJson = {
            title: rb.getText("EditProjectTemplateDialog.window.title"),
            id: projectTemplate.getId(),
            name: projectTemplate.getName(),
            description: projectTemplate.getDescription(),
            processStartDateStr: projectTemplate.getProcessStartDateStr(),
            //processNotificationType: projectTemplate.getProcessNotificationType(),
            //sheetDefinitions: projectTemplate.getSheetDefinitions(),
            //requireComments:requireComments,
            //dimensions:projectTemplate.getDimensions(),
            dimensionCrossingName: projectTemplate.getDimensionCrossingFilepath()
        };
        var model = new sas.risk.common.models.RiskJSONModel(editProjectJson);
        dialog.setModel(model);
        dialog.open();
        //S1477186 disabling project dimensions
        // sas.stress.config.StressAPI.getDimensions().done(function (dimensions) {
        //     var sortedDimensionList = _.sortBy(dimensions, "name");
        //     _.each(projectTemplate.getDimensionKeys(), function (dimensionKey) {
        //         _.find(sortedDimensionList, function (dimension) {
        //             if (dimension.id === dimensionKey) {
        //                 dimension.selected = true;
        //             }
        //         });
        //     });
        //     model.setProperty("/dimensionKeys", sortedDimensionList);
        // });


        sas.stress.config.StressAPI.loadSheetTemplates(true).done(function (worksheets) {
            var sortedDefs = _.sortBy(worksheets, "name");
            _.each(sortedDefs, function (worksheet) {
                var isSelected = projectTemplate.getSheetTemplateKeys().indexOf(worksheet.id) !== -1;
                worksheet.selected = isSelected;
                worksheet.nameVersion =  sas.stress.wkbench.StressUtil.getNameVersion(worksheet.name, worksheet.version);
            });
            model.setProperty("/worksheets", sortedDefs);
        }).fail(function (jqxhr) {  // S1391333
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });

        sas.stress.wkbench.StressAPI.loadProcessDefinitions().done(function (definitions) {
            var sortedDefs = _.sortBy(definitions, "name");
            var rb = sas.stress.wkbench.StressUtil.getBundle();
            sortedDefs.unshift({ id: 0, name: rb.getText("none.txt") });
            model.setProperty("/definitions", sortedDefs);
            var processDefnNameSelect = sap.ui.getCore().byId("CreateOrEditProjectTemplate.processDefnNameSelect");
            processDefnNameSelect.setSelectedKey(projectTemplate.getProcessDefinitionKey());

            var processStartDatePicker = sap.ui.getCore().byId("CreateOrEditProjectTemplate.processStartDatePicker");
            var processStartDatePickerLabel = sap.ui.getCore().byId("CreateOrEditProjectTemplate.processStartDatePickerLabel");

            if (projectTemplate.getProcessDefinitionKey() > 0) {
                processStartDatePicker.setEnabled(true);
                //processStartDatePicker.setValue(dateFormat.format(new Date(), true));
                processStartDatePicker.setValue(projectTemplate.getProcessStartDateStr());
                processStartDatePickerLabel.toggleStyleClass("ui-state-disabled", false);
            } else {
                var dateFormat = sap.ui.core.format.DateFormat.getDateInstance({pattern: "yyyy-MM-dd"});
                processStartDatePicker.setValue(dateFormat.format(new Date(), true));
            }

        }).fail(function (jqxhr) {  // S1391333
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });
    },

    editProject: function(project) {
        var projectIsLocked=this._getProject()?this._getProject().isLocked():true;
        var requireComments=this._getProject()?this._getProject().doesRequireComments():true;
        var rb = sas.stress.wkbench.StressUtil.getBundle();
        var existingDialog = sap.ui.getCore().byId("sas.stress.wkbench.view.EditProjectDialog");
        if (existingDialog) {
            existingDialog.destroy();
        }
        var dialog = sap.ui.jsfragment("sas.stress.wkbench.view.EditProjectDialog", this);

        var editProjectJson = {
            title: rb.getText("editProjectDialog.window.title"),
            id: project.getId(),
            name: project.getName(),
            description: project.getDescription(),
            asOfDate: project.getAsOfDate(),
            processStartDate: project.getProcessStartDate(),
            processNotificationType: project.getProcessNotificationType(),
            sheetDefinitions: project.getSheetDefinitions(),
            projectIsLocked:!projectIsLocked,
            requireComments:requireComments,
            dimensions:project.getDimensions(),
            dimensionCrossingName: project.getDimensionCrossingName()
        };
        var model = new sas.risk.common.models.RiskJSONModel(editProjectJson);
        dialog.setModel(model);
        dialog.open();
        //S1477186 disabling project dimensions
        // sas.stress.config.StressAPI.getDimensions().done(function (dimensions) {
        //     var sortedDimensionList = _.sortBy(dimensions, "name");
        //     _.each(sortedDimensionList,function(dimension){
        //         dimension.editable=!projectIsLocked;    // S1388579
        //     });
        //     _.each(project.getDimensions(), function (dimensionKey) {
        //         _.find(sortedDimensionList, function (dimension) {
        //             if (dimension.id === dimensionKey) {
        //                    dimension.selected = true;
        //             }
        //         });
        //     });
        //     model.setProperty("/dimensionKeys", sortedDimensionList);
        // });

        var associatedSheets = [];

        _.each(model.getProperty("/sheetDefinitions"), function (sheetDefinition) {
            associatedSheets.push(sheetDefinition.sheetTemplateKey);
        });

        sas.stress.config.StressAPI.loadSheetTemplates(false).done(function (worksheets) {
            var sortedDefs = _.sortBy(worksheets, "name");
            var worksheets = [];
            var packageSheets = [];
            _.each(sortedDefs, function (worksheet) {
                //S1312525: get all the worksheets, including the ones from packages.
                var isPackageSheet = worksheet.taxonomyPackageOwners && worksheet.taxonomyPackageOwners.length > 0;
                var isSelected = associatedSheets.indexOf(worksheet.id) !== -1;
                worksheet.selected = isSelected;
                worksheet.editable = !projectIsLocked && !isPackageSheet;
                worksheet.nameVersion = sas.stress.wkbench.StressUtil.getNameVersion(worksheet.name, worksheet.version);
                if (!isPackageSheet) {
                    // keep the normal worksheets on the top
                    worksheets.push(worksheet);
                } else if (isSelected) {
                    //only include the package sheets if they belong to the current project.
                    packageSheets.push(worksheet);
                }
            });
            model.setProperty("/worksheets", worksheets.concat(packageSheets));
        }).fail(function (jqxhr) {  // S1391333
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });

        sas.stress.wkbench.StressAPI.loadProcessDefinitions().done(function (definitions) {
            var sortedDefs = _.sortBy(definitions, "name");
            sortedDefs.unshift({ id: 0, name: rb.getText("none.txt") });
            model.setProperty("/definitions", sortedDefs);
            var processDefnNameSelect = sap.ui.getCore().byId("EditProjectDialog.processDefnNameSelect");
            processDefnNameSelect.setSelectedKey(project.getProcessDefId());

            var processNotification = sap.ui.getCore().byId("EditProjectDialog.processNotification");
            //processNotification.setSelected(project.getProcessNotificationType());
            var processStartDatePicker = sap.ui.getCore().byId("EditProjectDialog.processStartDatePicker");
            var processStartDatePickerLabel = sap.ui.getCore().byId("EditProjectDialog.processStartDatePickerLabel");

            if (project.getProcessDefId() > 0) {
                processStartDatePicker.setEnabled(true && !projectIsLocked);
                processStartDatePickerLabel.toggleStyleClass("ui-state-disabled", false);
                processNotification.setEnabled(true && !projectIsLocked);
            } else {
                processNotification.setSelected(false);
                //S1279351: default to today
                var dateFormat = sap.ui.core.format.DateFormat.getDateInstance({pattern: "yyyy-MM-dd"});
                processStartDatePicker.setValue(dateFormat.format(new Date(), true));
            }
        }).fail(function (jqxhr) {  // S1391333
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
         });
    },
    /* Called when user changes value in the Name text input in the Edit Project dialog.
     * Checks if the value is not empty and if the value is not equal to the name of another project. */
    editProjectDialogNameInputLiveChange: function (event) {
        var editProjectDialog = sap.ui.getCore().byId("sas.stress.wkbench.view.EditProjectDialog");
        var projectId = editProjectDialog.getModel().getProperty("/id");
        var projectBeingEdited = projectId && sas.stress.wkbench.model.Project.getProject(projectId);

        var rb = sas.stress.wkbench.StressUtil.getBundle();
        var nameInput = event.getSource();
        var newName = nameInput.getValue();
        //Check if name is empty.
        if (newName === null || newName.trim() === "") {
            nameInput.setValueStateText(rb.getText("error.requiredField.txt"));
            nameInput.setValueState(sap.ui.core.ValueState.Error);
        } else {
            //Check if project with this name already exists.
            var project = _.find(sas.stress.wkbench.model.Project._projects, function (project) {
                return (project.getName() !== null) &&  (!projectBeingEdited
                    || project.getName().trim().toLowerCase() !== projectBeingEdited.getName().trim().toLowerCase())
                    && (project.getName().trim().toLowerCase() === newName.trim().toLowerCase());
            });
            if (project) {
                nameInput.setValueStateText(rb.getText("error.nameAlreadyInUse.txt"));
                nameInput.setValueState(sap.ui.core.ValueState.Error);
            } else {
                //Name is good.
                nameInput.setValueStateText("");
                nameInput.setValueState(sap.ui.core.ValueState.None);
            }
        }
        this.toggleEditProjectDialogSaveButton();
    },

    createOrEditProjectTemplateNameInputLiveChange: function (event) {
        var CreateOrEditProjectTemplate = sap.ui.getCore().byId("sas.stress.wkbench.view.CreateOrEditProjectTemplateDialog");
        var projectTemplateId = CreateOrEditProjectTemplate.getModel().getProperty("/name");
        var projectBeingEdited = projectTemplateId && sas.stress.wkbench.model.ProjectTemplate.getProjectTemplate(projectTemplateId);

        var rb = sas.stress.wkbench.StressUtil.getBundle();
        var nameInput = event.getSource();
        var newName = nameInput.getValue();
        //Check if name is empty.
        if (newName === null || newName.trim() === "") {
            nameInput.setValueStateText(rb.getText("error.requiredField.txt"));
            nameInput.setValueState(sap.ui.core.ValueState.Error);
        } else {
            //Check if project Template with this name already exists.
            var projectTemplate = _.find(sas.stress.wkbench.model.ProjectTemplate._projectTemplates, function (projectTemplate) {
                return (projectTemplate.getName() !== null) &&  (!projectBeingEdited
                    || projectTemplate.getName().trim().toLowerCase() !== projectBeingEdited.getName().trim().toLowerCase())
                    && (projectTemplate.getName().trim().toLowerCase() === newName.trim().toLowerCase());
            });
            if (projectTemplate) {
                nameInput.setValueStateText(rb.getText("error.nameAlreadyInUse.txt"));
                nameInput.setValueState(sap.ui.core.ValueState.Error);
            } else {
                //Name is good.
                nameInput.setValueStateText("");
                nameInput.setValueState(sap.ui.core.ValueState.None);
            }
        }
        this.toggleEditProjectTemplateDialogSaveButton();
    },



    ///* Called when user changes value in the Process Start Date field. Checks if the value is valid. */
    //editProjectTemplateDialogProcessStartDateChange: function (event) {
    //    var datePicker = event.getSource();
    //    sas.stress.wkbench.StressUtil.handleDatePickerChange(event, datePicker.getEnabled());
    //    this.toggleEditProjectTemplateDialogSaveButton();
    //},

    /* Called when user changes value in the Process Start Date field. Checks if the value is valid. */
    editProjectDialogProcessStartDateChange: function (event) {
        var datePicker = event.getSource();
        sas.stress.wkbench.StressUtil.handleDatePickerChange(event, datePicker.getEnabled());
        this.toggleEditProjectDialogSaveButton();
    },

    /* Called when user changes value in the As of Date field in the Edit Project dialog. Checks if the value is valid. */
    editProjectDialogAsOfDateChange: function (event) {
        sas.stress.wkbench.StressUtil.handleDatePickerChange(event, true);
        this.toggleEditProjectDialogSaveButton();
    },

    //worksheetListSelectionChange: function (event) {
    //
    //    var workSheetList = sap.ui.getCore().byId("EditProjectDialog.workSheetList").getSelectedItems();
    //
    //    var sheetDefinitionList = [];
    //    _.each(workSheetList, function (workSheet) {
    //        sheetDefinitions.push({sheetTemplateKey: workSheet.key});
    //    });
    //
    //    //Call midtier with this list of sheet definitions and get the list of dimensions corresponding to this list.
    //
    //    this.retrieveDimensionList(event);
    //    this.toggleEditProjectDialogSaveButton();
    //},

    toggleEditProjectDialogSaveButton: function () {
        var isValid = sap.ui.getCore().byId("EditProjectDialog.nameInput").getValueState() === sap.ui.core.ValueState.None &&
            (!sap.ui.getCore().byId("EditProjectDialog.processStartDatePicker").getEnabled() || sap.ui.getCore().byId("EditProjectDialog.processStartDatePicker").getValueState() === sap.ui.core.ValueState.None) &&
            sap.ui.getCore().byId("EditProjectDialog.asOfDatePicker").getValueState() === sap.ui.core.ValueState.None;
            //&& sap.ui.getCore().byId("EditProjectDialog.workSheetList").getSelectedItems().length !== 0;        // worksheet is not required per Jeff
        sap.ui.getCore().byId("EditProjectDialog.saveProjectButton").setEnabled(isValid);
    },

    toggleEditProjectTemplateDialogSaveButton: function () {
        var isValid = sap.ui.getCore().byId("CreateOrEditProjectTemplate.nameInput").getValueState() === sap.ui.core.ValueState.None;
            // &&
           // (!sap.ui.getCore().byId("CreateOrEditProjectTemplate.processStartDatePicker").getEnabled() || sap.ui.getCore().byId("CreateOrEditProjectTemplate.processStartDatePicker").getValueState() === sap.ui.core.ValueState.None)

        //&& sap.ui.getCore().byId("EditProjectDialog.workSheetList").getSelectedItems().length !== 0;        // worksheet is not required per Jeff
        sap.ui.getCore().byId("CreateOrEditProjectTemplate.saveProjectButton").setEnabled(isValid);
    },


    editDimensionCrossingFileUploadComplete: function (event) {
        var view = this.getView();
        var controller = this;
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
       // var editProjectDialog = sap.ui.getCore().byId("sas.stress.config.view.EditProjectDialog");
        // the fixes for S1315620... this will inclide fixes for S1312975
        // get the status and the responseRaw
        var status = event.getParameter("status");
        var rawResponse=event.getParameter("responseRaw");

       // editProjectDialog.setBusy(false);
       // editProjectDialog.close();
        if (status===200) { // if status is good... use the responseRaw to update the lineitems
            if (rawResponse&&rawResponse.length){
                var responseJSON = JSON.parse(rawResponse);
                var dimensionCrossingFileName = responseJSON.name;
                var projectId = responseJSON.id;

                var view = this.getView();
                var dialog = sap.ui.getCore().byId("sas.stress.wkbench.view.EditProjectDialog");
                var editProjectDialogModelData = dialog.getModel().getData();

                controller._loadProjects();

                //var projectsModel = view.table.getModel();
                //var projectsModelData = projectsModel.getData();
                //var project = null,bClearFilters=false;
                //for (var i = projectsModelData.length - 1; i >= 0; i--) {
                //    if (projectsModelData[i].id === editedProject.projectKey) {
                //        project = projectsModelData[i]; //get project from model data.
                //        break;
                //    }
                //}

                //var lineItems = [];
                //_.each(responseJSON.lineItems, function (lineItem) {    // fixes for S1261325
                //    var hierarchyItem = new sas.stress.config.model.HierarchyLineItem(lineItem.liKey, lineItem.hierarchyKey,
                //        lineItem.liOrder, lineItem.contra, lineItem.aggregationType, lineItem.levelNames, lineItem.memberName,
                //        lineItem.format, lineItem.formula, lineItem.levelNum, lineItem.workgroupNames);
                //    lineItems.push(hierarchyItem);
                //});
                //if (lineItems.length > 0) {
                //    var hierarchyKey = lineItems[0].getHierarchyKey();
                //    //Update hierarchy in cache.
                //    var hierarchy = sas.stress.config.model.Hierarchy.getHierarchy(hierarchyKey);
                //    if (hierarchy) {
                //        hierarchy.setLineItems(lineItems);
                //        hierarchy.levelNames = responseJSON.hierarchy.levelNames; //fixes for S1261325
                //    }
                //
                //    //Update hierarchy in tree.
                //    var hierarchyRootNode = _.find(view.hierarchiesTreeTable.getModel().getData().children,
                //        function (hierarchyNode) {
                //            return hierarchyNode.hierarchyKey === hierarchyKey;
                //        }
                //    );
                //    if (hierarchyRootNode) {
                //        hierarchyRootNode.children = []; //delete existing line items.
                //        hierarchyRootNode.topLevelNames = responseJSON.hierarchy.levelNames; //fixes for S1261325
                //        _.each(hierarchy.getLineItems(), function (lineItem) {
                //            controller._addToTreeNode(hierarchyRootNode, lineItem);
                //        });
                //        view.hierarchiesTreeTable.getModel().refresh();
                //        controller._applyCategoryFilter();
                //        controller.hierarchiesTreeTableRowSelectionChange();
                //    }
                //}
            }
        } else {  //STATUS != OK;  use the responseRaw to display the error messages
            if (rawResponse&&rawResponse.length) {   //S1312975
                sas.stress.wkbench.StressUtil.handleServerFailure({responseText:rawResponse});
            } else {   //S1312975
                sas.hc.m.MessageDialog.error({text: rb.getText('error.message.txt'), title: rb.getText('error.window.title')});
            }
        }
        // refresh the hierarchy list in case the name or category is modified by another version
        //controller.refreshList();
    },

    editDimensionCrossingFileUploadTypeMissmatch: function (event) {
        var aFileTypes = event.getSource().getFileType();
        $.each(aFileTypes, function (key, value) {
            aFileTypes[key] = "*." + value;
        });
        var sSupportedFileTypes = aFileTypes.join(", ");
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        sas.hc.m.MessageDialog.error({
            text: rb.getText("fileUploadTypeMismatch.msg.fmt.txt", [event.getParameter("fileType"), sSupportedFileTypes]),
            title: this._getProjectDialogTitle()
        });

    },

    editDimensionCrossingFileUploadFileSizeExceed: function (event) {
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        sas.hc.m.MessageDialog.error({
            text: rb.getText("fileUploadFileSizeExceed.msg.fmt.txt", [event.getSource().getMaximumFileSize()]),
            title: this._getProjectDialogTitle()
        });
    },

    handleProjectTemplateDialogSaveButtonPress: function (event) {
        var view = this.getView();
        var projectId = sap.ui.getCore().byId("sas.stress.wkbench.view.CreateOrEditProjectTemplateDialog").getModel().getData().id;
        if (projectId != null) {
            this.editProjectTemplateDialogSaveButtonPress(event);
        } else {
            this.newProjectTemplateDialogSaveButtonPress(event);
        }
    },

    handleProjectDialogSaveButtonPress: function (event) {
        var view = this.getView();
        var projectId = sap.ui.getCore().byId("sas.stress.wkbench.view.EditProjectDialog").getModel().getData().id;
        if (projectId != null) {
            this.editProjectDialogSaveButtonPress(event);
        } else {
            this.newProjectDialogSaveButtonPress(event);
        }
    },

    /* Called when the Save button in the New Project Template dialog is pressed. */
    newProjectTemplateDialogSaveButtonPress: function (event) {
        var view = this.getView();
        var dialog = sap.ui.getCore().byId("sas.stress.wkbench.view.CreateOrEditProjectTemplateDialog");
        var newProjectTemplateDialogModelData = dialog.getModel().getData();
        var processDefnNameSelected = sap.ui.getCore().byId("CreateOrEditProjectTemplate.processDefnNameSelect").getSelectedItem();

        var dateTimeFormat = sap.ui.core.format.DateFormat.getDateTimeInstance({pattern: "yyyy-MM-dd", nosec: true, notzone: true});

        //var asOfDateSelected = sap.ui.core.format.DateFormat.getDateInstance().parse(sap.ui.getCore().byId("EditProjectDialog.asOfDatePicker").getValue());
        //var asOfDasteStr = dateTimeFormat.format(new Date(asOfDateSelected), false);
        //var processStartDateSelected = sap.ui.core.format.DateFormat.getDateInstance().parse(sap.ui.getCore().byId("CreateOrEditProjectTemplate.processStartDatePicker").getValue());
        //var processStartDateStr = (processStartDateSelected) ? dateTimeFormat.format(new Date(processStartDateSelected), false) : null;
        //var processNotificationSelected = (sap.ui.getCore().byId("EditProjectDialog.processNotification").getSelected() ? 1 : 0);
        var dimensionList = sap.ui.getCore().byId("CreateOrEditProjectTemplate.dimensionList").getSelectedItems();
        var workSheetList = sap.ui.getCore().byId("CreateOrEditProjectTemplate.workSheetList").getSelectedItems();
        //var requireCommentsSelected = (sap.ui.getCore().byId("EditProjectDialog.requireComments").getSelected() ? 1 : 0);

        var sheetTemplateKeys = [];

        _.each(workSheetList, function (workSheet) {
            sheetTemplateKeys.push(workSheet.key);
        });

        var dimensionKeys = [];

        _.each(dimensionList, function (dimension) {
            dimensionKeys.push(dimension.id);
        });


       // var startDate = (processDefnNameSelected.getKey() > 0 ? processStartDateStr : null);
       // var emailSelected = (processDefnNameSelected.getKey() > 0 ? processNotificationSelected : 0);

        var dimCrossingFileName = sap.ui.getCore().byId("CreateOrEditProjectTemplate.dimensionCrossingFileUploader").getValue();

        var projectTemplate = {
            name: newProjectTemplateDialogModelData.name.trim(),
            description: newProjectTemplateDialogModelData.description,
            processDefinitionKey: parseInt(processDefnNameSelected.getKey()) > 0 ? parseInt(processDefnNameSelected.getKey()) : null,
            //processStartDateStr: startDate,
            sheetTemplateKeys: sheetTemplateKeys,
            dimensionKeys: dimensionKeys,
            dimensionCrossingFilepath: dimCrossingFileName
        };
        sas.stress.wkbench.model.ProjectTemplate.createProjectTemplate(projectTemplate).done(function (projectTemplateSaved) {
            var projectTemplatesModel = sap.ui.getCore().byId("ProjectTemplateDialog.ProjectTemplateTable").getModel();

            var projectTemplate = new sas.stress.wkbench.model.ProjectTemplate(
                projectTemplateSaved.getId(),
                projectTemplateSaved.getName(),
                projectTemplateSaved.getDescription(),
                projectTemplateSaved.getProcessDefinitionKey(),
                //projectTemplateSaved.getProcessStartDateStr(),
                projectTemplateSaved.getDimensionKeys(),
                projectTemplateSaved.getSheetTemplateKeys(),
                projectTemplateSaved.getDimensionCrossingFilepath(),
                projectTemplateSaved.getCreatedByUser(),
                projectTemplateSaved.getCreatedTime(),
                projectTemplateSaved.getUpdatedByUser(),
                projectTemplateSaved.getLastModified()
            );

            projectTemplatesModel.getData().push(projectTemplate);

            //Upload file if one is selected.
            //var dimensionCrossingFileUploader = sap.ui.getCore().byId("CreateOrEditProjectTemplate.dimensionCrossingFileUploader");
            //if (dimensionCrossingFileUploader.getValue()) { //File selected?
            //    dimensionCrossingFileUploader.setUploadUrl("rest/projectTemplates/" + projectTemplateSaved.getId() + "/crossing");
            //    dimensionCrossingFileUploader.upload();
            //}
            projectTemplatesModel.refresh();
            dialog.close();
            //Open the project
            //sas.stress.wkbench.ViewManager.loadHomePage(project);
        }).fail(function (jqxhr) {
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });
    },

    /* Called when the Save button in the New Project dialog is pressed. */
    newProjectDialogSaveButtonPress: function (event) {
        var view = this.getView();
        var dialog = sap.ui.getCore().byId("sas.stress.wkbench.view.EditProjectDialog");
        var newProjectDialogModelData = dialog.getModel().getData();
        var processDefnNameSelected = sap.ui.getCore().byId("EditProjectDialog.processDefnNameSelect").getSelectedItem();

        var dateTimeFormat = sap.ui.core.format.DateFormat.getDateTimeInstance({pattern: "yyyy-MM-dd", nosec: true, notzone: true});

        var asOfDateSelected = sap.ui.core.format.DateFormat.getDateInstance().parse(sap.ui.getCore().byId("EditProjectDialog.asOfDatePicker").getValue());
        var asOfDasteStr = dateTimeFormat.format(new Date(asOfDateSelected), false);
        var processStartDateSelected = sap.ui.core.format.DateFormat.getDateInstance().parse(sap.ui.getCore().byId("EditProjectDialog.processStartDatePicker").getValue());
        var processStartDateStr = (processStartDateSelected) ? dateTimeFormat.format(new Date(processStartDateSelected), false) : null;
        var processNotificationSelected = (sap.ui.getCore().byId("EditProjectDialog.processNotification").getSelected() ? 1 : 0);
        //S1477186 disabling project dimensions
        //var dimensionList = sap.ui.getCore().byId("EditProjectDialog.dimensionList").getSelectedItems();
        var dimensionList = [];
        var workSheetList = sap.ui.getCore().byId("EditProjectDialog.workSheetList").getSelectedItems();
        var requireCommentsSelected = (sap.ui.getCore().byId("EditProjectDialog.requireComments").getSelected() ? 1 : 0);

        var sheetDefinitions = [];

        _.each(workSheetList, function (workSheet) {
            sheetDefinitions.push({sheetTemplateKey: workSheet.key});
        });

        var dimensionsSelected = [];

        _.each(dimensionList, function (dimension) {
            dimensionsSelected.push(dimension.id);
        });


        var startDate = (processDefnNameSelected.getKey() > 0 ? processStartDateStr : null);
        var emailSelected = (processDefnNameSelected.getKey() > 0 ? processNotificationSelected : 0);

        var project = {
            projectName: newProjectDialogModelData.name.trim(),
            description: newProjectDialogModelData.description,
            processDefinitionKey: parseInt(processDefnNameSelected.getKey()),
            asOfDateStr: asOfDasteStr,
            processStartDateStr: startDate,
            processNotificationType: emailSelected,
            sheetDefinitions: sheetDefinitions,
            requireComments:requireCommentsSelected,
            dimensions: dimensionsSelected
        };
        sas.stress.wkbench.model.Project.createProject(project).done(function (project) {
                var projectsModel = view.table.getModel();
                projectsModel.getData().push({
                    id: project.getId(),
                    name: project.getName(),
                    description: project.getDescription(),
                    createdTime: project.getCreatedTime(),
                    lastModified: project.getLastModified(),
                    createdByUser: project.getCreatedByUser(),
                    updatedByUser: project.getUpdatedByUser(),
                    asOfDate: project.getAsOfDate(),
                    processDefId:project.getProcessDefId(),
                    processDefinitionName:project.getProcessDefinitionName(),
                    processStartDate: project.getProcessStartDate(),
                    sheetDefinitions: project.getSheetDefinitions(),
                    processNotificationType: project.getProcessNotificationType(),
                    requireComments:project.doesRequireComments(),
                    dimensions:project.getDimensions()
                });

            //Upload file if one is selected.
            var dimensionCrossingFileUploader = sap.ui.getCore().byId("EditProjectDialog.dimensionCrossingFileUploader");
            if (dimensionCrossingFileUploader.getValue()) { //File selected?
                dimensionCrossingFileUploader.setUploadUrl("rest/projects/" + project.getId() + "/crossing");
                dimensionCrossingFileUploader.upload();
            }
            projectsModel.refresh();
            dialog.close();
            //Open the project
            sas.stress.wkbench.ViewManager.loadHomePage(project);
        }).fail(function (jqxhr) {
                sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });
    },

    clearDimensionCrossingNameButtonPress: function() {

        var dimensionCrossingFileUploader = sap.ui.getCore().byId("EditProjectDialog.dimensionCrossingFileUploader");
        if (dimensionCrossingFileUploader.getValue()) { //File selected?
            dimensionCrossingFileUploader.setValue();
            sap.ui.getCore().byId("EditProjectDialog.saveProjectButton").setEnabled(true);
            this.toggleEditProjectDialogSaveButton();
        }
    },

    projectTemplateClearDimensionCrossingNameButtonPress: function() {

        var dimensionCrossingFileUploader = sap.ui.getCore().byId("CreateOrEditProjectTemplate.dimensionCrossingFileUploader");
        if (dimensionCrossingFileUploader.getValue()) { //File selected?
            dimensionCrossingFileUploader.setValue();
            sap.ui.getCore().byId("CreateOrEditProjectTemplate.saveProjectButton").setEnabled(true);
            this.toggleEditProjectTemplateDialogSaveButton();
        }
    },

    /* Called when the Save button in the Edit Project Template dialog is pressed. */
    //DD Needs work
    editProjectTemplateDialogSaveButtonPress: function () {
        var view = this.getView();
        var controller = this;
        var dialog = sap.ui.getCore().byId("sas.stress.wkbench.view.CreateOrEditProjectTemplateDialog");
        var processDefnNameSelected = sap.ui.getCore().byId("CreateOrEditProjectTemplate.processDefnNameSelect").getSelectedItem();

        var dateTimeFormat = sap.ui.core.format.DateFormat.getDateTimeInstance({pattern: "yyyy-MM-dd", nosec: true, notzone: true});

        //var editProcessStartDateSelected = sap.ui.core.format.DateFormat.getDateInstance().parse(sap.ui.getCore().byId("CreateOrEditProjectTemplate.processStartDatePicker").getValue());
        //var editProcessStartDateStr = (editProcessStartDateSelected) ? dateTimeFormat.format(new Date(editProcessStartDateSelected), false) : null;

        //var editProcessNotificationSelected = (sap.ui.getCore().byId("EditProjectDialog.processNotification").getSelected() ? 1 : 0);

        var dimensions = [];
        _.each(sap.ui.getCore().byId("CreateOrEditProjectTemplate.dimensionList").getSelectedItems(), function (dimension) {
            dimensions.push(dimension.id);
        });

        var workSheetList = sap.ui.getCore().byId("CreateOrEditProjectTemplate.workSheetList").getSelectedItems();

        var processDefinitionName = (processDefnNameSelected.getKey() > 0 ? processDefnNameSelected.getText() : "");
        //var requireCommentsSelected = (sap.ui.getCore().byId("EditProjectDialog.requireComments").getSelected() ? 1 : 0);

        var sheetDefinitions = [];

        _.each(workSheetList, function (workSheet) {
            sheetDefinitions.push(workSheet.key);
        });

        //var startDate = (processDefnNameSelected.getKey() > 0 ? editProcessStartDateStr : null);
        //var emailSelected = (processDefnNameSelected.getKey() > 0 ? editProcessNotificationSelected : 0);

        var dimCrossingFileName = sap.ui.getCore().byId("CreateOrEditProjectTemplate.dimensionCrossingFileUploader").getValue();

        var model = dialog.getModel();
        var editedProject = {
            id: model.getProperty("/id"),
            name: sap.ui.getCore().byId("CreateOrEditProjectTemplate.nameInput").getValue(),
            description: sap.ui.getCore().byId("CreateOrEditProjectTemplate.descriptionInput").getValue(),
            processDefinitionKey: parseInt(processDefnNameSelected.getKey()) > 0 ? parseInt(processDefnNameSelected.getKey()) : null,
            //processStartDateStr: startDate,
            //processStartDateStr: editProcessStartDateStr,
            //processNotificationType: emailSelected,
            dimensionKeys: dimensions,
            sheetTemplateKeys: sheetDefinitions,
            dimensionCrossingKey:model.getProperty("/dimensionCrossingKey"),
            dimensionCrossingFilePath:dimCrossingFileName
        };

        var projectsModel = sap.ui.getCore().byId("ProjectTemplateDialog.ProjectTemplateTable").getModel();
        var projectsModelData = projectsModel.getData();
        var project = null;
        for (var i = projectsModelData.length - 1; i >= 0; i--) {
            if (projectsModelData[i].id === editedProject.id) {
                project = projectsModelData[i]; //get project from model data.
                break;
            }
        }

        sas.stress.wkbench.model.ProjectTemplate.editProjectTemplate(editedProject).done(function (updatedProject) {
            // Update the project in the model.
            // To maintain any sort already applied on the table, we remove the project from the model data,
            // update the project properties, and add it to the model data.
            if (project !== null) {
                projectsModelData.splice(i, 1); //remove project from model data.
                projectsModel.refresh();

                project.id = updatedProject.getId();
                project.name = updatedProject.getName();
                project.description = updatedProject.getDescription();
                project.processDefinitionKey = updatedProject.getProcessDefinitionKey();
                //project.processStartDateStr = updatedProject.getProcessStartDateStr();
                project.dimensionKeys=updatedProject.getDimensionKeys();
                project.sheetTemplateKeys = updatedProject.getSheetTemplateKeys();
                project.dimensionCrossingFilepath = updatedProject.getDimensionCrossingFilepath();
                project.createdByUser = updatedProject.getCreatedByUser();
                project.createdTime = updatedProject.getCreatedTime();
                project.updatedByUser = updatedProject.getUpdatedByUser();
                project.lastModified = updatedProject.getLastModified();

                //Upload file if one is selected.
                //var dimensionCrossingFileUploader = sap.ui.getCore().byId("CreateOrEditProjectTemplate.dimensionCrossingFileUploader");
                //if (dimensionCrossingFileUploader.getValue()) { //File selected?
                //    dimensionCrossingFileUploader.setUploadUrl("rest/projectTemplates/" + updatedProject.getId() + "/crossing");
                //    dimensionCrossingFileUploader.upload();
                //} else { //User wants to delete the file that was saved earlier.
                //    sas.stress.wkbench.model.ProjectTemplate.deleteDimensionCrossingFile(updatedProject.getId()).done(function () {
                //        project.dimensionCrossingKey = 0;
                //        project.dimensionCrossingName = "";
                //    });
                //}

                projectsModelData.push(project); //add project to model data.
                projectsModel.refresh();
                //controller.selectProjectByName(project.name);

                // reload the tasks for the project
                //controller._loadTasks();
            }
            dialog.close();
        }).fail(function (jqxhr) {
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });
    },

    /* Called when the Save button in the Edit Project dialog is pressed. */
    editProjectDialogSaveButtonPress: function () {
        var view = this.getView();
        var controller = this;
        var dialog = sap.ui.getCore().byId("sas.stress.wkbench.view.EditProjectDialog");
        var processDefnNameSelected = sap.ui.getCore().byId("EditProjectDialog.processDefnNameSelect").getSelectedItem();

        var dateTimeFormat = sap.ui.core.format.DateFormat.getDateTimeInstance({pattern: "yyyy-MM-dd", nosec: true, notzone: true});

        var editProcessStartDateSelected = sap.ui.core.format.DateFormat.getDateInstance().parse(sap.ui.getCore().byId("EditProjectDialog.processStartDatePicker").getValue());
        var editProcessStartDateStr = (editProcessStartDateSelected) ? dateTimeFormat.format(new Date(editProcessStartDateSelected), false) : null;
        var editAsOfDateSelected = sap.ui.core.format.DateFormat.getDateInstance().parse(sap.ui.getCore().byId("EditProjectDialog.asOfDatePicker").getValue());
        var editAsOfDasteStr = dateTimeFormat.format(new Date(editAsOfDateSelected), false);

        var editProcessNotificationSelected = (sap.ui.getCore().byId("EditProjectDialog.processNotification").getSelected() ? 1 : 0);
        //S1477186 disabling project dimensions
        // var dimensions = [];
        //_.each(sap.ui.getCore().byId("EditProjectDialog.dimensionList").getSelectedItems(), function (dimension) {
        //    dimensions.push(dimension.id);
        //});

        var workSheetList = sap.ui.getCore().byId("EditProjectDialog.workSheetList").getSelectedItems();

        var processDefinitionName = (processDefnNameSelected.getKey() > 0 ? processDefnNameSelected.getText() : "");
        var requireCommentsSelected = (sap.ui.getCore().byId("EditProjectDialog.requireComments").getSelected() ? 1 : 0);

        var sheetDefinitions = [];

        _.each(workSheetList, function (workSheet) {
            sheetDefinitions.push({sheetTemplateKey: workSheet.key});
        });

        var startDate = (processDefnNameSelected.getKey() > 0 ? editProcessStartDateStr : null);
        var emailSelected = (processDefnNameSelected.getKey() > 0 ? editProcessNotificationSelected : 0);

        var model = dialog.getModel();
        var editedProject = {
            projectKey: model.getProperty("/id"),
            projectName: model.getProperty("/name"),
            description: model.getProperty("/description"),
            processDefinitionKey: parseInt(processDefnNameSelected.getKey()),
            processDefinitionName: processDefinitionName,
            asOfDateStr: editAsOfDasteStr,
            processStartDateStr: startDate,
            processNotificationType: emailSelected,
            //dimensions: dimensions, //S1477186 disabling project dimensions
            dimensions: model.getProperty("/dimensions"),
            sheetDefinitions: sheetDefinitions,
            requireComments:requireCommentsSelected,
            dimensionCrossingKey:model.getProperty("/dimensionCrossingKey"),
            dimensionCrossingName:model.getProperty("/dimensionCrossingName"),
        };

        var projectsModel = view.table.getModel();
        var projectsModelData = projectsModel.getData();
        var project = null,bClearFilters=false;
        for (var i = projectsModelData.length - 1; i >= 0; i--) {
            if (projectsModelData[i].id === editedProject.projectKey) {
                project = projectsModelData[i]; //get project from model data.
                break;
            }
        }
        if (project&&project.asOfDate!==editAsOfDasteStr){
            bClearFilters=true;
        }
        sas.stress.wkbench.model.Project.editProject(editedProject).done(function (updatedProject) {
            // Update the project in the model.
            // To maintain any sort already applied on the table, we remove the project from the model data,
            // update the project properties, and add it to the model data.
            if (project !== null) {
                projectsModelData.splice(i, 1); //remove project from model data.
                projectsModel.refresh();

                project.name = updatedProject.getName();
                project.description = updatedProject.getDescription();
                project.lastModified = updatedProject.getLastModified();
                project.updatedByUser = updatedProject.getUpdatedByUser();
                //Convert millisecs to date format
                //var dateFormat = sap.ui.core.format.DateFormat.getDateInstance({ pattern: "yyyy-MM-dd" });
                //var dateStr = dateFormat.format(new Date(updatedProject.getAsOfDate()), true);
                //project.asOfDate = dateStr;
                project.asOfDate = updatedProject.getAsOfDate();
                project.processStartDate = updatedProject.getProcessStartDate();
                project.processNotificationType = updatedProject.getProcessNotificationType();
                project.processDefinitionName = updatedProject.getProcessDefinitionName();
                project.processDefId = updatedProject.getProcessDefId();
                project.dimensions=updatedProject.getDimensions();
                project.sheetDefinitions = updatedProject.getSheetDefinitions();
                project.requireComments = updatedProject.doesRequireComments();
                project.dimensionCrossingKey = updatedProject.getDimensionCrossingKey();
                project.dimensionCrossingName = updatedProject.getDimensionCrossingName();

                if (bClearFilters){    // if the as-of-date is updated, clear the filters
                    // fixes for S1272838
                    updatedProject.clearFilters(); // fixes for S1268615... clear the project filters
                }

                //Upload file if one is selected.
                var dimensionCrossingFileUploader = sap.ui.getCore().byId("EditProjectDialog.dimensionCrossingFileUploader");
                if (dimensionCrossingFileUploader.getValue()) { //File selected?
                    dimensionCrossingFileUploader.setUploadUrl("rest/projects/" + updatedProject.getId() + "/crossing");
                    dimensionCrossingFileUploader.upload();
                } else { //User wants to delete the file that was saved earlier.
                    sas.stress.wkbench.model.Project.deleteDimensionCrossingFile(updatedProject.getId()).done(function () {
                        project.dimensionCrossingKey = 0;
                        project.dimensionCrossingName = "";
                    });
                }

                projectsModelData.push(project); //add project to model data.
                projectsModel.refresh();
                controller.selectProjectByName(project.name);

                // reload the tasks for the project
                controller._loadTasks();
            }
            dialog.close();
        }).fail(function (jqxhr) {
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });
    },

    selectProjectByName: function (projectName) {
        var view = this.getView();
        // Ensure no selection exists in the table already.
        view.table.clearSelection();
        // We have to loop through the model because the row we want to select might not have been loaded yet
        var allProjects = view.table.getModel().getData();
        for (var i = 0; i < allProjects.length; i++) {
            var project = this._getProject(i);
            if (null !== project && project.name.toLocaleLowerCase() === projectName.toLocaleLowerCase()) {
                // Project found, select it.
                view.table.setSelectedIndex(i);
                // Scroll to the selected project.
                view.table.setFirstVisibleRow(i);
                // We found the project, no need to keep searching.
                break;
            }
        }
    },

    _getProject: function (index) {
        var view = this.getView();
        // If index wasn't specified, then we are getting the currently selected index from the table.
        if (undefined === index) {
            index = view.table.getSelectedIndex();
        }
        var project = null;
        if (index >= 0) {
            var context = view.table.getContextByIndex(index);
            var projectObj = context.getObject();
            // fixes for S1282574
            var name = projectObj.name;
            var theproject = _.find(view.table.getModel().getData(), function (project) {
                if (project.name === name) {
                    return project;
                }
            });
            return theproject; //project = sas.stress.wkbench.model.Project.getProject(theproject.id);
        }
        return project;
    },

    _getProjectTemplate: function (index) {

        // If index wasn't specified, then we are getting the currently selected index from the table.
        if (undefined === index) {
            index = sap.ui.getCore().byId("ProjectTemplateDialog.ProjectTemplateTable").getSelectedIndex();
        }
        var projectTemplate = null;
        if (index >= 0) {
            var context = sap.ui.getCore().byId("ProjectTemplateDialog.ProjectTemplateTable").getContextByIndex(index);
            var projectTemplateObj = context.getObject();
            // fixes for S1282574
            var id = projectTemplateObj.id;
            var theprojectTemplate = _.find(sap.ui.getCore().byId("ProjectTemplateDialog.ProjectTemplateTable").getModel().getData(), function (projectTemplate) {
                if (projectTemplate.id === id) {
                    return projectTemplate;
                }
            });
            return theprojectTemplate; //project = sas.stress.wkbench.model.Project.getProject(theproject.id);
        }
        return projectTemplate;
    },

    projectAccessControlsButtonPress: function (event) {
        // get the project that is currently selected
        var project = this._getProject();
        var that = this;
        if (null !== project) {
            //S1273684: make sure the project is there before continue
            sas.stress.wkbench.model.Project.loadProject(project.getId(), true).done(function () {
                that.editAccessControls(project);
            }).fail(function (jqxhr) {
                sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
                that._loadProjects();
                that.getView().table.clearSelection();
            });
        }
    },

    editAccessControls: function (project) {
        this.accessControlsDialog = new sas.stress.wkbench.widget.AccessControlsDialog({});
        var controller = this;
        var view = this.getView();
        view.table.setBusy(true);
        sas.stress.wkbench.StressAPI.getProjectAccessControls(project.getId()).done(function (accessControls) {
            view.table.setBusy(false);
            controller.accessControlsDialog.setup(project.getName(), accessControls, "project",
                $.proxy(controller.projectAccessControlsSaveButtonPress, controller));
        }).fail(function (jqxhr) {
            view.table.setBusy(false);
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });
    },

    projectAccessControlsSaveButtonPress: function (workgroups) {
        var controller = this;
        var project = this._getProject();
        if (null !== project) {
            sas.stress.wkbench.StressAPI.saveProjectAccessControls(project.getId(), workgroups).done(function () {
                controller.accessControlsDialog.setBusy(false);
                controller.accessControlsDialog.close();
            }).fail(function (jqxhr) {
                controller.accessControlsDialog.setBusy(false);
                sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            });
        }
    },

    lockProjectButtonPress: function (event) {

        var that = this;
        var rb = sas.stress.wkbench.StressUtil.getBundle();
        var view = this.getView();
        var context = view.table.getContextByIndex(view.table.getSelectedIndices()[0]);
        var project = context.getObject();

        var locking = !project.locked;

        var reasonText = new sas.hc.m.TextArea({
            autoExpandable: false,
            maxLength: 1024
        });

        var historyShade = new sas.hc.ui.commons.windowshade.WindowShade({
            openFirstSection: false,
            width: "100%"
        }).addStyleClass("lockHistoryShade");

        var historySection = new sas.hc.ui.commons.windowshade.WindowShadeSection({
            title: rb.getText("lockProjectDialog.showHistory.txt")
        }).addStyleClass("lockHistory");

        var historyLayout = new sas.hc.ui.layout.VerticalLayout({width: "100%"}).addStyleClass("lockHistoryLayout");

        _.each(project.lockHistory, function(entry) {

            var icon = (entry.eventCode === "ProjectLocked") ? sas.icons.HC.LOCKCLOSED : sas.icons.HC.LOCKOPEN;

            historyLayout.addContent(
                new sas.hc.m.ResponsiveToolbar({
                    content: [
                        new sas.hc.m.Label({text: entry.user}).addStyleClass("lockHistoryAuthor"),
                        new sas.hc.m.ToolbarSpacer(),
                        new sas.hc.m.Button({
                            enabled: false,
                            icon: icon
                        })
                    ]
                })
            );

            if (entry.reason) {
                historyLayout.addContent(
                    new sas.hc.m.Text({
                        text: entry.reason
                    }).addStyleClass("lockHistoryComment")
                );
            }

            var dateFormat = sap.ui.core.format.DateFormat.getDateTimeInstance();

            historyLayout.addContent(
                new sas.hc.m.Text({
                    text: dateFormat.format(new Date(entry.createdDTTM), false)   // fixes for S1304245
                }).addStyleClass("lockHistoryDate")
            );
        });

        var historyContainer = new sas.hc.m.ScrollContainer({
            content: historyLayout,
            height: "200px",
            horizontal: true,
            vertical: true
        }).addStyleClass("lockHistoryScroll");

        historySection.addContent(historyContainer);
        historyShade.addSection(historySection);

        var dateFormat = sap.ui.core.format.DateFormat.getDateTimeInstance();
        var defaultDate = (project.lockTimestamp) ? dateFormat.format(new Date(project.lockTimestamp)) : dateFormat.format(new Date(), false);

        var lockDate = new sas.hc.m.DateTimePicker({
            value: defaultDate
        });

        lockDate.setEnabled(locking);

        var form = new sas.hc.ui.layout.form.Form({
            layout: new sas.hc.ui.layout.form.ResponsiveGridLayout(),
            formContainers: [
                new sas.hc.ui.layout.form.FormContainer({
                    formElements: [
                        new sas.hc.ui.layout.form.FormElement({
                            label: rb.getText("lockProjectDialog.comment.txt"),
                            fields: [reasonText]
                        }),
                        new sas.hc.ui.layout.form.FormElement({
                            label: rb.getText("lockProjectDialog.lock.txt"),
                            fields: [lockDate]
                        }),
                        new sas.hc.ui.layout.form.FormElement({
                            fields: [historyShade]
                        })
                    ]
                })
            ]
        });
        form.addStyleClass("stressFormSmall");

        var title = (locking) ? rb.getText("lockProjectDialog.lock.window.title") : rb.getText("lockProjectDialog.unlock.window.title");

        var dialog = new sas.hc.m.Dialog({
            id: "sas.stress.wkbench.view.EditProjectDialog",
            title: title,
            content: form,
            beginButton: new sas.hc.m.Button({
                id: this.createId("EditProjectDialog.saveProjectButton"),
                text: rb.getText("comment.dialog.ok.txt"),
                enabled: true,
                press: function () {
                    // defect S1442944.. convert the date in GMT string to a ISO format string
                    sas.stress.wkbench.StressAPI.updateProjectLock(project, {reason: reasonText.getValue(), locked: locking, lockTimestamp: new Date(lockDate.getDateValue()).toISOString()}).done(function (tasksJSON) {
                        that._loadProjects();

                        view.lockProjectMenu.setText((!locking) ? rb.getText("lockProjectDialog.lock.window.title") : rb.getText("lockProjectDialog.unlock.window.title"));
                        view.projectAccessControlsMenu.setEnabled(that.capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.PROJECT.UPDATE) &&
                            project && !locking);

                        dialog.close();
                    }).fail (function(jqxhr) {
                        sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr, title);
                    });
                }
            }),
            endButton: new sas.hc.m.Button({
                text: rb.getText("comment.dialog.cancel.txt"),
                press: function () {
                    dialog.close();
                }
            }),
            afterClose: function () {
                dialog.destroy();
            }
        });

        dialog.open();

    },

    _getProjectDialog: function () {
        return sap.ui.getCore().byId("sas.stress.config.view.EditHierarchyDialog");
    },
    _getProjectDialogTitle: function () {
        var projectDialog = this._getProjectDialog();
        return projectDialog ? projectDialog.getModel().getData().title : "";
    },

    deleteProjectButtonPress: function (event) {
        var view = this.getView();
        var that = this;
        var table = view.table;
        var itemsSelected = table.getSelectedIndices();
        if (itemsSelected.length === 0) {
            return;
        }
        var projectIdList = [];
        var projectNameList = [];
        //Get a list of project ids from the item list
        for (var i = 0; i < itemsSelected.length; i++) {
            var item = itemsSelected[i];
            var project = this._getProject(item);
            if (null !== project) {
                projectIdList[i] = project.getId();
                projectNameList[i] =  project.getName();
            }
        }

        var rb = sas.stress.wkbench.StressUtil.getBundle();
        var projectDeleteMsg = "";
        projectDeleteMsg = sas.stress.wkbench.StressUtil.getConfirmationMessage(rb.getText("project.delete.confirmation.txt"), projectNameList);
        sas.hc.m.MessageDialog.warning({
            text: projectDeleteMsg,
            title: rb.getText("project.delete.window.title"),
            buttonActions: [sas.hc.m.MessageDialogAction.Delete, sas.hc.m.MessageDialogAction.Cancel],
            buttonPress: function (event) {
                var action = event.getParameter("origin").getMessageDialogAction();
                if (action === sas.hc.m.MessageDialogAction.Delete) {
                    //Delete all the items in the itemsSelected array
                    table.setBusy(true);
                    sas.stress.wkbench.model.Project.deleteProjects(projectIdList).done(function () {
                        for (var a = 0; a < projectIdList.length; a++) {
                            var projectsModel = view.table.getModel();
                            var projectsModelData = projectsModel.getData();
                            for (var i = projectsModelData.length - 1; i >= 0; i--) {
                                if (projectsModelData[i].id === projectIdList[a]) {
                                    if (sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.project_key") === projectsModelData[i].id) {
                                        // If deleting most-recent project, clear user preference
                                        sas.stress.wkbench.UserPrefs.save("sas.stress.wkbench.project_key", "");
                                    }
                                    projectsModelData.splice(i, 1);
                                    break;
                                }
                            }
                            // Delete the task column visibility settings for the project
                            sas.stress.wkbench.UserPrefs.delete("sas.stress.wkbench.visible_task_columns", projectIdList[a]);
                        }
                        projectsModel.refresh();
                        table.setBusy(false);
                        table.clearSelection();
                    }).fail(function (jqxhr) {
                        table.setBusy(false);
                        sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
                        // if we want to auto-refresh, we will lose the row selection
                        //that._loadProjects();
                        //table.clearSelection();
                    });
                }
            }
        });
    },

    deleteProjectTemplateButtonPress: function (event) {
        var view = this.getView();
        var that = this;
        //var table = view.table;

        var itemsSelected = sap.ui.getCore().byId("ProjectTemplateDialog.ProjectTemplateTable").getSelectedIndices();

        //var itemsSelected = table.getSelectedIndices();
        if (itemsSelected.length === 0) {
            return;
        }

        var projectTemplateIdList = [];
        var projectTemplateNameList = [];
        //Get a list of project ids from the item list
        for (var i = 0; i < itemsSelected.length; i++) {
            var item = itemsSelected[i];
            var projectTemplate = this._getProjectTemplate(item);
            if (null !== projectTemplate) {
                projectTemplateIdList[i] = projectTemplate.getId();
                projectTemplateNameList[i] =  projectTemplate.getName();
            }
        }

        var rb = sas.stress.wkbench.StressUtil.getBundle();
        var projectDeleteMsg = "";
        projectDeleteMsg = sas.stress.wkbench.StressUtil.getConfirmationMessage(rb.getText("project.delete.confirmation.txt"), projectTemplateNameList);
        sas.hc.m.MessageDialog.warning({
            text: projectDeleteMsg,
            title: rb.getText("project.delete.window.title"),
            buttonActions: [sas.hc.m.MessageDialogAction.Delete, sas.hc.m.MessageDialogAction.Cancel],
            buttonPress: function (event) {
                var action = event.getParameter("origin").getMessageDialogAction();
                if (action === sas.hc.m.MessageDialogAction.Delete) {
                    //Delete all the items in the itemsSelected array
                    sap.ui.getCore().byId("ProjectTemplateDialog.ProjectTemplateTable").setBusy(true);
                    sas.stress.wkbench.model.ProjectTemplate.deleteProjectTemplates(projectTemplateIdList).done(function () {
                        for (var a = 0; a < projectTemplateIdList.length; a++) {
                            var projectsModel = sap.ui.getCore().byId("ProjectTemplateDialog.ProjectTemplateTable").getModel();
                            var projectsModelData = projectsModel.getData();
                            for (var i = projectsModelData.length - 1; i >= 0; i--) {
                                if (projectsModelData[i].id === projectTemplateIdList[a]) {
                                    //if (sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.project_key") === projectsModelData[i].id) {
                                    //    // If deleting most-recent project, clear user preference
                                    //    sas.stress.wkbench.UserPrefs.save("sas.stress.wkbench.project_key", "");
                                    //}
                                    projectsModelData.splice(i, 1);
                                    break;
                                }
                            }
                            // Delete the task column visibility settings for the project
                            //sas.stress.wkbench.UserPrefs.delete("sas.stress.wkbench.visible_task_columns", projectIdList[a]);
                        }
                        projectsModel.refresh();
                        sap.ui.getCore().byId("ProjectTemplateDialog.ProjectTemplateTable").setBusy(false);
                        sap.ui.getCore().byId("ProjectTemplateDialog.ProjectTemplateTable").clearSelection();
                    }).fail(function (jqxhr) {
                        sap.ui.getCore().byId("ProjectTemplateDialog.ProjectTemplateTable").setBusy(false);
                        sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
                        // if we want to auto-refresh, we will lose the row selection
                        //that._loadProjects();
                        //table.clearSelection();
                    });
                }
            }
        });
    },


    /*
     * the event handler when adding a column to the visible column list...
     * the new column will be used during filters
     */
    onColumnVisibility: function () {   // S1389396
        var view = this.getView();
        var that = this;
        _.each(view.table.getColumns(), function (column) {
            var name = column.getProperty("name");
            if (name && name.length) {
                that._projectsTableVisibleColumns[name] = column.getVisible();
            }
        });
        this.filterProjects();
    }
});