jQuery.sap.require("sas.stress.wkbench.ViewManager");
jQuery.sap.require("sas.stress.wkbench.widget.SlimsheetSource");
jQuery.sap.require("sas.stress.wkbench.CellPickerUtil");
jQuery.sap.require("sas.stress.wkbench.model.Project");
jQuery.sap.require("sas.stress.wkbench.model.Sheet");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.risk.common.models.RiskJSONModel");

sap.ui.controller("sas.stress.wkbench.view.ScenarioEditor", {
    onInit: function () {
        var view = this.getView();
        var viewData = view.getViewData();
        var SourceClass = viewData.sourceClass;
        var source = new SourceClass(viewData.sheetId, viewData.filters);
        var model = new sas.risk.common.models.RiskJSONModel({
            source: source,
            /* comment properties */
            comments:[],
            isAdd:true,
            autoCloseCommentPaneOnEsc:false,
            enableCommentPane:true
        });
        view.setModel(model);

        // need to clear out the tokens displayed for the new sheet
        var wkBenchView = sas.stress.wkbench.ViewManager.getWorkbench();
        wkBenchView.tokenizer.destroyTokens();
        this.displaySheetFilters(viewData.sheetId, viewData.filters, viewData.iterationId);
        this.unSavedComments={};
        this.originalComment="";

        this.allComments=[], this.nextCellIndex=0, this.previousCellIndex=-1;

        this.noData;
        this.navigating=false;
    },
    onAfterRendering: function () {
        var view = this.getView();
        var liveModel = view.getModel("live");
        var selected=liveModel.getProperty("/selected");    // see if there is any selected cell.. if yes, keep it
        if (sas.stress.wkbench.CellPickerUtil.getOperationMode() === sas.stress.wkbench.CellPickerUtil.NORMAL_MODE) {
            liveModel.setProperty("/selected", null);
            if (selected){
                liveModel.setProperty("/selected", selected);
            }
            liveModel.setProperty("/editing", null);
            liveModel.setProperty("/informula", []);
        }
        liveModel.setProperty("/slimsheet", view.slimsheet);
        view.slimsheet.removeCellWithSelectedClass();

        // update commentsMap here S1343769
        this._updateCommentsMap();

        sas.stress.wkbench.ViewManager.getWorkbench().setBusy(false);
        sas.stress.wkbench.ViewManager.getWorkbench().getController().closeBzDialog();
    },
    onExit: function () {
        // free the key from the list of stored scenario editor views, this will free memory too
        var key = this.getView().getId();
        var toBeDestroyedViews = sas.stress.wkbench.ViewManager.getAllScenarioEditorViews();
        toBeDestroyedViews[key] = undefined;
        delete toBeDestroyedViews[key];
    },
    cellSelectedAction: function (e) {
        var self = this;
        var liveModel = this.getView().getModel("live");
        var selected = e.getParameter("selected");

        if (selected) {
            var cellRef = new sas.stress.wkbench.model.CellRef(e.getParameter("col")/*-1*/, e.getParameter("row"), e.getParameter("filter"));
            if (sas.stress.wkbench.CellPickerUtil.getOperationMode() === sas.stress.wkbench.CellPickerUtil.EDIT_MODE) {
                this._addSelectedCellToFormula(cellRef);
            } else {
                this.getView().slimsheet.selectCell(cellRef.row, cellRef.column/*+1*/, cellRef.filterId);
                liveModel.setProperty("/editing", null);
                liveModel.setProperty("/informula", []);

                /*var cellRange = liveModel.getProperty("/selecteds");
                _.each(cellRange, function(cell){
                    self.getView().slimsheet.deSelectCell(cell.row, cell.column/!*+1*!/, cell.filterId, false);
                });*/
                liveModel.setProperty("/selected", null);//so that the next line triggers its listeners
                liveModel.setProperty("/selected", cellRef);
                liveModel.setProperty("/selecteds", []);

                this.getView().slimsheet.focus({selected:selected});
            }
            this.originalComment="";
            this._updateNavigationIndex(cellRef.colShort + cellRef.rowShort, cellRef.row, cellRef.column);
            var commentedCell=_.find(this.allComments,function(value){
                return value.key===cellRef.colShort+cellRef.rowShort;
            });
            this._selectSelectedCellInList(commentedCell?commentedCell.index:-1);
        }
        this._refreshModel(selected,cellRef);
        this._setFocusOnCommentPaneVisible();

        /*
         hide the comment pane per requirement
        (When comment is mandatory, the comment bar and should automatically pop-in when a value is changed in a cell
         In case user navigates to another cell without a comment or change, the comment bar should collapse.)
         */
        this._hideCommentPane();
        /**/
    },
    rangeSelectCells : function(e){
        var self = this;
        var view = this.getView();
        var model = view.getModel();
        var liveModel = view.getModel("live");
        var cellRange = e.getParameter("cellRange");
        var filter = e.getParameter("filter");
        if (sas.stress.wkbench.CellPickerUtil.getOperationMode() !== sas.stress.wkbench.CellPickerUtil.EDIT_MODE) {
            model.setProperty("/enableCommentPane", false);
            var cellRefs=[];
            view.slimsheet.rangeSelectCells(cellRange, filter);
            for (var rowIndex = cellRange.previousRow; rowIndex <= cellRange.currentRow; rowIndex++) {
                for (var columnIndex = cellRange.previousColumn; columnIndex <= cellRange.currentColumn; columnIndex++) {
                    var cellRef = new sas.stress.wkbench.model.CellRef(columnIndex, rowIndex, filter);
                    cellRefs.push(cellRef);
                }
            }
            liveModel.setProperty("/selected", null);//so that the next line triggers its listeners
            liveModel.setProperty("/selecteds", []);//so that the next line triggers its listeners
            liveModel.setProperty("/selecteds", cellRefs);//so that the next line triggers its listeners

            this.getView().slimsheet.focus();
        }
    },
    cellDeSelectedAction: function(e){
        var liveModel = this.getView().getModel("live");
        var cellRef = new sas.stress.wkbench.model.CellRef(e.getParameter("col")/*-1*/, e.getParameter("row"), e.getParameter("filter"));
        if (sas.stress.wkbench.CellPickerUtil.getOperationMode() !== sas.stress.wkbench.CellPickerUtil.EDIT_MODE) {
            this.getView().slimsheet.deSelectCell(cellRef.row, cellRef.column/*+1*/, cellRef.filterId, e.getParameter("ctrlKey"));
            liveModel.setProperty("/selected", null);//so that the next line triggers its listeners
            liveModel.setProperty("/editing", null);
            liveModel.setProperty("/informula", []);

            var cellRange = liveModel.getProperty("/selecteds");
            if (cellRange&&cellRange.length){
                // need to remove the un-selected from the array if selected cells
                var newRange = _.filter(cellRange, function(selectedCell){
                    return selectedCell.column !== cellRef.column && selectedCell.row !== cellRef.row;
                });
                liveModel.setProperty("/selecteds", null);
                liveModel.setProperty("/selecteds", newRange||[]);
                cellRange=[];
            }

        }
    },
    _addSelectedCellToFormula: function (selected) {
        // Get the current cell info from the Cell Editor
        var cellEditor = sas.stress.wkbench.ViewManager.getCellEditor();
        var liveModel = cellEditor.getModel("live");
        var editing = liveModel.getProperty("/editing");
        if (!editing){
            return;
        }
        //var cellEditorScenario = editing.scenario;
        var cellEditorSheetId = editing.sheetId;
        var cellEditorScenarioId = editing.scenarioId;
        var cellEditorScenarioName = editing.scenarioName;

        // The selected cell info
        //var selectedCellScenario = selected.scenario;
        var selectedCellScenarioId = selected.scenarioId;
        var selectedCellScenarioName = selected.scenarioName;
        var selectedCellSheetId = selected.sheetId;

        // Compare selected cell with the current cell to determine proper cell reference.
        // ie, if on the same sheet and scenario, then we do not include them in the cell ref,
        // just use row and column ref (Local Reference)
        var cellName;
        if ((selectedCellScenarioId === cellEditorScenarioId) && (selectedCellSheetId === cellEditorSheetId)) {
            cellName = selected.toString(sas.stress.wkbench.model.CellRef.FMT_CELL_SHORT_LOCAL);
        } else if (cellEditorScenarioName === selectedCellScenarioName) {
            cellName = selected.toString(sas.stress.wkbench.model.CellRef.FMT_CELL_SHORT_LOCAL_BY_SCENARIO);
        } else {
            cellName = selected.toString(sas.stress.wkbench.model.CellRef.FMT_CELL_SHORT);
        }
        var StressUtil = sas.stress.wkbench.StressUtil;
        // Find the caret in the formula
        var formulaField = sas.stress.wkbench.ViewManager.getCellEditor().formulaField;
        /* S1344039 start */
        var pos= -1;
        pos=StressUtil.getCaretPosition(formulaField.getId(),false);    // S1396510
        StressUtil.setCaretPosition(formulaField.getId(),pos,false);    // S1396510
        var inPlaceEdit=sas.stress.wkbench.ViewManager.getCellEditor().getController()._getInPlaceEditControl();
        if (inPlaceEdit&&inPlaceEdit.style.visibility==="visible"){
            pos=StressUtil.getCaretPosition(inPlaceEdit.id);
            if (pos<0) {
                pos = StressUtil.getCaretPosition(formulaField.getId())-2/* reduce by 2 because the input adds 2 from the position?? - Srini */;
            }
        }
        /* S1344039 end */
        // The original formula
        var oldFormula = formulaField.getValue();
        // Insert cell name into formula at caret position to create new formula
        var newFormula = oldFormula.substr(0, pos) + cellName + oldFormula.substr(pos);
        // Set the formula field value with the new formula
        formulaField.setValue(newFormula);
        StressUtil.setCaretPosition(formulaField.getId(),pos+cellName.length,false);    // RQSSTR-796: set up the position of the cursor
        formulaField.fireLiveChange({length: cellName.length}); // RQSSTR-796: pass the length to be added to the cursor position
    },
    _onPropertyPaneVisible: function (e) {
        var view = this.getView();
        var that=this;
        setTimeout(function(){
            // HC 8.0 changes... now i need to do in a timeout, else the correct width is not available
            var liveModel=view.getModel("live");
            var selected=liveModel.getProperty("/selected");
            view.slimsheet.rerender();
            if (selected) {
                var selectCell = sas.stress.wkbench.CellPickerUtil.getOperationMode() !== sas.stress.wkbench.CellPickerUtil.EDIT_MODE;
                view.slimsheet.scrollToAndSelectCell(selected.row, selected.column, selectCell);
            }else{
                that._autoSelectFirstComment();
            }

            that._setFocusOnCommentPaneVisible();
        },0);
    },
    _onSelectionChange:function(){
        this._setFocusOnCommentPaneVisible();
    },
    _autoSelectFirstComment:function(key){
        var view=this.getView();
        var item = view.commentsList.getItemByKey(key||"0");
        if (item) {
            view.commentsList.setSelectedItemId(item.getId());
            view.commentsList.fireChange({selectedItem: item});//for some odd reason i had to manually fire it.
        }else{
            if (view.layout.getPropertyPaneVisible()){
                this._showOrHideNoData(true,sas.stress.wkbench.StressUtil.getBundle().getText("cell.comments.manager.no.cell.selected.txt"));
            }
        }
    },
    displaySheetFilters: function (sheetId, filterId, iterationId) {
        var that = this;
        var scview = that.getView();
        var facetPanel = scview.facetPanel;
        var facetModel = new sas.risk.common.models.RiskJSONModel();

        facetPanel.removeAllFacets();
        var modelData = [];

        facetModel.setData({"data": modelData});
        var project = sas.stress.wkbench.model.Project.getCurrentProject();
        var sheet = project.getSheet(sheetId);
        if (sheet) {
            sheet.loadSheetFilters(project.getId()).done(function (items) {
                _.each(items, function (item, idx) {
                    var name = item.hierarchy.name;
                    var nameVersion = sas.stress.wkbench.StressUtil.getNameVersion(name, item.hierarchy.version);
                    var facet = sap.ui.getCore().byId(scview.createId(idx.toString()));
                    if (facet === undefined) {
                        facet = new sas.stress.wkbench.widget.DescriptionFacet({
                            id: scview.createId(idx.toString()),
                            text: nameVersion,
                            fieldName: name,
                            searchThreshold: 10,
                            key: item.hierarchyKey/*,
                            format: item.format*/	// S1472103
                        });
                    }
                    facetPanel.addFacet(facet);

                    // If this is a scenario facet filter then sort alphanumeric, otherwise leave it as it came from midtier
                    var lineItems = /*(item.hierarchy.code === "SCENARIO") ? item.lineItems.sort(function (a, b) {
                        return a.memberName&& b.memberName?a.memberName.localeCompare(b.memberName):a.name.localeCompare(b.name);
                    }) :*/ item.lineItems;  // FIXID S1359443

                    _.each(lineItems, function (lineItem) {
                        var val = {};
                        var liName= lineItem.memberName?lineItem.memberName:lineItem.name;
                        val[name] = liName;
                        var indexOfNameInPath=lineItem.path.indexOf(liName);
                        var liPath=lineItem.path.substring(0,indexOfNameInPath);
                        val.path = liPath;// S1373008
                        val.liKey = lineItem.liKey;
                        val.hierarchyKey = lineItem.hierarchyKey;
                        val.variableShockInfo=lineItem.variableShockInfo;
                        val.format = lineItem.format ? lineItem.format : "1.2"; // S1472103
                        modelData.push(val);
                    });
                });
                facetModel.setData({"data": modelData});
                var filterProvider = new sas.stress.wkbench.widget.DescriptionModelFilterProvider({
                    dataPath: "/data",
                    facetPanel: facetPanel
                });
                filterProvider.setModel(facetModel);
                facetPanel.setModel(facetModel);
                // select the defaults...
                var facets= facetPanel.getFacets();
                /*
                 this is the sequence of filters returned and displayed... we
                 will try to use this for selecting the tokens.. else, the
                 tokens are displayed in the order of the "preferred Sheet filters"
                  */
                var sequence=new Array(facets.length);
                _.each(facets,function(facet, index){
                    sequence[index]=Number(facet.getProperty("key"));
                });
                var defFilters = sheet.getPreferredSheetFilter(sequence);
                var selectedFacets = [];
                var selectedFacet = {facetKey: 0, selectedFacetItem: undefined};
                _.each(defFilters, function (eachFilter) {
                    selectedFacet = {facetKey: 0, selectedFacetItem: undefined};
                    var facetKey;
                    var selectableFacet = _.find(facets,function(facet,key){
                        facetKey=key;
                        return facet.getProperty("key")===eachFilter.hierarchyKey.toString();
                    });
                    if (selectableFacet) {
                        var itemToSelect = _.find(selectableFacet.getFacetItems(), function (item) {
                            return item.liKey === eachFilter.liKey;
                        });
                        if (itemToSelect) {
                            // the item is found therefore set it as selected
                            itemToSelect.selected = true;
                            // store the key and item
                            selectedFacet.facetKey = facetKey;
                            selectedFacet.selectedFacetItem = itemToSelect;

                            selectedFacets.push(selectedFacet);
                        }
                    }
                });
                // loop and fire the facet item as selected in the facets
                _.each(selectedFacets, function (selectedFacet) {
                    var facet = facets[selectedFacet.facetKey];
                    // fixes for: S1237759; somehow the set selected item is no longer working
                    try {
                        facet.list.setSelectedItem(selectedFacet.selectedFacetItem);
                    } catch (e) {
                    }
                    // fixes for: S1229696; fire the selection change on the facet's list control..
                    facet._fireSelectionChange({
                        listItem: selectedFacet.selectedFacetItem,
                        listItems: [selectedFacet.selectedFacetItem],
                        selected: true
                    });
                });
                // these are default filters therefore the apply button needs to be disabled
                scview.applyFilterButton.setEnabled(false);
            });
        }
    },
    pressFilterApplyButton: function (wkBenchView/*project*/) {
        var that = this;
        var view = this.getView();
        var viewData = view.getViewData();
        var wkBenchController = wkBenchView.getController();
        var project = wkBenchController.project;	// makes sure we have the latest and greatest
        var sheet = project.getSheet(viewData.sheetId);

        if (wkBenchController.navChecker.getDirty()) {	// fixes for S1236895
            var rb = sas.stress.wkbench.StressUtil.getBundle();
            // show the warning dialog asking for refresh before if unsaved changes exist
            sas.hc.m.MessageDialog.warning({
                text: rb.getText("project.sheet.filters.apply.question.txt"),
                title: rb.getText("project.sheet.filters.apply.title"),
                buttonActions: [sas.hc.m.MessageDialogAction.Apply, sas.hc.m.MessageDialogAction.Cancel],
                buttonPress: function (event) {
                    var action = event.getParameter("origin").getMessageDialogAction();
                    if (action === sas.hc.m.MessageDialogAction.Apply) {
                        // user decided to discard the changes... process the apply pressed button event

                        // fixes for S1265734... see if we are in the EDIT_MODE.. if yes, leave
                        if (sas.stress.wkbench.CellPickerUtil.getOperationMode() === sas.stress.wkbench.CellPickerUtil.EDIT_MODE) {
                            sas.stress.wkbench.ViewManager.getCellEditor().getController().leaveEditMode();
                        }
                        that._pressFilterApplyButton(project, sheet).done(function () {
                            // we want to discard the changes.. the navchecker is not dirty
                            wkBenchController.navChecker.setDirty(false);
                            // update the facet Panel
                            that._updateFacetPanel();
                        }).fail(function (jqxhr) {
                            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr, rb.getText("project.sheet.filters.apply.title"));
                        });
                    } else {
                        view.applyFilterButton.setEnabled(false);
                        that._updateFiltersDisplay();   // revert back to the original filters fixes for S1246706
                        view.setBusy(false);
                    }
                }
            });

        } else {
            // fixes for S1265734... see if we are in the EDIT_MODE.. if yes, leave
            if (sas.stress.wkbench.CellPickerUtil.getOperationMode() === sas.stress.wkbench.CellPickerUtil.EDIT_MODE) {
                sas.stress.wkbench.ViewManager.getCellEditor().getController().leaveEditMode();
            }
            that._pressFilterApplyButton(project, sheet).done(function () {
                    that._updateFacetPanel();
            }).fail(function (jqxhr) {
                sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr,rb.getText("project.sheet.filters.apply.title"));
            });
        }
    },
    _pressFilterApplyButton: function (project) {
        var deferred = $.Deferred();
        var view = this.getView();
        var facetPanel = view.facetPanel;
        var wkbenchview = sas.stress.wkbench.ViewManager.getWorkbench();
        var source=view.slimsheet.getSlimsheetSource();
        source.destroy();
        //source.info=undefined;

        var projectFilters=project.getFilters();
        _.each(facetPanel.getFacets(), function (facet) {
            _.each(facet.getSelected(), function (item) {
                var projectFilter=_.find(projectFilters,function(filter){
                    return filter.hierarchyKey.toString()===facet.getKey();
                });
                if (projectFilter){
                    projectFilter.liKey=item.liKey;
                }else{

                }
            });
        });

        // save the filters
        sas.stress.wkbench.UserPrefs.save("sas.stress.wkbench.project_filters",JSON.stringify(projectFilters),project.getId());
        //wkbenchview.setModel();//clear out view model mistake .. fixes for S1312558
        wkbenchview.getController().openIfRequiredAndUpdateBzDialog();
        // S1510277 .. process all sheets because of filter changes will have different evaluations for sheets
        wkbenchview.getController()._processAllSheets = true;
        // S1431608.. do not ignore the preferred sheet
        wkbenchview.getController().refreshProject(false, false).done(function(project){
            deferred.resolve();
        }).fail(function(jqxhr){
            deferred.reject(jqxhr);
        });

        return deferred;
    },
    _updateFacetPanel: function () {
        var view = this.getView();
        // re-render the slimsheet.. needed when we apply the filter.. new data is now available
        // fixes for S1271169.. we need to clear the info so that it can re-loaded
/*
        var source=view.slimsheet.getSlimsheetSource();
        source.destroy();
*/
        //source.info=undefined;
        view.slimsheet.rerender();
        var wkBenchView = sas.stress.wkbench.ViewManager.getWorkbench();

        wkBenchView.tokenizer.destroyTokens();
        var facets = view.facetPanel.getFacets();
        _.each(facets, function (facet) {
            facet._fireSelectionChange();
        });

        view.applyFilterButton.setEnabled(false);
        view.setBusy(false);
    },
    _updateFiltersDisplay: function () {
        var view = this.getView();
        var viewData = view.getViewData();
        var wkbenchview = sas.stress.wkbench.ViewManager.getWorkbench();
        var project = wkbenchview.getController().project;	// makes sure we have the latest and greatest
        var sheet = project.getSheet(viewData.sheetId);

        if (wkbenchview.tokenizer.getTokens().length>0) {
            var selectedFacets = view.facetPanel.getFacets();
            _.each(selectedFacets, function (selectedFacet, index) {
                var token = wkbenchview.tokenizer.getTokens()[index];
                if (token) {
                    selectedFacet.resetToken(token);
                }
            });
        }else{
            view.facetPanel.removeAllFacets();
            //wkbenchview.tokenizer.destroyTokens();
        }
        this.displaySheetFilters(viewData.sheetId, viewData.filters, viewData.iterationId);
        view.applyFilterButton.setEnabled(false);
        view.setBusy(false);
    },
    /**
     * fixes for S1266122
     *
     * We need to look at the tokens on the workbench to see if the selecteditem in the facet
     * has the token, if it does, enabled =false else enabled =true
     * @private
     */
    _enableButton:function(){
        var view = this.getView();
        var enable=false;
        var selectedFacets = view.facetPanel.getFacets();
        var wkbenchview = sas.stress.wkbench.ViewManager.getWorkbench();

        var selected;
        if (wkbenchview.tokenizer.getTokens().length>0) {
            _.each(selectedFacets, function (selectedFacet, index) {
                var items=selectedFacet.getFacetItems();
                selected=_.find(items,function(item){
                    return item.selected;
                });

                if (selected){
                    var found=_.find(wkbenchview.tokenizer.getTokens(),function(token){
                        return token.getKey()===selected.liKey; // S1303232.. it's all keys for the defect fixes
                    });
                    // there is a selected item and it was either found or not
                    // if not found, then we need to enable else same item disable
                    enable=!found||enable;
                }
            });
        }
        view.applyFilterButton.setEnabled(enable);
        delete selected;
    },
    _getFullParentInfo:function(parentNames,parents,currentParentLiKey){
        if (!currentParentLiKey){
            return "";
        }
        var parentLiKey=currentParentLiKey;
        var path=(parents[parentLiKey]&&parents[parentLiKey].name) ? parents[parentLiKey].name + " > " : "";
        parentLiKey=parents[parentLiKey]?parents[parentLiKey].key:undefined;
        path=this._getFullParentInfo(parentNames,parents,parentLiKey) + path;

        return path;
    },
    _onEditComment:function(e){
        var view=this.getView();
        var model=view.getModel();
        var liveModel=sas.stress.wkbench.ViewManager.getWorkbench().getModel("live");
        var cellRef=liveModel.getProperty("/selected");
        var value= e.getParameter("value");
        var changed=true;
        var unsavedComment={cellRef:cellRef, comment:value};
        if (value&&this.originalComment!==value){
            changed=true;
        }else{
            changed=false;
        }

        this._enablePostButton(changed);
        var newComment=model.getProperty("/comments")[0];
        newComment.comment=value;
        this.unSavedComments[cellRef.colShort+cellRef.rowShort]=unsavedComment;

        //view.cancelCommentButton.setEnabled(true);
    },
    _onSaveCommentClick:function(){
        var view = this.getView();
        var model=view.getModel();
        var newComment = model.getProperty("/comments")[0];//view.commentTextArea.getValue();
        var liveModel=sas.stress.wkbench.ViewManager.getWorkbench().getModel("live");
        var slimsheet= liveModel.getProperty("/slimsheet");
        var userName = sas.stress.wkbench.StressUtil.getUserName();
        var cellRef=liveModel.getProperty("/selected");
        var data=cellRef.getFilterData()/*getScenarioData()*/;
        data.addComment("$"+cellRef.colLong+"$"+cellRef.rowLong,cellRef.row,cellRef.column,userName,newComment.comment);

        //slimsheet.getSlimsheetSource().clearInfo();
        slimsheet.rerender();
        //sas.applicationSwitcher.updateDirtyFlag(sas.config.getAppId(), true);// Let Appswitcher know
        // fixes for S1203977
        var wkbenchView=sas.stress.wkbench.ViewManager.getWorkbench();
        wkbenchView.getController().navChecker.setDirty(true);
        wkbenchView.getController().hasUnSavedValues=true;
        liveModel.setProperty("/toggleButtons", true);//Update property in model

        this._refreshModel(true,cellRef);
        model.setProperty("/isAdd",false);

        //view.cellComments.rerender();
        this._enablePostButton(false);
        /* New comment posted.. need to update the comments map and the navigation index */
        this.updateModel(true, cellRef);
    },
    /**
     * This will be called from the cell comment editor controller on saveing a new formula or value for a cell
     * and the comments are mandatory
     *
     * (S1342467) - the cell comments navigations need to be updated if a cell is overridden and the comments are
     * mandatory
     *
     * @param modelRefreshed
     * @param cellRef
     */
    updateModel:function(modelRefreshed,cellRef){
        if (!modelRefreshed) {
            this._refreshModel(true, cellRef);
        }
        if (cellRef) {
            this._updateCommentsMap(cellRef);

            this._updateNavigationIndex(cellRef.colShort + cellRef.rowShort, cellRef.column, cellRef.row);
            var commentedCell = _.find(this.allComments, function (value) {
                return value.key === cellRef.colShort + cellRef.rowShort;
            });
            this._selectSelectedCellInList(commentedCell ? commentedCell.index : 0);
        }
    },
    _onCancelEdit:function() {
        var view = this.getView();
        var model=view.getModel();
        var liveModel=sas.stress.wkbench.ViewManager.getWorkbench().getModel("live");
        var cellRef=liveModel.getProperty("/selected");
        var autoCloseCommentPaneOnEsc=model.getProperty("/autoCloseCommentPaneOnEsc");
        if (autoCloseCommentPaneOnEsc&&view.layout.getPropertyPaneVisible()){
            view.layout.setPropertyPaneVisible(false);  // minimize the pane
        }
        model.setProperty("/autoCloseCommentPaneOnEsc",false);    // reset the flag so that it cannot close
        var comments=model.getProperty("/comments");
        comments[0].comment=this.originalComment?this.originalComment:"";   // S1343514
        model.setProperty("/comments", comments);
        this._enablePostButton(false);
        delete this.unSavedComments[cellRef.colShort+cellRef.rowShort];
    },
    _setFocusOnCommentPaneVisible:function(){
        var self=this;
        var view = this.getView();
        if (view.layout.getPropertyPaneVisible()&&view.layout.getSelectedPropertyPaneIndex()===1){
            /*the focus into the textarea*/

            // S1344598
            if(view.commentPropPane.getContent()[0] && view.commentPropPane.getContent()[0].getCenter() && view.commentPropPane.getContent()[0].getCenter().getContent()[0].getItems()[0]) {
                var commentTextArea = view.commentPropPane.getContent()[0].getCenter().getContent()[0].getItems()[0];//view.commentPropPane.getContent()[0].getContent()[0].getContent()[1];
                /*in a timeout to queue the events*/
                if (commentTextArea&&commentTextArea.getValue) {
                    // S1342467.. focus on the comment text area and move the cursor to the end of the string/comment
                    sas.stress.wkbench.StressUtil.setCaretPosition(commentTextArea.getId(),commentTextArea.getValue().length,true);
                }else if (commentTextArea.getContent){
                    var commentTextArea=commentTextArea.getItem()[1];
                    // S1342467.. focus on the comment text area and move the cursor to the end of the string/comment
                    sas.stress.wkbench.StressUtil.setCaretPosition(commentTextArea.getId(),commentTextArea.getValue().length,true);
                }
                self._showOrHideNoData(
                    view.getModel().getProperty("/showHideNoData"),
                    sas.stress.wkbench.StressUtil.getBundle().getText("cell.comments.manager.no.cell.selected.txt")
                );
                view.nextButton.setEnabled(self.nextCellIndex<=(self.allComments.length-1));
                view.previousButton.setEnabled(self.previousCellIndex>-1);

                /* keep the cell highlighted... Ritesh spoke about this.. keeps Excel like functionality */
                var liveModel=view.getModel("live");
                var selected=liveModel.getProperty("/selected");
                var slimSheet=liveModel.getProperty("/slimsheet");
                if (selected) {
                    // use the static because we do not want to fire the cell selected action.. only highlight the cell
                    sas.stress.wkbench.widget.Slimsheet.selectCell(slimSheet, selected.row, selected.column, selected.filterId);
                }
            }
        }
    },
    /**
     * Refreshes the model of the view for the comment manager
     *
     * @param selected
     * @param cellRef
     * @private
     */
    _refreshModel:function(selected,cellRef){
        var that=this;
        var view=this.getView();
        var model=view.getModel();
        var comments = [];
        var isCellEditable=!!cellRef && cellRef.getInfoData().isEditable(cellRef.row, cellRef.column, true);

        if (selected&&cellRef) {
            var data = cellRef.getFilterData()/*getScenarioData()*/;
            var key = "$" + cellRef.colLong + "$" + cellRef.rowLong;
            var currentProject = sas.stress.wkbench.model.Project.getCurrentProject();	// the common overlay that is stored on the project
            var dataKey= cellRef.getInfoData().getDatakeyFromRowColumn(key);
            var projectOverlay = currentProject.getCellOverlayByDataKey((overlay ? overlay.dataKey : dataKey));	// make sure we have something

            var unsavedComment = false;
            var savedComment=data.cComments[key]?data.cComments[key]:(projectOverlay && projectOverlay.comment && projectOverlay.comment .timestamp? projectOverlay.comment : undefined);
            if (data.ncComments.hasOwnProperty(key)||(savedComment&&savedComment.timestamp===undefined)) {  // if no time stamp then it is unsaved item
                unsavedComment = true;
                // put into unsaved comments
                if (savedComment) {
                    var unsaved = {
                        cellRef: cellRef,
                        comment: savedComment.comment ? savedComment.comment : savedComment.value
                    };
                    unsaved.comment = savedComment.comment ? savedComment.comment : savedComment.value;
                    this.unSavedComments[cellRef.colShort + cellRef.rowShort] = unsaved;
                }
            }
            var overlay = data.ncComments["$" + cellRef.colLong + "$" + cellRef.rowLong];	// the general overlay for the sheet
            var comment = projectOverlay && projectOverlay.comment ? projectOverlay.comment.value : "";//overlay?overlay.comment:"";
            var user = overlay ? overlay.user : (projectOverlay && projectOverlay.comment? projectOverlay.comment.user: "");
            var timestamp = overlay ? overlay.timestamp : (projectOverlay && projectOverlay.comment? projectOverlay.comment.timestamp: "");
            var dttm = "";
            var fmt = sap.ui.core.format.DateFormat.getDateTimeInstance();
            if (timestamp) {
                dttm = fmt.format(new Date(timestamp), false);
            }
            if (!dttm){
                this.originalComment=comment;
            }

            if (!unsavedComment&&isCellEditable) {
                comments.push({
                    comment: this.unSavedComments[cellRef.colShort+cellRef.rowShort]?this.unSavedComments[cellRef.colShort+cellRef.rowShort].comment:"",
                    user: "",
                    dttm: "",
                    enableCommentField: true
                });
            }else if(isCellEditable){
                comments.push({
                    comment: comment,
                    user: dttm ? user : "",
                    dttm: dttm,
                    enableCommentField: dttm?false:true
                });
            }
            if (savedComment&&savedComment.timestamp) {
                dttm = fmt.format(new Date(savedComment.timestamp), false);
                comments.push({
                    comment: savedComment.comment?savedComment.comment:savedComment.value,
                    user: savedComment.user,
                    dttm: dttm,
                    enableCommentField: false
                });
            }
            model.setProperty("/comments", []);
            model.setProperty("/comments", comments);
            model.setProperty("/isAdd", comment.length === 0 && dttm === "");
            model.setProperty("/showHideNoData",dttm === "");
            //model.setProperty("/enableCommentPane",isCellEditable);

            if (isCellEditable) {
                this._showOrHideNoData(comments.length === 1)
            }else{
                // non-editable cell remove the comments
                //model.setProperty("/comments", []);
                this._showOrHideNoData(comments.length===0);
            }
            if (this.unSavedComments[cellRef.colShort+cellRef.rowShort]) {
                this._enablePostButton(true);
            }else{
                this._enablePostButton(false);
            }
        }else{
            var eachComment={
                comment:"",
                user:"",
                dttm:"",
                enableCommentField:false
            };
            comments.push(eachComment);
            model.setProperty("/comments",[]);
            //model.setProperty("/enableCommentPane",false);
            if (view.layout.getPropertyPaneVisible()/*&&view.layout.getSelectedPropertyPaneIndex()===1*/){
                this._showOrHideNoData(true,sas.stress.wkbench.StressUtil.getBundle().getText("cell.comments.manager.no.cell.selected.txt"));
            }
        }
    },
    /**
     * Factory method to create the content of the comment manager layout using factory binding of SAP UI5
     *
     * @param sId
     * @param context
     * @returns {sas.hc.ui.layout.VerticalLayout}
     * @private
     */
    _createCommentContent:function(sId, context){
        var comment=context.getObject();
        var controller=this;//sas.stress.wkbench.ViewManager.getWorkbench().getScenarioEditorView().getController();
        var commentUserLabel = new sas.hc.m.Label({
            text: {
                path:"user",
                formatter:function(usr){
                    return usr?usr:sas.stress.wkbench.StressUtil.getBundle().getText("commentManager.newComment.txt");
                }
            },
            design:{
                path:"user",
                formatter:function(usr){
                    return usr?sap.m.LabelDesign.Bold:sap.m.LabelDesign.Standard;
                }
            }
        }).addStyleClass("stressCellCommentManagerUserLabel");
        var commentTimestampLabel = new sas.hc.m.Text({
            text: "{dttm}",
            visible:{
                path:"dttm",
                formatter:function(dt){
                    return dt?true:false;
                }
            }
        }).addStyleClass("popoverCmntDate stressCellCommentTextAreaStyle");
        var commentTextArea = new sas.stress.wkbench.widget.TextAreaAuto({
            value: {
                path:"comment",
                formatter:function(c){
                    return c;
                }
            },
            width: "98.4%",
            rows: 3,
            maxLength: 2048,
            autoExpandable: true,
            textAlign: sap.ui.core.TextAlign.Begin,
            enterPressed: $.proxy(controller._onSaveCommentClick, controller),
            escapePressed: $.proxy(controller._onCancelEdit, controller),
            editable: "{enableCommentField}",
            liveChange: $.proxy(controller._onEditComment,controller),
            onFocus: $.proxy(controller._onCommentTextFocus,controller)
        }).addStyleClass("stressCellCommentTextAreaStyle");

        var addButton=new sas.hc.m.Button({
            text: sas.stress.wkbench.StressUtil.getBundle().getText("commentManager.button.post.txt"),
            visible: {
                path:"dttm",
                formatter:function(dt){
                    return dt?false:true;
                }
            },
            enabled: "{commentChangeModel>/saveButtonEnabled}",
            press: $.proxy(controller._onSaveCommentClick,controller)
        }).addStyleClass("sasPrimaryButton");
        var cancelButton=new sas.hc.m.Button({
            text: sas.stress.wkbench.StressUtil.getBundle().getText("scenarioeditor.sheet.comments.button.cancel.tip"),
            visible: {
                path:"dttm",
                formatter:function(dt){
                    return dt?false:true;
                }
            },
            enabled: "{commentChangeModel>/saveButtonEnabled}", // S1343514
            press: $.proxy(controller._onCancelEdit,controller)
        });
        var buttonLayout=new sas.hc.ui.layout.HorizontalLayout({
            content:[
                /*new sas.hc.m.ToolbarSpacer({ width: "9.3rem" }),*/
                addButton,
                cancelButton
            ],
            visible: {
                path:"dttm",
                formatter:function(dt){
                    return dt?false:true;
                }
            }
        }).addStyleClass("stressCellCommentManagerButtonLayout");
        var commentLayout= new sas.hc.m.VBox({
            width:"100%",
            items: [
                commentUserLabel,
                commentTextArea,
                commentTimestampLabel,
                buttonLayout,
                //noData,
                new sas.hc.ui.commons.HorizontalDivider({
                }).addStyleClass("stressRemoveDividerMargin")
            ]
        });

        if (comment.dttm===""){
            commentLayout.addStyleClass("stressEditableComment");
        }

        return commentLayout;
    },
    /**
     * Enable or Disable the "Post" button in the comment manager
     * @param enable
     * @private
     */
    _enablePostButton:function(enable){
        var buttonsModel=this.getView().cellComments.getModel("commentChangeModel");
        if (buttonsModel) {
            buttonsModel.setProperty("/saveButtonEnabled", enable);
        }
    },
    _showOrHideNoData:function(show,txt) {
        var that = this;
        var view = this.getView();
        // the nodata could have been destroyed..... recreate it
        if (this.noData===undefined||this.noData.bIsDestroyed) {
            that.noData = new sas.hc.m.ZeroState({
                //icon: alertIcon?sas.icons.HC.ZEROSTATEALERT: sas.icons.HC.ZEROSTATEZERO,  // icon not allowed in property pane
                text: txt||sas.stress.wkbench.StressUtil.getBundle().getText("home.tasks.details.comments.nodata.txt")
            });
            setTimeout(function () {
                view.commentPropPane.getContent()[0].getCenter().getContent()[0].addItem(that.noData);
            }, 0);
        }
        this.noData.setVisible(show);
    },
    _onNavigatePrevious:function(){
        var view=this.getView();
        var liveModel=view.getModel("live");
        var slimSheet=liveModel.getProperty("/slimsheet");
        if (this.previousCellIndex>-1){
            this.nextCellIndex=this.previousCellIndex+1;    // this next is the previous + 1
            var previousCell=this.allComments[this.previousCellIndex--];

            slimSheet.scrollToAndSelectCell(previousCell.row,previousCell.col);
            this._selectSelectedCellInList(this.previousCellIndex+1);
        }
    },
    _onNavigateNext:function(){
        var view=this.getView();
        var liveModel=view.getModel("live");
        var slimSheet=liveModel.getProperty("/slimsheet");
        var size=this.allComments.length;//_.size(this.allComments);
        if (this.nextCellIndex< /*this.allComments.length*/size){
            this.previousCellIndex=this.nextCellIndex-1;    // the previous index is one before the next
            var nextCell=this.allComments[this.nextCellIndex++];

            slimSheet.scrollToAndSelectCell(nextCell.row,nextCell.col);
            this._selectSelectedCellInList(this.nextCellIndex-1);
        }
    },
    _updateNavigationIndex:function(selectedCellKey, currentRow, currentCol){
        var self=this;
        _.find(this.allComments, function(value, key){
            if (value.row>-1&&value.col>-1) {
                if (value.key === selectedCellKey) {
                    self.nextCellIndex = key + 1;
                    self.previousCellIndex = key - 1;
                    return true;
                } else {
                    if (value.row > currentRow) {
                        self.nextCellIndex = key;
                        self.previousCellIndex = key - 1;
                        return true;
                    } else if (value.row === currentRow) {
                        if (value.col >= currentCol) {
                            self.nextCellIndex = key;
                            self.previousCellIndex = key - 1;

                            return true;
                        }
                    }
                }
            }
            return false;
        });
    },
    _onCommentTextFocus:function(){
        /* keep the cell highlighted... Ritesh spoke about this.. keeps Excel like functionality */
        var liveModel=this.getView().getModel("live");
        var selected=liveModel.getProperty("/selected");
        var slimSheet=liveModel.getProperty("/slimsheet");
        // use the static because we do not want to fire the cell selected action.. only highlight the cell
        sas.stress.wkbench.widget.Slimsheet.selectCell(slimSheet, selected.row,selected.column, selected.filterId);
    },
    _updateCommentsMap:function(cellRef){
        var comments=[];
        var rb=sas.stress.wkbench.StressUtil.getBundle();
        /*comments.push({
            key: rb.getText("cell.comment.list.txt"),
            row:-1,
            col:-1,
            index:0
        });*/
        var view = this.getView();
        var viewData = view.getViewData();
        var model=view.getModel();
        var sheet=sas.stress.wkbench.model.Project.getCurrentProject().getSheet(viewData.sheetId);
        if (sheet) {
            var index = 0;
            if (!cellRef) {
                for (var i in sheet.getInfo().getFilters().getData().ncComments) {
                    var ol = sheet.getInfo().getFilters().getData().ncComments[i];
                    var ref = new sas.stress.wkbench.model.CellRef(ol.columnIndex/*-1*/, ol.rowIndex, sheet.getInfo().getFilters()/*.getParameter("scenario")*/);
                    comments.push({
                        key: ref.colShort + ref.rowShort,
                        row: ol.rowIndex,
                        col: ol.columnIndex,
                        index: index++
                    });
                }
                this.allComments = _(comments).chain().sortBy(function (cell) {
                    return cell.col;
                }).sortBy(function (cell) {
                    return cell.row;
                }).value();
            } else {
                //S1344598
                var insertIndex = this.allComments.length;  // assume to be inserted last
                var exists = false;
                _.find(this.allComments, function (value, key) {
                    // same cell new comment added
                    if (value.key === cellRef.colShort + cellRef.rowShort) {
                        insertIndex = key;
                        exists = true;
                        return true;
                    } else {
                        // look for the correct place
                        if (value.row > cellRef.row) {
                            insertIndex = key;
                            return true;
                        } else if (value.row === cellRef.row) {
                            if (cellRef.column > value.col) {
                                insertIndex = key + 1;
                                return true;
                            } else if (cellRef.column < value.col) {
                                insertIndex = key;
                                return true;
                            }
                        }
                    }
                });
                if (!exists) {  // if an existing comment is updated, do not do anything
                    var comment = {
                        key: cellRef.colShort + cellRef.rowShort,
                        row: cellRef.row,
                        col: cellRef.column,
                        index: insertIndex
                    };
                    this.allComments.splice(insertIndex, 0, comment);
                    this.allComments.join();

                    /* S1343257..
                       we need to update index of the comment next to insertIndex to be the next value
                       for all the comments after
                     */
                    for (var index=insertIndex+1;index<this.allComments.length;++index) {
                        comment = this.allComments[index];
                        comment.index = index;
                    }
                }
            }
        }
        model.setProperty("/allComments", this.allComments);

        var compareValues=function(v1, v2){
            return (v1 > v2)
                ? 1
                : (v1 < v2 ? -1 : 0);
        };
    },
    /**
     * The event handler for the comment selection
     *
     * @param e
     * @private
     */
    _commentSelected:function(e){
        var view=this.getView();
        var liveModel=view.getModel("live");
        var model=view.getModel();
        var allComments=model.getProperty("/allComments");
        var item=e.getParameter("selectedItem");
        //if ("0"!==item.getKey()) {
            var selectedCommentIndex = Number(item.getKey());
            var commentData = allComments[selectedCommentIndex];

            var slimSheet = liveModel.getProperty("/slimsheet");
            slimSheet.scrollToAndSelectCell(commentData.row, commentData.col);
        //}else{
            //this._refreshModel(false,undefined);
            /*this.previousCellIndex=0;
            this.nextCellIndex=0;*/
        //}
    },
    /**
     * Select the item in the comments list. The cell reference would be the highlighted cell on the worksheet
     * @param key
     * @private
     */
    _selectSelectedCellInList:function(key){
        var view=this.getView();
        var item;
        if (key>-1) {
            item=view.commentsList.getItemByKey(key.toString());
        }else{
            item=view.commentsList.getItems()[0];   // the "select an Item" value
        }
        if (item) {
            view.commentsList.setSelectedItemId(item.getId());
        }
    },
    /**
     * S1343227 and S1344598
     */
    clearUnSavedComments:function(cellRef){
        var self=this;
        if (cellRef){
            var key=cellRef.colShort+cellRef.rowShort;
            delete self.unSavedComments[key];
        }else {
            _.each(this.unSavedComments, function (value, key) {
                delete self.unSavedComments[key];
            });
            this.unSavedComments = {};
            this.originalComment = "";
        }
    },
    _hideCommentPane:function(){
        var view=this.getView();
        var model=view.getModel();
        var selectedItem=view.commentsList.getSelectedItem().getKey();
        // make sure a cell without a comment is selected to auto hide!!!
        if (selectedItem==="0") {
            var autoCloseCommentPaneOnEsc = model.getProperty("/autoCloseCommentPaneOnEsc");
            if (autoCloseCommentPaneOnEsc && view.layout.getPropertyPaneVisible()) {
                view.layout.setPropertyPaneVisible(false);  // minimize the pane
            }
            model.setProperty("/autoCloseCommentPaneOnEsc", false);    // reset the flag so that it cannot close
        }
    }
});