jQuery.sap.require("sas.hc.m.BusyDialog");
jQuery.sap.require("sas.hc.m.Dialog");
jQuery.sap.require("sas.hc.m.MessageToast");
jQuery.sap.require("sas.risk.common.models.RiskJSONModel");
jQuery.sap.require("sas.stress.reports.StressAPI");
jQuery.sap.require("sas.stress.scenarios.ScenarioAPI");
jQuery.sap.require("sas.stress.wkbench.StressAPI");
jQuery.sap.require("sas.stress.wkbench.LeftNavManager");
jQuery.sap.require("sas.stress.wkbench.StressFixes");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.stress.wkbench.ViewManager");
jQuery.sap.require("sas.stress.wkbench.model.CellHistoryItem");
jQuery.sap.require("sas.stress.wkbench.model.Project");
jQuery.sap.require("sas.stress.wkbench.widget.DescriptionFacet");
jQuery.sap.require("sas.stress.wkbench.widget.DescriptionModelFilterProvider");


sap.ui.controller("sas.stress.wkbench.view.Workbench", {
    onInit: function () {
        this.router = sap.ui.core.routing.Router.getRouter("router");
        this.router.attachExactRouteMatched("Workbench", this._init, this);

        // show groove bar
        sas.stress.wkbench.LeftNavManager.getInstance().showGrooveBar(sas.stress.wkbench.LeftNavManager.PROJECTS);
    },
    _init: function (event) {
        var projectId = event.getParameter("arguments").projectId;

        var that = this;
        that._processAllSheets=true;    //S1371893
        // open the busy dialog
        that.openIfRequiredAndUpdateBzDialog(sas.stress.wkbench.StressUtil.getBundle().getText("project.loading.data.message.txt"));
        var rb = sas.stress.wkbench.StressUtil.getBundle();

        // S1273684: don't use the cached project since it might be stale or deleted by others
        sas.stress.wkbench.model.Project.loadProject(projectId, true).done(function (project) {
            // make sure there are sheets
            if (project._getFilteredSheets().length===0) {
                that.closeBzDialog();
                if (!sas.stress.wkbench.CapabilityManager.getInstance().isCapableOf(sas.stress.wkbench.CapabilityManager.PROJECT.UPDATE)) {
                    // fixes for CET defect S1266931
                    var messages = [];
                    messages.push(rb.getText("list.bullet.txt") + rb.getText("workbench.error.no.templates.txt"));
                    messages.push(rb.getText("list.bullet.txt") + rb.getText("workbench.error.no.permissions.txt"));
                    messages.push(rb.getText("list.bullet.txt") + rb.getText("workbench.error.no.data.txt"));

                    //message+=rb.getText("contact.sys.admin.txt");
                    var json = {
                        message: rb.getText("workbench.error.nosheets.main.fmt", project.getName()),
                        details: messages
                    };
                    sas.stress.wkbench.StressUtil.handleServerFailure(json, rb.getText("workbench.load.title.txt"));

                    return sas.stress.wkbench.ViewManager.loadHomePage(project);
                }
            }

            that.refresh(project).done(function(){
                // fixes for CET defect S1266931
                that.closeBzDialog();
            }).fail(function(jqxhr){
                that.closeBzDialog();
                sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
                return sas.stress.wkbench.ViewManager.loadHomePage(project);
            });
        }).fail(function (jqxhr) {
            that.closeBzDialog();

            // update the response.. The details contain the message from the server
            jqxhr.responseJSON.details.push(rb.getText("list.bullet.txt")+jqxhr.responseJSON.message);
            // the message from the server is updated to contain a UI message
            jqxhr.responseJSON.message= rb.getText("workbench.error.noproject.main.fmt",sas.stress.wkbench.ViewManager.projectName);
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr,rb.getText("workbench.load.title.txt"));
            // reset it
            sas.stress.wkbench.ViewManager.projectName="";
            return sas.stress.wkbench.ViewManager.loadProjectManager();
        });

        this.navChecker = sas.stress.wkbench.StressUtil.createNavChecker(true, this.getView());
    },
    /*
     * the optional argument 'bEnableSave' is for enabling the save
     * during 'refresh and merge' option select on the conflict resolution dialog
     */
    refresh: function (project, bEnableSave) {
        var def = $.Deferred();
        var that = this;
        this.project = project;
        var view = this.getView();
        var rb = sas.stress.wkbench.StressUtil.getBundle();

        if (view.getModel() === undefined && bEnableSave === undefined) { // initial
            /*view.setBusyIndicatorDelay(0);
             view.setBusy(true);*/
            sas.stress.wkbench.ViewManager.loadCellEditor();
            sas.stress.wkbench.CellPickerUtil.deactivateEditMode();//page refresh we reset the cell picker edit mode

            // S1510277.. a change did not come through from RFW 3.1. see defect for details
            var preferredSheetId=undefined;/*sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.sheet_key", project.getId())||
                (this._getFilteredSheets()[0]?this._getFilteredSheets()[0].getId():undefined);
            if (project.getSheet(preferredSheetId)===undefined){    // make sure the sheet exists
                preferredSheetId=this._getFilteredSheets().length>0?this._getFilteredSheets()[0].getId():undefined;
            }*/

            this.project.getMyTasks().done(function(myTasks){
                that.myTasks= myTasks;  // store it
                that.project.loadSheetsData(preferredSheetId, that.navChecker.getDirty()).done(function (newproject) {
                    that._refreshModel(newproject, myTasks, bEnableSave, rb);

                    def.resolve(that);
                }).fail(function (jqxhr) {
                    if (typeof(jqxhr)==="string"&&(jqxhr===sas.stress.wkbench.model.Sheet.NO_PERMISSONS||jqxhr===sas.stress.wkbench.model.Sheet.NO_DATA)) {
                        if (!sas.stress.wkbench.CapabilityManager.getInstance().isCapableOf(sas.stress.wkbench.CapabilityManager.PROJECT.UPDATE)) {
                            // fixes for CET defect S1266931
                            var messages = [];
                            messages.push(rb.getText("list.bullet.txt") + rb.getText("workbench.error.no.templates.txt"));
                            messages.push(rb.getText("list.bullet.txt") + rb.getText("workbench.error.no.permissions.txt"));
                            messages.push(rb.getText("list.bullet.txt") + rb.getText("workbench.error.no.data.txt"));

                            var json = {
                                message: rb.getText("workbench.error.nosheets.main.fmt", project.getName()),
                                details: messages
                            };
                            def.reject(json);
                        }else {
                            that._refreshModel(project, myTasks, bEnableSave, rb);
                            def.resolve(that);
                        }
                    }
                    def.reject(jqxhr);
                });
            });
        } else {
            var liveModel = that.getView().getModel("live");
            var defSheetId = sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.sheet_key", project.getId());
            var sheetId=defSheetId||liveModel.getProperty("/previousSheetId")||filteredSheets[0].getId();
            that.getView().selectSheet(sheetId);
            def.resolve(that);
        }
        //Note: flipping the busy flag is in ScenarioEditor.controller's onAfterRendering
        return def;
    },
    /**
     * refresh the model..
     *
     * @param project       - the project
     * @param myTasks       - the tasks of the user
     * @param bEnableSave   - do we want to enable the save button (if dirty then yes)
     * @param rb            - the resource bundle
     *
     * @private
     */
    _refreshModel: function(newProject, myTasks, bEnableSave, rb){
        var that = this;
        var view = this.getView();

        that._processAllSheets=true;    // S1371893

        var json = {};
        if (!newProject){
            newProject = that.project;
        }
        json.name = newProject?newProject.getName():that.project.getName();
        json.asOfDate = newProject.getAsOfDate();
        json.actions = myTasks.length?myTasks[0].getActions():[];
        json.isProjectLocked = that.project.isLocked();   // S1388587 for task action menu item to be disabled

        // categories first
        var _categories = {};
        json.categories = [];
        // the all category will contain the total count and it goes first
        var filteredSheets = newProject.getSheets();	// fixes for S1231681... all the sheets should be used to figure out the categories
        _categories[rb.getText("filter.category.all.txt")] = {
            count: filteredSheets.length,
            key: "0",
            name: rb.getText("filter.category.all.txt")
        };
        _.each(filteredSheets, function (sheet) {
            var categoryKey = sheet.getSheetCategory().getId();
            var categoryName = sheet.getSheetCategory().getName();

            var sheetCategoryCount = _categories[categoryName] || {count: 0, key: "0", name: ""};
            // increment the count &&
            sheetCategoryCount.count++;
            _categories[categoryName] = {count: sheetCategoryCount.count, key: categoryKey, name: categoryName};
        });
        var idx = 0;
        _.each(_categories, function (categorydetails, category) {
            json.categories.push({
                key: categorydetails.key,
                name: category,
                count: categorydetails.count.toString()
            });
            idx++;
        });

        // handle process orchestration parameters, if they exist. Need to do this prior to selecting the category
        // or loading the filtered sheets.
        var currentSheet;
        var paramsModel = sap.ui.getCore().getModel("WorkbenchParams");
        if (paramsModel && paramsModel.getData()) {
            if (paramsModel.getData().sheetName) {
                currentSheet = paramsModel.getData().sheetName;

                // S1251269: If a worksheet was specified in process orchestration, then we want to use the "ALL" category.
                var prefs = {"sas.stress.wkbench.category_key": 0};
                sas.stress.wkbench.UserPrefs.save(prefs, newProject.getId());   // S1431593
            }
            sap.ui.getCore().setModel(undefined, "WorkbenchParams");
        }

        // the sheets next
        filteredSheets = that._getFilteredSheets();
        json.sheets=[];
        var foundSheet = false;
        _.each(filteredSheets, function (sheet) {
            var icon=null;
            if(sheet.errored){ // S1266931
                icon=sas.icons.HC.WARNINGSTATUS; // error sheets get an icon too
            }
            json.sheets.push({name: sheet.getNameVersion(), key: sheet.getId(), icon: icon});
            // Was a starting sheet specified? (Process Orchestration)
            if (currentSheet && currentSheet.toLocaleLowerCase() === sheet.getName().toLocaleLowerCase()) {
                // Save the key of the worksheet that was passed in so it is the default worksheet
                sas.stress.wkbench.UserPrefs.save({"sas.stress.wkbench.sheet_key": sheet.getId()}, newProject.getId());
                delete currentSheet;
                foundSheet = true;
            }
        });
        if (currentSheet && !foundSheet) {
            // Invalid worksheet name specified.
            sas.hc.m.MessageDialog.error({
                text: rb.getText("workbench.processOrchestration.invalidSheetName.fmt.txt", [currentSheet]),
                title: rb.getText("workbench.load.title.txt")
            });

            return;
        }
        // iteration last
        var projectIteration = newProject.getCurrentIterationNo();
        json.iterations = [];
        for (var i = projectIteration; i > 0; i--) {
            json.iterations.push({
                text: i,
                key: i.toString()
            });
        }
        var iteration=sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.project_iteration_id",newProject.getId())||projectIteration;
        if (iteration !== null) {
            view.iterationSelect.setSelectedKey(iteration.toString());
        }

        var model= new sas.risk.common.models.RiskJSONModel(json);
        that.getView().setModel(model);
        var liveModel = that.getView().getModel("live") || new sas.risk.common.models.RiskJSONModel({saveButtonEnabled: ((bEnableSave !== undefined) ? bEnableSave : false)});
        liveModel.setProperty("/iterationNumber", sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.project_iteration_id", newProject.getId()) || newProject.getCurrentIterationNo());
        liveModel.setProperty("/copyButtonEnabled",false);
        var isNodeExpanded = liveModel.getProperty("/isNodeExpanded");
        liveModel.setProperty("/isNodeExpanded", (isNodeExpanded === undefined || isNodeExpanded === null) ? true: isNodeExpanded);
        that.getView().setModel(liveModel, "live");//live selections,formula editing decors
        // select a sheet if there are any sheets present
        if (filteredSheets.length>0) {
            that._autoSelectFirstSheet();
        }else{
            view.setNoData(that);
        }
    },
    onBreadCrumbClick: function (e) {
        var item = e.getParameter("item");
        var that = this;
        switch (item.getProperty("key")) {
            case "ProjectManager":
                var keys= _.keys(sas.stress.wkbench.model.Project._projects);
                for (var i=keys.length-1;i>=0;--i){
                    sas.stress.wkbench.model.Project.clearProject(keys[i]);
                }
                keys=[];
                sas.stress.wkbench.ViewManager.loadProjectManager();
                break;
            default:
                sas.stress.wkbench.ViewManager.loadHomePage(that.project);
                break;
        }
    },

    _getFilteredSheets: function () {
        //return this.project._getFilteredSheets();   // S1347872.. common method now in project because
        var sheets = this.project.getSheets();
        if (this._processAllSheets===false){    // S1371893
        	var preferredCategory = sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.category_key", this.project.getId());
        	if (preferredCategory && preferredCategory !== "-1") {
                // HF..S1352166
                // general purpose guard.. make sure the preferred category exists
	            var categoryFound=_.find(this.project.getSheetCategories(),function(category){
	                return category.getId()===preferredCategory;
	            });
            	if (categoryFound) {
                    // HF..S1352166
                    sheets = _.filter(sheets, function (sheet) { // need to sert by name
                    	return (sheet.getSheetCategory().getId() === preferredCategory);
                    });
            	}else{
                	// remove the preference
                	sas.stress.wkbench.UserPrefs.delete({"sas.stress.wkbench.category_key":undefined}, this.project.getId());
            	}
        	}
        }
        return _.sortBy(sheets, function(sheet){ // need to sert by name
            return sheet.getName().toLocaleLowerCase();
        });
    },
    onExit: function () {
        // we would need to destory all the scenario editors that have been created
        var tobeDestroyedScenarioEditors = sas.stress.wkbench.ViewManager.getAllScenarioEditorViews();
        var length = _.size(tobeDestroyedScenarioEditors);
        _.each(tobeDestroyedScenarioEditors, function (scenarioeditorView, key) {
            // destroy everything related to the scenario editor
            scenarioeditorView.destroy();
        });
        if (this.project) {
            this.project.destroy();
            delete this.project;
        }
        this.router.detachExactRouteMatched("Workbench", this._init, this);
        // clear the View on the router otherwise the next router.navTo() would fail with a blank page
        this.router.setView("sas.stress.wkbench.view.Workbench", null);
        this.navChecker.stop();
    },
    openSheetNew: function (e) {
        var that=this;
        var view = this.getView();
        var liveModel = view.getModel("live");
        var sheetId = view.getSelectedSheetId();
        var sheet = this.project.getSheet(sheetId);
        var rb = sas.stress.wkbench.StressUtil.getBundle();
        // if the sheet is marked as "errored" during load
        function displayErrorMessage() {
            this.closeBzDialog();
            // show a dialog that mentions the sheet has an issue
            var json = {
                message: rb.getText("workbench.error.invalid.sheets.fmt", sheet.getName()),
                details: sheet.getNoVisibilityReasons()
            };
            sas.stress.wkbench.StressUtil.handleServerFailure({responseJSON: json});
            // revert the drop down to the previous selected sheet
            sheetId = liveModel.getProperty("/previousSheetId");
            if (sheetId) {
                var oldItem = view.sheetSelect.getItemByKey(sheetId);
                view.sheetSelect.setSelectedItem(oldItem);
            } else {
                // we will need reload the sheet
                var iterationId = sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.project_iteration_id", that.project.getId()) || that.project.getCurrentIterationNo();
                sas.stress.wkbench.ViewManager.loadScenarioEditor(sheetId, view.getDefaultFilterId(sheetId), iterationId);
            }
            return;
        }

        if (sheet.errored){ // S1266931
            return displayErrorMessage.call(this);
        }else if (sheet.getInfo()===undefined){
            // open the busy dialog
            this.openIfRequiredAndUpdateBzDialog(rb.getText("project.loading.sheet.message.fmt",sheet.getName()));
            this.project.loadSheetsData(sheetId,this.navChecker.getDirty()).done(function(){
                var filteredSheets = that._getFilteredSheets();
                var modelData=view.getModel().getData();
                modelData.sheets=[];
                _.each(filteredSheets, function (sheet) {
                    var icon=null;
                    if(sheet.errored){ // S1266931
                        icon=sas.icons.HC.WARNINGSTATUS; // error sheets get an icon too
                    }
                    modelData.sheets.push({name: sheet.getNameVersion(), key: sheet.getId(), icon: icon});
                });
                view.getModel().setData(modelData);

                sheet = that.project.getSheet(sheetId);
                if (sheet.errored){
                    return displayErrorMessage.call(that);
                }else{
                    sas.stress.wkbench.UserPrefs.save({"sas.stress.wkbench.sheet_key": sheetId},that.project.getId());
                    that._refreshSheets();
                }
            });
        } else{
            sas.stress.wkbench.UserPrefs.save({"sas.stress.wkbench.sheet_key": sheetId},this.project.getId());
            this._refreshSheets();
        }
    },
    _autoSelectFirstSheet: function () {
        var workbenchView = this.getView();
        this.openIfRequiredAndUpdateBzDialog(sas.stress.wkbench.StressUtil.getBundle().getText("project.loading.data.message.txt"));
        var sheets = this._getFilteredSheets();
        if (sheets.length) {
            var defaultSheetId = workbenchView.getDefaultSheetId(sheets);

            var item = workbenchView.sheetSelect.getItemByKey(defaultSheetId);
            workbenchView.sheetSelect.setSelectedItemId(item.getId());
            workbenchView.sheetSelect.fireChange({selectedItem: item});//for some odd reason i had to manually fire it.

            // S1406702: upcoming defect.. add the defect here..
            // select the sheet category after the default sheet is figured out because the category for the sheet would have changed
            // if so, we will default to the "All" category.. user preferred sheet takes precedence over category
            var preferredCategory = sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.category_key", this.project.getId());
            // use the preferred category key if it is present
            if (preferredCategory) {
                var sheet;
                var category = _.find(this.project.getSheetCategories(), function(sheetCategory){
                    return sheetCategory.getId() === preferredCategory;
                });
                if (category) {
                    sheet = _.find(category.getSheets(), function (sheet) {
                        return sheet.getId() === defaultSheetId;
                    });
                    if (sheet){
                        // fixes for S1231681
                        workbenchView.sheetCategorySelect.setSelectedKey(Number(preferredCategory));

                    }else{
                        sheet = this.project.getSheet(defaultSheetId);
                        preferredCategory = sheet.getSheetCategory().getId();
                    }
                }else{
                    sheet = this.project.getSheet(defaultSheetId);
                    preferredCategory = sheet.getSheetCategory().getId();
                }
                this._handleSheetCategoryChange(false, "sas.stress.wkbench.category_key", preferredCategory);
            }
        } else {
            //sas.stress.wkbench.ViewManager.getWorkbench().setBusy(false);
            // close the busy dialog
            // fixes for CET defect S1266931
            this.closeBzDialog();
        }
    },
    navigateToHomePage: function () {
        sas.stress.wkbench.ViewManager.loadHomePage(this.project);
    },
    changeIteration: function () {
        var that=this;
        var view = this.getView();
        var project = sas.stress.wkbench.model.Project.getCurrentProject();
        var selectedProjectIterationNo = view.getSelectedIterationId();

        if (this.navChecker.getDirty()){
            var rb = sas.stress.wkbench.StressUtil.getBundle();
            // show the warning dialog asking for refresh before if unsaved changes exist
            sas.hc.m.MessageDialog.warning({
                text: rb.getText("project.iteration.change.apply.question.fmt", project.getCurrentIterationNo()),
                title: rb.getText("workbench.change.iteration.title.txt"),
                buttonActions: [sas.hc.m.MessageDialogAction.Apply, sas.hc.m.MessageDialogAction.Cancel],
                buttonPress: function (event) {
                    var action = event.getParameter("origin").getMessageDialogAction();
                    if (action === sas.hc.m.MessageDialogAction.Apply) {
                        // common method
                        that._performProjectIterationChange(selectedProjectIterationNo);
                    } else {
                        var iterationId=view.getModel("live").getProperty("/iterationNumber")||sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.project_iteration_id",that.project.getId());
                        view.iterationSelect.setSelectedKey(iterationId.toString());
                    }
                }
            });
        } else {
            // common method
            that._performProjectIterationChange(selectedProjectIterationNo);
        }
    },
    /*
     * Common Method
     * @param selectedProjectIterationNo
     * @private
     */
    _performProjectIterationChange:function(selectedProjectIterationNo) {
        var view = this.getView();
        var liveModel=view.getModel("live");
        var project = sas.stress.wkbench.model.Project.getCurrentProject();

        this.navChecker.setDirty(false);    // S1395400 the navchecker is not dirty.. user decided to change iteration
        liveModel.setProperty("/iterationNumber", selectedProjectIterationNo);
        liveModel.setProperty("/toggleButtons", false); // S1395400 save is disabled, user decided to discard the changes
        liveModel.setProperty("/saveButtonEnabled", false); // S1395400 save is disabled, user decided to discard the changes
        //project.setCurrentIterationNo(Number(selectedProjectIterationNo));  // S1403022 .. should set at all.. it is what is as received from server
        this._handleProjectIterationChange("sas.stress.wkbench.project_iteration_id", selectedProjectIterationNo).done(function () {
            // reset the actions for the task action buttons visibility
            var model=view.getModel();
            var actions=model.getProperty("/actions")||[];
            model.setProperty("/actions",[]);
            model.setProperty("/actions",actions);
        }).fail(function (jqxhr) {
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr,sas.stress.wkbench.StressUtil.getBundle().getText("workbench.change.iteration.title.txt"));
        });
    },
    _handleProjectIterationChange:function(preferenceKey, preferenceVal){
        var deferred= $.Deferred();
        // S1395400 no need to remove the model
        this.openIfRequiredAndUpdateBzDialog();
        if (preferenceKey&&preferenceVal) {
            sas.stress.wkbench.UserPrefs.save(preferenceKey, preferenceVal, this.project.getId());
        }

        this.refreshProject(false).done(function(project){   // bEnableSave===undefined
            deferred.resolve();
        }).fail(function(jqxhr){
            deferred.reject(jqxhr);
        });

        return deferred;
    },
    _handleSheetCategoryChange:function(selectSheet, preferenceKey, preferenceVal){
        var view = this.getView();
        var filteredSheets = this._getFilteredSheets();
        var modelData=view.getModel().getData();

        if (selectSheet) {
            // display the bz dialog
            this.openIfRequiredAndUpdateBzDialog();
        }
        if (preferenceKey&&preferenceVal) {
            sas.stress.wkbench.UserPrefs.save(preferenceKey, preferenceVal, this.project.getId());
        }
        modelData.sheets=[];
        if (filteredSheets.length === 0) {
            this.closeBzDialog();

            // fixes for CET defect S1266931
            var messages=[];
            messages.push(rb.getText("workbench.error.no.templates.txt"));
            messages.push(rb.getText("workbench.error.no.permissions.txt"));
            messages.push(rb.getText("workbench.error.no.data.txt"));

            var json={message:rb.getText("workbench.error.nosheets.main.fmt", this.project.getName()),details:messages};
            sas.stress.wkbench.StressUtil.handleServerFailure({responseJSON:json},rb.getText("workbench.change.sheet.category.title.txt"));

            return;
        }
        _.each(filteredSheets, function (sheet) {
            var icon=null;
            if(sheet.errored){ // S1266931
                icon=sas.icons.HC.WARNINGSTATUS; // error sheets get an icon too
            }
            modelData.sheets.push({name: sheet.getNameVersion(), key: sheet.getId(), icon: icon});
        });
        view.getModel().setData(modelData);
        if (selectSheet) {
            // select the sheet because user changed category
            var liveModel = view.getModel("live");
            var defSheetId = sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.sheet_key", this.project.getId());
            // HF..S1352166
            if (this.project.getSheet(defSheetId) === undefined) {
                defSheetId = liveModel.getProperty("/previousSheetId") || filteredSheets[0].getId();
            }
            var sheetId = defSheetId || liveModel.getProperty("/previousSheetId") || filteredSheets[0].getId();
            view.selectSheet(sheetId);
        }
    },
    changeSheetCategory: function (e) {
        var view = this.getView();
        var project = sas.stress.wkbench.model.Project.getCurrentProject();
        var sheetCategoryKey = view.getSelectedSheetCategoryKey();
        var liveModel = view.getModel("live");
        // save the new selected sheet category
        sas.stress.wkbench.UserPrefs.save("sas.stress.wkbench.category_key", sheetCategoryKey, this.project.getId());

        var previousSheetId=liveModel.getProperty("/previousSheetId");
        var foundPreviousSheet=_.find(this._getFilteredSheets(),function(sheet){
            if (previousSheetId===undefined){
                return false;
            }
            return (sheet.getId().toString()===previousSheetId.toString());
        });
        if (foundPreviousSheet===undefined) {
            sas.stress.wkbench.UserPrefs.save("sas.stress.wkbench.sheet_key",this._getFilteredSheets()[0].getId(), project.getId());
            liveModel.setProperty("/previousSheetId", undefined);
        }
        //view.sheetSelect.removeAllItems();
        this._handleSheetCategoryChange(true);
    },
    openCellCommentPopup: function (e) {
        var view = this.getView();
        var liveModel = view.getModel("live");
        var selected = liveModel.getProperty("/selected");
        var slimsheet = liveModel.getProperty("/slimsheet");
        // the li_key column removed therefore, the actual column index is required.. scenarios do not exist therefore, use filter
        var cell = sas.stress.wkbench.widget.Slimsheet.findCell(slimsheet, selected.row, slimsheet.getSlimsheetSource().getCalculatedColumn(selected.column/*+1*/, selected.filterId/*.scenarioId*/), false);
        if (selected && slimsheet && cell) {
            var existingPopup=sap.ui.getCore().byId("sas.stress.wkbench.view.CellCommentPopup");
            if (existingPopup){
                existingPopup.destroy();
            }
            var cellCommentEditor = sas.stress.wkbench.ViewManager.loadCellCommentEditor(selected,true,false);
            var cellCommentPopup = new sas.hc.m.ResponsivePopover({
                id: "sas.stress.wkbench.view.CellCommentPopup",
                title: sas.stress.wkbench.StressUtil.getBundle().getText("cell.comment.dialog.title.txt"),
                verticalScrolling: false,
                placement: sap.m.PlacementType.Top,
                modal: true,
                initialFocus: cellCommentEditor.commentTextArea,
                content: cellCommentEditor,
                offsetY:10
            });
            cellCommentPopup.addStyleClass("stressCommentPopover");
            if (this.project.doesRequireComments()) {
                cellCommentPopup.addStyleClass("stressRequiredField");
            }
            cellCommentEditor.attachClosing(function () {
                this.destroy();
            }, cellCommentPopup);
            cellCommentPopup.openBy(cell);
        }
    },
    pressLogButton: function () {
        var that = this;
        var logDialogView = sas.stress.wkbench.ViewManager.loadLogDialogView();
        var logDialog = new sas.hc.m.Dialog({
            title: sas.stress.wkbench.StressUtil.getBundle().getText("cell.log.dialog.window.title"),
            /*width:"1600px",*/
            contentWidth: "85%",
            contentHeight:"28rem",
            content: logDialogView,
            beginButton: logDialogView.discardbutton,
            endButton: logDialogView.endbutton,
            resizable:true,
            afterClose: function () {
                logDialog.destroy();
                // select the last cell selected
                var liveModel=that.getView().getModel("live");
                var selected=liveModel.getProperty("/selected");
                var slimsheet = liveModel.getProperty("/slimsheet");
                if (selected) {
                    slimsheet.fireCellSelected({
                        selected: true,
                        row: selected.row,
                        col: selected.column/*+1*/,
                        filter: selected.getSheetFilters()/*scenario:selected.getScenario()*/
                    });
                }else{
                    slimsheet.focus();
                }
            },
            afterOpen: function () {
                // once the dialog opens, lets sort... this should help in aligning right along with the css
                // disable default sort for now - the 'special' sort is done by the midtier
                // logDialogView.oTable.sort(logDialogView.oTable.getColumns()[5], sas.hc.ui.table.SortOrder.Descending);
            },
            verticalScrolling: false,
            initialFocus: logDialogView.endbutton
        });
        logDialogView.attachClosing(function () {
            logDialog.close();
        }, logDialog);
        logDialog.open();

        // open the busy dialog on top of the log dialog
        that.openIfRequiredAndUpdateBzDialog(sas.stress.wkbench.StressUtil.getBundle().getText("project.loading.data.message.txt"));
    },
    closeBzDialog: function () {
        var view=this.getView();
        if(this.bzdlg) {
            this.bzdlg.close();
            this.bzdlg.destroy();
            this.bzdlg = undefined;
        }
        view.setBusy(false);
    },
    openIfRequiredAndUpdateBzDialog: function(title){
        var view=this.getView();
        var updatedText="";
        view.setBusyIndicatorDelay(0);
        view.setBusy(true);
        if (!this.bzdlg){
            if (!title) {
                this.bzdlg = new sas.hc.m.BusyDialog({id: "StressBusyDialog", title: sas.stress.wkbench.StressUtil.getBundle().getText("project.loading.data.message.txt")});
            }else{
                this.bzdlg = new sas.hc.m.BusyDialog({id: "StressBusyDialog", title: title});   // no busy dialog open with the title provided
            }
            this.bzdlg.open();
        } else{
            updatedText=sas.stress.wkbench.StressUtil.getBundle().getText("project.loading.sheet.message.fmt",title);
            this.bzdlg.setTitle(updatedText);   // update the title
        }
    },
    pressImportButton: function () {
        var that = this;
        var view = sas.stress.wkbench.ViewManager.loadWorksheetImport();
        var dialog = new sas.hc.m.Dialog({
            title: sas.stress.wkbench.StressUtil.getBundle().getText("worksheet.import.title.txt"),
            content: view,
            beginButton: view.okButton,
            endButton: view.cancelButton,
            afterClose: function () {
                this.destroy();
            },
            initialFocus: view.cancelButton
        });
        view.attachClosing(function (e) {
            this.destroy();
        }, dialog);
        view.attachUploading(function (e) {
            if (e.getParameter("action") === "START") {
                view.setBusy(true);
            } else {
                var response = e.getParameter("response");
                if (response !== "Done") {
                    sas.stress.wkbench.StressUtil.handleServerFailure(response, sas.stress.wkbench.StressUtil.getBundle().getText("worksheet.import.title.txt"));   //S1432295
                    view.setBusy(false);
                } else {
                    that.getView().setModel();//clear out view model
                    that.openIfRequiredAndUpdateBzDialog();
                    that._processAllSheets=true;    // S1371893
                    that.refreshProject(false).done(function(){     // no need to call loadsheet data twice PErformance S1304571
                        view.setBusy(false);
                        dialog.close();
                        that.closeBzDialog();
                        // need to clear the source, else after import workbench does not come up
                        //that.cleanUpScenarioViewsSource();
                    }).fail(function(jqxhr){
                        that.closeBzDialog();
                        sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr, rb.getText("worksheet.import.title.txt"));
                        view.setBusy(false);
                        dialog.close();
                    });
                }
            }
        }, dialog);
        dialog.open();
    },
    pressExportButton: function () {
        var that=this;
        var view = this.getView();
        view.setBusyIndicatorDelay(0);
        view.setBusy(true);

        var rb = sas.stress.wkbench.StressUtil.getBundle();
        var loadSelectSheetsForDownload = that._displaySelectSheetsForDownload();

        var dialog = new sas.hc.m.Dialog({
            title: sas.stress.wkbench.StressUtil.getBundle().getText("worksheet.export.sheet.templates.title.txt"),
            content: loadSelectSheetsForDownload,
            beginButton: loadSelectSheetsForDownload.okButton,
            endButton: loadSelectSheetsForDownload.cancelButton,
            contentHeight: "30rem",
            contentWidth: "62rem",
            afterClose: function () {
                dialog.destroy();
            },
            initialFocus: loadSelectSheetsForDownload.cancelButton,
            resizable: true
        });
        loadSelectSheetsForDownload.attachClosing(function (e) {
            if (e.getParameter("action") === "CANCEL") {
                this.close();
                view.setBusy(false);
            } else {
                var response = e.getParameter("response");
                var selectedSheets = e.getParameter("data");
                if (response !== "Done") {
                    sas.stress.wkbench.StressUtil.handleServerFailure(response);
                    loadSelectSheetsForDownload.setBusy(false);
                } else {
                    loadSelectSheetsForDownload.setBusy(false);
                    this.close();

                    // need to return the selectedSheets some where
                 // make sure the project is upto date..
                    sas.stress.wkbench.StressAPI.verifyProjectStatusWithServer(that.project).done(function (valid) {
                        if (valid === true) {
                            that.openIfRequiredAndUpdateBzDialog(rb.getText("export.excel.busy.indicator.message.txt"));
                            // the iteration
                            var iterationNum = sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.project_iteration_id", that.project.getId()) || that.project.getCurrentIterationNo();
                            // download
                            sas.stress.wkbench.StressAPI.exportSheetsToExcel(that.project.getId(), iterationNum, {sheetIds: selectedSheets, filters: that.project.getFilters()}).done(function () {
                                view.setBusy(false);
                                that.closeBzDialog();
                            }).fail(function (error) {
                                that.closeBzDialog();
                                view.setBusy(false);
                                sas.stress.wkbench.StressUtil.handleServerFailure(error,rb.getText("worksheet.export.title.txt"));
                            });
                        } else {
                            // show the warning dialog asking for refresh before export
                            sas.hc.m.MessageDialog.warning({
                                text: rb.getText("exportToExcel.project.refresh.question.txt"),
                                title: rb.getText("exportToExcel.project.refresh.title"),
                                buttonActions: [sas.hc.m.MessageDialogAction.Yes, sas.hc.m.MessageDialogAction.No],
                                buttonPress: function (event) {
                                    var action = event.getParameter("origin").getMessageDialogAction();
                                    var rb = sas.stress.wkbench.StressUtil.getBundle();
                                    // user answered YES
                                    if (action === undefined) {
                                        jQuery.sap.log.error("Internal method invoked!!!");
                                    }
                                    if (action === sas.hc.m.MessageDialogAction.Yes) {
                                        // we need to refresh
                                        that.refreshProject(false).done(function (project) {
                                            // then export to excel
                                            var iterationNum = sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.project_iteration_id", that.project.getId()) || that.project.getCurrentIterationNo();
                                            sas.stress.wkbench.StressAPI.exportSheetsToExcel(that.project.getId(), iterationNum, {sheetIds: selectedSheets, filters: that.project.getFilters()})
                                                .done(function () {
                                                    view.setBusy(false);
                                                    that.closeBzDialog();
                                                    var rb = sas.stress.wkbench.StressUtil.getBundle();
                                                    // show the message toast
                                                    sap.m.MessageToast.show(rb.getText("refresh.project.successful.msg.txt"), {at: sap.ui.core.Popup.Dock.CenterTop});
                                                })
                                                .fail(function (error) {
                                                    view.setBusy(false);
                                                    that.closeBzDialog();
                                                    sas.hc.m.MessageDialog.error({
                                                        text:rb.getText("excel.generate.failed.message.txt"),
                                                        title:rb.getText("error.window.title")
                                                    });
                                                });
                                        }).fail(function(jqxhr){
                                            view.setBusy(false);
                                            that.closeBzDialog();
                                            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr, rb.getText("worksheet.export.title.txt"));
                                        });
                                    } else {
                                        view.setBusy(false);
                                        that.closeBzDialog();
                                        // show the message toast
                                        sas.hc.m.MessageDialog.warning({
                                            text:rb.getText("exportToExcel.project.refresh.needed.txt"),
                                            title:rb.getText("exportToExcel.cancel.project.refresh.title")
                                        });
                                    }
                                }
                            });
                        }
                    }).fail(function (jqxhr) {
                        view.setBusy(false);
                        sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr,rb.getText("worksheet.export.title.txt"));
                    });
                }
            }
        }, dialog);
        dialog.open();
    },
    pressExportAllButton: function () {
        var that=this;
        var view = this.getView();
        view.setBusyIndicatorDelay(0);
        view.setBusy(true);

        var rb = sas.stress.wkbench.StressUtil.getBundle();
        // make sure the project is upto date..
        sas.stress.wkbench.StressAPI.verifyProjectStatusWithServer(this.project).done(function (valid) {
            if (valid === true) {
                that.openIfRequiredAndUpdateBzDialog(rb.getText("export.excel.busy.indicator.message.txt"));
                // the iteration
                var iterationNum = sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.project_iteration_id", that.project.getId()) || that.project.getCurrentIterationNo();
                // download
                sas.stress.wkbench.StressAPI.exportToExcel(that.project.getId(), iterationNum, that.project.getFilters()).done(function () {
                    view.setBusy(false);
                    that.closeBzDialog();
                }).fail(function (error) {
                    that.closeBzDialog();
                    view.setBusy(false);
                    sas.stress.wkbench.StressUtil.handleServerFailure(error,rb.getText("worksheet.export.title.txt"));
                });
            } else {
                // show the warning dialog asking for refresh before export
                sas.hc.m.MessageDialog.warning({
                    text: rb.getText("exportToExcel.project.refresh.question.txt"),
                    title: rb.getText("exportToExcel.project.refresh.title"),
                    buttonActions: [sas.hc.m.MessageDialogAction.Yes, sas.hc.m.MessageDialogAction.No],
                    buttonPress: function (event) {
                        var action = event.getParameter("origin").getMessageDialogAction();
                        var rb = sas.stress.wkbench.StressUtil.getBundle();
                        // user answered YES
                        if (action === undefined) {
                            jQuery.sap.log.error("Internal method invoked!!!");
                        }
                        if (action === sas.hc.m.MessageDialogAction.Yes) {
                            // we need to refresh
                            that.refreshProject(false).done(function (project) {
                                // then export to excel
                                var iterationNum = sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.project_iteration_id", that.project.getId()) || that.project.getCurrentIterationNo();
                                sas.stress.wkbench.StressAPI.exportToExcel(that.project.getId(), iterationNum, that.project.getFilters())
                                    .done(function () {
                                        view.setBusy(false);
                                        that.closeBzDialog();
                                        var rb = sas.stress.wkbench.StressUtil.getBundle();
                                        // show the message toast
                                        sap.m.MessageToast.show(rb.getText("refresh.project.successful.msg.txt"), {at: sap.ui.core.Popup.Dock.CenterTop});
                                    })
                                    .fail(function (error) {
                                        view.setBusy(false);
                                        that.closeBzDialog();
                                        sas.hc.m.MessageDialog.error({
                                            text:rb.getText("excel.generate.failed.message.txt"),
                                            title:rb.getText("error.window.title")
                                        });
                                    });
                            }).fail(function(jqxhr){
                                view.setBusy(false);
                                that.closeBzDialog();
                                sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr, rb.getText("worksheet.export.title.txt"));
                            });
                        } else {
                            view.setBusy(false);
                            that.closeBzDialog();
                            // show the message toast
                            sas.hc.m.MessageDialog.warning({
                                text:rb.getText("exportToExcel.project.refresh.needed.txt"),
                                title:rb.getText("exportToExcel.cancel.project.refresh.title")
                            });
                        }
                    }
                });
            }
        }).fail(function (jqxhr) {
            view.setBusy(false);
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr,rb.getText("worksheet.export.title.txt"));
        });
    },
    pressRefreshButton: function (e, bSave) {
        var controller = this;
        var rb = sas.stress.wkbench.StressUtil.getBundle();
        var project = this.project;
        // does this project need to be refreshed? the user's un-saved changes would be lost
        sas.stress.wkbench.StressAPI.verifyProjectStatusWithServer(project).done(function (valid) {
            if (!valid) {
                controller.openIfRequiredAndUpdateBzDialog();   // fixes for S1266018
                controller._processAllSheets=true;    // S1371893
                // S1431608 on project refresh and clash.. we need all the data for all the sheets
                controller._loadFreshProject(project,false, true).done(function (freshProject) {
                    var uncommittedPayload = project.getUncommittedPayload();
                    var conflicts = controller._calculateConflicts(freshProject, project, uncommittedPayload);
                    if (conflicts.length) {
                        var existingDialog = sap.ui.getCore().byId("sas.stress.wkbench.view.CellConflictDialog");
                        if (existingDialog) {
                            existingDialog.destroy();
                        }
                        var cellconflictDialog = sap.ui.jsfragment("sas.stress.wkbench.view.CellConflictDialog", controller);
                        var conflictModel = new sas.risk.common.models.RiskJSONModel({
                            modelData: conflicts,
                            saveProject: false,
                            freshProject: freshProject,
                            project: project
                        });
                        cellconflictDialog.setModel(conflictModel);
                        cellconflictDialog.open();
                    } else {
                        freshProject.commit();//make this project the real one
                        controller.refresh(freshProject).done(function () {
                            controller.mergeProject(freshProject, project);
                            //sas.applicationSwitcher.updateDirtyFlag(sas.config.getAppId(), !!uncommittedPayload.length);
                            controller.navChecker.setDirty(!!uncommittedPayload.length);
                            var liveModel = sas.stress.wkbench.ViewManager.getWorkbench().getModel("live");
                            liveModel.setProperty("/toggleButtons", null);
                            liveModel.setProperty("/toggleButtons", !!uncommittedPayload.length);
                            liveModel.checkUpdate(true);//make that cell editor refresh
                            sap.m.MessageToast.show(rb.getText("refresh.project.successful.msg.txt"), {at: sap.ui.core.Popup.Dock.CenterTop});
                        }).fail(function (jqxhr) {
                            /*sas.hc.m.MessageDialog.error({
                                text:rb.getText("error.message"),
                                title:rb.getText("workbench.refresh.project.title.txt")
                            });*/
                            controller.closeBzDialog();
                            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr,rb.getText("workbench.refresh.project.title.txt"));
                        });
                    }
                }).fail(function (jqxhr) {
                    sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr,rb.getText("workbench.refresh.project.title.txt"));
                    controller.closeBzDialog();
                });
            } else {
                sas.hc.m.MessageDialog.information({
                    text:rb.getText("cell.conflictDialog.no.server.changes.txt"),
                    title:rb.getText("cell.conflictDialog.no.server.changes.title.txt")
                });
                controller.closeBzDialog();
            }
        }).fail(function (jqxhr) {
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr,rb.getText("workbench.refresh.project.title.txt"));
            controller.closeBzDialog();
        });
    },
    refreshProject: function (bEnableSave, ignorePreferredSheet) {
        var deferred = $.Deferred();
        var projectId = this.project.getId();
        var that = this;

        // reload the project and wait for the completion
        this._loadFreshProject(this.project,true, ignorePreferredSheet).done(function (freshProject) {
            // the old project has been deleted, delete the slimsheet sources that contain references to the dead objects
            that.cleanUpScenarioViewsSource();
            //Update property in model.. we should lose the changes!!! no saving!!
            var project = sas.stress.wkbench.model.Project.getProject(projectId);
            var liveModel = sas.stress.wkbench.ViewManager.getWorkbench().getModel("live");
            if (liveModel&&!bEnableSave) { //S1395400 if not defined remove the items else leave them be
                liveModel.setProperty("/selected", null);
                //liveModel.setProperty("/previousSheetId", undefined);
                liveModel.checkUpdate(true);//make that cell editor refresh
            }

            that.refresh(project, bEnableSave).done(function () {
                deferred.resolve();
            });
        }).fail(function (jqxhr) {
            deferred.reject(jqxhr);
        });

        return deferred;
    },
    /**
     * this will refresh the sheets and make sure if the project is dirty, reload the data.
     * Thusly, helping in displaying the right data (S1343769)
     *
     * @param afterSave - boolean indicating if the refresh is occurring after a save.. only true when called on successful save
     * @private
     */
    _refreshSheets:function(afterSave){ //S1343769
        var that=this;
        var view = this.getView();
        var liveModel=view.getModel("live");
        var reloadDuetoDiscard=liveModel.getProperty("/reloadDueDiscard");
        var sheetId = view.getSelectedSheetId();
        var sheet = this.project.getSheet(sheetId);

        // preference to user updates before the dependent sheet updates
        var wasProjectUpdated=this.navChecker.getDirty()?this.navChecker.getDirty():this.project.wasProjectUpdatedAfterLastLoad();
        var isDirty=afterSave||reloadDuetoDiscard||wasProjectUpdated;  //S1343769
        var defs=[];

        var info;
        var filteredSheets; // S1371893
        if (that._processAllSheets===false){
            filteredSheets=this._getFilteredSheets();    // fixes for S1276689
        }else{
            filteredSheets=this.project.getSheets();
        }
        // open the busy dialog
        if (this.bzdlg) {
            this.openIfRequiredAndUpdateBzDialog(sheet.getName());
        } else {
            // open the busy dialog
            this.openIfRequiredAndUpdateBzDialog();
        }
        var t0 = performance.now();
        _.each(filteredSheets, function (s) {
            info = s.getInfo();
            if (info&& !s.errored) {
                defs.push(info.getFilters().loadCellData(isDirty));
            }
        });
        if (defs.length) {
            $.when.apply($, defs).done(function () {
                var i;
                for (i = 0; i < filteredSheets.length; i++) {
                    var info = filteredSheets[i].getInfo();//.getSheetScenario(scenarioId);
                    if (info&&!filteredSheets[i].errored) {
                        info.getFilters().getData().processOverlays();
                    }
                }
            });
        }
        var t1= performance.now();
        console.log("Sheet Data Processed in: " + (t1-t0) + " milliseconds");
        if (view.getScenarioEditorView() && sas.stress.wkbench.CellPickerUtil.getOperationMode()!== sas.stress.wkbench.CellPickerUtil.EDIT_MODE){
            var scenarioViewLiveModel=view.getScenarioEditorView().getModel("live");
            if (scenarioViewLiveModel) {
                scenarioViewLiveModel.setProperty("/selected", null);//so that the next line triggers its listeners
                scenarioViewLiveModel.setProperty("/editing", null);
                scenarioViewLiveModel.setProperty("/informula", []);
            }
        }
        liveModel.setProperty("/previousSheetId", sheetId);
        liveModel.setProperty("/reloadDueDiscard",false);
        var iterationId = sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.project_iteration_id", that.project.getId()) || that.project.getCurrentIterationNo();
        sas.stress.wkbench.ViewManager.loadScenarioEditor(sheetId, view.getDefaultFilterId(sheetId), iterationId);
		that._processAllSheets=false;   // S1371893
    },
    cellconflictDialog_mergePressed: function (e) {
        var wkbenchController = this;
        var rb = sas.stress.wkbench.StressUtil.getBundle();
        var dialog = sap.ui.getCore().byId("sas.stress.wkbench.view.CellConflictDialog");
        var model = dialog.getModel();
        var bSave = model.getProperty("/saveProject");
        var freshProject = model.getProperty("/freshProject");
        var project = model.getProperty("/project");
        dialog.close();
        freshProject.commit();//make this project the real one
        if (bSave) {
            wkbenchController.mergeProject(freshProject, project);
            wkbenchController.saveProject(freshProject).done(function () {
                wkbenchController.refresh(freshProject).done(function () {
                    wkbenchController.navChecker.setDirty(false);
                    var liveModel = sas.stress.wkbench.ViewManager.getWorkbench().getModel("live");
                    liveModel.setProperty("/toggleButtons", null);
                    liveModel.setProperty("/toggleButtons", false);
                    liveModel.checkUpdate(true);//make that cell editor refresh

                    var selected=liveModel.getProperty("/selected");
                    wkbenchController.getView().getScenarioEditorView().getController().updateModel(true, selected);
                }).fail(function (jqxhr) {
                    sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr,rb.getText("workbench.conflict.merge.fail.title.txt"));
                    /*sas.hc.m.MessageDialog.error({
                        text:rb.getText("error.message"),
                        title:rb.getText("workbench.conflict.merge.fail.title.txt")
                    });*/
                });
            }).fail(function () {
                sas.hc.m.MessageDialog.error({
                    text:rb.getText("error.message"),
                    title:rb.getText("workbench.conflict.merge.fail.title.txt")
                });
            });
        } else {
            sap.m.MessageToast.show(rb.getText("refresh.project.successful.msg.txt"), {at: sap.ui.core.Popup.Dock.CenterTop});
            wkbenchController.refresh(freshProject).done(function () {
                wkbenchController.mergeProject(freshProject, project);
                wkbenchController.navChecker.setDirty(true);
                wkbenchController.hasUnsavedValues = false;
                var liveModel = sas.stress.wkbench.ViewManager.getWorkbench().getModel("live");
                liveModel.setProperty("/toggleButtons", null);
                liveModel.setProperty("/toggleButtons", true);
                liveModel.checkUpdate(true);//make that cell editor refresh
            }).fail(function (jqxhr) {
                sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr,rb.getText("workbench.conflict.merge.fail.title.txt"));
                /*sas.hc.m.MessageDialog.error({
                    text:rb.getText("error.message"),
                    title:rb.getText("workbench.conflict.merge.fail.title.txt")
                });*/
            });
        }
    },
    cellconflictDialog_discardSelectedPressed: function (e) {
        var wkbenchController = this;
        var rb = sas.stress.wkbench.StressUtil.getBundle();
        var dialog = sap.ui.getCore().byId("sas.stress.wkbench.view.CellConflictDialog");
        var model = dialog.getModel();
        var bSave = model.getProperty("/saveProject");
        var freshProject = model.getProperty("/freshProject");
        var project = model.getProperty("/project");
        dialog.close();

        var table = dialog.getCellConflictsTable();
        var selectedIndices = table.getSelectedIndices();
        var i, conflict, data;
        for (i = 0; i < selectedIndices.length; i++) {
            conflict = table.getContextByIndex(selectedIndices[i]).getObject();
            var othersheet=project.getSheet(conflict.sheetId);
            if(!othersheet.errored) {
                data = othersheet.getInfo().getFilters()/*.getScenario(conflict.scenarioId)*/.getData();
                delete data.ncFormulas[conflict.key];
            }
        }
        var numberKept = table.getRows().length - selectedIndices.length;
        freshProject.commit();//make this project the real one
        if (bSave) {
            wkbenchController.mergeProject(freshProject, project);
            wkbenchController.saveProject(freshProject).done(function () {
                wkbenchController.refresh(freshProject).done(function () {
                    wkbenchController.navChecker.setDirty(false);
                    var liveModel = sas.stress.wkbench.ViewManager.getWorkbench().getModel("live");
                    liveModel.setProperty("/toggleButtons", null);
                    liveModel.setProperty("/toggleButtons", false);
                    liveModel.checkUpdate(true);//make that cell editor refresh

                    var selected=liveModel.getProperty("/selected");
                    wkbenchController.getView().getScenarioEditorView().getController().updateModel(true, selected);
                }).fail(function (jqxhr) {
                    sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr,rb.getText("workbench.conflict.descard.fail.title.txt"));
                    /*sas.hc.m.MessageDialog.error({
                        text:rb.getText("error.message"),
                        title:rb.getText("workbench.conflict.descard.fail.title.txt")
                    });*/
                });
            }).fail(function () {
                sas.hc.m.MessageDialog.error({
                    text:rb.getText("error.message"),
                    title:rb.getText("workbench.conflict.descard.fail.title.txt")
                });
            });
        } else {
            var viewDirty = (!!wkbenchController.hasUnsavedValues || numberKept > 0);
            wkbenchController.refresh(freshProject).done(function () {
                wkbenchController.mergeProject(freshProject, project);
                wkbenchController.navChecker.setDirty(viewDirty);
                var liveModel = sas.stress.wkbench.ViewManager.getWorkbench().getModel("live");
                liveModel.setProperty("/toggleButtons", null);
                liveModel.setProperty("/toggleButtons", viewDirty);
                liveModel.checkUpdate(true);//make that cell editor refresh
                sap.m.MessageToast.show(rb.getText("refresh.project.successful.msg.txt"), {at: sap.ui.core.Popup.Dock.CenterTop});
            }).fail(function (jqxhr) {
                sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr,rb.getText("workbench.conflict.descard.fail.title.txt"));
                /*sas.hc.m.MessageDialog.error({
                    text:rb.getText("error.message"),
                    title:rb.getText("workbench.conflict.descard.fail.title.txt")
                });*/
            });
        }
        wkbenchController.hasUnsavedValues = false;
    },
    cellconflictDialog_cancelPressed: function (e) {
        var dialog = sap.ui.getCore().byId("sas.stress.wkbench.view.CellConflictDialog");
        if (dialog) {
            dialog.close();
        }
    },
    mergeProject: function (freshProject, previousProject) {
        freshProject.mergeUncommittedOverlays(previousProject);//merge unsaved data from old project into fresh project
        previousProject.destroy();//destroy old project
        this.cleanUpScenarioViewsSource(); // fixes for S1274034
        this.project = freshProject;
    },
    /*
     * FIXES for S1274034
     */
    cleanUpScenarioViewsSource:function(){
        var tobeDestroyedScenarioEditors = sas.stress.wkbench.ViewManager.getAllScenarioEditorViews();
        var length = _.size(tobeDestroyedScenarioEditors);
        _.each(tobeDestroyedScenarioEditors, function (scenarioeditorView, key) {
            // destroy everything related to the scenario editor
            var source=scenarioeditorView.slimsheet.getSlimsheetSource();
            if (source) {
                source.destroy();
            }
        });
    },
    _loadFreshProject: function (project,deleteOldProject,ignorePreferredSheet) {
        var that=this;
        var def = $.Deferred();
        var defs=[],info;

        sas.stress.wkbench.model.Project.loadProject(project.getId(), deleteOldProject).done(function (freshProject) {
            //Preload the same scenarios on this fresh project as the ones loaded on the project we tried to save
            var preferredSheetId;
            if (ignorePreferredSheet === undefined || ignorePreferredSheet === false) { // S1431608
                preferredSheetId = sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.sheet_key", freshProject.getId()) ||
                    (that._getFilteredSheets()[0] ? that._getFilteredSheets()[0].getId() : undefined);
            }
            freshProject.loadSheetsData(preferredSheetId,false).done(function(){
                var sheets=that._getFilteredSheets();
                _.each(sheets, function (s) {
                    info = s.getInfo();
                    if (info&&!s.errored){    // fixes for S1297759 and S1347872
                        defs.push(info.getFilters().loadCellData());
                    }
                });
                if (defs.length) {
                    $.when.apply($, defs).done(function () {
                        var i;
                        _.each(sheets, function (s) {
                            info = s.getInfo();
                            if (info && !s.errored) {    // fixes for S1297759 and S1347872
                                info.getFilters().getData().processOverlays();
                            }
                        });
                        def.resolve(freshProject);
                    });
                }else {
                    def.resolve(freshProject);
                }
            }).fail(function(jqxhr) {
                def.reject(jqxhr);
            });
        }).fail(function(jqxhr){
            def.reject(jqxhr);
        });
        return def;
    },
    _calculateConflicts: function (freshProject, project, uncommittedPayload) {
        //Identify conflicts between local project and fresh project
        var i, j, sheet, filters/*scenario*/, overlay, oldValue, entry, data, freshValue, key, filtertokens = "";
        var columnId,rowId,sheetInfoData;
        var timestamp = project.getLastModified();
        var conflicts = [];
        var view = this.getView();
        for (i = 0; i < uncommittedPayload.length; i++) {
            entry = uncommittedPayload[i];
            sheet = freshProject.getSheet(entry.sheetId);
            if (sheet&&!sheet.errored) {
                //scenario=sheet.getInfo().getScenario(entry.scenarioId);
                filters = sheet.getInfo().getFilters();
                data = filters.getData();
                sheetInfoData = sheet.getInfo().getData();
                for (j = 0; j < entry.values.length; j++) {
                    overlay = entry.values[j];
                    oldValue = overlay.value.value;
                    columnId = sheetInfoData.getColLongFromDatakey(overlay.dataKey);
                    rowId = sheetInfoData.getRowLongFromDatakey(overlay.dataKey);
                    key = "$" + columnId + "$" + rowId;
                    freshValue = data.cFormulas[key];
                    if (freshValue && freshValue.timestamp > timestamp) {
                        filtertokens = "[";
                        var tokens = view.tokenizer.getTokens();
                        _.each(tokens, function (token, idx) {
                            if (idx > 0) {
                                filtertokens += ", ";
                            }
                            filtertokens += "\"" + token.getText() + "\"";
                        });
                        filtertokens += "]";

                        var rowIndex = sheet.getInfo().getData().getRowIndexByLongUsingDatakey(overlay.dataKey);
                        var columnIndex = sheet.getInfo().getData().getColShortFromDatakey(overlay.dataKey);
                        var cellref = columnIndex + (rowIndex + 1).toLocaleString();
                        /*column: columnIndex.toLocaleString(),
                         row: (rowIndex + 1).toLocaleString()*/
                        /*overlay.rowId*/
                        conflicts.push({
                            sheetId: sheet.getId(),
                            sheet: sheet.getName(),
                            /*scenarioId:scenario.getId(),
                             scenario:scenario.getName(),*/
                            tokens: filtertokens,
                            cellReference: cellref,
                            user: freshValue.user,
                            timestamp: freshValue.timestamp,
                            localValue: oldValue,
                            serverValue: freshValue.txt,
                            key: key
                        });
                    } else {
                        // there is a value that's changed
                        this.hasUnsavedValues = true;
                    }
                }
            }
        }
        return conflicts;
    },
    _onProjectSave:function(){
        var rb = sas.stress.wkbench.StressUtil.getBundle();
        this.saveProject().done(function(){
            sas.hc.m.MessageToast.show(rb.getText("save.successful.msg.txt"), {at: sap.ui.core.Popup.Dock.CenterTop});
        }).fail(function(jqXHR){
            sas.stress.wkbench.StressUtil.handleServerFailure(jqXHR,rb.getText("workbench.save.project.title.txt"));
        });
    },
    saveProject: function (project) {
        var deferred= $.Deferred();
        project = project || this.project;
        var view = this.getView();

        var that = this;
        // fixes for : no need to send the entity
        sas.stress.wkbench.StressAPI.saveProject(project).done(function () {
            that.navChecker.setDirty(false);
            that.hasUnsavedValues = false;
            var liveModel = view.getModel("live");
            liveModel.setProperty("/toggleButtons", false);
            var selected=liveModel.getProperty("/selected");    // S1431441
            // S1343227 fixes
            // we would need to update all the scenario editors that have been created
            var tobeDestroyedScenarioEditors = sas.stress.wkbench.ViewManager.getAllScenarioEditorViews();
            _.each(tobeDestroyedScenarioEditors, function (scenarioeditorView) {
                scenarioeditorView.getController().clearUnSavedComments();
                scenarioeditorView.getController().updateModel(true, selected); // S1431441
            });
            var slimSheet = liveModel.getProperty("/slimsheet");
            that._refreshSheets(true);  //S1343769.. after save reload the sheets

            slimSheet.rerender();   // need to rerender to remove the drilldownables if any reset
            if (selected) {
                //slimSheet.selectCell(selected.row, selected.column, selected.filterId);
                slimSheet.fireCellSelected({    // S1431441
                    selected:true,
                    row:selected.row,
                    col:selected.column,
                    filter:selected.getSheetFilters()
                });
            }

            deferred.resolve();
        }).fail(function (uncommittedPayload, jqXHR) {
            if (jqXHR !== undefined && jqXHR.status === 412) { 	// if the status === conflict (412) display the resolution dialog
                view.getModel("live").setProperty("/toggleButtons", true);
                var updatedByUser = "unknown";
                if (jqXHR !== undefined) {
                    var error = JSON.parse(jqXHR.responseText);
                    updatedByUser = error.details[0];
                }
                that.openIfRequiredAndUpdateBzDialog();
                that._loadFreshProject(project,false,true).done(function (freshProject) {   // ignore loading a sheet.. load all
                    var conflicts = that._calculateConflicts(freshProject, project, uncommittedPayload);
                    var conflictDialogView = sas.stress.wkbench.ViewManager.loadConflictDialogView(project, freshProject, conflicts, updatedByUser);
                    var conflictDialog = new sas.hc.m.Dialog({
                        id: "ConflictProjectDialog",
                        title: sas.stress.wkbench.StressUtil.getBundle().getText("projectConflictDialog.title.txt"),
                        content: conflictDialogView,
                        beginButton: conflictDialogView.okButton,
                        endButton: conflictDialogView.cancelButton,
                        afterClose: function () {
                            this.destroy();
                        },
                        initialFocus: conflictDialogView.okButton
                        // S1389044.. no need for constant height and width
                    });
                    conflictDialogView.attachClosing(function () {
                        conflictDialog.close();
                    }, conflictDialog);
                    that.closeBzDialog();
                    conflictDialog.open();
                }).fail(function (jqxhr) {
                    that.closeBzDialog();
                    deferred.reject(jqXHR);
                });
            } else {
                deferred.reject(jqXHR);
            }

            deferred.resolve();
        });

        return deferred;
    },
    _addTaskActions: function (actions) {
        var rb=sas.stress.wkbench.StressUtil.getBundle();
        var view = this.getView();
        var taskActions=actions||view.getModel().getProperty("/actions");

        view.actions.removeAllItems();
        var i = 0;
        _.each(actions, function (action) {
            var item = new sas.hc.m.MenuItem({
                key: i++,
                text: rb.getText(action.getLabel()),
                visible:action.getType()==="STATUS_UPDATE",
                press:function(e){
                    var key=e.getSource().getProperty("key");
                    var taskAction=taskActions[Number(key)];
                    var project=taskAction.getTask().getProject();

                    var callBack=function(comment){
                        sas.stress.wkbench.StressAPI.saveTaskStatus(taskAction,comment).done(function(){
                            sas.stress.wkbench.ViewManager.loadHomePage(project);
                        }).fail(function(jqxhr){    // S1388587
                            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
                        });
                    };
                    // call REST API
                    sas.stress.wkbench.StressAPI.loadTaskTransitionChangeSummary(taskAction).done(function(changes){
                        sas.stress.wkbench.widget.CommentDialog.show(rb.getText(taskAction.getLabel()), callBack, undefined, changes);
                    }).fail(function(jqxhr){
                        sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
                    });
                },
                /* if the view is dirty we need to disable the buttons so that life
                 is easier for us developers and does not give wrong indication to
                 the user
                 */
                enabled: {
                    // the parent propagation should help finding the property
                    path: "live>/toggleButtons",
                    formatter: function (enabled) {
                        return enabled ? false : true;
                    }
                }
            });

            view.actions.addItem(item);
        });

        return actions?(actions.length>0?true:false):false;
    },
    _addRemoveSheetTemplatesPress: function(){
        var self = this;
        var selectedCategoryKey = this.getView().getSelectedSheetCategoryKey();
        var addRemoveSheetTemplates = sas.stress.wkbench.ViewManager.loadAddRemoveSheetTemplates({selectedCategoryKey: selectedCategoryKey });
        var rb = sas.stress.wkbench.StressUtil.getBundle();
        var projectName = this.project.getName();


        var dialog = new sas.hc.m.Dialog({
            title: sas.stress.wkbench.StressUtil.getBundle().getText("worksheet.edit.sheet.templates.title.txt"),
            content: addRemoveSheetTemplates,
            beginButton: addRemoveSheetTemplates.okButton,
            endButton: addRemoveSheetTemplates.cancelButton,
            contentHeight: "38rem",
            contentWidth: "50rem",
            afterClose: function () {
                dialog.destroy();
            },
            initialFocus: addRemoveSheetTemplates.cancelButton,
            resizable: true
        });
        addRemoveSheetTemplates.attachClosing(function (e) {
            if (e.getParameter("action") === "CANCEL") {
                this.close();
            } else {
                var response = e.getParameter("response");
                var project = e.getParameter("data");
                if (response !== "Done") {
                    sas.stress.wkbench.StressUtil.handleServerFailure(response);
                    addRemoveSheetTemplates.setBusy(false);
                } else {
                    addRemoveSheetTemplates.setBusy(false);
                    this.close();

                    self.openIfRequiredAndUpdateBzDialog();
                    // remove the model.. we'll need to re-add it
                    self.getView().setModel(undefined);
                    // bEnableSave = undefined and do not ignorePreferredSheetId
                    self.refreshProject(undefined, false).done(function(){
                        self.closeBzDialog();
                    }).fail(function(e){
                        self.closeBzDialog();
                        if (e===sas.stress.wkbench.model.Sheet.NO_DATA){
                            if (!sas.stress.wkbench.CapabilityManager.getInstance().isCapableOf(sas.stress.wkbench.CapabilityManager.PROJECT.UPDATE)) {
                                var messages = [];
                                messages.push(rb.getText("list.bullet.txt") + rb.getText("workbench.error.no.templates.txt"));
                                messages.push(rb.getText("list.bullet.txt") + rb.getText("workbench.error.no.permissions.txt"));
                                messages.push(rb.getText("list.bullet.txt") + rb.getText("workbench.error.no.data.txt"));

                                //message+=rb.getText("contact.sys.admin.txt");
                                var json = {
                                    message: rb.getText("workbench.error.nosheets.main.fmt", projectName),
                                    details: messages
                                };
                                sas.stress.wkbench.StressUtil.handleServerFailure(json, rb.getText("workbench.load.title.txt"));

                                sas.stress.wkbench.ViewManager.loadHomePage(project);
                            }else{
                                // we would need to destory all the scenario editors that have been created
                                var tobeDestroyedScenarioEditors = sas.stress.wkbench.ViewManager.getAllScenarioEditorViews();
                                _.each(tobeDestroyedScenarioEditors, function (scenarioeditorView) {
                                    // destroy everything related to the scenario editor
                                    scenarioeditorView.destroy();
                                });

                                self.getView().tokenizer.destroyTokens();
                                self._refreshModel(project, self.myTasks, false, rb);
                            }
                        }else {
                            sas.stress.wkbench.StressUtil.handleServerFailure(e);
                        }
                    });
                }
            }
        }, dialog);
        dialog.open();
    },

    _displaySelectSheetsForDownload: function() {
        var self = this;
        var selectedCategoryKey = this.getView().getSelectedSheetCategoryKey();
        var selectedWorksheetId = this.getView().getSelectedSheetId();
        var loadSelectSheetsForDownload = sas.stress.wkbench.ViewManager.loadSelectSheetsForDownload({selectedCategoryKey: selectedCategoryKey, currentWorksheetId: selectedWorksheetId });

        return loadSelectSheetsForDownload;
    }
});