jQuery.sap.require("sas.hc.ui.commons.facetFilter.BaseFacet");
jQuery.sap.require("sas.risk.common.models.RiskJSONModel");
jQuery.sap.require("sas.stress.wkbench.ViewManager");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.hc.m.FlexItemData");

sas.hc.ui.commons.facetFilter.BaseFacet.extend("sas.stress.wkbench.widget.DescriptionFacet", {
    metadata: {
        properties: {
            "searchThreshold": {type: "int", defaultValue: 20},
            "showZeroCounts": {type: "boolean", defaultValue: true},
            "key": {type: "string", defaultValue: null},
            "showSelectedItemsFirst": {type: "boolean", defaultValue: false}
        }
    },

    renderer: "sas.hc.ui.commons.facetFilter.FacetRenderer",

    rb: sap.ui.getCore().getLibraryResourceBundle("sas.hc.ui.commons")

});

sas.stress.wkbench.widget.DescriptionFacet.prototype.init = function () {
    if (sas.hc.ui.commons.facetFilter.BaseFacet.prototype.init)
        sas.hc.ui.commons.facetFilter.BaseFacet.prototype.init.apply(this, arguments);

    var self = this;
    this.setUsesFacetItems(true);

    this._selectedValues = [];

    var searchField = new sas.hc.m.SearchField({
        width: "100%",
        visible: {
            path: "/items",
            formatter: function (value) {
                return $.isArray(value) && value.length > self.getSearchThreshold();
            }
        },
        layoutData: new sas.hc.m.FlexItemData({
            growFactor: 0,
            shrinkFactor: 0
        }),
        liveChange: [this._onSearchChange, this],
        filterMode: true
    });

    this.list = new sas.hc.m.List({
        width: "100%",
        layoutData: new sas.hc.m.FlexItemData({
            growFactor: 1,
            shrinkFactor: 1
        }),
        includeItemInSelection: true,
        showNoData: false,
        mode: sap.m.ListMode.SingleSelectLeft,
        items: {
            path: "/items",
            template: new sas.hc.m.StandardListItem({
                selected: "{selected}",
                title: "{value}",
                description: "{path}",
                info: "{liKey}",
                tooltip: {
                    parts: [
                        "value",
                        "path",
                        "liKey",
                        "format",	// S1472103
                        "variableShockInfo"
                    ],
                    formatter: function (value, path, liKey, format,variableShockInfo ) { // S1472103
                        var rb=sas.stress.wkbench.StressUtil.getBundle();
                        var shockInfo=self._getShockText(variableShockInfo);
                        if (shockInfo.length>0){
                            return path ? rb.getText("slimsheet.td.with.shocks.tip.fmt.txt", [path, liKey, format, shockInfo]): rb.getText("slimsheet.td.tip.fmt.txt", [value, liKey, format, variableShockInfo]);	// S1472103
                        }
                        return path ? rb.getText("slimsheet.td.tip.fmt.txt", [path, liKey, format]): rb.getText("slimsheet.td.tip.fmt.txt", [value, liKey, format]);	// S1472103
                    }
                }
            }),
            templateShareable:false
        },
        selectionChange: [this._onSelectionChange, this]
    }).addStyleClass("descriptionFacet");

    var model = new sas.risk.common.models.RiskJSONModel();
    this.setModel(model);

    var scroll = new sas.hc.m.ScrollContainer({
        width: "100%",
        horizontal: false,
        vertical: true,
        content: this.list
    }).addStyleClass("sasDescriptionFacet-scroll");

    var layout = new sas.hc.m.VBox({
        items: [searchField, scroll]
    });

    this.getSection().addContent(layout);
};

sas.stress.wkbench.widget.DescriptionFacet.prototype.reset = function () {
    var facetItems = this.getModel().getProperty("/items");
    if ($.isArray(facetItems)) {
        $.each(facetItems, function (index, item) {
            item.selected = false;
        });
    }

    this._selectedValues = [];
};

sas.stress.wkbench.widget.DescriptionFacet.prototype.onFacetItemsChange = function () {
    var model = this.getModel();
    var newSelectedValues = [];

    var items = this.getFacetItems();
    if ($.isArray(items)) {
        $.each(items, function (index, item) {
            if (item.selected) {
                newSelectedValues.push(item.value);
            }
        });
    }

    // prevent list from remembering selections - fixes issues
    // where checkboxes remain selected after all list items
    // are removed and then added back.
    this.list.setRememberSelections(false);

    this._selectedValues = newSelectedValues;
    model.setProperty("/items", items);
    model.setSizeLimit(items.length);

    this.list.setRememberSelections(true);
};

sas.stress.wkbench.widget.DescriptionFacet.prototype.isMatch = function (object) {
    var match = true;

    if (this.hasFilterCriteria()) {
        var filterValue = this.getFieldValue(object);
        match = this._selectedValues.indexOf(filterValue) >= 0;
    }

    return match;
};

sas.stress.wkbench.widget.DescriptionFacet.prototype.hasFilterCriteria = function () {
    return $.isArray(this._selectedValues) && this._selectedValues.length > 0;
};

sas.stress.wkbench.widget.DescriptionFacet.prototype.hasToken = function (token) {
    return this.hasFilterCriteria() && token.getFacetId() === this.getId() &&
        this._selectedValues.indexOf(token.getKey()) >= 0;
};

sas.stress.wkbench.widget.DescriptionFacet.prototype._sortFacetItems = function (items) {
};
/*We donot want things to be sorted... therefore, ignore the sorting please
 * fixes for S1228288.. Srini
 * David, G will need to investigate further if need be
 */
sas.stress.wkbench.widget.DescriptionFacet.prototype.sortFacetItems = function (items) {
};

sas.stress.wkbench.widget.DescriptionFacet.prototype.getTokens = function () {
    if (this.hasFilterCriteria()) {
        var self = this;
        var tokens = [];
        var rb=sas.stress.wkbench.StressUtil.getBundle();

        // defect S1303232.. we use the liKeys now, cannot use that, therefore, filter out the selected filters
        //  and use the liKey and value property off the selected filters for the tokens
        var facetItems = this.getModel().getProperty("/items");
        var selectedValues=_.filter(facetItems, function (item) {
            var indexOf=self._selectedValues.indexOf(item.liKey);
            if (indexOf>=0) {
                return true;
            }
            return false;
        });

        _.each(selectedValues, function (value) {
            var newToken = new sas.hc.ui.commons.facetFilter.FacetToken({
                facetId: self.getId(),
                key: value.liKey,
                text: value.value
            });
            newToken.variableShockInfo=value.variableShockInfo;
            newToken.format=value.format;	// S1472103
            self.setTokenTooltip(newToken);
            tokens.push(newToken);
        });
        return tokens;
    }
    return null;
};

sas.stress.wkbench.widget.DescriptionFacet.prototype.resetToken = function (token) {
    if (token.getFacetId() === this.getId() && $.isArray(this._selectedValues)) {
        var index = this._selectedValues.indexOf(token.getKey());
        if (index >= 0) {
            this._selectedValues.splice(index, 1);

            var facetItems = this.getModel().getProperty("/items");
            if ($.isArray(facetItems)) {
                $.each(facetItems, function (index, item) {
                    if (item.liKey === token.getKey()) {
                        item.selected = false;
                        return false;
                    }
                    return true;
                });
            }

            this.getModel().refresh();
            this.fireChange();
        }
    }
};

sas.stress.wkbench.widget.DescriptionFacet.prototype.refreshCounts = function () {
    this.getModel().refresh();
};

sas.stress.wkbench.widget.DescriptionFacet.prototype.getSelected = function () {
    var selectedItems = [];
    var items = this.getFacetItems();
    if ($.isArray(items)) {
        $.each(items, function (index, item) {
            if (item.selected) {
                selectedItems.push(item);
            }
        });
    }
    return selectedItems;
};

sas.stress.wkbench.widget.DescriptionFacet.prototype.getQuery = function () {
    var query = null;

    if ($.isArray(this._selectedValues) && this._selectedValues.length > 0) {
        var valuesString = "";
        $.each(this._selectedValues, function (index, value) {
            if (valuesString.length > 0) {
                valuesString += ",";
            }
            valuesString += ("\"" + value + "\"");
        });

        query = "in(" + this.getFieldName() + "," + valuesString + ")";
    }

    return query;
};

sas.stress.wkbench.widget.DescriptionFacet.prototype.setQuery = function (query) {
};

sas.stress.wkbench.widget.DescriptionFacet.prototype._onSearchChange = function (evt) {
    var filter = new sap.ui.model.Filter({
        path: "value",
        operator: sap.ui.model.FilterOperator.Contains,
        value1: evt.getParameter("newValue")
    });

    this.list.getBinding("items").filter([filter]);
};

/*
 * fixes for: S1229696
 * The default method for selection change on the list.
 *
 * The apply button will not be enabled if the user reverts the selections back to the original
 * before hitting "apply"
 */
sas.stress.wkbench.widget.DescriptionFacet.prototype._onSelectionChange = function (evt) {
    var self = this;
    var model = this.getModel();
    var facetItems = model.getProperty("/items");
    // keep internl list of selected values to improve filtering performance
    var newSelectedValues = [];

    newSelectedValues.push(evt.getParameter("listItem").getProperty("info"));  // faster look at the event parameter to get the value
    // remove the duplicates.. look at self._selectedValues and the newSelectedValues
    newSelectedValues = _.reduce(newSelectedValues, function (memo, selected) {
        var found = _.find(self._selectedValues, function (previous) {
            return previous === selected;
        });
        if (!found) {
            memo.push(selected);
        }
        return memo;
    }, []);

    // fixes for S1229696
    // the apply button is enabled only if there is change... if the user reverts before applying the button should be disabled
    if (newSelectedValues.length) {
        if (!_.isEqual(this._selectedValues, newSelectedValues)) {
            // if there are any previously selected items un-select them, each is a radio-button at most one
            _.each(facetItems, function (item) {
                if (item.liKey !== newSelectedValues[0] && item.selected) {
                    item.selected = false;
                }
            });
        }
    }
    // see if the button can be enabled fixes for S1266122
    sas.stress.wkbench.ViewManager.getWorkbench().getScenarioEditorView().getController()._enableButton();
};

/*
 * fixes for: S1229696
 * This method will be invoked for the all the facets that are selected after the apply is succesful
 */
sas.stress.wkbench.widget.DescriptionFacet.prototype._fireSelectionChange = function (evt) {
    var self = this;
    var model = this.getModel();
    var facetItems = model.getProperty("/items");
    // keep internl list of selected values to improve filtering performance
    var newSelectedValues = [];
    var selectedItem=evt.listItem||evt.getParameter("listItem");

    if ($.isArray(facetItems)) {
        $.each(facetItems, function (index, item) {
            if (item.selected) {
                if (item.liKey===selectedItem.liKey){
                    newSelectedValues.push(item.liKey);
                } else{
                    var index = self._selectedValues.indexOf(item.liKey);
                    // fixes for S1239083
                    if (index >= 0) {
                        self._selectedValues.splice(index, 1); // remove the item if it exists
                        item.selected=false;
                    }
                }
            }
        });
    }
    // prevent list from remembering selections - fixes issues
    // where checkboxes remain selected after all list items
    // are removed and then added back.
    if (newSelectedValues.length > 0) {
        this._selectedValues = newSelectedValues;
    }

    this.fireChange({});
};

/*
 * Enhance the token tooltip to display the datakey too.
 */
sas.stress.wkbench.widget.DescriptionFacet.prototype.setTokenTooltip = function (token) {
    var rb=sas.stress.wkbench.StressUtil.getBundle();
    var shockInfo=token.variableShockInfo;
    if (token.variableShockInfo===undefined||token.variableShockInfo===null){
        token.setTooltip(rb.getText("slimsheet.td.tip.fmt.txt",[token.getText(), token.getKey(), token.format])); // S1472103
    }else{
        token.setTooltip(rb.getText("slimsheet.td.with.shocks.tip.fmt.txt", [
            token.getText(), token.getKey(), token.format, this._getShockText(token.variableShockInfo)	 // S1472103
        ]));
    }
};

sas.stress.wkbench.widget.DescriptionFacet.prototype._getShockText=function(shockInfo){
    var shockVars="";
    if (shockInfo===undefined||shockInfo===null){
        return shockVars ;
    }else{
        switch(shockInfo.length){
            case 1:
                shockVars="\u0009"+shockInfo;
                break;
            default:
                var allVars="";
                _.each(shockInfo,function(vars){
                    if (allVars.length>0){
                        allVars+="\n";
                    }
                    allVars+="\u0009"+vars;
                });
                shockVars=allVars;
                break;
        }
        return shockVars;
    }
}