jQuery.sap.require("sas.hc.ui.commons.facetFilter.BaseFilterProvider");
jQuery.sap.require("sas.hc.ui.commons.facetFilter.FilterUtil");

sas.hc.ui.commons.facetFilter.BaseFilterProvider.extend("sas.stress.wkbench.widget.DescriptionModelFilterProvider", {
    metadata: {
        properties: {
            "dataPath": {type: "string", defaultValue: null},
            "searchFieldOperator": {type: "object", defaultValue: sap.ui.model.FilterOperator.Contains},
            "searchFieldPaths": {type: "string[]", defaultValue: null}
        }
    },

    logger: sas.log.getLogger("sas.stress.wkbench.widget.DescriptionModelFilterProvider", sas.log.Level.ERROR)
});

sas.stress.wkbench.widget.DescriptionModelFilterProvider.prototype.setDataPath = function(value) {
    this.setProperty("dataPath", value, true);

    if(this._dataPathBinding) {
        this._dataPathBinding.detachChange(this._modelChanged, this);
        this._dataPathBinding = null;
    }

    this._modelChanged();
};

sas.stress.wkbench.widget.DescriptionModelFilterProvider.prototype.setModel = function(oModel, sName) {
    if(sas.hc.ui.commons.facetFilter.BaseFilterProvider.prototype.setModel)
        sas.hc.ui.commons.facetFilter.BaseFilterProvider.prototype.setModel.apply(this, arguments);

    this._modelChanged();
};

sas.stress.wkbench.widget.DescriptionModelFilterProvider.prototype.applyFilter = function() {
    if(!this.getModel() || !this.getDataPath()) return;

    var self = this;
    var filteredData = [];
    var facetMap = {};

    var matchesAllOtherFacetIds = function(facetId, matchingFacetIds) {
        var rval = true;
        $.each(self.getFacets(), function(index, facet) {
            if(facet.getId() !== facetId && matchingFacetIds.indexOf(facet.getId()) === -1) {
                rval = false;
                return false;
            }
            return true;
        });
        return rval;
    };

    var incrementCount = function(facet, object) {
        var map = facetMap[facet.getId()];
        map.totalCount++;

        var value = facet.getFieldValue(object);
        if(map.countMap.hasOwnProperty(value)) {
            map.countMap[value]++;
        } else {
            map.countMap[value] = 1;
        }
    };

    // populate map for each facet
    $.each(this.getFacets(), function(index, facet) {
        var newMap = {
            countMap: {},
            totalCount: 0
        };
        facetMap[facet.getId()] = newMap;
    });

    // create list of filters for the SearchField and any external filters
    var filters = [];
    var searchField = self.getSearchFieldControl();
    if(searchField) {
        var searchFilter = this.createSearchFieldFilter(searchField.getValue());
        if(searchFilter) {
            filters.push(searchFilter);
        }
    }
    if($.isArray(this.getExternalFilters())) {
        filters = filters.concat(this.getExternalFilters());
    }

    // iterate through all the objects and determine which ones
    // should be displayed based on the current facet values.
    $.each(this._unfilteredData, function(objectIndex, object) {
        var matchingFacetIds = [];

        // apply filters if they exist
        if(filters.length > 0 && !sas.hc.ui.commons.facetFilter.
            FilterUtil.applyFilters(object, filters)) {
            // object did not match external filters so continue
            return true;
        }

        // see which facets are a match (i.e. show the object)
        $.each(self.getFacets(), function(index, facet) {
            if(facet.isMatch(object)) {
                matchingFacetIds.push(facet.getId());
            }
        });

        // if all the facets match, then add object to the filtered data
        if(matchingFacetIds.length === self.getFacets().length) {
            filteredData.push(object);
        }

        // for a given facet, we increment the count for the objects
        // that are a match on all the other facets.
        $.each(self.getFacets(), function(index, facet) {
            if(matchesAllOtherFacetIds(facet.getId(), matchingFacetIds)) {
                incrementCount(facet, object);
            }
        });
    });

    var i = 0;

    // update counts on all facets
    $.each(this.getFacets(), function(facetIndex, facet) {
        var map = facetMap[facet.getId()];
        facet.setTotalCount(map.totalCount);
        if(facet.getUsesFacetItems()) {
            var items = facet.getFacetItems();
            if($.isArray(items)) {
                $.each(items, function(index, item) {
                    if (self._unfilteredData[i]) {
                        item.path = self._unfilteredData[i].path;
                        item.liKey = self._unfilteredData[i].liKey;
                    }
                    if(map.countMap.hasOwnProperty(item.value)) {
                        item.count = map.countMap[item.value];
                    } else {
                        item.count = 0;
                    }
                    i++;
                });
            }
        }

        facet.refreshCounts();
    });

    // update the model with the filtered list of objects
    this._ignoreDataChange = true;
    this.getModel().setProperty(this.getDataPath(), filteredData);
    this._ignoreDataChange = false;

    var panel = this.getFacetPanelControl();
    if(panel) {
        panel.setTotalCount(filteredData.length);
    }

    this.fireFilterApplied();
};

sas.stress.wkbench.widget.DescriptionModelFilterProvider.prototype.createSearchFieldFilter = function(value) {
    var self = this;
    var rval;
    var filters = [];

    if($.isArray(this.getSearchFieldPaths())) {
        $.each(this.getSearchFieldPaths(), function(index, path) {
            var filter = new sap.ui.model.Filter({
                path: path,
                operator: self.getSearchFieldOperator(),
                value1: value
            });
            filters.push(filter);
        });
    }

    if(filters.length > 1) {
        // for multiple filters, we need to create a "multi-filter"
        // that is an "or" of all the filters.
        rval = new sap.ui.model.Filter({
            filters: filters,
            and: false
        });
    } else if(filters.length === 1) {
        rval = filters[0];
    }

    return rval;
};

sas.stress.wkbench.widget.DescriptionModelFilterProvider.prototype._modelChanged = function() {
    if(this._ignoreDataChange || !this.getDataPath() || !this.getModel()) return;

    var model = this.getModel();

    var tmp = model.getProperty(this.getDataPath()) || [];
    if($.isArray(tmp)) {
        this._unfilteredData = tmp;
    } else {
        this.logger.warning("_modelChanged: dataPath does not point to an array.");
        this._unfilteredData = [];
    }

    // update the facet items and apply the filter
    this._updateFacetItems();

    // create a binding so we know when the data in the model changes
    if(!this._dataPathBinding) {
        this._dataPathBinding = model.bindList(this.getDataPath());
        this._dataPathBinding.attachChange(this._modelChanged, this);
    }
};

sas.stress.wkbench.widget.DescriptionModelFilterProvider.prototype._updateFacetItems = function() {
    var self = this;
    var selectedValuesMap = {};
    var facetMap = {};

    var addFacetItem = function(facetId, value, liKey, format, variableShockInfo) {	// S1472103
        var map = facetMap[facetId];
        if(!map) {
            map = {
                values: [],
                items: []
            };
            facetMap[facetId] = map;
        }

        //if(map.values.indexOf(value) === -1) {    // do not ignore duplicates see S1303232
        map.values.push(value);
        map.items.push({
            value: value,
            count: 0,
            selected: isSelectedValue(facetId, value),
            format: format,	// S1472103
            variableShockInfo:variableShockInfo
        });
        //}
    };

    var addSelectedValue = function(facetId, value) {
        var values = selectedValuesMap[facetId];
        if(!values) {
            values = [];
            selectedValuesMap[facetId] = values;
        }
        values.push(value);
    };

    var isSelectedValue = function(facetId, value) {
        var values = selectedValuesMap[facetId];
        if(values) {
            return values.indexOf(value) >= 0;
        }
        return false;
    };

    // build map of currently selected values for each facet
    $.each(this.getFacets(), function(facetIndex, facet) {
        if(facet.getUsesFacetItems()) {
            if($.isArray(facet.getFacetItems())) {
                $.each(facet.getFacetItems(), function(index, item) {
                    if(item.selected) {
                        addSelectedValue(facet.getId(), item.value);
                    }
                });
            }
        }
    });

    // iterate through the model objects and create facet items for each facet
    $.each(this._unfilteredData, function(objectIndex, object) {
        $.each(self.getFacets(), function(index, facet) {
            if(facet.getUsesFacetItems()) {
                var tmp = facet.getFieldValue(object);
                if(tmp !== undefined && tmp !== null) {
                    addFacetItem(facet.getId(), tmp, object.liKey, object.format, object.variableShockInfo);	// S1472103
                }
            }
        });
    });

    // now set the facet items on each facet
    $.each(this.getFacets(), function(index, facet) {
        if(facet.getUsesFacetItems()) {
            var map = facetMap[facet.getId()];
            facet.setFacetItems(!!map ? map.items : []);
        }
    });

    // apply the filter so the counts are updated
    this.applyFilter();
};
